package VehicleDynamics "Package for vehicle dynamics modelling"
package SI = Modelica.SIunits;


annotation (
  Dymola(checkSum="148749709:2164628697"),
  Settings(
    NewStateSelection=true, 
    Evaluate=true, 
    OutputCPUtime=true, 
    Hidden(SupportConstrain=true)), 
  Coordsys(
    extent=[0, 0; 440, 456], 
    grid=[2, 2], 
    component=[20, 20]), 
  Window(
    x=0.45, 
    y=0.01, 
    width=0.35, 
    height=0.49, 
    library=1, 
    autolayout=1), 
  Icon, 
  Diagram, 
  Documentation(info="<HTML>
<H1>VehicleDynamics Library</H1>
<p> This library is used to model <b>vehicle
chassis</b>. The library is divided into suspensions and
components. Suspensions are complete front and rear suspensions
ready to use, e.g. MacPherson, Double wishbone and Multilink. The
components are used to build these suspensions, e.g. struts,
wishbones, and anti-roll linkages. Below, the model structure, its
hierarchical levels and the parametrisation are explained. There are 
also some predefined chassis that serves as examples of how the parts 
can be gathered, in figure 1 and figure 2, a BMW 3-series and a Dallara 
F3 are shown, respecitvely.
</p>
<p>
<IMG SRC=\"../Images/BMW.gif\" ALT=\"BMW\"> </p><p><b>Figure
1</b> A BMW 3-series sedan.</p>
<p>
<IMG SRC=\"../Images/Dallara.gif\" ALT=\"Dallara\"> </p><p><b>Figure
2</b> A Dallara formula 3 chassis.</p>
<H2>\"Modelling grammar\"</H2>
<H3> The structure of a chassis model</H3> <p>The Chassis library
is based on the ModelicaAdditions.MultiBody library and thus, the
Car models must follow the tree structure requirements presented
above. To handle this the direction of the car modelling is chosen
to go from chassis towards the ground via the wheels as
illustrated in figure 3. Since the model must start with an
inertial system, in this case the ground, there must be an
additional connection from ground to chassis. This is achieved
with a free motion joint.</p>
<p> This way of modelling may seem awkward but has some important
advantages: </p>
<ul>
<li> The car can be modelled the same way it is built up. Start with
chassis and add suspensions, anti roll linkages, dampers etc. All
this can then be tested without wheels and tyres by simply
attaching the model directly to the ground.
</li>
<li>All suspensions have the same direction in the tree, which make
them replaceable. This would not have been the case if the tree
direction had been through one wheel towards the car and then back
through the other wheels.
</li>
<li>The number of states can be reduced in a convenient way. For
example could a constant speed test be done by simply modifying
the free motion joint. 
</p>
<p> <IMG SRC=\"../Images/structure.png\" ALT=\"structure\"> </p><p><b>Figure
3</b> A complete chassis model. The tree structure starts with the
ground element. Via the freeMotion element, the Body Geometric Reference (BGR) of the car's
coordinate system is reached. From BGR, the tree is defined
towards the wheels and the road.</li>
</ul>
<H3>Hierarchical levels </H3> <p>The library is based on a
hierarchical structure within the total model. This is done to
give a clear overview of the model and simplify reusage of
components and suspensions. The hierarchy contains the following
levels that can bee seen in figure 4:</p>
<p>
<b>A. Vehicle level.</b> 
</p><p>
<b>B. Chassis level.</b>
</p><p>
<b>C. Suspension level.</b>
</p><p>
<b>D.  Component level.</b>
</p>
<p>
<IMG SRC=\"../Images/hierarchy.png\" ALT=\"hierarchy\"> </p><p><b>Figure
4</b> The hierarchical levels of a vehicle model. <b>a</b>, the
vehicle level, <b>b</b>, the chassis level, <b>c</b>, the suspension level, <b>c</b> the component level and <b>d</b>, the elementary level.</p>
<p><b>Vehicle level</b> The total model of the vehicle/car contains
typically a chassis model. Additionally, models for drivers,
power-trains, aerodynamics etc. can be added.</p>
<p><b>Chassis level</b> Within the chassis level, a complete
chassis is built up, typically with a front and a rear suspension,
wheels and a body</p>
<p><b>Suspension level</b> The idea with the suspension level is
to make it easy to reconfigure a car by just swapping suspension
and therefore, all suspension should share the same basic
interface, i.e. one MBS-cut for the connection to the body. There
should also be an MBS cut for each wheel (normally two) that is to
be connected to the suspension. Additionally, there may be some
extra connectors depending on the suspension. For example will a
steerable suspension also have a connector for a steering wheel. All availible Suspensions are gathered under Chassis.Suspensions.
</p>
<p><b>Component level</b> With the component level, the
foundation for efficient reuse of vehicle models is laid.
Components like a-arms, bushings, MacPherson struts, trailing arms,
multilinks, anti roll linkages, rack steerings etc. are available.
These components are based on the Modelica and ModelicaAdditions
libraries as well as some special models found under Utilities.  </p>
<H3>Parameterisation</H3><p> To specify a linkage, the geometry has
to be defined. Additionally, the mass and inertia properties of
the parts within the linkage can be defined. For an understandable
parameterisation of these properties, a systematic definition of
the parameter names is necessary.</p>
<b>Nomenclature </b><p>The geometry is mainly defied by the
connection joint locations, connection points. Additionally, the
direction(s) of a joints' degree(s) of freedom must be given if it
is (are) not defined by the connection joint points. The geometry
parameters are named according to <pre>
  <b>[geometry parameter]=[property][connection point]_[wheel no]
  [connection point]=[part 1][part 2][part n]</b>
</pre> while the mass and inertia properties are component specific and
are thus named according to<pre>
   <b>[component parameter]=[property][part]_[wheel no]</b>
</pre></p><p>The properties are named according to <pre>
  <b>[property]</b>
   <b>r</b>    - location
   <b>n</b>    - direction of rotation or translation
   <b>m</b>    - mass
   <b>rcm</b>  - location of centre of mass
   <b>c</b>    - stiffness
   <b>d</b>    - damping
   <b>f</b>    - force
   <b>t</b>    - torque
   <b>i</b>    - inertia element, (gear) ratio
   <b>q0</b>   - Relative offset of the unaffected state for spring/damper/strut,
          compared to the state given by the geometrical parameters. 
          Typically, if a strut is mounted in at rA and rB, then it Is 
          unaffected length is |rA-rB|+q0. 
   <b>qInit</b>- Initial value
</pre>and the parts are named according to<pre> 
  <b>[part]</b>
   <b>C</b> - chassis 
   <b>R</b> - steering (rack) 
   <b>U</b> - upright, part that holds the wheel 
   <b>P</b> - pivot element,used for example in Watt's linkage 
   <b>S</b> - strut, 1D force element 
   <b>L</b> - link or rod
   <b>B</b> - body or bushing
   <b>A</b> - antiroll 
   <b>X</b> - Undefined part/General part, for instance an anti-roll linkage 
       could be attached to links as well as uprights.
   <b>W</b> - wheel</pre>
When there is more then one part of the same type, a number is
added to the character. For example if there are more than one
link, as in a double-wishbone, they are numbered L1, L2, etc.,
starting at the front upper link.</p>
<p>The wheels are numbered from front left towards right and rear.
For a four wheel car this yields: <pre>
  <b>[wheel no]</b> 
   <b>1</b> - front left wheel
   <b>2</b> - front right wheel 
   <b>3</b> - rear left wheel 
   <b>4</b> - rear right wheel
</pre> Some examples of how parameters are named: <pre>
   <b>rCL1_2</b>  - Location of connection joint between chassis and link 1 at
             front right wheel. 
   <b>i22L1_3</b> - Inertia element i22 of link 1 at rear left wheel. 
   <b>nCL1_4</b>  - Direction of revolution of the joint that connects link 1 to 
             the chassis at the right rear wheel. This could for example be the rotation
             axis of a swing axle. 
   <b>rUL1L2_1</b>- Location of connection joint between the upright and link 1 
            and 2 at the front left wheel. This could for example be the upper spindle
             joint at a double wishbone suspension
</pre></p>
<p>
The frames are named after the part they are attached to. The
frames are normally located at the parts outermost point referred
to the tree structure.
Examples: <pre>
   <b>frame_L1L2</b>- A frame attached to links 1 and 2 at their outer end. For a 
               MacPherson linkage this would mean that the frame is attached to 
               the link at the location of the spindle. 
   <b>frame_U_2</b> - The frame attached at the outermost point of the right upright. 
               Typically this is the location of the wheel centre.
</pre></p>
<b>Coordinate system</b><p>The coordinate systems used within the library refers to the DIN standard, the x,y and z axes point forward, left and upward respectively, see figure 5.
</p>
<IMG SRC=\"../Images/coords.gif\" ALT=\"coordinate system\"> </p><p><b>Figure
5</b> The coordinate system used to define the suspension geometries.</p>
<b>Scaling</b><p>In many cases it is convenient to mirror
components in a car, for example left and right suspensions. To
handle this there is a three-dimensional scaleFactor. This can be
used to rescale and mirror objects, for example <pre>
   <b>scaleFactor={1,-1,1}</b></pre> mirrors the model around the xz-plane
</p>
<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.ave.kth.se/staff/fdn/johan/index.html\">Johan Andreasson</a><br>
Div. of Vehicle Dynamics<br>
Royal Institute of Technology<br>
Teknikringen 8c<br>
SE-10044 Stockholm, Sweden<br>
email: <A HREF=\"mailto:johan@fkt.kth.se\">johan@fkt.kth.se</A><br>
</dl>
<p><b>Release Notes:</b></p>
<ul>
<li><i>April 29, 2003</i><br>
       First beta release (version 0.5) implemented with largely extended
       vehicle components. Contains especially
       5 models of a typical sedan car with different levels of model
       details leading to 14 to 87 degrees of freedom.
       </li>
<li><i>Nov. 1, 2000</i><br>
       First pre beta release implemented, containing especially
       the model of a racing car.
</ul>
<p>
<b>Acknowledgements:</b><br>
Some components of this library, such as the base wheel, the Rill tyre model
and the aggregation joints for analytically solving kinematic loops, have been
developed by <a href=\"http://www.robotic.dlr.de/Martin.Otter\">Martin Otter</a>, from 
<a href=\"http://www.robotic.dlr.de\">DLR - Institute of Robotics and Mechatronics</a>,
Germany.<br>
Part of this library was developed with financial support from Dynasim AB, Sweden
and DLR - Institute of Robotics and Mechatronics, Germany.
Some of the models are originally developed for the Driving Dynamics project within the \"The Green Vehicle\"/FCHEV Programme.
</p>
<p><b>Copyright &copy; 2003, Modelica Association, Johan Andreasson, DLR and 
Dynasim</b></p>
</HTML>
"));


package Examples 
  import Modelica.SIunits;
  
  partial model CarBase 
    "A sedan car modeled with linkages as parameterisised curves based on wheel travel."
     
    
    extends Modelica.Icons.Example;
    inner replaceable block Road = Environments.NoGraphicsRoad extends 
      VehicleDynamics.Environments.Interfaces.TyreRoadInterface annotation (
        choicesAllMatching, extent=[-104, -16; -74, 24]);
    inner replaceable block Weather = Environments.ConstantWind extends 
      VehicleDynamics.Environments.Interfaces.WeatherInterface annotation (
        choicesAllMatching, extent=[-96, -52; -76, -32]);
    
    // INITIAL CONDITIONS
    parameter SI.Velocity v_x0=20 "|Initial conditions| start velocity x";
    parameter SI.Position r_x0=0 "|Initial conditions| start position x";
    parameter SI.Position r_y0=0 "|Initial conditions| start position y";
    parameter SI.Position r_z0=0.24 "|Initial conditions| start position z";
    
  protected 
    parameter SI.Position[3] rInit={r_x0,r_y0,r_z0};
    parameter Real[3] vInit={v_x0,0,0};
    parameter SI.Angle[3] phiInit={0,0,0};
    
    // DRIVER
    
    // STEERING
    parameter Real kLaneChange=1 
      "|Steering|LaneChange| Gain of Lane Change output, use this to turn on/off output";
    parameter SI.Time t1LaneChange=2.5 
      "|Steering|LaneChange| start time of sine1 ";
    parameter SI.Time t2LaneChange=5.5 
      "|Steering|LaneChange|start time of sine2";
    parameter SI.Time p1LaneChange=2 "|Steering|LaneChange|period of sine1";
    parameter SI.Time p2LaneChange=2 "|Steering|LaneChange| period of sine2";
    parameter SI.Angle a1LaneChange=0.6 
      "|Steering|LaneChange| amplitude of sine1";
    parameter SI.Angle a2LaneChange=-0.6 
      "|Steering|LaneChange| amplitude of sine2";
    parameter Boolean generateEvent=false 
      "|Steering|LaneChange| generate event for discontinuities";
    
    parameter Real kRampSteering=0 
      "|Steering|Ramp| Gain of Ramp output, use this to turn on/off output";
    parameter SI.Angle heightRampSteering=0.0 "|Steering|Ramp| Height of ramp";
    parameter SI.Time durationRampSteering(min=Modelica.Constants.SMALL) = 
      0.001 "|Steering|Ramp| Duration of ramp";
    parameter SI.Angle offsetRampSteering=0 "|Steering|Ramp| Ramp offset";
    parameter SI.Time startTimeRampSteering=1.5 
      "|Steering|Ramp| Output = offset for time < startTime";
    
    parameter Real kTableSteering=0 
      "|Steering|TimeTable| Gain of Table output, use this to turn on/off output";
    parameter Real tableSteering[:, :]=[0, 0; 1, 1; 2, 4] 
      "|Steering|TimeTable| Table matrix (time = first column)";
    
    parameter Real kRampDrive=0 
      "|Drive|Ramp| Gain of Ramp output, use this to turn on/off output";
    parameter Real heightRampDrive=500 "|Drive|Ramp| Height of ramp";
    parameter Real durationRampDrive(min=Modelica.Constants.SMALL) = 2 
      "|Drive|Ramp| Duration of ramp";
    parameter Real offsetRampDrive=0 "|Drive|Ramp| Ramp offset";
    parameter SI.Time startTimeRampDrive=1 
      "|Drive|Ramp| Output = offset for time < startTime";
    
  protected 
    Drivers.OpenLoopDriver driver(
      kLaneChange=kLaneChange, 
      t1LaneChange=t1LaneChange, 
      t2LaneChange=t2LaneChange, 
      p1LaneChange=p1LaneChange, 
      p2LaneChange=p2LaneChange, 
      a1LaneChange=a1LaneChange, 
      a2LaneChange=a2LaneChange, 
      generateEvent=generateEvent, 
      kRampSteering=kRampSteering, 
      heightRampSteering=heightRampSteering, 
      durationRampSteering=durationRampSteering, 
      offsetRampSteering=offsetRampSteering, 
      startTimeRampSteering=startTimeRampSteering, 
      tableSteering=tableSteering, 
      kTableSteering=kTableSteering, 
      kRampDrive=kRampDrive, 
      heightRampDrive=heightRampDrive, 
      durationRampDrive=durationRampDrive, 
      offsetRampDrive=offsetRampDrive, 
      startTimeRampDrive=startTimeRampDrive)
      annotation (extent=[-104, -16; -64, 24]);
    annotation (Diagram);
    annotation (Diagram);
    
  end CarBase;
  
  model SedanCar_Level1_LinkageTables 
    "Car model with suspensions toe in and camber charactistics from polynomials."
     
    
    annotation (Commands(
        file="Scripts/LaneChange1.mos" "Simulate double lane change", 
        file="Scripts/FrontLeftAboveView.mos" 
          "Front left above view of animation", 
        file="Scripts/FrontLeftView.mos" "Front left view of animation", 
        file="Scripts/HighAboveView.mos" "High above view of animation"), 
        Diagram);
    
    extends StandardCar(chassis(redeclare 
          VehicleDynamics.Chassis.Suspensions.SimpleSuspension frontSuspension, 
           redeclare 
          VehicleDynamics.Chassis.Suspensions.SimpleSuspensionNoSteer 
          rearSuspension));
    
    annotation (Diagram, Documentation(info="<HTML><h3><span lang=EN-GB>Level 1 Linkage tables</span></h3>
<p class=MsoNormal><span lang=EN-US>The wheel suspensions are described by
tables defining camber and toe-in angles as functions of wheel bounce, i.e., a
vertical motion of the wheel with constrained changes of the Camber and toe-in
angles.</span></p>
<p class=MsoNormal><span lang=EN-GB>This could easily be extended to handle
also camber and toe-in as functions of side force, which would make it possible
to mimic the behaviour of suspensions with bushings and other flexible
elements. However, the drawback with this method is that the characteristics
must either be measured, meaning that the suspension has to be built, or that
the suspension characteristics has to be calculated from a more detailed model.</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>Steering is defined by an Ackermann
function. The tables for camber and toe-in angles are implemented as scaled
polynomials. Dymolas symbolic engine differentiates these polynomials twice to
handle the reduction of degrees-of-freedom.</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>The complexity of the model is
characterized in the following table showing the number of degrees-of-freedom (DOFs).</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<div align=center>
<table class=MsoNormalTable border=1 cellspacing=0 cellpadding=0
 style='border-collapse:collapse;border:none'>
 <tr>
  <td width=109 valign=top style='width:81.9pt;border:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Individual DOFs</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Component count</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Total DOFs</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=Bodyafter><span lang=EN-US>Comment</span></p>
  </td>
 </tr>
 <tr>
  <td width=109 valign=top style='width:81.9pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Chassis</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=109 valign=top style='width:81.9pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Wheel bounce</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=109 valign=top style='width:81.9pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Wheel rotation</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=109 valign=top style='width:81.9pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Steering</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=109 valign=top style='width:81.9pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\"'>&nbsp;</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><b><span lang=EN-US>&nbsp;</span></b></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=109 valign=top style='width:81.9pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\"'>Total</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><b><span lang=EN-US>15</span></b></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
</table>
</div>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>The tires have each 2 state variables for the
deflection in x and y directions, i.e., 4*2 = 8 states. The total number of
states for the vehicle dynamics itself is thus 2*15 + 8 = 38.</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>The steering in the benchmark model is a
parameterized, given function which is filtered by a second order low pass
filter to model driving behavior. The driver model of the benchmark model
contains two additional state variables for the accelerator behavior. This is
not used in this model since the vehicle maneuver is made with idle gear. The
total number of state variables is thus 38 + 2 = 40.</span></p>
</HTML>"));
    
  end SedanCar_Level1_LinkageTables;
  
  model SedanCar_Level2_RigidLinkages 
    "Car model with rigid suspensions linkages." 
    
    annotation (Commands(
        file="Scripts/LaneChange2.mos" "Simulate double lane change", 
        file="Scripts/FrontLeftAboveView.mos" 
          "Front left above view of animation", 
        file="Scripts/FrontLeftView.mos" "Front left view of animation", 
        file="Scripts/HighAboveView.mos" "High above view of animation"), 
        Diagram);
    
    extends StandardCar(chassis(redeclare 
          VehicleDynamics.Chassis.Suspensions.MacPhersonSuspension2 
          frontSuspension, redeclare 
          VehicleDynamics.Chassis.Suspensions.TrailingArmSuspension 
          rearSuspension));
    annotation (Diagram, Documentation(info="<HTML>
<h3><span lang=EN-GB>Level 2 Linkage with ideal joints</span></h3>
<p class=Bodyafter><span lang=EN-US>The table description used in level 1is
limited to only Camber and toe-in angles. It would of course be possible to
extend to Caster angle trail as well as track width and wheel base translations.
However, in many cases, in particular when trying new designs, it?s easier to
describe the suspension in terms of the linkage that is used.</span></p>
<p class=Bodyafter><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-GB>The suspensions in level 2 consist of rigid
mechanical components, i.e. all flexible elements, except for the struts, are
replaced by ideal joints. Instead of a multi-link suspension, a trailing arm
with similar geometry is used. The advantage over level 1 is that the
suspension can be modelled with physical data and no precalculations or
measurements are therefore needed.</span></p>
<p class=Bodyafter><span lang=EN-US>&nbsp;</span></p>
<p class=Bodyafter><span lang=EN-US>The level 2 model uses a MacPherson type
front wheel suspension, with the wishbone attached to the chassis via an ideal
revolute joint (1 DOF). A strut is placed between the chassis and the wishbone
via two spherical joints. The eigen rotation of the strut around its axis (1 DOF)
is constrained by the distance constraint of an additional rod with two
spherical joints on each end (1 constraint). One of the spherical joints of
this rod is attached to the steering. In total, the suspension has therefore
one degree of freedom, if the steering angle is given. The anti-roll bar is
approximated by a spring/damper combination where the vertical force acting at
its mount point on the lower part of the MacPherson strut is proportional to
the relative vertical distance of the left and the right mount points. The rear
suspension is a type trailing arm with one DOF, the anti roll bar is modeled
like in the front suspension.</span></p>
<p class=Bodyafter><span lang=EN-US>&nbsp;</span></p>
<p class=Bodyafter><span lang=EN-US>When using base elements of the MultiBody
library to build up the MacPherson suspension, several non-linear algebraic
loops appear. By using composite joint models (e.g., an aggregation of a
revolute, a spherical and a universal joint) that contain analytic solutions of
the non-linear kinematic relationships within the aggregation, the non-linear
algebraic loops do no longer occur in the generated code. In the following
table the degrees of freedom of this vehicle model is summarized.</span></p>
<p class=Bodyafter><span lang=EN-US>  </span></p>
<p class=Bodyafter><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<div align=center>
<table class=MsoNormalTable border=1 cellspacing=0 cellpadding=0
 style='border-collapse:collapse;border:none'>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Individual DOFs</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Component count</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Total DOFs</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=Bodyafter><span lang=EN-US>Comment</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Chassis</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Wishbone rotation</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Steering</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Trailing arm rotation</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Wheel rotation</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\"'>&nbsp;</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><b><span lang=EN-US>&nbsp;</span></b></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.6pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\"'>Total</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><b><span lang=EN-US>15</span></b></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
</table>
</div>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=Bodyafter><span lang=EN-GB>Similar to the level 1 car, there are 4*2 =
8 states for the tire deflections and 2 unused states. As a result, this model
has also 40 states. Note, that the elasticity of the tires in vertical
direction has been modified slightly (both for the level 1 and the level 2
cars) in order to approximately compensate for the neglected bushings.</span></p>
</HTML>"));
  end SedanCar_Level2_RigidLinkages;
  
  model SedanCar_Level3_Bushings 
    "Car model with bushings in suspensions linkages and massless rods." 
    
    annotation (Commands(
        file="Scripts/LaneChange3.mos" "Simulate double lane change", 
        file="Scripts/FrontLeftAboveView.mos" 
          "Front left above view of animation", 
        file="Scripts/FrontLeftView.mos" "Front left view of animation", 
        file="Scripts/HighAboveView.mos" "High above view of animation"), 
        Diagram);
    
    extends StandardCar(chassis(redeclare 
          VehicleDynamics.Chassis.Suspensions.MultiLink4Suspension3 
          rearSuspension, redeclare 
          VehicleDynamics.Chassis.Suspensions.MacPhersonSuspension3 
          frontSuspension));
    annotation (Diagram, Documentation(info="<HTML>
<h3><span lang=EN-GB>Level 3 Linkage with bushings and massless bars</span></h3>
<p class=MsoNormal><span lang=EN-US>Using ideal joint models for the linkage is
not accurate enough for severe driving conditions since bushings with certain
flexibility are used in the real vehicle. </span><span lang=EN-GB>Flexible
elements are introduced in the suspensions</span><span lang=EN-US> of  the
level 3 model. </span><span lang=EN-GB>The front suspension has bushings in the
A-arm mounts</span><span lang=EN-US>. </span><span lang=EN-GB>The rear
multilink suspension has no ideal joints and the links are modelled as
mass-less bars. </span><span lang=EN-US>If the mass and inertia of the rod
connecting two bushings would not be neglected, 6 DOF would be added for every
such pushrod. However, the mass and inertia are usually very small compared to,
say, the wheel and carrier masses, and therefore it is a good approximation to
neglect the pushrod masses and inertias. </span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>If the bushings would be described solely
by springs, then no states would be added, since springs in series connection
leads to algebraic equations to solve for the spring deflections. Since
bushings have additionally a damping part, there are the states of the dampers
(= 2*6). Once the states of one damper are known, the states of the other
damper can be computed by relative kinematics. To summarize, a pushrod has 6
states, if the mass and inertia of the rod connecting the two bushings is
neglected. There are 3 such bushing pairs at each rear wheel, i.e. the number
of states is 2*3*6=36 states.</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>Additionally, the elasticity in the
steering is taken into account by having a spring/damper system in the rack
steering adding one additional DOF. </span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>The number of degrees of freedom is shown
below.</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<div align=center>
<table class=MsoNormalTable border=1 cellspacing=0 cellpadding=0
 style='border-collapse:collapse;border:none'>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Individual DOFs</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Component count</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Total DOFs</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=Bodyafter><span lang=EN-US>Comment</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Chassis</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Front wheel carrier</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>12</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Steering</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Rack steering </span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Rear wheel carrier</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>12</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Wheel rotation</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\";font-weight:normal'>&nbsp;</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\"'>Total</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><b><span lang=EN-US>36</span></b></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
</table>
</div>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>There are 4*2 = 8 states for the tire
deflections and 2 unused states. As a result, this model has 2*36 + 36 + 8 + 2
= 118 states. </span></p>
</HTML>"));
  end SedanCar_Level3_Bushings;
  
  model SedanCar_Level4_MassRods 
    "Car model with bushings in suspensions linkages and mass rods." 
    annotation (Commands(
        file="Scripts/LaneChange4.mos" "Simulate double lane change", 
        file="Scripts/FrontLeftAboveView.mos" 
          "Front left above view of animation", 
        file="Scripts/FrontLeftView.mos" "Front left view of animation", 
        file="Scripts/HighAboveView.mos" "High above view of animation"), 
        Diagram);
    
    extends StandardCar(chassis(redeclare 
          VehicleDynamics.Chassis.Suspensions.MacPhersonSuspension3 
          frontSuspension, redeclare 
          VehicleDynamics.Chassis.Suspensions.MultiLink4Suspension4 
          rearSuspension));
    annotation (Diagram, Documentation(info="<HTML>
<h3><span lang=EN-GB>Level 4 Linkage with bushings and non-massless bars</span></h3>
<p class=MsoNormal><span lang=EN-US>A slightly more detailed model is obtained
by not neglecting the masses of the push rods. The number of degrees of freedom
is shown below.</span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<div align=center>
<table class=MsoNormalTable border=1 cellspacing=0 cellpadding=0
 style='border-collapse:collapse;border:none'>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Individual DOFs</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Component count</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Total DOFs</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border:solid windowtext 1.0pt;
  border-left:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=Bodyafter><span lang=EN-US>Comment</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Chassis</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Front wheel carrier</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>12</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Steering</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Rack steering </span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Rear wheel carrier</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>12</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Pushrods of rear wheel carrier</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>6</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>2*3</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>36</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>Wheel rotation</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>1</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>4</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\";font-weight:normal'>&nbsp;</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
 <tr>
  <td width=118 valign=top style='width:88.75pt;border:solid windowtext 1.0pt;
  border-top:none;padding:0cm 5.4pt 0cm 5.4pt'>
  <h3 style='margin:0cm;margin-bottom:.0001pt'><span lang=EN-US
  style='font-family:\"Times New Roman\"'>Total</span></h3>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><span lang=EN-US>&nbsp;</span></p>
  </td>
  <td width=78 valign=top style='width:58.5pt;border-top:none;border-left:none;
  border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal align=right style='text-align:right'><b><span lang=EN-US>72</span></b></p>
  </td>
  <td width=120 valign=top style='width:90.0pt;border-top:none;border-left:
  none;border-bottom:solid windowtext 1.0pt;border-right:solid windowtext 1.0pt;
  padding:0cm 5.4pt 0cm 5.4pt'>
  <p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
  </td>
 </tr>
</table>
</div>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
<p class=MsoNormal><span lang=EN-US>This model has 2*72 + 8 + 2 = 154 states<b>.</b></span></p>
<p class=MsoNormal><span lang=EN-US>&nbsp;</span></p>
</HTML>"));
  end SedanCar_Level4_MassRods;
  
  model Dallara 
    "Car model with bushings in suspensions linkages and mass rods." 
    annotation (Commands(
        file="Scripts/LaneChangeDallara.mos" "Simulate double lane change", 
        file="Scripts/FrontLeftAboveView.mos" 
          "Front left above view of animation", 
        file="Scripts/FrontLeftView.mos" "Front left view of animation", 
        file="Scripts/HighAboveView.mos" "High above view of animation"), 
        Diagram);
    
    Chassis.FormulaChassis chassis(
      vInit=vInit, 
      rInit=rInit, 
      phiInit=phiInit) annotation (extent=[-20, -20; 40, 40]);
    annotation (Diagram, Documentation(info="<HTML>
<p>This is a model of a Dallara F3 race car. Typical for a formula car is stiff spring elements and the lack of bushings. This version uses inboard mouted struts both front and rear, adding more kinematical loops. In front, there is a BelleVille linkage that decouples roll and bounce stiffness. In the rear suspension, two struts are mounted onboard and the anti roll is handled by a bar with levers.
</p></HTML>"));
    Aerodynamics.BasicAirResistance airResistance
      annotation (extent=[-44, 24; -24, 44], rotation=180);
    extends CarBase(r_z0=0.26, phiInit={0,0.015,0});
    // For plotting
    output SI.Acceleration sideAcceleration=chassis.BGR.a[2];
    output SI.AngularVelocity yawVelocity=chassis.BGR.w[3];
    output SI.Velocity longitudinalSpeed=chassis.BGR.v[1];
    output SI.Velocity lateralSpeed=chassis.BGR.v[2];
  equation 
    connect(driver.flange_Lat, chassis.flange_SW)
      annotation (points=[-62, 10; -19.4, -2], style(color=0));
    connect(airResistance.frame_a, chassis.BGR)
      annotation (points=[-23.5, 34; -20, 22]);
  end Dallara;
  
  model StandardCar "Standard car chassis" 
    annotation (Commands(
        file="Scripts/LaneChangeStandard.mos" "Simulate double lane change", 
        file="Scripts/FrontLeftAboveView.mos" 
          "Front left above view of animation", 
        file="Scripts/FrontLeftView.mos" "Front left view of animation", 
        file="Scripts/HighAboveView.mos" "High above view of animation"), 
        Diagram);
    Chassis.StandardChassis chassis(
      vInit=vInit, 
      rInit=rInit, 
      phiInit=phiInit, 
      redeclare VehicleDynamics.Chassis.Suspensions.MultiLink4Suspension3 
        rearSuspension) annotation (extent=[-20, -8; 40, 52]);
    extends CarBase(redeclare block Road = 
          VehicleDynamics.Environments.SplitMueRoad);
    Aerodynamics.BasicAirResistance airResistance
      annotation (extent=[-60, 24; -40, 44], rotation=180);
    // For plotting
    output SI.Acceleration sideAcceleration=chassis.BGR.a[2];
    output SI.AngularVelocity yawVelocity=chassis.BGR.w[3];
    output SI.Velocity longitudinalSpeed=chassis.BGR.v[1];
    output SI.Velocity lateralSpeed=chassis.BGR.v[2];
    ModelicaAdditions.MultiBody.Parts.BoxBody load(
      r={-lLoad,0,0}, 
      r0=rLoad + {lLoad/2,0,0}, 
      LengthDirection={1,0,0}, 
      WidthDirection={0,1,0}, 
      Length=lLoad, 
      Width=wLoad, 
      Height=hLoad, 
      rho=rhoLoad, 
      Material={0,0,1,0.7}) annotation (extent=[-20, 60; 0, 80]);
    parameter SI.Density rhoLoad=0 "|Additional Load||Density of load";
    parameter SI.Length lLoad=0 "|Additional Load||Length of load";
    parameter SI.Length wLoad=0 "|Additional Load||Width of load";
    parameter SI.Length hLoad=0 "|Additional Load||Height of load";
    parameter SI.Position[3] rLoad={-2.6,0.4,1.5} 
      "|Additional Load||Position of load centre";
  equation 
    
    connect(driver.flange_Lat, chassis.flange_SW)
      annotation (points=[-62, 10; -19.4, 10], style(color=0));
    annotation (Documentation(info="<HTML>
<p>This is a model standard four wheeled car driven by an open loop driver, i.e. no feedback is considered. Only the front wheels can be steered. To the wheel flanges, a powertrain can be added.</p></HTML>"), 
         Diagram);
    connect(airResistance.frame_a, chassis.BGR)
      annotation (points=[-39.5, 34; -20, 34]);
    connect(load.frame_a, chassis.BGR)
      annotation (points=[-20.5, 70; -30, 70; -30, 34; -20, 34]);
  end StandardCar;
  
  model SedanCarWithTrailer "Car model with rigid suspensions linkages." 
    annotation (Commands(
        file="Scripts/LaneChangeTrailer.mos" "Simulate double lane change", 
        file="Scripts/FrontLeftAboveView.mos" 
          "Front left above view of animation", 
        file="Scripts/FrontLeftView.mos" "Front left view of animation", 
        file="Scripts/HighAboveView.mos" "High above view of animation"), 
        Diagram);
    
    extends StandardCar(chassis(redeclare 
          VehicleDynamics.Chassis.Suspensions.MacPhersonSuspension2 
          frontSuspension, redeclare 
          VehicleDynamics.Chassis.Suspensions.TrailingArmSuspension 
          rearSuspension));
    annotation (Diagram, Documentation(info="<HTML>
<p>In this model, the StandardCar example is extended with a trailer.
</p></HTML>"));
    Chassis.Trailer trailer annotation (extent=[42, -20; 102, 40]);
  equation 
    connect(chassis.BGR, trailer.frame_a)
      annotation (points=[-20, 34; -20, 40; 40, 40; 40, 10; 45, 10]);
  end SedanCarWithTrailer;
  
  model SuspensionTestRig "Rig for isolated testing of suspenson suspensions" 
    
    annotation (Commands(file="SCRIPT/TestRig.mos" 
          "Script to simulate test rig"), Diagram);
    
    parameter SI.Force[1] k={4000} "Additional force";
    parameter SI.Mass weightOnSuspension=400 
      "The amount of weight on the tested suspension.";
    parameter SI.Distance width=0.23 "Tyre width";
    parameter SI.Distance radius=0.295 "Tyre radius";
    parameter SI.Distance rimRadius=0.22 "Rim radius";
    parameter Boolean animation=true 
      "This should be false if the tyres should not be animated.";
    
    ModelicaAdditions.MultiBody.Parts.InertialSystem ground(ng={0,0,1})
      annotation (extent=[-80, 2; -60, 22]);
    annotation (
      Diagram, 
      Icon(Bitmap(extent=[-100, -100; 100, 100], name="images/MacPherson.png"), 
           Rectangle(extent=[-100, 100; 100, -100])), 
      Documentation(info="<HTML>
<p>This rig is used to test suspensions separated from the rest of the car. The proper load on the suspension can be set with a parameter. Additionally there are sensors to measuere the rotation of the wheel carrier (upright). The suspension to be mesured can be selected from the parameter dialog.
</p></HTML>"));
    replaceable VehicleDynamics.Chassis.Suspensions.MacPhersonSuspension2 
      suspension extends Chassis.Interfaces.SuspensionInterface annotation (
      choicesAllMatching, 
      extent=[-22, 0; 20, 40], 
      rotation=180);
    Utilities.Parts.Body leftWheel(
      shape="tire", 
      length=width, 
      material={0.2,0.2,0.2,0.2}, 
      width=2*radius, 
      height=2*radius, 
      extra=rimRadius/radius, 
      m=weightOnSuspension/2, 
      animation=animation, 
      lengthDirection={0,1,0})
      annotation (extent=[-10, 60; 10, 80], rotation=90);
    Utilities.Parts.Body rightWheel(
      shape="tire", 
      length=width, 
      material={0.2,0.2,0.2,0.2}, 
      width=2*radius, 
      height=2*radius, 
      extra=rimRadius/radius, 
      m=weightOnSuspension/2, 
      lengthDirection={0,-1,0}, 
      animation=animation) annotation (extent=[-10, -40; 10, -20], rotation=270);
    ModelicaAdditions.MultiBody.Forces.ExtLineForce leftForce(n={0,0,1})
      annotation (extent=[26, 74; 46, 94], rotation=180);
    ModelicaAdditions.MultiBody.Forces.ExtLineForce rightForce(n={0,0,1})
      annotation (extent=[28, -52; 48, -32], rotation=180);
    Modelica.Blocks.Sources.Pulse pulse1(
      amplitude=k, 
      period={4}, 
      offset=-k/3) annotation (extent=[64, 74; 84, 94], rotation=180);
    Modelica.Blocks.Sources.Pulse pulse2(
      amplitude=k, 
      period={8}, 
      offset=-k/3) annotation (extent=[64, -52; 84, -32], rotation=180);
    Modelica.Mechanics.Rotational.Fixed fixed
      annotation (extent=[-48, 18; -28, 38], rotation=270);
  equation 
    connect(ground.frame_b, suspension.frame_C) annotation (points=[-59.5, 12; 
          -39.175, 12; -39.175, 12.2; -18.85, 12.2]);
    connect(leftWheel.frame_a, suspension.frame_U_1) annotation (points=[-6.42918e-016, 
           59.5; -6.42918e-016, 54.625; 0.89, 54.625; 0.89, 41]);
    connect(rightWheel.frame_a, suspension.frame_U_2) annotation (points=[
          1.92876e-015, -19.5; 1.92876e-015, -14.625; 0.89, -14.625; 0.89, -1]);
    connect(leftForce.frame_b, leftWheel.frame_b) annotation (points=[25.5, 84; 
           12.75, 84; 12.75, 80.5; 6.42918e-016, 80.5]);
    connect(rightForce.frame_b, rightWheel.frame_b) annotation (points=[27.5, -42; 
           13.75, -42; 13.75, -40.5; -1.92876e-015, -40.5]);
    connect(pulse1.outPort, leftForce.inPort)
      annotation (points=[63, 84; 46, 84], style(color=3));
    connect(pulse2.outPort, rightForce.inPort)
      annotation (points=[63, -42; 48, -42], style(color=3));
    connect(fixed.flange_b, suspension.flange_SW) annotation (points=[-38, 28; 
          -28.425, 28; -28.425, 27.8; -18.85, 27.8], style(color=0));
  end SuspensionTestRig;
  
  model DoubleWishBoneTest 
    "Test of a double wishbone suspension mounted in a rig. The suspension can easilly be replaced by selecting another suspension in the parameter dialog."
     
    
    annotation (Commands(file="SCRIPT/TestRig2.mos" 
          "Script to simulate test rig"), Diagram);
    
    extends Modelica.Icons.Example;
    SuspensionTestRig suspensionTestRig(
      animation=false, 
      redeclare VehicleDynamics.Chassis.Suspensions.DoubleWishBoneSuspension 
        suspension, 
      k={2500}, 
      weightOnSuspension=150) annotation (extent=[-38, -40; 42, 40]);
    annotation (Documentation(info="<HTML>
<p>This an example showing the use of the SuspensionTestRig for a MacPhersonSuspension
</p></HTML>"));
  end DoubleWishBoneTest;
end Examples;


package Chassis "Modelica library to model vehicle chassis." 
  extends Modelica.Icons.Library;
  
  annotation (
    Coordsys(
      extent=[0, 0; 840, 1086], 
      grid=[2, 2], 
      component=[20, 20]), 
    Window(
      x=0.45, 
      y=0.01, 
      width=0.53, 
      height=0.95, 
      library=1, 
      autolayout=1), 
    Documentation(info="<HTML>
</HTML> 
"));
  package Interfaces 
    extends Modelica.Icons.Library;
    annotation (Coordsys(
        extent=[0, 0; 632, 426], 
        grid=[2, 2], 
        component=[20, 20]), Window(
        x=0.14, 
        y=0.24, 
        width=0.4, 
        height=0.4, 
        library=1, 
        autolayout=1));
    partial model SuspensionInterface 
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Icon(
          Rectangle(extent=[-90, 90; 70, -90], style(fillColor=8)), 
          Text(extent=[-90, 40; 70, -40], string="%name"), 
          Text(
            extent=[-100, -40; 80, -100], 
            string="left", 
            style(color=61))));
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_U_1
        annotation (extent=[6, -120; -24, -90], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_U_2
        annotation (extent=[-24, 90; 6, 120], rotation=90);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[70, 24; 100, 54], rotation=180);
    end SuspensionInterface;
    
    partial model ChassisInterface 
      parameter Real[3] vInit={10,0,0} 
        "|Initial conditions| initial speed of vehicle";
      parameter SI.Position[3] rInit={0,0,0.3} 
        "|Initial conditions| initial position of vehicle";
      parameter SI.Angle[3] phiInit={0,0,0} 
        "|Initial conditions| initial rotation of vehicle";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Diagram, 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Icon);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_SW
        annotation (extent=[-108, -50; -88, -30]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b BGR 
        "Body Geometric Reference. All coordinates within the chassis are defined from here"
        annotation (extent=[-110, 30; -90, 50], rotation=180);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_2
        annotation (extent=[-70, 60; -50, 80], rotation=180);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_4
        annotation (extent=[50, 60; 70, 80], rotation=180);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_1
        annotation (extent=[-70, -80; -50, -60]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_3
        annotation (extent=[50, -80; 70, -60]);
    end ChassisInterface;
    
    model Interface "Interface for linkages" 
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon, 
        Diagram, 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUL1L2: location of outer ball joint, resolved in frame_C
  rCL1:   location of inner front joint, resolved in frame_C
  rCL2:   location of inner rear joint, resolved in frame_C
  rCS:    location of strut mount in chassis, resolved in frame_C
  rUS:    location of strut mount in upright, resolved in frame_C
  rUW:    location of frame_W, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Utilties.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-120, -16; -90, 14]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_U
        annotation (extent=[90, -14; 120, 16]);
    end Interface;
    
    partial model SteerableSuspensionInterface 
      extends VehicleDynamics.Chassis.Interfaces.SuspensionInterface;
      
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_SW
        annotation (extent=[70, -54; 100, -24]);
    end SteerableSuspensionInterface;
    
    model NoSteerSuspensionInterface 
      extends SuspensionInterface;
    end NoSteerSuspensionInterface;
  end Interfaces;
  
  package Suspensions 
    "Suspensions, models ready to be used as front or rear suspensions." 
    
    extends Modelica.Icons.Library;
    
    annotation (
      Coordsys(
        extent=[0, 0; 744, 786], 
        grid=[1, 1], 
        component=[20, 20]), 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.47, 
        height=0.7, 
        library=1, 
        autolayout=1), 
      Documentation(info="<HTML> 
<p>
In this library, suspension models are gahtered. To make the models easily repleaceable, the all share the same basic interface, i.e. one frame_C that is to be connected to the chassis and one frame_W per wheel, normally two, to connect to the wheels. Additionally, some supensions has a rotaty flage for the steering wheel.
</p>
</HTML> 
"));
    
    model MacPhersonSuspension5 
      "MacPherson suspension with bushings, including anti roll bar linkage." 
      
      extends Chassis.Interfaces.SteerableSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Diagram, 
        Images(Parameters(name="|Geometric reference|", source=
                "images/MacPherson_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="MacPherson", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
This version contains bushings in the A-arm mount and in the anti-roll linkage. Furthermore there is an elacticity in the steering.</p>
</HTML>"));
      Components.RackSteering4 rackSteering(
        rXL_1=data.rUL3_1, 
        rRL_1=data.rRL3_1, 
        rX_1=data.rUW_1, 
        rXL_2=data.rUL3_2, 
        rRL_2=data.rRL3_2, 
        rX_2=data.rUW_2, 
        ratioWheelToRack=data.ratioWheelToRack, 
        iSW=data.iSW, 
        data_R=data.data_R) annotation (extent=[20, -41; 60, -1]);
      Components.MacPherson4 MacPherson_1(
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCS=data.rCS_1, 
        rUS=data.rUS_1, 
        rUL1L2=data.rUL1L2_1, 
        rUW=data.rUW_1, 
        q0S=data.q0S_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_S=data.data_S_1, 
        mL1L2=1, 
        i11L1L2=1, 
        i22L1L2=1, 
        i33L1L2=1, 
        rRL3=data.rRL3_1, 
        rUL3=data.rUL3_1)
        annotation (extent=[-30.5, -69.5; 9, -30], rotation=270);
      Components.MacPherson4 MacPherson_2(
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCS=data.rCS_2, 
        rUS=data.rUS_2, 
        rUL1L2=data.rUL1L2_2, 
        rUW=data.rUW_2, 
        q0S=data.q0S_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_S=data.data_S_2, 
        mL1L2=1, 
        i11L1L2=1, 
        i22L1L2=1, 
        i33L1L2=1, 
        rRL3=data.rRL3_2, 
        rUL3=data.rUL3_2) annotation (extent=[10, 30; -30, 70], rotation=90);
      parameter Chassis.Data.MacPhersonData data
        annotation (extent=[-114, 40; -80, 80]);
      Chassis.Components.BushingAntiRollBar antiRollBar(
        scaleFactor=data.scaleFactor, 
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL4_1, 
        rXL_1=data.rUL4_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL4_2, 
        rXL_2=data.rUL4_2, 
        rX_2=data.rUW_2, 
        data=data.data_A)
        annotation (extent=[-71.5, -19.5; -36.5, 15.5], rotation=270);
    equation 
      connect(rackSteering.frame_C, frame_C)
        annotation (points=[60, -11.8; 78, -11.8; 78, 34], style(thickness=4));
      connect(rackSteering.flange_a, flange_SW)
        annotation (points=[62, -28; 80, -28; 80, -44], style(color=0));
      connect(MacPherson_1.frame_C, frame_C) annotation (points=[-10, -29.0125; 
             -10, 26; 74, 26], style(thickness=4));
      connect(MacPherson_2.frame_C, frame_C)
        annotation (points=[-8, 28; -8, 26; 78, 26], style(thickness=4));
      connect(MacPherson_1.frame_U, frame_U_1)
        annotation (points=[-10, -70; -10, -100], style(thickness=4));
      connect(MacPherson_2.frame_U, frame_U_2)
        annotation (points=[-10, 72; -10, 96], style(thickness=4));
      connect(antiRollBar.frame_C, frame_C)
        annotation (points=[-38, -2; -10, -2; -10, 26; 74, 26]);
      connect(antiRollBar.frame_X_2, MacPherson_2.frame_U)
        annotation (points=[-61, 17; -61, 52; -10, 72], style(thickness=4));
      connect(antiRollBar.frame_X_1, MacPherson_1.frame_U)
        annotation (points=[-60, -20; -60, -56; -12, -70], style(thickness=4));
      connect(rackSteering.frame_X_2, MacPherson_2.frame_S) annotation (points=
            [28, 2; 28, 20; 34, 24; 34, 30; 28, 34; 28, 80; 6, 80; 6, 33], 
          style(thickness=4));
      connect(MacPherson_1.frame_S, rackSteering.frame_X_1) annotation (points=
            [4, -32.9625; 4, -80; 28, -80; 28, -40], style(thickness=4));
    end MacPhersonSuspension5;
    
    model MacPhersonSuspension3 
      "MacPherson suspension with bushings and a simplfied anti roll bar." 
      
      extends Chassis.Interfaces.SteerableSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Diagram, 
        Images(Parameters(name="|Geometric reference|", source=
                "images/MacPherson_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="MacPherson", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
This version contains bushings in the A-arm mount and there is an elacticity in the steering.</p>
</HTML>"));
      Components.RackSteering4 rackSteering(
        rXL_1=data.rUL3_1, 
        rRL_1=data.rRL3_1, 
        rX_1=data.rUW_1, 
        rXL_2=data.rUL3_2, 
        rRL_2=data.rRL3_2, 
        rX_2=data.rUW_2, 
        ratioWheelToRack=data.ratioWheelToRack, 
        iSW=data.iSW, 
        data_R=data.data_R) annotation (extent=[20, -41; 60, -1]);
      Components.MacPherson4 MacPherson_1(
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCS=data.rCS_1, 
        rUS=data.rUS_1, 
        rUL1L2=data.rUL1L2_1, 
        rUW=data.rUW_1, 
        q0S=data.q0S_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_S=data.data_S_1, 
        mL1L2=1, 
        i11L1L2=1, 
        i22L1L2=1, 
        i33L1L2=1, 
        rRL3=data.rRL3_1, 
        rUL3=data.rUL3_1)
        annotation (extent=[-30.5, -69.5; 9, -30], rotation=270);
      Components.MacPherson4 MacPherson_2(
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCS=data.rCS_2, 
        rUS=data.rUS_2, 
        rUL1L2=data.rUL1L2_2, 
        rUW=data.rUW_2, 
        q0S=data.q0S_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_S=data.data_S_2, 
        mL1L2=1, 
        i11L1L2=1, 
        i22L1L2=1, 
        i33L1L2=1, 
        rRL3=data.rRL3_2, 
        rUL3=data.rUL3_2) annotation (extent=[10, 30; -30, 70], rotation=90);
      parameter Chassis.Data.MacPhersonData data
        annotation (extent=[-114, 40; -80, 80]);
      Chassis.Components.SimpleAntiRollBar antiRollBar(
        scaleFactor=data.scaleFactor, 
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL4_1, 
        rXL_1=data.rUL4_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL4_2, 
        rXL_2=data.rUL4_2, 
        rX_2=data.rUW_2, 
        data=data.data_A)
        annotation (extent=[-71.5, -19.5; -36.5, 15.5], rotation=270);
    equation 
      connect(rackSteering.frame_C, frame_C)
        annotation (points=[60, -11.8; 78, -11.8; 78, 34], style(thickness=4));
      connect(rackSteering.flange_a, flange_SW)
        annotation (points=[62, -28; 80, -28; 80, -44], style(color=0));
      connect(MacPherson_1.frame_C, frame_C) annotation (points=[-10, -29.0125; 
             -10, 26; 74, 26], style(thickness=4));
      connect(MacPherson_2.frame_C, frame_C)
        annotation (points=[-8, 28; -8, 26; 78, 26], style(thickness=4));
      connect(MacPherson_1.frame_U, frame_U_1)
        annotation (points=[-10, -70; -10, -100], style(thickness=4));
      connect(MacPherson_2.frame_U, frame_U_2)
        annotation (points=[-10, 72; -10, 96], style(thickness=4));
      connect(antiRollBar.frame_C, frame_C)
        annotation (points=[-38, -2; -10, -2; -10, 26; 74, 26]);
      connect(antiRollBar.frame_X_2, MacPherson_2.frame_U)
        annotation (points=[-61, 17; -61, 52; -10, 72], style(thickness=4));
      connect(antiRollBar.frame_X_1, MacPherson_1.frame_U)
        annotation (points=[-60, -20; -60, -56; -12, -70], style(thickness=4));
      connect(rackSteering.frame_X_2, MacPherson_2.frame_S) annotation (points=
            [28, 2; 28, 20; 34, 24; 34, 30; 28, 34; 28, 80; 6, 80; 6, 33], 
          style(thickness=4));
      connect(MacPherson_1.frame_S, rackSteering.frame_X_1) annotation (points=
            [4, -32.9625; 4, -80; 28, -80; 28, -40], style(thickness=4));
    end MacPhersonSuspension3;
    
    model MultiLink4Suspension5 
      "MultiLink4 suspension containing links with mass properties and an anti roll bar linkage"
       
      
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/MultiLink4_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="MultiLink", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.51, 
          height=0.84), 
        Diagram, 
        Documentation(info="<HTML>
<p>
This Multilink 4 suspensio is basically a trailing arm mounted in the chassis with a bushing. To stabilise the motion, three additional links  connects the chassis and the arm. These have bushings at each end.
</p>
<p>
Advantages:
Good compromise between a controlable wheel motion and comfort.
</p>
<p>
Disadvantages:
More expensive than trailing arm suspensions. Requires more space.</p>
<p>
Usage:
The multilink is common in rear suspensions of more expensive cars from mid size and up. There are both MultiLink4 and MultiLink5 suspensions.
</p>
<p>
This version contains no ideal joints and the masses of the links are included. In addition the anti roll bar is also modelled as a linkage with bushings.</p>
</HTML>"));
      Chassis.Components.Strut Strut_2(
        rB=data.rCS_2, 
        s0=sqrt((data.rCS_2 - data.rUS_2)*(data.rCS_2 - data.rUS_2)) + data.
            q0S_1, 
        data=data.data_S_2, 
        rA=data.rUS_2 - data.rUW_2) annotation (extent=[20, 0; 60, 40]);
      Components.MultiLink42 multiLink_2(
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCL3=data.rCL3_2, 
        rCU=data.rCU_2, 
        rUL1=data.rUL1_2, 
        rUL2=data.rUL2_2, 
        rUL3=data.rUL3_2, 
        rUW=data.rUW_2, 
        data_CL1=data.data_CL1_2, 
        data_UL1=data.data_UL1_2, 
        data_CL2=data.data_CL2_2, 
        data_UL2=data.data_UL2_2, 
        data_CL3=data.data_CL3_2, 
        data_UL3=data.data_UL3_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_CU=data.data_CU_2)
        annotation (extent=[-30, 80; 10, 40], rotation=270);
      Components.MultiLink42 multiLink_1(
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCL3=data.rCL3_1, 
        rCU=data.rCU_1, 
        rUL1=data.rUL1_1, 
        rUL2=data.rUL2_1, 
        rUL3=data.rUL3_1, 
        rUW=data.rUW_1, 
        data_CL1=data.data_CL1_1, 
        data_UL1=data.data_UL1_1, 
        data_CL2=data.data_CL2_1, 
        data_UL2=data.data_UL2_1, 
        data_CL3=data.data_CL3_1, 
        data_UL3=data.data_UL3_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_CU=data.data_CU_1)
        annotation (extent=[-30, -80; 10, -40], rotation=270);
      Chassis.Components.Strut Strut_1(
        rB=data.rCS_1, 
        data=data.data_S_1, 
        rA=data.rUS_1 - data.rUW_1, 
        s0=sqrt((data.rCS_1 - data.rUS_1)*(data.rCS_1 - data.rUS_1)) + data.
            q0S_1) annotation (extent=[20, -40; 60, 0]);
      Chassis.Components.BushingAntiRollBar antiRollBar(
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL4_1, 
        rXL_1=data.rUL4_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL4_2, 
        rXL_2=data.rUL4_2, 
        rX_2=data.rUW_2, 
        data=data.data_A) annotation (extent=[-80, -20; -40, 20], rotation=270);
      parameter Chassis.Data.Multilink4Data data
        annotation (extent=[-100, 60; -80, 80]);
    equation 
      connect(antiRollBar.frame_X_2, multiLink_2.frame_U)
        annotation (points=[-68, 22; -68, 80; -10.2, 80], style(thickness=4));
      connect(multiLink_2.frame_U, frame_U_2)
        annotation (points=[-10, 80; -10, 90], style(thickness=4));
      connect(multiLink_1.frame_U, frame_U_1)
        annotation (points=[-10, -80; -10, -90], style(thickness=4));
      connect(antiRollBar.frame_X_1, multiLink_1.frame_U)
        annotation (points=[-68, -22; -68, -80; -10, -80], style(thickness=4));
      connect(antiRollBar.frame_C, frame_C)
        annotation (points=[-40, 0; 80, 0; 80, 36], style(thickness=4));
      connect(multiLink_2.frame_C, frame_C) annotation (points=[-10, 40; -10, 0; 
             80, 0; 80, 36], style(thickness=4));
      connect(multiLink_1.frame_C, frame_C) annotation (points=[-10, -38; -10, 
            0; 80, 0; 80, 32], style(thickness=4));
      connect(Strut_1.frame_b, frame_C)
        annotation (points=[60, -20.2; 80, -20.2; 80, 38], style(thickness=4));
      connect(Strut_2.frame_b, frame_C)
        annotation (points=[60, 19.8; 80, 19.8; 80, 40], style(thickness=4));
      connect(Strut_1.frame_a, multiLink_1.frame_U) annotation (points=[20, -20; 
             -40, -20; -40, -60; -10, -82], style(thickness=4));
      connect(Strut_2.frame_a, multiLink_2.frame_U) annotation (points=[20, 20; 
             -40, 20; -40, 60; -10, 82], style(thickness=4));
    end MultiLink4Suspension5;
    
    model MultiLink4Suspension4 
      "MultiLink4 suspension containing links with mass properties and simplified anti roll"
       
      
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/MultiLink4_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="MultiLink", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.51, 
          height=0.84), 
        Diagram, 
        Documentation(info="<HTML>
<p>
This Multilink 4 suspensio is basically a trailing arm mounted in the chassis with a bushing. To stabilise the motion, three additional links  connects the chassis and the arm. These have bushings at each end.
</p>
<p>
Advantages:
Good compromise between a controlable wheel motion and comfort.
</p>
<p>
Disadvantages:
More expensive than trailing arm suspensions. Requires more space.</p>
<p>
Usage:
The multilink is common in rear suspensions of more expensive cars from mid size and up. There are both MultiLink4 and MultiLink5 suspensions.
</p>
<p>
This version contains no ideal joints and the masses of the links are included.</p>
</HTML>"));
      Chassis.Components.Strut Strut_2(
        rB=data.rCS_2, 
        s0=sqrt((data.rCS_2 - data.rUS_2)*(data.rCS_2 - data.rUS_2)) + data.
            q0S_1, 
        data=data.data_S_2, 
        rA=data.rUS_2 - data.rUW_2) annotation (extent=[20, 0; 60, 40]);
      Components.MultiLink42 multiLink_2(
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCL3=data.rCL3_2, 
        rCU=data.rCU_2, 
        rUL1=data.rUL1_2, 
        rUL2=data.rUL2_2, 
        rUL3=data.rUL3_2, 
        rUW=data.rUW_2, 
        data_CL1=data.data_CL1_2, 
        data_UL1=data.data_UL1_2, 
        data_CL2=data.data_CL2_2, 
        data_UL2=data.data_UL2_2, 
        data_CL3=data.data_CL3_2, 
        data_UL3=data.data_UL3_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_CU=data.data_CU_2)
        annotation (extent=[-30, 80; 10, 40], rotation=270);
      Components.MultiLink42 multiLink_1(
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCL3=data.rCL3_1, 
        rCU=data.rCU_1, 
        rUL1=data.rUL1_1, 
        rUL2=data.rUL2_1, 
        rUL3=data.rUL3_1, 
        rUW=data.rUW_1, 
        data_CL1=data.data_CL1_1, 
        data_UL1=data.data_UL1_1, 
        data_CL2=data.data_CL2_1, 
        data_UL2=data.data_UL2_1, 
        data_CL3=data.data_CL3_1, 
        data_UL3=data.data_UL3_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_CU=data.data_CU_1)
        annotation (extent=[-30, -80; 10, -40], rotation=270);
      Chassis.Components.Strut Strut_1(
        rB=data.rCS_1, 
        data=data.data_S_1, 
        rA=data.rUS_1 - data.rUW_1, 
        s0=sqrt((data.rCS_1 - data.rUS_1)*(data.rCS_1 - data.rUS_1)) + data.
            q0S_1) annotation (extent=[20, -40; 60, 0]);
      Chassis.Components.SimpleAntiRollBar antiRollBar(
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL4_1, 
        rXL_1=data.rUL4_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL4_2, 
        rXL_2=data.rUL4_2, 
        rX_2=data.rUW_2, 
        data=data.data_A) annotation (extent=[-80, -20; -40, 20], rotation=270);
      parameter Chassis.Data.Multilink4Data data
        annotation (extent=[-100, 60; -80, 80]);
    equation 
      connect(antiRollBar.frame_X_2, multiLink_2.frame_U)
        annotation (points=[-68, 22; -68, 80; -10.2, 80], style(thickness=4));
      connect(multiLink_2.frame_U, frame_U_2)
        annotation (points=[-10, 80; -10, 90], style(thickness=4));
      connect(multiLink_1.frame_U, frame_U_1)
        annotation (points=[-10, -80; -10, -90], style(thickness=4));
      connect(antiRollBar.frame_X_1, multiLink_1.frame_U)
        annotation (points=[-68, -22; -68, -80; -10, -80], style(thickness=4));
      connect(antiRollBar.frame_C, frame_C)
        annotation (points=[-40, 0; 80, 0; 80, 36], style(thickness=4));
      connect(multiLink_2.frame_C, frame_C) annotation (points=[-10, 40; -10, 0; 
             80, 0; 80, 36], style(thickness=4));
      connect(multiLink_1.frame_C, frame_C) annotation (points=[-10, -38; -10, 
            0; 80, 0; 80, 32], style(thickness=4));
      connect(Strut_1.frame_b, frame_C)
        annotation (points=[60, -20.2; 80, -20.2; 80, 38], style(thickness=4));
      connect(Strut_2.frame_b, frame_C)
        annotation (points=[60, 19.8; 80, 19.8; 80, 40], style(thickness=4));
      connect(Strut_1.frame_a, multiLink_1.frame_U) annotation (points=[20, -20; 
             -40, -20; -40, -60; -10, -82], style(thickness=4));
      connect(Strut_2.frame_a, multiLink_2.frame_U) annotation (points=[20, 20; 
             -40, 20; -40, 60; -10, 82], style(thickness=4));
    end MultiLink4Suspension4;
    
    model MultiLink4Suspension3 "MultiLink4 suspension with massless rods" 
      
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/MultiLink4_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="MultiLink", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.51, 
          height=0.84), 
        Diagram, 
        Documentation(info="<HTML>
<p>
This Multilink 4 suspensio is basically a trailing arm mounted in the chassis with a bushing. To stabilise the motion, three additional links  connects the chassis and the arm. These have bushings at each end.
</p>
<p>
Advantages:
Good compromise between a controlable wheel motion and comfort.
</p>
<p>
Disadvantages:
More expensive than trailing arm suspensions. Requires more space.</p>
<p>
Usage:
The multilink is common in rear suspensions of more expensive cars from mid size and up. There are both MultiLink4 and MultiLink5 suspensions.
</p>
<p>
This version contains no ideal joints and the links are considered massless.</p>
</HTML>"));
      Chassis.Components.Strut Strut_2(
        rB=data.rCS_2, 
        s0=sqrt((data.rCS_2 - data.rUS_2)*(data.rCS_2 - data.rUS_2)) + data.
            q0S_1, 
        data=data.data_S_2, 
        rA=data.rUS_2 - data.rUW_2) annotation (extent=[20, 0; 60, 40]);
      Components.MultiLink4 multiLink_2(
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCL3=data.rCL3_2, 
        rCU=data.rCU_2, 
        rUL1=data.rUL1_2, 
        rUL2=data.rUL2_2, 
        rUL3=data.rUL3_2, 
        rUW=data.rUW_2, 
        data_CL1=data.data_CL1_2, 
        data_UL1=data.data_UL1_2, 
        data_CL2=data.data_CL2_2, 
        data_UL2=data.data_UL2_2, 
        data_CL3=data.data_CL3_2, 
        data_UL3=data.data_UL3_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_CU=data.data_CU_2)
        annotation (extent=[-30, 80; 10, 40], rotation=270);
      Components.MultiLink4 multiLink_1(
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCL3=data.rCL3_1, 
        rCU=data.rCU_1, 
        rUL1=data.rUL1_1, 
        rUL2=data.rUL2_1, 
        rUL3=data.rUL3_1, 
        rUW=data.rUW_1, 
        data_CL1=data.data_CL1_1, 
        data_UL1=data.data_UL1_1, 
        data_CL2=data.data_CL2_1, 
        data_UL2=data.data_UL2_1, 
        data_CL3=data.data_CL3_1, 
        data_UL3=data.data_UL3_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_CU=data.data_CU_1)
        annotation (extent=[-30, -80; 10, -40], rotation=270);
      Chassis.Components.Strut Strut_1(
        rB=data.rCS_1, 
        data=data.data_S_1, 
        rA=data.rUS_1 - data.rUW_1, 
        s0=sqrt((data.rCS_1 - data.rUS_1)*(data.rCS_1 - data.rUS_1)) + data.
            q0S_1) annotation (extent=[20, -40; 60, 0]);
      Chassis.Components.SimpleAntiRollBar antiRollBar(
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL4_1, 
        rXL_1=data.rUL4_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL4_2, 
        rXL_2=data.rUL4_2, 
        rX_2=data.rUW_2, 
        data=data.data_A) annotation (extent=[-80, -20; -40, 20], rotation=270);
      parameter Chassis.Data.Multilink4Data data
        annotation (extent=[-100, 60; -80, 80]);
    equation 
      connect(antiRollBar.frame_X_2, multiLink_2.frame_U)
        annotation (points=[-68, 22; -68, 80; -10.2, 80], style(thickness=4));
      connect(multiLink_2.frame_U, frame_U_2)
        annotation (points=[-10, 80; -10, 90], style(thickness=4));
      connect(multiLink_1.frame_U, frame_U_1)
        annotation (points=[-10, -80; -10, -90], style(thickness=4));
      connect(antiRollBar.frame_X_1, multiLink_1.frame_U)
        annotation (points=[-68, -22; -68, -80; -10, -80], style(thickness=4));
      connect(antiRollBar.frame_C, frame_C)
        annotation (points=[-40, 0; 80, 0; 80, 36], style(thickness=4));
      connect(multiLink_2.frame_C, frame_C) annotation (points=[-10, 40; -10, 0; 
             80, 0; 80, 36], style(thickness=4));
      connect(multiLink_1.frame_C, frame_C) annotation (points=[-10, -38; -10, 
            0; 80, 0; 80, 32], style(thickness=4));
      connect(Strut_1.frame_b, frame_C)
        annotation (points=[60, -20.2; 80, -20.2; 80, 38], style(thickness=4));
      connect(Strut_2.frame_b, frame_C)
        annotation (points=[60, 19.8; 80, 19.8; 80, 40], style(thickness=4));
      connect(Strut_1.frame_a, multiLink_1.frame_U) annotation (points=[20, -20; 
             -40, -20; -40, -60; -10, -82], style(thickness=4));
      connect(Strut_2.frame_a, multiLink_2.frame_U) annotation (points=[20, 20; 
             -40, 20; -40, 60; -10, 82], style(thickness=4));
    end MultiLink4Suspension3;
    
    model TrailingArmSuspension 
      "Trailing arm suspension with anti roll linkage." 
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/TrailingArm_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="TrailingArm", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram, 
        Documentation(info="<HTML>
<p>
Trailing arm suspension employs two trailing arms which are pivoted to the car body at the arm's front end's mountin points. To the other end of the arm, the wheel is mouted. The suspension is normally handled by a coil spring at each arm. The suspending motion allow no camber control to the motion induced by body roll. A more common approach is instead the semi trailing arm is that the arm's mouting points are located along a line that is not perpedicular to the car's longitudinal axis, generating a a varying camber, depending on body roll. The drawback is that the camber angle change also occure when going over uneven roads.
</p>
<p>
Advantages:
Cheap and simple, can be made compact.
</p>
<p>
Disadvantages:
Less refined than multi-link suspension which has replaced the trailing arm suspension in more expensive cars.
</p>
<p>
Usage:
Very common until the early 90's. Now often used together with a twist-beam.
</p>
</HTML>"));
      Chassis.Components.Strut Strut_2(
        rB=data.rCS_2, 
        data=data.data_S_2, 
        s0=sqrt((data.rUS_2 - data.rCS_2)*(data.rUS_2 - data.rCS_2)) + data.
            q0S_2, 
        rA=data.rUS_2 - data.rUW_2) annotation (extent=[-20, 0; 20, 40]);
      Chassis.Components.Strut Strut_1(
        rB=data.rCS_1, 
        data=data.data_S_1, 
        s0=sqrt((data.rUS_1 - data.rCS_1)*(data.rUS_1 - data.rCS_1)) + data.
            q0S_1, 
        rA=data.rUS_1 - data.rUW_1) annotation (extent=[-20, -40; 20, 0]);
      Chassis.Components.SimpleAntiRollBar antiRollLinkage(
        rCA_1=data.rCA_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rX_2=data.rUW_2, 
        data=data.data_A, 
        rAL_1=data.rAL1_1, 
        rXL_1=data.rUL1_1, 
        rAL_2=data.rAL1_2, 
        rXL_2=data.rUL1_2) annotation (extent=[-80, -20; -40, 20], rotation=270);
      Chassis.Components.TrailingArm leftTrailingArm(
        rCU=data.rCU_1, 
        rUW=data.rUW_1, 
        nCU=data.nCU_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1) annotation (extent=[-30, -40; 10, -80], rotation=180);
      Chassis.Components.TrailingArm rightTrailingArm(
        rCU=data.rCU_2, 
        rUW=data.rUW_2, 
        nCU=data.nCU_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2) annotation (extent=[-30, 40; 10, 80], rotation=180);
      parameter Chassis.Data.TrailingArmData data
        annotation (extent=[-100, 52; -80, 72]);
    equation 
      connect(antiRollLinkage.frame_X_2, rightTrailingArm.frame_U)
        annotation (points=[-68, 20; -68, 80; -10, 80], style(thickness=4));
      connect(antiRollLinkage.frame_X_1, leftTrailingArm.frame_U)
        annotation (points=[-68, -20; -68, -80; -10, -80], style(thickness=4));
      connect(leftTrailingArm.frame_C, frame_C) annotation (points=[10, -60; 40, 
             -60; 40, 40; 78, 40], style(thickness=4));
      connect(rightTrailingArm.frame_C, frame_C) annotation (points=[12, 60; 40, 
             60; 40, 40; 78, 40], style(thickness=4));
      connect(Strut_2.frame_b, frame_C) annotation (points=[20, 20; 40, 20; 40, 
             40; 80, 40], style(thickness=4));
      connect(Strut_1.frame_b, frame_C) annotation (points=[20, -20; 40, -20; 
            40, 40; 74, 40], style(thickness=4));
      connect(Strut_2.frame_a, rightTrailingArm.frame_U)
        annotation (points=[-20, 20; -40, 20; -40, 80; -10, 80]);
      connect(Strut_1.frame_a, leftTrailingArm.frame_U) annotation (points=[-20, 
             -20; -40, -20; -40, -80; -10, -80], style(thickness=4));
      connect(antiRollLinkage.frame_C, frame_C) annotation (points=[-40, 0; 40, 
             0; 40, 40; 80, 40], style(thickness=4));
      connect(leftTrailingArm.frame_U, frame_U_1)
        annotation (points=[-10, -82; -10, -90], style(thickness=4));
      connect(rightTrailingArm.frame_U, frame_U_2)
        annotation (points=[-10, 82; -10, 90], style(thickness=4));
    end TrailingArmSuspension;
    
    model SimpleSuspension 
      "Steerable suspension without geometry. Camber and toe are give as functions of bounce."
       
      
      extends Chassis.Interfaces.SteerableSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="Simple", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram, 
        Documentation(info="<HTML>
<p>
The suspension characteristics is modelled as camber and toe-in as functions of the bounce motion. This could easily be extended to handle also camber and toe-in as functions of side force, which would make it possible to mimic the behaviour of suspensions with bushings and other flexible elements. 
</p>
<p>
Advantages:
Fast simulations, possible to do studies of how suspension characterisitcs affects the vehicle's behaviour without the need to specify the type of suspension.
</p>
<p>
Disadvantages:
When modelling a specific suspension, the characteristics must either be measured, meaning that the suspension has to be built, or that the suspension characteristics has to be calculated from a more detailed model. 
</p>
</HTML>"));
      Chassis.Components.AckermannSteering steering(
        wheelbase=data.wheelbase, 
        trackwidth=data.trackwidth, 
        ratio=data.ratio) annotation (extent=[-60, -20; -20, 20]);
      Chassis.Components.Simple simple_1(
        scaleFactor=data.scaleFactor, 
        rUW=data.rUW_1, 
        q0Strut=data.q0S_1, 
        leftLinkage=true, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_S=data.data_S_1, 
        steeringPoly=data.data_Link1_steering, 
        camberPoly=data.data_Link1_camber, 
        rCU=data.rCU_1) annotation (extent=[-30, -80; 30, -20], rotation=270);
      Chassis.Components.Simple simple_2(
        rUW=data.rUW_2, 
        q0Strut=data.q0S_2, 
        leftLinkage=false, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_S=data.data_S_2, 
        steeringPoly=data.data_Link2_steering, 
        camberPoly=data.data_Link2_camber, 
        rCU=data.rCU_2) annotation (extent=[-30, 80; 30, 20], rotation=270);
      parameter Chassis.Data.SimpleDataFront data
        annotation (extent=[-108, 40; -88, 60]);
      Utilities.Forces.SpringDamperTable1D antiRoll(data=data.data_A)
        annotation (extent=[20, -12; 40, 8], rotation=90);
    equation 
      
      connect(steering.flange_a, flange_SW) annotation (points=[-20, -2; 80, -2; 
             80, -40; 84, -40], style(color=0));
      connect(steering.flange_b2, simple_2.steerAxis)
        annotation (points=[-52, 16; -52, 48; -30, 48], style(color=0));
      connect(steering.flange_b1, simple_1.steerAxis)
        annotation (points=[-52, -16; -52, -46; -32, -46], style(color=0));
      connect(simple_2.frame_U, frame_U_2)
        annotation (points=[0, 86; 0, 90], style(thickness=4));
      connect(simple_1.frame_U, frame_U_1)
        annotation (points=[0, -86; 0, -90], style(thickness=4));
      connect(simple_1.frame_C, frame_C) annotation (points=[0, -18; 0, 10; 60, 
             10; 60, 40; 72, 40], style(thickness=4));
      connect(simple_2.frame_C, frame_C) annotation (points=[0, 20; 0, 10; 60, 
            10; 60, 40; 72, 40], style(thickness=4));
      connect(antiRoll.flange_a, simple_1.bounceAxis)
        annotation (points=[30, -12; 30, -46], style(color=58));
      connect(antiRoll.flange_b, simple_2.bounceAxis)
        annotation (points=[30, 8; 30, 46], style(color=58));
    end SimpleSuspension;
    
    model SimpleSuspensionNoSteer 
      "Suspension without geometry. No steering possibility. Camber and toe are give as functions of bounce."
       
      
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="Simple", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram, 
        Documentation(info="<HTML>
<p>
This suspension is identical to SimpleSuspension except that this is not steerable.
</p>
</HTML>"));
      Chassis.Components.Simple simple_1(
        scaleFactor=data.scaleFactor, 
        rUW=data.rUW_1, 
        q0Strut=data.q0S_1, 
        leftLinkage=true, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_S=data.data_S_1, 
        steeringPoly=data.data_Link1_steering, 
        camberPoly=data.data_Link1_camber, 
        rCU=data.rCU_1) annotation (extent=[-30, -80; 30, -20], rotation=270);
      Chassis.Components.Simple simple_2(
        rUW=data.rUW_2, 
        q0Strut=data.q0S_2, 
        leftLinkage=false, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_S=data.data_S_2, 
        steeringPoly=data.data_Link2_steering, 
        camberPoly=data.data_Link2_camber, 
        rCU=data.rCU_2) annotation (extent=[-30, 80; 30, 20], rotation=270);
      parameter Chassis.Data.SimpleDataRear data
        annotation (extent=[-108, 40; -88, 60]);
      Modelica.Mechanics.Rotational.Fixed fixed
        annotation (extent=[-62, -10; -42, 10], rotation=270);
      Utilities.Forces.SpringDamperTable1D antiRoll(data=data.data_A)
        annotation (extent=[20, -8; 40, 12], rotation=90);
    equation 
      
      connect(simple_2.frame_U, frame_U_2)
        annotation (points=[0, 86; 0, 90], style(thickness=4));
      connect(simple_1.frame_U, frame_U_1)
        annotation (points=[0, -86; 0, -90], style(thickness=4));
      connect(simple_1.frame_C, frame_C) annotation (points=[0, -18; 0, 10; 60, 
             10; 60, 40; 72, 40], style(thickness=4));
      connect(simple_2.frame_C, frame_C) annotation (points=[0, 20; 0, 10; 60, 
            10; 60, 40; 72, 40], style(thickness=4));
      connect(fixed.flange_b, simple_2.steerAxis)
        annotation (points=[-52, 0; -40, 0; -40, 46; -32, 46], style(color=0));
      connect(fixed.flange_b, simple_1.steerAxis) annotation (points=[-52, 0; -40, 
             0; -40, -46; -32, -46], style(color=0));
      connect(antiRoll.flange_a, simple_1.bounceAxis)
        annotation (points=[30, -8; 30, -48], style(color=58));
      connect(antiRoll.flange_b, simple_2.bounceAxis)
        annotation (points=[30, 12; 30, 46], style(color=58));
    end SimpleSuspensionNoSteer;
    
    model MacPhersonSuspension2 "MacPherson suspension with ideal joints." 
      extends Chassis.Interfaces.SteerableSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Diagram, 
        Images(Parameters(name="|Geometric reference|", source=
                "images/MacPherson_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="MacPherson", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
This version does not contain bushings and is completely rigid except for the strut.</p>
</HTML>"));
      Components.RackSteering4 rackSteering(
        rXL_1=data.rUL3_1, 
        rRL_1=data.rRL3_1, 
        rX_1=data.rUW_1, 
        rXL_2=data.rUL3_2, 
        rRL_2=data.rRL3_2, 
        rX_2=data.rUW_2, 
        ratioWheelToRack=data.ratioWheelToRack, 
        iSW=data.iSW, 
        data_R=data.data_R) annotation (extent=[20, -41; 60, -1]);
      Components.MacPherson MacPherson_1(
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCS=data.rCS_1, 
        rUS=data.rUS_1, 
        rUL1L2=data.rUL1L2_1, 
        rUW=data.rUW_1, 
        q0S=data.q0S_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        data_S=data.data_S_1, 
        mL1L2=1, 
        i11L1L2=1, 
        i22L1L2=1, 
        i33L1L2=1, 
        rRL3=data.rRL3_1, 
        rUL3=data.rUL3_1)
        annotation (extent=[-30.5, -69.5; 9, -30], rotation=270);
      Components.MacPherson MacPherson_2(
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCS=data.rCS_2, 
        rUS=data.rUS_2, 
        rUL1L2=data.rUL1L2_2, 
        rUW=data.rUW_2, 
        q0S=data.q0S_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        data_S=data.data_S_2, 
        mL1L2=1, 
        i11L1L2=1, 
        i22L1L2=1, 
        i33L1L2=1, 
        rRL3=data.rRL3_2, 
        rUL3=data.rUL3_2) annotation (extent=[10, 30; -30, 70], rotation=90);
      parameter Chassis.Data.MacPhersonData data
        annotation (extent=[-114, 40; -80, 80]);
      Chassis.Components.SimpleAntiRollBar antiRollBar(
        scaleFactor=data.scaleFactor, 
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL4_1, 
        rXL_1=data.rUL4_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL4_2, 
        rXL_2=data.rUL4_2, 
        rX_2=data.rUW_2, 
        data=data.data_A)
        annotation (extent=[-71.5, -19.5; -36.5, 15.5], rotation=270);
    equation 
      connect(rackSteering.frame_C, frame_C) annotation (points=[60.6, -12.8; 
            85, -12.8; 85, 39], style(thickness=4));
      connect(rackSteering.flange_a, flange_SW)
        annotation (points=[61, -29; 85, -29; 85, -39], style(color=0));
      connect(MacPherson_1.frame_C, frame_C) annotation (points=[-10.9475, -29.0125; 
             -10.9475, 39; 85, 39], style(thickness=4));
      connect(MacPherson_2.frame_C, frame_C)
        annotation (points=[-10.2, 29; -10.2, 39; 85, 39], style(thickness=4));
      connect(MacPherson_1.frame_U, frame_U_1) annotation (points=[-10.5525, -70.4875; 
             -10.5525, -80.3656; -9.5525, -80.3656; -9.5525, -90.2438; -9, -90.2438; 
             -9, -105], style(thickness=4));
      connect(MacPherson_2.frame_U, frame_U_2) annotation (points=[-9.8, 71; -9.8, 
             81.75; -8.8, 81.75; -8.8, 92.5; -10, 92.5; -10, 96], style(
            thickness=4));
      connect(antiRollBar.frame_C, frame_C)
        annotation (points=[-35.975, -2.175; -10, -2.175; -10, 39; 85, 39]);
      connect(antiRollBar.frame_X_2, MacPherson_2.frame_U) annotation (points=[
            -61.175, 16.375; -61.175, 71; -9.8, 71], style(thickness=4));
      connect(antiRollBar.frame_X_1, MacPherson_1.frame_U) annotation (points=[
            -60.825, -20.025; -60.825, -70.4875; -10.5525, -70.4875], style(
            thickness=4));
      connect(rackSteering.frame_X_2, MacPherson_2.frame_S) annotation (points=
            [27.8, 0; 27.8, 20; 34, 24; 34, 30; 28, 34; 28, 80; 3, 80; 3, 33], 
          style(thickness=4));
      connect(MacPherson_1.frame_S, rackSteering.frame_X_1) annotation (points=
            [2.0875, -32.9625; 2.0875, -80; 27.8, -80; 27.8, -42], style(
            thickness=4));
    end MacPhersonSuspension2;
    
    model DoubleWishBoneSuspension 
      "Double wish-bone (A-arm) suspension with ideal joints." 
      
      extends Chassis.Interfaces.SteerableSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/DoubleWishBone_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 24], 
            string="DoubleWishBone", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram, 
        Documentation(info="<HTML>
<p>
Basically, double wishbones suspension always maintains the wheel perpendicular to the road surface, irrespective of the wheel's bouncing and steering movement, which gives a good camber control.
Traditional double wishbones consists of 2 parellel wishbones of equal length which has the drawback of excessive tyre scrubbing due to variation in track width as the wheel moved off the neutral position.
This effect can be reduced with unequal-length non-parellel wishbones that are more common nowadays. Anti-dive or anti-squat function is achieved by tilting the wishbones relative to another. 
</p>
<p>
Advantages:
The double wishbone suspension gives the designer good control over the wheel's motion and many tuning possibilites.
</p>
<p>
Disadvantages:
The double wishbone suspension is more expensive and requires more space than for example the MacPherson suspension.
</p>
<p>
Usage:
The double wishbone is used both in front and rear suspensions, mostly in race and high performance cars.
</p>
</HTML>"));
      Chassis.Components.Strut strut_2(
        scaleFactor=data.scaleFactor, 
        rA=data.rSL3L4_2 - data.rUL3L4_2, 
        rB=data.rCS_2, 
        s0=sqrt((data.rSL3L4_2 - data.rCS_2)*(data.rSL3L4_2 - data.rCS_2)) + 
            data.q0S_2, 
        data=data.data_S_2) annotation (extent=[-20, 0; 20, 40]);
      /*  Chassis.Components.DoubleWishBone doubleWishBone_1(
    scaleFactor=data.scaleFactor, 
    rCL1=data.rCL1_1, 
    rCL2=data.rCL2_1, 
    rCL3=data.rCL3_1, 
    rCL4=data.rCL4_1, 
    rUL3L4=data.rUL3L4_1, 
    rUL1L2=data.rUL1L2_1, 
    rUW=data.rUW_1, 
    rRL5=data.rRL5_1, 
    rUL5=data.rUL5_1) annotation (extent=[-22, -80; 18, -40], rotation=270);
  Chassis.Components.DoubleWishBone doubleWishBone_2(
    scaleFactor=data.scaleFactor, 
    rCL1=data.rCL1_2, 
    rCL2=data.rCL2_2, 
    rCL3=data.rCL3_2, 
    rCL4=data.rCL4_2, 
    rUL3L4=data.rUL3L4_2, 
    rUL1L2=data.rUL1L2_2, 
    rUW=data.rUW_2, 
    rRL5=data.rRL5_2, 
    rUL5=data.rUL5_2) annotation (extent=[-24, 80; 16, 40], rotation=270);
  Chassis.Components.RackSteering rackSteering(
    scaleFactor=data.scaleFactor, 
    rX_1=data.rUW_1, 
    rX_2=data.rUW_2, 
    ratioWheelToRack=data.ratioWheelToRack, 
    rXL_1=data.rUL5_1, 
    rRL_1=data.rRL5_1, 
    rXL_2=data.rUL5_2, 
    rRL_2=data.rRL5_2) annotation (extent=[26, -52; 66, -12]);
*/
      
      Chassis.Components.DoubleWishBone doubleWishBone_1(
        scaleFactor=data.scaleFactor, 
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCL3=data.rCL3_1, 
        rCL4=data.rCL4_1, 
        rUL3L4=data.rUL3L4_1, 
        rUL1L2=data.rUL1L2_1, 
        rUW=data.rUW_1, 
        rRL5=data.rRL5_1, 
        rUL5=data.rUL5_1) annotation (extent=[-30, -80; 10, -40], rotation=270);
      Chassis.Components.DoubleWishBone doubleWishBone_2(
        scaleFactor=data.scaleFactor, 
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCL3=data.rCL3_2, 
        rCL4=data.rCL4_2, 
        rUL3L4=data.rUL3L4_2, 
        rUL1L2=data.rUL1L2_2, 
        rUW=data.rUW_2, 
        rRL5=data.rRL5_2, 
        rUL5=data.rUL5_2) annotation (extent=[-30, 80; 10, 40], rotation=270);
      Chassis.Components.RackSteering rackSteering(
        scaleFactor=data.scaleFactor, 
        rX_1=data.rUW_1, 
        rX_2=data.rUW_2, 
        ratioWheelToRack=data.ratioWheelToRack, 
        rXL_1=data.rUL5_1, 
        rRL_1=data.rRL5_1, 
        rXL_2=data.rUL5_2, 
        rRL_2=data.rRL5_2) annotation (extent=[26, -40; 66, 0]);
      
      Chassis.Components.SimpleAntiRollBar antiRollBar(
        scaleFactor=data.scaleFactor, 
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL6_1, 
        rXL_1=data.rUL6_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL6_2, 
        rXL_2=data.rUL6_2, 
        rX_2=data.rUW_2, 
        data=data.data_A) annotation (extent=[-80, -20; -40, 20], rotation=270);
      Chassis.Components.Strut strut_1(
        scaleFactor=data.scaleFactor, 
        rA=data.rSL3L4_1 - data.rUL3L4_1, 
        rB=data.rCS_1, 
        s0=sqrt((data.rSL3L4_1 - data.rCS_1)*(data.rSL3L4_1 - data.rCS_1)) + 
            data.q0S_1, 
        data=data.data_S_1) annotation (extent=[-20, -40; 20, 0]);
      parameter Chassis.Data.DoubleWishBoneData data
        annotation (extent=[-114, 40; -80, 80]);
    equation 
      connect(doubleWishBone_1.frame_L3L4, strut_1.frame_a) annotation (points=
            [-30.6, -60; -33, -60; -33, -20; -20, -20], style(thickness=4));
      connect(doubleWishBone_2.frame_L3L4, strut_2.frame_a) annotation (points=
            [-30.6, 60; -33, 60; -33, 20; -22, 20], style(thickness=4));
      connect(frame_U_2, doubleWishBone_2.frame_U)
        annotation (points=[-4, 90; -9.8, 82], style(thickness=4));
      connect(doubleWishBone_1.frame_U, frame_U_1)
        annotation (points=[-9.8, -82; -4, -90], style(thickness=4));
      connect(antiRollBar.frame_X_1, doubleWishBone_1.frame_U) annotation (
          points=[-68, -20; -68, -86; -4, -86; -9.8, -84], style(thickness=4));
      connect(antiRollBar.frame_X_2, doubleWishBone_2.frame_U) annotation (
          points=[-68, 22; -68, 87; -4, 87; -9.8, 82], style(thickness=4));
      connect(doubleWishBone_2.frame_C, frame_C) annotation (points=[-10.2, 40; 
             -4, 32; 28, 32; 28, 38; 76, 38], style(thickness=4));
      connect(strut_2.frame_b, frame_C) annotation (points=[20, 20; 28, 20; 28, 
             38; 78, 38], style(thickness=4));
      connect(strut_1.frame_b, frame_C) annotation (points=[20, -20; 28, -20; 
            28, 38; 80, 38], style(thickness=4));
      connect(doubleWishBone_1.frame_C, frame_C) annotation (points=[-10.2, -40; 
             -4, -32; 28, -32; 28, 38; 74, 38], style(thickness=4));
      connect(antiRollBar.frame_C, frame_C) annotation (points=[-40, 0; 28, 0; 
            28, 38; 78, 38], style(thickness=4));
      connect(rackSteering.flange_a, flange_SW)
        annotation (points=[65, -28; 72, -40], style(color=0));
      connect(rackSteering.frame_C, frame_C)
        annotation (points=[64.6, -11.8; 66, 38; 76, 38], style(thickness=4));
      connect(doubleWishBone_1.frame_S, rackSteering.frame_X_1)
        annotation (points=[2, -44; 31.8, -40]);
      connect(rackSteering.frame_X_2, doubleWishBone_2.frame_S)
        annotation (points=[34, 0; 32, 34; 2, 44]);
    end DoubleWishBoneSuspension;
    
    model DoubleWishBoneSuspension2 
      "Double wish-bone (A-arm) suspension with an onboard belle-ville to decouple suspension and anti-roll characteristics."
       
      
      extends Chassis.Interfaces.SteerableSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/BelleVille_text.png")), 
        Icon(Text(
            extent=[-90, 96; 70, 40], 
            string="DoubleWishBone", 
            style(color=0)), Text(
            extent=[-90, 64; 70, 8], 
            string="MonoDamperBelleVille", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.47, 
          height=0.96), 
        Diagram, 
        Documentation(info="<HTML>
<p>
The double wishbone linkages are suspended by a single strut with BelleVille anti roll spring. The purpose is to decouple pitch and roll motion of cars. In racing this is desired while it makes tuning more convinient. 
</p>
<p>
Usage:
In some race cars
</p>
</HTML>"));
      Chassis.Components.DoubleWishBone LeftDoubleWishBone(
        scaleFactor=data.scaleFactor, 
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCL3=data.rCL3_1, 
        rCL4=data.rCL4_1, 
        rUL3L4=data.rUL3L4_1, 
        rUL1L2=data.rUL1L2_1, 
        rUW=data.rUW_1, 
        rRL5=data.rRL5_1, 
        rUL5=data.rUL5_1) annotation (extent=[-30, -80; 10, -40], rotation=270);
      Chassis.Components.DoubleWishBone rightDoubleWishBone(
        scaleFactor=data.scaleFactor, 
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCL3=data.rCL3_2, 
        rCL4=data.rCL4_2, 
        rUL3L4=data.rUL3L4_2, 
        rUL1L2=data.rUL1L2_2, 
        rUW=data.rUW_2, 
        rRL5=data.rRL5_2, 
        rUL5=data.rUL5_2) annotation (extent=[-30, 80; 10, 40], rotation=270);
      Chassis.Components.RackSteering rackSteering(
        scaleFactor=data.scaleFactor, 
        rX_1=data.rUW_1, 
        rX_2=data.rUW_2, 
        ratioWheelToRack=data.ratioWheelToRack, 
        rXL_1=data.rUL5_1, 
        rRL_1=data.rRL5_1, 
        rXL_2=data.rUL5_2, 
        rRL_2=data.rRL5_2) annotation (extent=[20, -40; 60, 0]);
      
      Chassis.Components.MonoDamperBelleVilleAntiRoll 
        MonoDamperBelleVilleAntiRoll(
        scaleFactor=data.scaleFactor, 
        rCP=data.rCP, 
        rCS=data.rCS, 
        rPS=data.rPS, 
        nCP=data.nCP, 
        q0S=data.q0S, 
        rPL_1=data.rPL6_1, 
        rXL_1=data.rUL3L4_1, 
        rX_1=data.rUL3L4_1, 
        rPL_2=data.rPL6_2, 
        rXL_2=data.rUL3L4_2, 
        rX_2=data.rUL3L4_2, 
        data=data.data_S, 
        data_A=data.data_A) annotation (extent=[-40, -20; 0, 20]);
      parameter Chassis.Data.DoubleWishBoneData2 data
        annotation (extent=[-100, 20; -60, 60]);
    equation 
      connect(rightDoubleWishBone.frame_U, frame_U_2)
        annotation (points=[-10, 84; -10, 90], style(thickness=4));
      connect(rackSteering.frame_C, frame_C)
        annotation (points=[60, -12; 60, 40; 76, 40], style(thickness=4));
      connect(rackSteering.flange_a, flange_SW)
        annotation (points=[60, -28; 60, -40; 86, -40], style(color=0));
      connect(MonoDamperBelleVilleAntiRoll.FromChassis2, rightDoubleWishBone.
        frame_L3L4)
        annotation (points=[-34, 22; -30.6, 60], style(thickness=4));
      connect(MonoDamperBelleVilleAntiRoll.FromChassis1, frame_C)
        annotation (points=[2, 0; 20, 0; 20, 40; 70, 40]);
      connect(rightDoubleWishBone.frame_C, frame_C) annotation (points=[-10, 40; 
             -10, 30; 20, 30; 20, 40; 76, 40; 75.25, 40.125; 74, 40], style(
            thickness=4));
      connect(LeftDoubleWishBone.frame_C, frame_C) annotation (points=[-10, -40; 
             -10, -28; 20, -28; 20, 40; 74, 40], style(thickness=4));
      connect(LeftDoubleWishBone.frame_U, frame_U_1)
        annotation (points=[-10, -84; -10, -90], style(thickness=4));
      connect(MonoDamperBelleVilleAntiRoll.FromChassis, LeftDoubleWishBone.
        frame_L3L4)
        annotation (points=[-34, -22; -30.6, -60], style(thickness=4));
      connect(rightDoubleWishBone.frame_S, rackSteering.frame_X_2) annotation (
          points=[2, 44; 2, 36; 26, 36; 28, 2], style(fillColor=10, fillPattern
            =1));
      connect(rackSteering.frame_X_1, LeftDoubleWishBone.frame_S) annotation (
          points=[28, -42; 28, -46; 16, -48; 16, -42; 4, -44], style(fillColor=
              10, fillPattern=1));
    end DoubleWishBoneSuspension2;
    
    model DoubleWishBoneSuspension3 
      "Double wish-bone (A-arm) suspension with onboard mounted struts." 
      
      extends Chassis.Interfaces.SteerableSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/TwinStrut_text.png")), 
        Icon(Text(
            extent=[-90, 104; 70, 48], 
            string="DoubleWishBone", 
            style(color=0)), Text(
            extent=[-90, 70; 70, 14], 
            string="TwinDamperTorsion", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram, 
        Documentation(info="<HTML>
<p>
This suspension consist of a double wish bone linkage with a speclial linkage to mount the struts between the mouting points of the wishbones. The purpose is to be able to mount these inside the body of a formula car.
<p>
Usage:
In some formula race cars.
</p>
</HTML>"));
      Chassis.Components.DoubleWishBone LeftDoubleWishBone(
        scaleFactor=data.scaleFactor, 
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCL3=data.rCL3_1, 
        rCL4=data.rCL4_1, 
        rUL3L4=data.rUL3L4_1, 
        rUL1L2=data.rUL1L2_1, 
        rUW=data.rUW_1, 
        rRL5=data.rRL5_1, 
        rUL5=data.rUL5_1) annotation (extent=[-30, -80; 10, -40], rotation=270);
      Chassis.Components.DoubleWishBone rightDoubleWishBone(
        scaleFactor=data.scaleFactor, 
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCL3=data.rCL3_2, 
        rCL4=data.rCL4_2, 
        rUL3L4=data.rUL3L4_2, 
        rUL1L2=data.rUL1L2_2, 
        rUW=data.rUW_2, 
        rRL5=data.rRL5_2, 
        rUL5=data.rUL5_2) annotation (extent=[-30, 80; 10, 40], rotation=270);
      Chassis.Components.RackSteering rackSteering(
        scaleFactor=data.scaleFactor, 
        rXL_1=data.rUL5_1, 
        rRL_1=data.rRL5_1, 
        rX_1=data.rUW_1, 
        rXL_2=data.rUL5_2, 
        rRL_2=data.rRL5_2, 
        rX_2=data.rUW_2, 
        ratioWheelToRack=data.ratioWheelToRack)
        annotation (extent=[20, -40; 60, 0]);
      Chassis.Components.TwinDamperTorsionAntiRoll twinDamperTorsionAntiRoll(
        scaleFactor=data.scaleFactor, 
        rCP_1=data.rCP_1, 
        rCS_1=data.rCS_1, 
        rPS_1=data.rPS_1, 
        rPL1_1=data.rPL6_1, 
        rX_1=data.rUL3L4_1, 
        rXL1_1=data.rUL6_1, 
        rPL2_1=data.rPL7_1, 
        rAL2_1=data.rAL7_1, 
        rCA_1=data.rCA_1, 
        nP_1=data.nP_1, 
        q0S_1=data.q0S_1, 
        rCP_2=data.rCP_2, 
        rCS_2=data.rCS_2, 
        rPS_2=data.rPS_2, 
        rPL1_2=data.rPL6_2, 
        rX_2=data.rUL3L4_2, 
        rXL1_2=data.rUL6_2, 
        rPL2_2=data.rPL7_2, 
        rAL2_2=data.rAL7_2, 
        rCA_2=data.rCA_2, 
        nP_2=data.nP_2, 
        q0S_2=data.q0S_1, 
        data_A=data.data_A, 
        data_S_1=data.data_S_1, 
        data_S_2=data.data_S_2)
        annotation (extent=[-46, 20; -6, -20], rotation=180);
      parameter Data.DoubleWishBoneData3 data
        annotation (extent=[-102, 26; -82, 46]);
    equation 
      connect(LeftDoubleWishBone.frame_U, frame_U_1)
        annotation (points=[-10, -84; -10, -90]);
      connect(rightDoubleWishBone.frame_U, frame_U_2)
        annotation (points=[-10, 84; -10, 90]);
      connect(rightDoubleWishBone.frame_L3L4, twinDamperTorsionAntiRoll.
        frame_X_2) annotation (points=[-30.6, 60; -37.8, 60; -37.8, 22], style(
            thickness=4));
      connect(twinDamperTorsionAntiRoll.frame_X_1, LeftDoubleWishBone.
        frame_L3L4)
        annotation (points=[-38, -22; -38, -60; -30, -60], style(thickness=4));
      connect(rightDoubleWishBone.frame_C, frame_C) annotation (points=[-10, 38; 
             -10, 30; 10, 30; 10, 40; 78, 40], style(thickness=4));
      connect(twinDamperTorsionAntiRoll.FromChassis, frame_C)
        annotation (points=[-6, 0; 10, 0; 10, 40; 74, 40], style(thickness=4));
      connect(LeftDoubleWishBone.frame_C, frame_C) annotation (points=[-10, -40; 
             -10, -30; 10, -30; 10, 40; 74, 40], style(thickness=4));
      connect(rackSteering.flange_a, flange_SW)
        annotation (points=[64, -28; 70, -28], style(color=0));
      connect(rackSteering.frame_C, frame_C)
        annotation (points=[60, -11.8; 60, 40; 74, 40], style(thickness=4));
      connect(rightDoubleWishBone.frame_S, rackSteering.frame_X_2)
        annotation (points=[4, 44; 2, 36; 30, 34; 30, 2]);
      connect(rackSteering.frame_X_1, LeftDoubleWishBone.frame_S)
        annotation (points=[28, -42; 26, -64; 14, -60; 10, -42; 4, -42]);
    end DoubleWishBoneSuspension3;
    
    model FixedAxleSuspension 
      "Suspension with wheel mount points fixed rigidly to the chassis" 
      
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      
      ModelicaAdditions.MultiBody.Parts.FrameTranslation rSuspension(r=
            Utilities.Functions.EWM((data.rUW_1 + data.rUW_2)/2, data.
            scaleFactor)) annotation (extent=[20, 30; 40, 50], rotation=180);
      Utilities.Parts.Body rLeftWheel(r=Utilities.Functions.EWM(data.rUW_1 - (
            data.rUW_1 + data.rUW_2)/2, data.scaleFactor))
        annotation (extent=[-20, -80; 0, -60], rotation=270);
      Utilities.Parts.Body rRightWheel(r=Utilities.Functions.EWM(data.rUW_2 - (
            data.rUW_1 + data.rUW_2)/2, data.scaleFactor))
        annotation (extent=[-20, 60; 0, 80], rotation=90);
      annotation (
        Diagram, 
        Icon(Text(extent=[-90, 90; 68, 20], string="FixedAxle")), 
        Documentation(info="<HTML>
<p>
The wheels are mounted rigidly to the chassis.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
</p>
Very poor ride and handling
<p>
Usage:
Some cheap trailiers.
</p>
</HTML>
"));
      parameter Data.FixedAxleData data annotation (extent=[-86, 10; -66, 30]);
    equation 
      connect(rLeftWheel.frame_a, rSuspension.frame_b) annotation (points=[-10, 
             -58; -10, 40; 20, 40], style(fillColor=0, fillPattern=1));
      connect(rRightWheel.frame_a, rSuspension.frame_b) annotation (points=[-10, 
             60; -10, 40; 20, 40], style(fillColor=0, fillPattern=1));
      connect(rRightWheel.frame_b, frame_U_2) annotation (points=[-10, 80; -10, 
             90], style(fillColor=0, fillPattern=1));
      connect(rSuspension.frame_a, frame_C) annotation (points=[40, 40; 70, 40], 
           style(fillColor=0, fillPattern=1));
      connect(rLeftWheel.frame_b, frame_U_1)
        annotation (points=[-10, -82; -10, -94]);
    end FixedAxleSuspension;
    
    model FiveLinkSuspension 
      "Linkage with five links to contrain the motion. Ideal joints are used." 
      
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/FiveLink_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            string="MultiLink", 
            style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.51, 
          height=0.84), 
        Diagram, 
        Documentation(info="<HTML>
<p>
This suspension uses five links to contrain the constrain the motion of the upright. This version uses ideal joints which can be replaced by bushings by replacing the links with pushrods. 
</p>
<p>
Advantages:
Good posibilties to control the upright motion.
</p>
<p>
Disadvantages:
Requires space, expensive</p>
<p>
This version contains ideal joints and the masses of the links are included.</p>
</HTML>"));
      Chassis.Components.Strut Strut_2(
        rB=data.rCS_2, 
        s0=sqrt((data.rCS_2 - data.rUS_2)*(data.rCS_2 - data.rUS_2)) + data.
            q0S_1, 
        data=data.data_S_2, 
        rA=data.rUS_2 - data.rUW_2) annotation (extent=[20, 0; 60, 40]);
      Components.FiveLink fiveLink_2(
        rCL1=data.rCL1_2, 
        rCL2=data.rCL2_2, 
        rCL3=data.rCL3_2, 
        rCL4=data.rCL4_2, 
        rCL5=data.rCL5_2, 
        rUL1=data.rUL1_2, 
        rUL2=data.rUL2_2, 
        rUL3=data.rUL3_2, 
        rUL4=data.rUL4_2, 
        rUL5=data.rUL5_2, 
        rUW=data.rUW_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        scaleFactor=data.scaleFactor)
        annotation (extent=[-30, 80; 10, 40], rotation=270);
      Components.FiveLink fiveLink_1(
        rCL1=data.rCL1_1, 
        rCL2=data.rCL2_1, 
        rCL3=data.rCL3_1, 
        rCL4=data.rCL4_1, 
        rCL5=data.rCL5_1, 
        rUL1=data.rUL1_1, 
        rUL2=data.rUL2_1, 
        rUL3=data.rUL3_1, 
        rUL4=data.rUL4_1, 
        rUL5=data.rUL5_1, 
        rUW=data.rUW_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        scaleFactor=data.scaleFactor)
        annotation (extent=[-30, -80; 10, -40], rotation=270);
      Chassis.Components.Strut Strut_1(
        rB=data.rCS_1, 
        data=data.data_S_1, 
        rA=data.rUS_1 - data.rUW_1, 
        s0=sqrt((data.rCS_1 - data.rUS_1)*(data.rCS_1 - data.rUS_1)) + data.
            q0S_1) annotation (extent=[20, -40; 60, 0]);
      Chassis.Components.SimpleAntiRollBar antiRollBar(
        rCA_1=data.rCA_1, 
        rAL_1=data.rAL6_1, 
        rXL_1=data.rUL6_1, 
        rX_1=data.rUW_1, 
        rCA_2=data.rCA_2, 
        rAL_2=data.rAL6_2, 
        rXL_2=data.rUL6_2, 
        rX_2=data.rUW_2, 
        data=data.data_A) annotation (extent=[-80, -20; -40, 20], rotation=270);
      parameter Chassis.Data.FiveLinkData data
        annotation (extent=[-100, 60; -80, 80]);
    equation 
      connect(antiRollBar.frame_X_2, fiveLink_2.frame_U)
        annotation (points=[-68, 22; -68, 80; -10.2, 80], style(thickness=4));
      connect(fiveLink_2.frame_U, frame_U_2)
        annotation (points=[-10, 80; -10, 90], style(thickness=4));
      connect(fiveLink_1.frame_U, frame_U_1)
        annotation (points=[-10, -80; -10, -90], style(thickness=4));
      connect(antiRollBar.frame_X_1, fiveLink_1.frame_U)
        annotation (points=[-68, -22; -68, -80; -10, -80], style(thickness=4));
      connect(antiRollBar.frame_C, frame_C)
        annotation (points=[-40, 0; 80, 0; 80, 36], style(thickness=4));
      connect(fiveLink_2.frame_C, frame_C) annotation (points=[-10, 40; -10, 0; 
             80, 0; 80, 36], style(thickness=4));
      connect(fiveLink_1.frame_C, frame_C) annotation (points=[-10, -38; -10, 0; 
             80, 0; 80, 32], style(thickness=4));
      connect(Strut_1.frame_b, frame_C)
        annotation (points=[60, -20.2; 80, -20.2; 80, 38], style(thickness=4));
      connect(Strut_2.frame_b, frame_C)
        annotation (points=[60, 19.8; 80, 19.8; 80, 40], style(thickness=4));
      connect(Strut_1.frame_a, fiveLink_1.frame_U) annotation (points=[20, -20; 
             -40, -20; -40, -60; -10, -82], style(thickness=4));
      connect(Strut_2.frame_a, fiveLink_2.frame_U) annotation (points=[20, 20; 
            -40, 20; -40, 60; -10, 82], style(thickness=4));
    end FiveLinkSuspension;
    
    model SimpleTwistBeamSuspension 
      "A simple twist beam suspension where the twist beam is replaced by a roational spring damper."
       
      
      extends Chassis.Interfaces.NoSteerSuspensionInterface;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Images(Parameters(name="|Geometric reference|", source=
                "images/SimpleTwistBeam_text.png")), 
        Icon(Text(
            extent=[-90, 90; 70, 34], 
            style(color=0), 
            string="TwistBeam")), 
        Window(
          x=0.09, 
          y=0.05, 
          width=0.39, 
          height=0.74), 
        Diagram, 
        Documentation(info="<HTML>
<p>
A twist beam suspension is basically a trailing arm suspension where the trailing arms are connected by a twist beam. This then replaces the anti roll linkage and the whole thing can be manufacutred in one piece giving an compact and cheap suspension.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
</p>
Limited possibilites to adjust ride and handling.
<p>
Usage:
Most cars up to C class.
</p>
</HTML>
"));
      Utilities.Forces.SpringDamperTableRot1D twistBeam(data=data.data_A)
        annotation (extent=[-10, -10; 10, 10], rotation=90);
      parameter VehicleDynamics.Chassis.Data.SimpleTwistBeamData data
        annotation (extent=[-92, 46; -66, 78]);
      VehicleDynamics.Chassis.Components.Strut Strut_2(
        rB=data.rCS_2, 
        data=data.data_S_2, 
        s0=sqrt((data.rUS_2 - data.rCS_2)*(data.rUS_2 - data.rCS_2)) + data.
            q0S_2, 
        rA=data.rUS_2 - data.rUW_2, 
        scaleFactor=data.scaleFactor) annotation (extent=[-20, 0; 20, 40]);
      VehicleDynamics.Chassis.Components.Strut Strut_1(
        rB=data.rCS_1, 
        data=data.data_S_1, 
        s0=sqrt((data.rUS_1 - data.rCS_1)*(data.rUS_1 - data.rCS_1)) + data.
            q0S_1, 
        rA=data.rUS_1 - data.rUW_1, 
        scaleFactor=data.scaleFactor) annotation (extent=[-20, -40; 20, 0]);
      VehicleDynamics.Chassis.Components.TrailingArm leftTrailingArm(
        rCU=data.rCU_1, 
        rUW=data.rUW_1, 
        nCU=data.nCU_1, 
        rCMU=data.rCMU_1, 
        mU=data.mU_1, 
        i11U=data.i11U_1, 
        i22U=data.i22U_1, 
        i33U=data.i33U_1, 
        i21U=data.i21U_1, 
        i31U=data.i31U_1, 
        i32U=data.i32U_1, 
        scaleFactor=data.scaleFactor)
        annotation (extent=[-30, -40; 10, -80], rotation=180);
      VehicleDynamics.Chassis.Components.TrailingArm rightTrailingArm(
        rCU=data.rCU_2, 
        rUW=data.rUW_2, 
        nCU=data.nCU_2, 
        rCMU=data.rCMU_2, 
        mU=data.mU_2, 
        i11U=data.i11U_2, 
        i22U=data.i22U_2, 
        i33U=data.i33U_2, 
        i21U=data.i21U_2, 
        i31U=data.i31U_2, 
        i32U=data.i32U_2, 
        scaleFactor=data.scaleFactor)
        annotation (extent=[-30, 40; 10, 80], rotation=180);
    equation 
      connect(leftTrailingArm.frame_C, frame_C) annotation (points=[10, -60; 40, 
             -60; 40, 40; 78, 40], style(thickness=4));
      connect(rightTrailingArm.frame_C, frame_C) annotation (points=[12, 60; 40, 
             60; 40, 40; 78, 40], style(thickness=4));
      connect(Strut_2.frame_b, frame_C) annotation (points=[20, 20; 40, 20; 40, 
             40; 80, 40], style(thickness=4));
      connect(Strut_1.frame_b, frame_C) annotation (points=[20, -20; 40, -20; 
            40, 40; 74, 40], style(thickness=4));
      connect(Strut_2.frame_a, rightTrailingArm.frame_U)
        annotation (points=[-20, 20; -40, 20; -40, 80; -10, 80]);
      connect(Strut_1.frame_a, leftTrailingArm.frame_U) annotation (points=[-20, 
             -20; -40, -20; -40, -80; -10, -80], style(thickness=4));
      connect(leftTrailingArm.frame_U, frame_U_1)
        annotation (points=[-10, -82; -10, -115], style(thickness=4));
      connect(rightTrailingArm.frame_U, frame_U_2)
        annotation (points=[-7.8, 80.6; 0, 100], style(thickness=4));
      connect(twistBeam.flange_a, leftTrailingArm.axis)
        annotation (points=[0, -10; 0, -48], style(color=0));
      connect(twistBeam.flange_b, rightTrailingArm.axis)
        annotation (points=[0, 10; 0, 46], style(color=0));
    end SimpleTwistBeamSuspension;
  end Suspensions;
  
  package Data "Contains data for different suspensions" 
    extends Modelica.Icons.Library;
    
    record MacPhersonData "Data record for MacPhersonSuspension" 
      
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //mcpherson
      parameter SI.Position[3] rCL1_1={-0.0075,0.3753,-0.0358} 
        "|Geometry|Left MacPherson|";
      parameter SI.Position[3] rCL2_1={-0.3175,0.3535,-0.02735} 
        "|Geometry|Left MacPherson|";
      parameter SI.Position[3] rUL1L2_1={0.0058556,0.69905,-0.0725} 
        "|Geometry|Left MacPherson|";
      parameter SI.Position[3] rUW_1={-0.00013,0.725,0.0450} 
        "|Geometry|Left MacPherson|";
      parameter SI.Position[3] rCS_1={-0.0291,0.541,0.561} 
        "|Geometry|Left MacPherson|";
      parameter SI.Position[3] rUS_1={-0.00012,0.5791,0.042} 
        "|Geometry|Left MacPherson|";
      parameter SI.Distance q0S_1=3516/20000 "|Geometry|Left MacPherson|";
      
      parameter SI.Position[3] rCL1_2=Utilities.Functions.EWM({1,-1,1}, rCL1_1)
         "|Geometry|Right MacPherson|";
      parameter SI.Position[3] rCL2_2=Utilities.Functions.EWM({1,-1,1}, rCL2_1)
         "|Geometry|Right MacPherson|";
      parameter SI.Position[3] rUL1L2_2=Utilities.Functions.EWM({1,-1,1}, 
          rUL1L2_1) "|Geometry|Right MacPherson|";
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Right MacPherson|";
      parameter SI.Position[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Right MacPherson|";
      parameter SI.Position[3] rUS_2=Utilities.Functions.EWM({1,-1,1}, rUS_1) 
        "|Geometry|Right MacPherson|";
      parameter SI.Distance q0S_2=q0S_1 "|Geometry|Right MacPherson|";
      
      //steering rack
      parameter SI.Position[3] rUL3_1={-0.1283,0.643,0.07344} 
        "|Geometry|Steering|";
      parameter SI.Position[3] rRL3_1={-0.164,0.3047,0.0945} 
        "|Geometry|Steering|";
      parameter SI.Position[3] rUL3_2=Utilities.Functions.EWM({1,-1,1}, rUL3_1)
         "|Geometry|Steering|";
      parameter SI.Position[3] rRL3_2=Utilities.Functions.EWM({1,-1,1}, rRL3_1)
         "|Geometry|Steering|";
      parameter Real ratioWheelToRack=0.03/4 
        "|Geometry|Steering| rotatational to translational ratio [m/rad]";
      parameter SI.Inertia iSW=0.04 "|Mass and Inertia|Steering|";
      
      //torsion anti roll
      parameter SI.Position[3] rUL4_1={-0.05624,0.54401,0.25183} 
        "|Geometry|Anti roll|";
      parameter SI.Position[3] rAL4_1={-0.04359,0.55115,0.02503} 
        "|Geometry|Anti roll|";
      parameter SI.Position[3] rCA_1={-0.24055,0.3452,0.0353} 
        "|Geometry|Anti roll|";
      
      parameter SI.Position[3] rUL4_2=Utilities.Functions.EWM({1,-1,1}, rUL4_1)
         "|Geometry|Anti roll|";
      parameter SI.Position[3] rAL4_2=Utilities.Functions.EWM({1,-1,1}, rAL4_1)
         "|Geometry|Anti roll|";
      parameter SI.Position[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1) 
        "|Geometry|Anti roll|";
      
      annotation (Images(Parameters(name="|Geometry|Left MacPherson|", source=
                "images/MacPherson_text.png")), Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
      parameter SI.Position[3] rCMU_1={0.118,0.028,0.0776} - rUL1L2_1 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.4 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.2959 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.2867 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.0963 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=0.036 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=-0.126 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.016 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -20000; 
             1, 20000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.385, 1336; 0.518, 1455; 0.995, 1851; 
            1.455, 2255]) "|Forces|Left MacPherson|Strut data"
        annotation (extent=[-67.5, -45.5; -32.5, -10.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[-1, -100; 0, 
             0; 1, 100], fc=[-1, -33626; 0, 0; 1, 33626]) 
        "|Forces|Anti Roll|Anti Roll data"
        annotation (extent=[-15.5, -3.5; 19.5, 31.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces|Right MacPherson|Strut data"
        annotation (extent=[40.5, -45.5; 75.5, -10.5]);
      parameter Chassis.Data.BumpBushingData data_CL1_1(
        r_min=-0.001*{0.2,0.2,2}, 
        r_max=0.001*{0.2,0.2,2}, 
        bumpFactor=10, 
        nx=cross(rCL2_1 - rCL1_1, {0,0,1}), 
        ny=cross(rCL2_1 - rCL1_1, {1,0,0}), 
        Cdeg=diagonal({19500,19500,1050,11500,115000,3500}), 
        Ddeg=diagonal({254,254,35,175,175,45})) 
        "|Forces|Left MacPherson|Bushing data, chassis to link1"
        annotation (extent=[-48, 58; -8, 98]);
      parameter Chassis.Data.BumpBushingData data_CL1_2(
        r_rela0=data_CL1_1.r_rela0, 
        r_min=data_CL1_1.r_min, 
        r_max=data_CL1_1.r_max, 
        phi_rela0=data_CL1_1.phi_rela0, 
        phi_min=data_CL1_1.phi_min, 
        phi_max=data_CL1_1.phi_max, 
        bumpFactor=data_CL1_1.bumpFactor, 
        Cdeg=data_CL1_1.Cdeg, 
        Ddeg=data_CL1_1.Ddeg, 
        C=data_CL1_1.C, 
        D=data_CL1_1.D, 
        linear=data_CL1_1.linear, 
        eps=data_CL1_1.eps, 
        nx=cross(rCL2_2 - rCL1_2, {0,0,1}), 
        ny=cross(rCL2_2 - rCL1_2, {1,0,0})) 
        "|Forces|Right MacPherson|Bushing data, chassis to link1"
        annotation (extent=[8, 58; 48, 98]);
      parameter Chassis.Data.BumpBushingData data_CL2_1(
        r_min=-0.001*{2.5,2.5,10}, 
        r_max=0.001*{2.5,2.5,10}, 
        bumpFactor=10, 
        nx=cross(rCL2_1 - rCL1_1, {0,0,1}), 
        ny=cross(rCL2_1 - rCL1_1, {1,0,0}), 
        Cdeg=diagonal({1150,1150,200,1775,1775,1215}), 
        Ddeg=diagonal({12,12,2.3,17.5,17.5,13})) 
        "|Forces|Left MacPherson|Bushing data, chassis to link2"
        annotation (extent=[-48, 14; -8, 54]);
      parameter Chassis.Data.BumpBushingData data_CL2_2(
        r_rela0=data_CL1_1.r_rela0, 
        r_min=data_CL2_1.r_min, 
        r_max=data_CL2_1.r_max, 
        phi_rela0=data_CL2_1.phi_rela0, 
        phi_min=data_CL2_1.phi_min, 
        phi_max=data_CL2_1.phi_max, 
        bumpFactor=data_CL2_1.bumpFactor, 
        Cdeg=data_CL2_1.Cdeg, 
        Ddeg=data_CL2_1.Ddeg, 
        C=data_CL2_1.C, 
        D=data_CL2_1.D, 
        linear=data_CL2_1.linear, 
        eps=data_CL2_1.eps, 
        nx=cross(rCL2_2 - rCL1_2, {0,0,1}), 
        ny=cross(rCL2_2 - rCL1_2, {1,0,0})) 
        "|Forces|Right MacPherson|Bushing data, chassis to link2"
        annotation (extent=[8, 14; 48, 54]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_R(fd=[-1, -25.2; 0, 
             0; 1, 25.2], fc=[-1, -1.72*0.3*180/3.14159; 0, 0; 1, 1.72*0.3*180/
            3.14159]) "|Forces|Steering|Steering wheel axis compliance"
        annotation (extent=[-11.5, -59.5; 23.5, -24.5]);
    end MacPhersonData;
    
    record SimpleData 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      
      parameter SI.Position[3] rUW_1={0,0.88,0.002} "|Geometry||";
      parameter SI.Position[3] rCU_1={0,0.78,0.002} "|Geometry||";
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry||";
      parameter SI.Position[3] rCU_2=Utilities.Functions.EWM({1,-1,1}, rCU_1) 
        "|Geometry||";
      
      parameter Real ratio=0.03/0.16;
      
      parameter SI.Distance q0S_1=0.1758 "|Geometry||";
      parameter SI.Distance q0S_2=q0S_1 "|Geometry||";
      
      //steering
      parameter Real wheelbase=2.55;
      parameter Real trackwidth=rUW_1[2] - rUW_2[2];
      
      parameter SI.Position[3] rCMU_1={0.118,0.028,0.0776} 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.0 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.299 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.287 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.093 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=0.03 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=-0.12 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.01 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      annotation (Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
      Table data_Link_1 annotation (extent=[-60, -20; -20, 20]);
      Table data_Link_2(bounce2steer=data_Link_1.bounce2steer, bounce2camber=
            data_Link_1.bounce2camber) annotation (extent=[20, -20; 60, 20]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fd=[-1.4950, -719; 
             -0.996, -531; -0.518, -413; -0.389, -376; -0.260, -327; -0.130, -166; 
             -0.052, -82; 0, 0; 0.052, 154; 0.130, 419; 0.260, 1122; 0.389, 
            1336; 0.518, 1466; 0.996, 1891; 1.495, 2263], fc=[-1, -20000; 1, 
            20000]) "|Forces||Strut data"
        annotation (extent=[-73.5, -75.5; -38.5, -40.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces||Strut data"
        annotation (extent=[34.5, -75.5; 69.5, -40.5]);
    end SimpleData;
    
    record BodyData "Contains Body and Body shape data" 
      
      parameter SI.Position rcmB[3]={-2.55*0.396,0,2.55*0.23 - 0.14} 
        "|Mass and Inertia|Vector from frame_a to center of mass, resolved in frame_a";
      parameter SI.Mass mB=1200 "|Mass and Inertia|Mass of body";
      parameter SI.Inertia i11B=730 
        "|Mass and Inertia|(1,1) element of inertia tensor";
      parameter SI.Inertia i22B=1350 
        "|Mass and Inertia|(2,2) element of inertia tensor";
      parameter SI.Inertia i33B=1820 
        "|Mass and Inertia|(3,3) element of inertia tensor";
      parameter SI.Inertia i21B=0 
        "|Mass and Inertia|(2,1) element of inertia tensor";
      parameter SI.Inertia i31B=0 
        "|Mass and Inertia|(3,1) element of inertia tensor";
      parameter SI.Inertia i32B=0 
        "|Mass and Inertia|(3,2) element of inertia tensor";
      parameter String shape="sphere" "|Animation|Name of shape";
      parameter SI.Position r0[3]={-2.55*0.396,0,2.55*0.23 - 0.14} 
        "|Animation|Vector from frame_a to shape origin. resolved in frame_a";
      parameter SI.Position lengthDirection[3]={1,0,0} 
        "|Animation|Vector in length direction, resolved in frame_a";
      parameter SI.Position widthDirection[3]={0,1,0} 
        "|Animation|Vector in width direction, resolved in frame_a";
      parameter SI.Length length=0.1 "|Animation|length of shape";
      parameter SI.Length width=0.1 "|Animation|width of shape";
      parameter SI.Length height=0.1 "|Animation|height of shape";
      parameter Real material[4]={1,0,0,0.5} 
        "|Animation|Color and specular coefficient";
      parameter Real extra=0.0 
        "|Animation|Additional parameter for cone and pipe";
      
      annotation (Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=1)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
    end BodyData;
    
    record Multilink4Data "Data record for MultiLink4Suspension" 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //multilink4
      parameter SI.Position[3] rCL1_1={-2.6430,0.231,-0.022} 
        "|Geometry|Left multilink|Lower link to chassis/body";
      parameter SI.Position[3] rCL2_1={-2.5039,0.4150,0.174} 
        "|Geometry|Left multilink|Upper link to chassis/body";
      parameter SI.Position[3] rCL3_1={-2.4617,0.4750,0.0637} 
        "|Geometry|Left multilink|Steering link to chassis/body";
      parameter SI.Position[3] rCU_1={-2.0825,0.6200,0.0105} 
        "|Geometry|Left multilink|Trailing arm to chassis/body";
      parameter SI.Position[3] rUL1_1={-2.6170,0.6672,-0.05327} 
        "|Geometry|Left multilink|Lower link to trailing arm";
      parameter SI.Position[3] rUL2_1={-2.5178,0.6678,0.1607} 
        "|Geometry|Left multilink|Upper link to trailing arm";
      parameter SI.Position[3] rUL3_1={-2.4270,0.612,0.050} 
        "|Geometry|Left multilink|Steering link to trailing arm";
      parameter SI.Position[3] rUW_1={-2.557,0.7373,0.0290} 
        "|Geometry|Left multilink|Wheel centre";
      
      parameter SI.Position[3] rCL1_2=Utilities.Functions.EWM({1,-1,1}, rCL1_1)
         "|Geometry|Right multilink|Lower link to chassis/body";
      parameter SI.Position[3] rCL2_2=Utilities.Functions.EWM({1,-1,1}, rCL2_1)
         "|Geometry|Right multilink|Upper link to chassis/body";
      parameter SI.Position[3] rCL3_2=Utilities.Functions.EWM({1,-1,1}, rCL3_1)
         "|Geometry|Right multilink|Steering link to chassis/body";
      parameter SI.Position[3] rCU_2=Utilities.Functions.EWM({1,-1,1}, rCU_1) 
        "|Geometry|Right multilink|Trailing arm to chassis/body";
      parameter SI.Position[3] rUL1_2=Utilities.Functions.EWM({1,-1,1}, rUL1_1)
         "|Geometry|Right multilink|Lower link to trailing arm";
      parameter SI.Position[3] rUL2_2=Utilities.Functions.EWM({1,-1,1}, rUL2_1)
         "|Geometry|Right multilink|Upper link to trailing arm";
      parameter SI.Position[3] rUL3_2=Utilities.Functions.EWM({1,-1,1}, rUL3_1)
         "|Geometry|Right multilink|Steering link to trailing arm";
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Right multilink|Wheel centre";
      
      // strut
      parameter SI.Position[3] rCS_1={-2.55 - 0.0621,0.518,0.450} 
        "|Geometry|Left strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_1={-2.55 - 0.063305,0.581,-0.046} 
        "|Geometry|Left strut|Strut ot trailing arm";
      parameter SI.Position q0S_1=0.116 
        "|Geometry|Left strut|Additional length for unloaded strut";
      
      parameter SI.Position[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Right strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_2=Utilities.Functions.EWM({1,-1,1}, rUS_1) 
        "|Geometry|Right strut|Strut ot trailing arm";
      parameter SI.Position q0S_2=q0S_1 
        "|Geometry|Right strut|Additional length for unloaded strut";
      
      //torsion anti roll
      parameter SI.Position[3] rUL4_1={-2.55 - 0.03595,0.4590,0.0021} 
        "|Geometry|Anti roll, left|Anti roll bar link to trailing arm";
      parameter SI.Position[3] rAL4_1={-2.55 - 0.040,0.4951,0.07755} 
        "|Geometry|Anti roll, left|Anti roll bar link to anti roll bar";
      parameter SI.Position[3] rCA_1={-2.55 - 0.2510,0.4653,0.0630} 
        "|Geometry|Anti roll, left|Anti roll bar to chassis/body";
      
      parameter SI.Position[3] rUL4_2=Utilities.Functions.EWM({1,-1,1}, rUL4_1)
         "|Geometry|Anti roll, right|Anti roll bar link to trailing arm";
      parameter SI.Position[3] rAL4_2=Utilities.Functions.EWM({1,-1,1}, rAL4_1)
         "|Geometry|Anti roll, right|Anti roll bar link to anti roll bar";
      parameter SI.Position[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1) 
        "|Geometry|Anti roll, right|Anti roll bar to chassis/body";
      parameter SI.Position[3] rCMU_1={0.118,0.028,0.0776} 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.0 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.0929 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.287 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.293 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=-0.03 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=0.12 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.01 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      annotation (
        Images(Parameters(name="|Geometry|Left multilink|", source=
                "images/MultiLink4_text.png")), 
        Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")), 
        Diagram);
      
      parameter Chassis.Data.BumpBushingData data_CU_1(
        r_min=-0.001*{6.82,2.52,4.87}, 
        r_max=0.001*{6.57,2.53,4.7}, 
        Cdeg=diagonal({456,179,643,1526,1720,1875}), 
        Ddeg=diagonal({7,4,10,22,228,321})) 
        "|Forces|Left multilink|Bushing data, Trailing arm to chassis/body"
        annotation (extent=[-50, 56; -10, 96]);
      parameter Chassis.Data.BumpBushingData data_CU_2(
        nx=Utilities.Functions.EWM(data_CU_1.nx, {1,-1,1}), 
        ny=Utilities.Functions.EWM(data_CU_1.ny, {1,-1,1}), 
        r_rela0=data_CU_1.r_rela0, 
        r_min=data_CU_1.r_min, 
        r_max=data_CU_1.r_max, 
        phi_rela0=data_CU_1.phi_rela0, 
        phi_min=data_CU_1.phi_min, 
        phi_max=data_CU_1.phi_max, 
        bumpFactor=data_CU_1.bumpFactor, 
        Cdeg=data_CU_1.Cdeg, 
        Ddeg=data_CU_1.Ddeg, 
        C=data_CU_1.C, 
        D=data_CU_1.D, 
        linear=data_CU_1.linear, 
        eps=data_CU_1.eps) 
        "|Forces|Right multilink|Bushing data, Trailing arm to chassis/body"
        annotation (extent=[4, 56; 44, 96]);
      parameter Chassis.Data.BumpBushingData data_CL1_1(
        bumpFactor=10, 
        Ddeg=diagonal({64,64,4,61,61,10}), 
        nx={0,0,1}, 
        r_min=-0.001*{4,4,6}, 
        r_max=0.001*{4,4,6}, 
        Cdeg=diagonal({6310,6310,447,6230,6230,1030})) 
        "|Forces|Left multilink|Bushing data, chassis to link1"
        annotation (extent=[-50, 4; -10, 44]);
      parameter Chassis.Data.BumpBushingData data_CL1_2(
        nx=Utilities.Functions.EWM(data_CL1_1.nx, {1,-1,1}), 
        ny=Utilities.Functions.EWM(data_CL1_1.ny, {1,-1,1}), 
        r_rela0=data_CL1_1.r_rela0, 
        r_min=data_CL1_1.r_min, 
        r_max=data_CL1_1.r_max, 
        phi_rela0=data_CL1_1.phi_rela0, 
        phi_min=data_CL1_1.phi_min, 
        phi_max=data_CL1_1.phi_max, 
        bumpFactor=data_CL1_1.bumpFactor, 
        Cdeg=data_CL1_1.Cdeg, 
        Ddeg=data_CL1_1.Ddeg, 
        C=data_CL1_1.C, 
        D=data_CL1_1.D, 
        linear=data_CL1_1.linear, 
        eps=data_CL1_1.eps) 
        "|Forces|Right multilink|Bushing data, chassis to link1"
        annotation (extent=[6, 4; 46, 44]);
      parameter Chassis.Data.BumpBushingData data_UL1_1(
        bumpFactor=10, 
        Ddeg=diagonal({15,15,206,180,180,31.5}), 
        nx={0,0,1}, 
        r_min=-0.001*{4,4,4}, 
        r_max=0.001*{4,4,4}, 
        Cdeg=diagonal({11600,10600,7512,9373,9373,1625})) 
        "|Forces|left multilink|Bushing data, trailing arm to link1"
        annotation (extent=[-98, 4; -58, 44]);
      parameter Chassis.Data.BumpBushingData data_UL1_2(
        nx=Utilities.Functions.EWM(data_UL1_1.nx, {1,-1,1}), 
        ny=Utilities.Functions.EWM(data_UL1_1.ny, {1,-1,1}), 
        r_rela0=data_UL1_1.r_rela0, 
        r_min=data_UL1_1.r_min, 
        r_max=data_UL1_1.r_max, 
        phi_rela0=data_UL1_1.phi_rela0, 
        phi_min=data_UL1_1.phi_min, 
        phi_max=data_UL1_1.phi_max, 
        bumpFactor=data_UL1_1.bumpFactor, 
        Cdeg=data_UL1_1.Cdeg, 
        Ddeg=data_UL1_1.Ddeg, 
        C=data_UL1_1.C, 
        D=data_UL1_1.D, 
        linear=data_UL1_1.linear, 
        eps=data_UL1_1.eps) 
        "|Forces|Right multilink|Bushing data, trailing arm to link1"
        annotation (extent=[56, 4; 96, 44]);
      parameter Chassis.Data.BumpBushingData data_CL2_1(
        Ddeg=diagonal({75,75,7,31,31,14}), 
        nx={0,0,1}, 
        r_min=-0.001*{4,4,6}, 
        r_max=0.001*{4,4,6}, 
        bumpFactor=10, 
        Cdeg=diagonal({25413,25413,3112,2.122,2.122,1.011})) 
        "|Forces|Left multilink|Bushing data, chassis to link2"
        annotation (extent=[-50, -40; -10, 0]);
      parameter Chassis.Data.BumpBushingData data_CL2_2(
        nx=Utilities.Functions.EWM(data_CL2_1.nx, {1,-1,1}), 
        ny=Utilities.Functions.EWM(data_CL2_1.ny, {1,-1,1}), 
        r_rela0=data_CL2_1.r_rela0, 
        r_min=data_CL2_1.r_min, 
        r_max=data_CL2_1.r_max, 
        phi_rela0=data_CL2_1.phi_rela0, 
        phi_min=data_CL2_1.phi_min, 
        phi_max=data_CL2_1.phi_max, 
        bumpFactor=data_CL2_1.bumpFactor, 
        Cdeg=data_CL2_1.Cdeg, 
        Ddeg=data_CL2_1.Ddeg, 
        C=data_CL2_1.C, 
        D=data_CL2_1.D, 
        linear=data_CL2_1.linear, 
        eps=data_CL2_1.eps) 
        "|Forces|Right multilink|Bushing data, chassis to link2"
        annotation (extent=[6, -40; 46, 0]);
      parameter Chassis.Data.BumpBushingData data_UL2_1(
        nx={0,0,1}, 
        r_min=-0.001*{4,4,6}, 
        r_max=0.001*{4,4,6}, 
        Cdeg=diagonal({11900,11900,301,3.239,3.239,0.839}), 
        Ddeg=diagonal({2715,2715,10,813,813,222})) 
        "|Forces|left multilink|Bushing data, trailing arm to link2"
        annotation (extent=[-98, -40; -58, 0]);
      parameter Chassis.Data.BumpBushingData data_UL2_2(
        nx=Utilities.Functions.EWM(data_UL2_1.nx, {1,-1,1}), 
        ny=Utilities.Functions.EWM(data_UL2_1.ny, {1,-1,1}), 
        r_rela0=data_UL2_1.r_rela0, 
        r_min=data_UL2_1.r_min, 
        r_max=data_UL2_1.r_max, 
        phi_rela0=data_UL2_1.phi_rela0, 
        phi_min=data_UL2_1.phi_min, 
        phi_max=data_UL2_1.phi_max, 
        bumpFactor=data_UL2_1.bumpFactor, 
        Cdeg=data_UL2_1.Cdeg, 
        Ddeg=data_UL2_1.Ddeg, 
        C=data_UL2_1.C, 
        D=data_UL2_1.D, 
        linear=data_UL2_1.linear, 
        eps=data_UL2_1.eps) 
        "|Forces|Right multilink|Bushing data, trailing arm to link2"
        annotation (extent=[56, -40; 96, 0]);
      parameter Chassis.Data.BumpBushingData data_CL3_1(
        nx={0,0,1}, 
        r_min=-0.001*{4,4,6}, 
        r_max=0.001*{4,4,6}, 
        bumpFactor=10, 
        Cdeg=diagonal({25422,25422,327,2.22,2.22,1.01}), 
        Ddeg=diagonal({627,627,6,31,31,14})) 
        "|Forces|Left multilink|Bushing data, chassis to link3"
        annotation (extent=[-50, -86; -10, -46]);
      parameter Chassis.Data.BumpBushingData data_CL3_2(
        nx=Utilities.Functions.EWM(data_CL3_1.nx, {1,-1,1}), 
        ny=Utilities.Functions.EWM(data_CL3_1.ny, {1,-1,1}), 
        r_rela0=data_CL3_1.r_rela0, 
        r_min=data_CL3_1.r_min, 
        r_max=data_CL3_1.r_max, 
        phi_rela0=data_CL3_1.phi_rela0, 
        phi_min=data_CL3_1.phi_min, 
        phi_max=data_CL3_1.phi_max, 
        bumpFactor=data_CL3_1.bumpFactor, 
        Cdeg=data_CL3_1.Cdeg, 
        Ddeg=data_CL3_1.Ddeg, 
        C=data_CL3_1.C, 
        D=data_CL3_1.D, 
        linear=data_CL3_1.linear, 
        eps=data_CL3_1.eps) 
        "|Forces|Right multilink|Bushing data, chassis to link3"
        annotation (extent=[6, -88; 46, -48]);
      parameter Chassis.Data.BumpBushingData data_UL3_1(
        nx={0,0,1}, 
        r_min=-0.001*{4,4,6}, 
        r_max=0.001*{4,4,6}, 
        bumpFactor=10, 
        Cdeg=diagonal({27625,27625,312,4143,4143,1236}), 
        Ddeg=diagonal({485,485,9.2,84.5,84.5,26})) 
        "|Forces|left multilink|Bushing data, trailing arm to link3"
        annotation (extent=[-98, -86; -58, -46]);
      parameter Chassis.Data.BumpBushingData data_UL3_2(
        nx=Utilities.Functions.EWM(data_UL3_1.nx, {1,-1,1}), 
        ny=Utilities.Functions.EWM(data_UL3_1.ny, {1,-1,1}), 
        r_rela0=data_UL3_1.r_rela0, 
        r_min=data_UL3_1.r_min, 
        r_max=data_UL3_1.r_max, 
        phi_rela0=data_UL3_1.phi_rela0, 
        phi_min=data_UL3_1.phi_min, 
        phi_max=data_UL3_1.phi_max, 
        bumpFactor=data_UL3_1.bumpFactor, 
        Cdeg=data_UL3_1.Cdeg, 
        Ddeg=data_UL3_1.Ddeg, 
        C=data_UL3_1.C, 
        D=data_UL3_1.D, 
        linear=data_UL3_1.linear, 
        eps=data_UL3_1.eps) 
        "|Forces|Right multilink|Bushing data, trailing arm to link3"
        annotation (extent=[56, -86; 96, -46]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[0, 0], fc=[-1, 
             -5432; 1, 5432]) "|Forces|Left multilink|Anti Roll data"
        annotation (extent=[-17.5, -57.5; 17.5, -22.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces|Left multilink|Strut data"
        annotation (extent=[38.5, -99.5; 73.5, -64.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -24000; 
             1, 24000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.389, 1336; 0.518, 1466; 0.996, 1891; 
            1.495, 2263]) "|Forces|Left multilink|Strut data"
        annotation (extent=[-69.5, -99.5; -34.5, -64.5]);
    end Multilink4Data;
    
    record TrailingArmData "Data record for Simpletrailing arm4Suspension" 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //trailing arm
      parameter SI.Position[3] rCU_1={-2.085,0.600,0.010} 
        "|Geometry|Left trailing arm|Trailing arm to chassis/body";
      parameter Real[3] nCU_1={0,1,0} 
        "|Geometry|Left trailing arm|Trailing arm to chassis/body";
      parameter SI.Position[3] rUW_1={-2.55,0.733,0.030} 
        "|Geometry|Left trailing arm|Wheel centre";
      
      parameter SI.Position[3] rCU_2=Utilities.Functions.EWM({1,-1,1}, rCU_1) 
        "|Geometry|Right trailing arm|Trailing arm to chassis/body";
      parameter Real[3] nCU_2=Utilities.Functions.EWM({1,-1,1}, nCU_1) 
        "|Geometry|Right trailing arm|Trailing arm to chassis/body";
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Right trailing arm|Wheel centre";
      
      // strut
      parameter SI.Position[3] rCS_1={-2.55 - 0.061,0.518,0.450} 
        "|Geometry|Left strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_1={-2.55 - 0.0605,0.581,-0.046} 
        "|Geometry|Left strut|Strut ot trailing arm";
      parameter SI.Position q0S_1=0.116 
        "|Geometry|Left strut|Additional length for unloaded strut";
      
      parameter SI.Position[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Right strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_2=Utilities.Functions.EWM({1,-1,1}, rUS_1) 
        "|Geometry|Right strut|Strut ot trailing arm";
      parameter SI.Position q0S_2=q0S_1 
        "|Geometry|Right strut|Additional length for unloaded strut";
      
      //torsion anti roll
      parameter SI.Position[3] rUL1_1={-2.55 - 0.0395,0.490,0.002} 
        "|Geometry|Anti roll, left|Anti roll bar link to trailing arm";
      parameter SI.Position[3] rAL1_1={-2.55 - 0.040,0.491,0.0775} 
        "|Geometry|Anti roll, left|Anti roll bar link to anti roll bar";
      parameter SI.Position[3] rCA_1={-2.55 - 0.210,0.463,0.060} 
        "|Geometry|Anti roll, left|Anti roll bar to chassis/body";
      
      parameter SI.Position[3] rUL1_2=Utilities.Functions.EWM({1,-1,1}, rUL1_1)
         "|Geometry|Anti roll, right|Anti roll bar link to trailing arm";
      parameter SI.Position[3] rAL1_2=Utilities.Functions.EWM({1,-1,1}, rAL1_1)
         "|Geometry|Anti roll, right|Anti roll bar link to anti roll bar";
      parameter SI.Position[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1) 
        "|Geometry|Anti roll, right|Anti roll bar to chassis/body";
      parameter SI.Position[3] rCMU_1={-0.418,0.028,0.0776} 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.0 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.099 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.287 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.293 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=0.03 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=-0.12 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.01 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      annotation (
        Images(Parameters(name="|Geometry|Left trailing arm|", source=
                "images/TrailingArm_text.png")), 
        Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")), 
        Diagram);
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[0, 0], fc=[-1, 
             -5432; 1, 5432]) "|Forces|Left multilink|Anti Roll data"
        annotation (extent=[-17.5, -57.5; 17.5, -22.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces|Left multilink|Strut data"
        annotation (extent=[38.5, -99.5; 73.5, -64.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fd=[-1.4950, -719; 
             -0.996, -531; -0.518, -413; -0.389, -376; -0.260, -327; -0.130, -166; 
             -0.052, -82; 0, 0; 0.052, 154; 0.130, 419; 0.260, 1122; 0.389, 
            1336; 0.518, 1466; 0.996, 1891; 1.495, 2263], fc=[-1, -20000; 1, 
            20000]) "|Forces|Left multilink|Strut data"
        annotation (extent=[-69.5, -99.5; -34.5, -64.5]);
    end TrailingArmData;
    
    record SimpleTwistBeamData "Data record for Simpletrailing arm4Suspension" 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //trailing arm4
      parameter SI.Position[3] rCU_1={-2.085,0.600,0.010} 
        "|Geometry|Left trailing arm|Trailing arm to chassis/body";
      parameter Real[3] nCU_1={0,1,0} 
        "|Geometry|Left trailing arm|Trailing arm to chassis/body";
      parameter SI.Position[3] rUW_1={-2.55,0.733,0.030} 
        "|Geometry|Left trailing arm|Wheel centre";
      
      parameter SI.Position[3] rCU_2=Utilities.Functions.EWM({1,-1,1}, rCU_1) 
        "|Geometry|Right trailing arm|Trailing arm to chassis/body";
      parameter Real[3] nCU_2=Utilities.Functions.EWM({1,-1,1}, nCU_1) 
        "|Geometry|Right trailing arm|Trailing arm to chassis/body";
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Right trailing arm|Wheel centre";
      
      // strut
      parameter SI.Position[3] rCS_1={-2.55 - 0.061,0.518,0.450} 
        "|Geometry|Left strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_1={-2.55 - 0.0605,0.581,-0.046} 
        "|Geometry|Left strut|Strut ot trailing arm";
      parameter SI.Position q0S_1=2469/24000 
        "|Geometry|Left strut|Additional length for unloaded strut";
      
      parameter SI.Position[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Right strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_2=Utilities.Functions.EWM({1,-1,1}, rUS_1) 
        "|Geometry|Right strut|Strut ot trailing arm";
      parameter SI.Position q0S_2=q0S_1 
        "|Geometry|Right strut|Additional length for unloaded strut";
      
      //torsion anti roll
      parameter SI.Position[3] rCMU_1={-0.418,0.028,0.0776} 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.0 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.099 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.287 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.293 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=0.03 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=-0.12 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.01 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      annotation (
        Images(Parameters(name="|Geometry|Left trailing arm|", source=
                "images/SimpleTwistBeam_text.png")), 
        Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")), 
        Diagram);
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[0, 0], fc=[-1, 
             -25432; 1, 25432]) "|Forces|Left multilink|Anti Roll data"
        annotation (extent=[-17.5, -57.5; 17.5, -22.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces|Left multilink|Strut data"
        annotation (extent=[38.5, -99.5; 73.5, -64.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -24000; 
             1, 24000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.389, 1336; 0.518, 1466; 0.996, 1891; 
            1.495, 2263]) "|Forces|Left multilink|Strut data"
        annotation (extent=[-69.5, -99.5; -34.5, -64.5]);
    end SimpleTwistBeamData;
    
    record BumpBushingData 
      parameter SI.Position[3] rA={0,0,0} 
        "|General geometry|Position|vector from frame_A to bushing centre";
      parameter SI.Position[3] rB={0,0,0} 
        "|General geometry|Position|vector from frame_B to bushing centre";
      parameter Real[3] nx={1,0,0} 
        "|General geometry|Orientation|x direction of bushing, resolved in frame_A";
      parameter Real[3] ny={0,1,0} 
        "|General geometry|Orientation|y direction of bushing, resolved in frame_A";
      
      parameter SI.Position[3] r_rela0={0,0,0} 
        "|General geometry|Pretention|Unstretched translational spring vector";
      parameter SI.Position[3] r_min={-0.05,-0.03,-0.06} 
        "|Forces|Bump|vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Position[3] r_max={0.05,0.03,0.06} 
        "|Forces|Bump|vector to bump stop from reference point in positive direction, resolved in frame_A";
      
      parameter SI.Angle[3] phi_rela0={0,0,0} 
        "|General geometry|Pretention|Unstretched rotational spring vector";
      parameter SI.Angle[3] phi_min={-0.05,-0.03,-0.06} 
        "|Forces|Bump|rotational vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Angle[3] phi_max={0.05,0.03,0.06} 
        "|Forces|Bump|rotational vector to bump stop from reference point in positive direction, resolved in frame_A";
      parameter Real bumpFactor=10 
        "|Forces|Bump|scaleFactor of spring force at after bump";
      
      parameter Real[6, 6] Cdeg=[10000000, 0, 0, 0, 0, 0; 0, 10000000, 0, 0, 0, 
           0; 0, 0, 10000000, 0, 0, 0; 0, 0, 0, 100, 0, 0; 0, 0, 0, 0, 100, 0; 
          0, 0, 0, 0, 0, 100] 
        "|Forces|Linear|Stiffness matrix [N/mm and Nmm/degree]";
      parameter Real[6, 6] Ddeg=[100000, 0, 0, 0, 0, 0; 0, 1000000, 0, 0, 0, 0; 
           0, 0, 100000, 0, 0, 0; 0, 0, 0, 100, 0, 0; 0, 0, 0, 0, 100, 0; 0, 0, 
           0, 0, 0, 100] 
        "|Forces|Linear|Damping matrix [Ns/mm and Nmms/degree]";
      
      parameter Real pi=Modelica.Constants.pi;
      
      parameter Real C[6, 6]=Cdeg*[1000, 0, 0, 0, 0, 0; 0, 1000, 0, 0, 0, 0; 0, 
           0, 1000, 0, 0, 0; 0, 0, 0, .180/pi, 0, 0; 0, 0, 0, 0, .180/pi, 0; 0, 
           0, 0, 0, 0, .180/pi] 
        "|Forces|Linear|Stiffness matrix [N/m and Nm/rad]";
      parameter Real D[6, 6]=Ddeg*[1000, 0, 0, 0, 0, 0; 0, 1000, 0, 0, 0, 0; 0, 
           0, 1000, 0, 0, 0; 0, 0, 0, .180/pi, 0, 0; 0, 0, 0, 0, .180/pi, 0; 0, 
           0, 0, 0, 0, .180/pi] 
        "|Forces|Linear|Stiffness matrix [Ns/m and Nms/rad]";
      parameter Boolean linear=true 
        "|Advanced||turn on/off linearisation of cosine and sine in rotation vector equation";
      parameter Real eps=0.0000001 "|Advanced||Guard against division by zero";
      
      parameter Boolean animation=true "True, if animation shall be enabled";
      parameter SI.Position width=0.04 
        "|Animation|if animation = true| width bushing";
      parameter SI.Position length=0.04 
        "|Animation|if animation = true| width bushing";
      parameter SI.Position height=0.04 
        "|Animation|if animation = true| width bushing";
      parameter Real material[4]={0.2,0.2,0.2,0.2} 
        "|Animation|if animation = true| Color and specular coefficient of spring";
      
      annotation (Icon(
          Rectangle(extent=[-90, 90; 90, -90], style(fillColor=51)), 
          Text(
            extent=[-80, 82; 82, 18], 
            style(fillColor=51, fillPattern=1), 
            string="Data"), 
          Text(
            extent=[-90, 0; 90, -90], 
            style(fillColor=51, fillPattern=1), 
            string="%name")));
    end BumpBushingData;
    
    record Table 
      parameter Real[:, 2] bounce2steerDegmm=[-88, -0.408; -68, 0.269; -48, 
          0.180; -28, 0.08; -20, -0.049; -8, -0.01; 0, 0; 12, 0.01; 32, 0.02; 
          52, 0; 72, -0.01];
      parameter Real[:, 2] bounce2camberDegmm=[-88, 0.79; -68, 0.70; -48, 0.55; 
           -28, 0.40; -20, 0.31; -8, 0.15; 0, 0; 12, -0.23; 32, -0.64; 52, -1.58; 
           72, -2.24];
      
      parameter Real[:, 2] bounce2steer=bounce2steerDegmm*[0.001, 0; 0, pi/180];
      parameter Real[:, 2] bounce2camber=bounce2camberDegmm*[0.001, 0; 0, pi/
          180];
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-100, 104; 102, -100], style(color=0, fillColor=51)), 
             
          Text(extent=[-100, 100; 100, 18], string="%name"), 
          Text(extent=[-80, 20; 84, -42], string="steer=f(bounce)"), 
          Text(extent=[-80, -28; 84, -90], string="camber=f(bounce)")), 
        Window(
          x=0.03, 
          y=0.02, 
          width=0.92, 
          height=0.95));
      
      constant Real pi=Modelica.Constants.pi;
    end Table;
    
    record SimpleDataFront 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      
      parameter SI.Position[3] rUW_1={-0.0001,0.727,0.040} "|Geometry||";
      parameter SI.Position[3] rCU_1={0.00586,0.6905,-0.071} "|Geometry||";
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry||";
      parameter SI.Position[3] rCU_2=Utilities.Functions.EWM({1,-1,1}, rCU_1) 
        "|Geometry||";
      parameter Real ratio=0.05833 
        "|Geometry|Steering|Steering wheel to Ackermann angle";
      
      parameter SI.Position q0S_1=0.1758*230/275*(1 - 0.396)*2*0.95 
        "|Geometry||";
      parameter SI.Position q0S_2=q0S_1 "|Geometry||";
      
      //steering
      parameter Real wheelbase=2.55 "|Geometry|Steering|Ackermann wheel base";
      parameter Real trackwidth=rUW_1[2] - rUW_2[2] 
        "|Geometry|Steering|Ackermann track width";
      
      parameter SI.Position[3] rCMU_1={0.118,0.028,0.0776} 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.0 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.299 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.287 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.093 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=0.03 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=-0.12 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.01 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      annotation (Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fd=[-1.4950, -719; 
             -0.996, -531; -0.518, -413; -0.389, -376; -0.260, -327; -0.130, -166; 
             -0.052, -82; 0, 0; 0.052, 154; 0.130, 419; 0.260, 1122; 0.389, 
            1336; 0.518, 1466; 0.996, 1891; 1.495, 2263], fc=[-1, -20000; 1, 
            20000]) "|Forces|Strut|"
        annotation (extent=[-79.5, -59.5; -44.5, -24.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces|Strut|"
        annotation (extent=[40.5, -59.5; 75.5, -24.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[0, 0], fc=[-1, 
             -51000; 0, 0; 1, 51000]) "|Forces|Anti Roll|"
        annotation (extent=[-19.5, -59.5; 15.5, -24.5]);
      parameter Utilities.Polynomial.ScaledPolynomial data_Link1_steering(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.082, 
        u_max=0.078, 
        y_min=-0.010297, 
        y_max=0.0012217, 
        c={6.30814489981295e+000,-1.63208206468340e+001,1.59971846025538e+001,-9.43372508316803e+000,
            2.72106634745229e+000,7.28435761181664e-001}, 
        y_umin=-0.0019065, 
        der_y_umin=0.1959, 
        y_umax=-0.010294, 
        der_y_umax=-0.1366, 
        original=[-8.20000000000000e-002, -1.91986217719376e-003; -6.20000000000000e-002, 
             -4.20000000000000e-002; -2.20000000000000e-002, -1.60000000000000e-002; 
             -2.00000000000000e-003, 0.00000000000000e+000; 
            1.80000000000000e-002, 3.80000000000000e-002; 5.80000000000000e-002, 
             7.80000000000000e-002; 6.98131700797732e-004, 
            1.22173047639603e-003; 1.22173047639603e-003, 8.72664625997165e-004; 
             1.74532925199433e-004, 0.00000000000000e+000; -1.57079632679490e-003, 
             -4.01425727958696e-003; -7.15584993317675e-003, -1.02974425867665e-002])
         "|Geometry|toe in as function of bounce|"
        annotation (extent=[-56, 0; -26, 28]);
      parameter Utilities.Polynomial.ScaledPolynomial data_Link1_camber(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.082, 
        u_max=0.078, 
        y_min=-0.0055851, 
        y_max=0.027576, 
        c={-4.98260440195846e+000,1.28215982847983e+001,-1.17916202644075e+001,
            5.83708966179418e+000,-2.88532206648880e+000,9.99807679818011e-001}, 
           
        y_umin=0.02757, 
        der_y_umin=-0.59801, 
        y_umax=-0.0056199, 
        der_y_umax=-0.04407, 
        original=[-8.20000000000000e-002, 2.75762021815104e-002; -6.20000000000000e-002, 
             -4.20000000000000e-002; -2.20000000000000e-002, -1.60000000000000e-002; 
             -2.00000000000000e-003, 0.00000000000000e+000; 
            1.80000000000000e-002, 3.80000000000000e-002; 5.80000000000000e-002, 
             7.80000000000000e-002; 1.79768912955416e-002, 
            1.09955742875643e-002; 5.75958653158129e-003, 4.01425727958696e-003; 
             5.23598775598299e-004, 0.00000000000000e+000; -2.96705972839036e-003, 
             -4.71238898038469e-003; -5.58505360638185e-003, -5.58505360638185e-003])
         "|Geometry|camber as function of bounce|"
        annotation (extent=[-18, 0; 12, 28]);
      
      parameter Utilities.Polynomial.ScaledPolynomial data_Link2_steering(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.082, 
        u_max=0.078, 
        y_min=-0.010297, 
        y_max=0.0012217, 
        c={6.30814489981295e+000,-1.63208206468340e+001,1.59971846025538e+001,-9.43372508316803e+000,
            2.72106634745229e+000,7.28435761181664e-001}, 
        y_umin=-0.0019065, 
        der_y_umin=0.1959, 
        y_umax=-0.010294, 
        der_y_umax=-0.1366, 
        original=[-8.20000000000000e-002, -1.91986217719376e-003; -6.20000000000000e-002, 
             -4.20000000000000e-002; -2.20000000000000e-002, -1.60000000000000e-002; 
             -2.00000000000000e-003, 0.00000000000000e+000; 
            1.80000000000000e-002, 3.80000000000000e-002; 5.80000000000000e-002, 
             7.80000000000000e-002; 6.98131700797732e-004, 
            1.22173047639603e-003; 1.22173047639603e-003, 8.72664625997165e-004; 
             1.74532925199433e-004, 0.00000000000000e+000; -1.57079632679490e-003, 
             -4.01425727958696e-003; -7.15584993317675e-003, -1.02974425867665e-002])
         "|Geometry|toe in as function of bounce|"
        annotation (extent=[-58, 40; -28, 68]);
      parameter Utilities.Polynomial.ScaledPolynomial data_Link2_camber(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.082, 
        u_max=0.078, 
        y_min=-0.0055851, 
        y_max=0.027576, 
        c={-4.98260440195846e+000,1.28215982847983e+001,-1.17916202644075e+001,
            5.83708966179418e+000,-2.88532206648880e+000,9.99807679818011e-001}, 
           
        y_umin=0.02757, 
        der_y_umin=-0.59801, 
        y_umax=-0.0056199, 
        der_y_umax=-0.04407, 
        original=[-8.20000000000000e-002, 2.75762021815104e-002; -6.20000000000000e-002, 
             -4.20000000000000e-002; -2.20000000000000e-002, -1.60000000000000e-002; 
             -2.00000000000000e-003, 0.00000000000000e+000; 
            1.80000000000000e-002, 3.80000000000000e-002; 5.80000000000000e-002, 
             7.80000000000000e-002; 1.79768912955416e-002, 
            1.09955742875643e-002; 5.75958653158129e-003, 4.01425727958696e-003; 
             5.23598775598299e-004, 0.00000000000000e+000; -2.96705972839036e-003, 
             -4.71238898038469e-003; -5.58505360638185e-003, -5.58505360638185e-003])
         "|Geometry|camber as function of bounce|"
        annotation (extent=[-20, 40; 10, 68]);
      
    end SimpleDataFront;
    
    record SimpleDataRear 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      
      parameter SI.Position[3] rCU_1={-2.53,0.71,0.030} "|Geometry||";
      parameter SI.Position[3] rUW_1={-2.55,0.733,0.030} "|Geometry||";
      
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry||";
      parameter SI.Position[3] rCU_2=Utilities.Functions.EWM({1,-1,1}, rCU_1) 
        "|Geometry||";
      
      parameter SI.Position q0S_1=0.1758*230/275*(0.396)*2*0.84 "|Geometry||";
      parameter SI.Position q0S_2=q0S_1 "|Geometry||";
      
      parameter SI.Position[3] rCMU_1={0.118,0.028,0.0776} 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.0 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.299 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.287 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.093 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=0.03 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=-0.12 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.01 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      annotation (Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -24000; 
             1, 24000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.389, 1336; 0.518, 1466; 0.996, 1891; 
            1.495, 2263]) "|Forces|Left multilink|Strut data"
        annotation (extent=[-67.5, -77.5; -32.5, -42.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces|Left multilink|Strut data"
        annotation (extent=[40.5, -77.5; 75.5, -42.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[0, 0], fc=[-1, 
             -27000; 1, 27000]) "|Forces|Left multilink|Anti Roll data"
        annotation (extent=[-17.5, -77.5; 17.5, -42.5]);
      parameter Utilities.Polynomial.ScaledPolynomial data_Link1_steering(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.088, 
        u_max=0.072, 
        y_min=-0.0071209, 
        y_max=0.0046949, 
        c={-4.08651809141416e+000,-2.61358219299911e+001,1.58089719564480e+002,
            -2.88178014646844e+002,2.43334996812024e+002,-9.95716531245717e+001,
            1.71349749888266e+001,4.37595591060174e-004}, 
        y_umin=-0.0071158, 
        der_y_umin=1.2654, 
        y_umax=-0.00017177, 
        der_y_umax=0.023088, 
        original=[-8.80000000000000e-002, -7.12094334813686e-003; -6.80000000000000e-002, 
             -4.80000000000000e-002; -2.80000000000000e-002, -2.00000000000000e-002; 
             -8.00000000000000e-003, 0.00000000000000e+000; 
            1.20000000000000e-002, 3.20000000000000e-002; 5.20000000000000e-002, 
             7.20000000000000e-002; 4.69493568786475e-003, 
            3.14159265358979e-003; 1.39626340159546e-003, -8.55211333477221e-004; 
             -1.74532925199433e-004, 0.00000000000000e+000; 
            1.74532925199433e-004, 3.49065850398866e-004; 0.00000000000000e+000, 
             -1.74532925199433e-004]) annotation (extent=[-40, 0; -10, 28]);
      parameter Utilities.Polynomial.ScaledPolynomial data_Link1_camber(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.088, 
        u_max=0.072, 
        y_min=-0.039095, 
        y_max=0.013788, 
        c={7.73060054654145e+000,-1.81856097590213e+001,1.38416784600087e+001,-4.61422796271159e+000,
            2.24932992091360e-001,9.97996235422114e-001}, 
        y_umin=0.013682, 
        der_y_umin=0.074345, 
        y_umax=-0.03934, 
        der_y_umax=-0.51823, 
        original=[-8.80000000000000e-002, 1.37881010907552e-002; -6.80000000000000e-002, 
             -4.80000000000000e-002; -2.80000000000000e-002, -2.00000000000000e-002; 
             -8.00000000000000e-003, 0.00000000000000e+000; 
            1.20000000000000e-002, 3.20000000000000e-002; 5.20000000000000e-002, 
             7.20000000000000e-002; 1.22173047639603e-002, 
            9.59931088596881e-003; 6.98131700797732e-003, 5.41052068118242e-003; 
             2.61799387799149e-003, 0.00000000000000e+000; -4.01425727958696e-003, 
             -1.11701072127637e-002; -2.75762021815104e-002, -3.90953752446730e-002])
        annotation (extent=[0, 0; 30, 28]);
      parameter Utilities.Polynomial.ScaledPolynomial data_Link2_steering=
          data_Link1_steering;
      parameter Utilities.Polynomial.ScaledPolynomial data_Link2_camber=
          data_Link1_camber;
    end SimpleDataRear;
    
    record PartData 
      "Contains Body and Body shape dataRigid body with cylindrical shape (also used for animation)"
       
      
      parameter SI.Position r[3]={0.2,0,0} 
        "|Geometry||Vector from frame_a to frame_b, resolved in frame_a";
      parameter SI.Length radius(min=0) = 0.1 
        "|Mass and Inertia|Default cylinder parameters|Radius of cylinder";
      parameter SI.Length innerRadius(
        min=0, 
        max=radius) = 0 
        "|Mass and Inertia|Default cylinder parameters|Inner radius of cylinder";
      parameter Real rho(min=0) = 7.7 
        "|Mass and Inertia|Default cylinder parameters|Density of material [g/cm^3]";
      
      parameter Real material[4]={1,0,0,0.5} 
        "|Animation||Color and specular coefficient";
      parameter Boolean animation=true 
        "|Animation||True if part should be animated";
      parameter String shape="cylinder" "|Animation||shape";
      parameter SI.Length height=radius "|Animation||height of shape";
      parameter SI.Length width=radius "|Animation||width of shape";
      parameter SI.Position r0[3]={0,0,0} 
        "|Animation|Vector from frame_a to shape frame, resolved in frame_a";
      parameter SI.Length length=Utilities.Functions.length(r - r0) 
        "|Animation||length of shape";
      parameter SI.Position lengthDirection[3]=r - r0 
        "|Animation|Vector in direction of cylinder axis, resolved in frame_a";
      parameter SI.Position widthDirection[3]={0,0,1} 
        "|Animation|Vector in direction perpendiculare to cylinder axis, resolved in frame_a";
      parameter Real extra=1.0 "|Animation||Additional parameter";
      parameter Boolean mass=true 
        "|Mass and Inertia||True if mass should be considered";
      
      parameter SI.Position rCM[3]=r0 + length/2*Utilities.Functions.normalize(
          lengthDirection) 
        "|Mass and Inertia|General parameters|Vector from frame_a to center of mass, resolved in frame_a";
      parameter SI.Mass m=mo - mi "Mass of body [kg]";
      parameter SI.Inertia I11=I[1, 1] 
        "|Mass and Inertia|General parameters|(1,1) element of inertia tensor";
      parameter SI.Inertia I22=I[2, 2] 
        "|Mass and Inertia|General parameters|(2,2) element of inertia tensor";
      parameter SI.Inertia I33=I[3, 3] 
        "|Mass and Inertia|General parameters|(3,3) element of inertia tensor";
      parameter SI.Inertia I21=I[2, 1] 
        "|Mass and Inertia|General parameters|(2,1) element of inertia tensor";
      parameter SI.Inertia I31=I[3, 1] 
        "|Mass and Inertia|General parameters|(3,1) element of inertia tensor";
      parameter SI.Inertia I32=I[3, 2] 
        "|Mass and Inertia|General parameters|(3,2) element of inertia tensor";
      
    protected 
      constant Real pi=Modelica.Constants.pi;
      parameter SI.Mass mo=1000*rho*pi*length*radius*radius;
      parameter SI.Mass mi=1000*rho*pi*length*innerRadius*innerRadius;
      parameter SI.Inertia i22=(mo*(length*length + 3*radius*radius) - mi*(
          length*length + 3*innerRadius*innerRadius))/12;
      parameter Real Scyl[3, 3]=Utilities.Functions.from_nxy(lengthDirection, 
          widthDirection);
      parameter SI.Inertia I[3, 3]=Scyl*diagonal({(mo*radius*radius - mi*
          innerRadius*innerRadius)/2,i22,i22})*transpose(Scyl);
    equation 
      
      annotation (Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=1)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")), Documentation(info="Parameters for rigid body. By default the mass properties of the body are
computed from the cylinder data. Optionally, the cylinder may be hollow.
These data can be overrided by adding other data for mass, inertia and centre of gravity.
Parameter vectors are defined with respect to frame_a in [m].
Parameters:
  r[3]       : Position vector from the origin of frame_a to the origin of
               frame_b.
  Axis[3]    : Unit vector in direction of the cylinder axis
               (will be normalized)
  Length     : Length of cylinder in [m].
  Radius     : Radius of cylinder in [m].
  InnerRadius: Inner radius of cylinder in [m].
  rho        : Density of material in [g/cm^3], e.g.
                  steel: 7.7 .. 7.9
                  wood : 0.4 .. 0.8
  Material(4): = {r, g, b, specular}.
               Color and specular coefficient of the box.
               [r,g,b] affects the color of diffuse and ambient reflected
               light. Specular is a coefficient defining white specular
               reflection. What color that is reflected also depends on the
               color of the light sources. Note, r g, b and specular are
               given in the range 0-1. Specular=1 gives a metallic appearance."));
      
    end PartData;
    
    record DoubleWishBoneData 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //double wishbone
      parameter Real[3] rCL1_1={-0.005,0.375,0.230} 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL2_1={-0.315,0.355,0.230} 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL1L2_1={0.005,0.690,0.230} 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL3_1={-0.005,0.375,-0.040} 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL4_1={-0.315,0.355,-0.030} 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL3L4_1={0.005,0.690,-0.070} 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUW_1={-0.000,0.727,0.040} 
        "|Geometry|Double wish bone|";
      
      parameter Real[3] rCL1_2=Utilities.Functions.EWM({1,-1,1}, rCL1_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL2_2=Utilities.Functions.EWM({1,-1,1}, rCL2_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL3_2=Utilities.Functions.EWM({1,-1,1}, rCL3_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL4_2=Utilities.Functions.EWM({1,-1,1}, rCL4_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL1L2_2=Utilities.Functions.EWM({1,-1,1}, rUL1L2_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL3L4_2=Utilities.Functions.EWM({1,-1,1}, rUL3L4_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Double wish bone|";
      
      // strut
      parameter Real[3] rCS_1={-0.030,0.545,0.565} "|Geometry|Strut|";
      parameter Real[3] rSL3L4_1={-0.000,0.580,0.040} "|Geometry|Strut|";
      parameter Real q0S_1=3516/20000 "|Geometry|Strut|";
      
      parameter Real[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Strut|";
      parameter Real[3] rSL3L4_2=Utilities.Functions.EWM({1,-1,1}, rSL3L4_1) 
        "|Geometry|Strut|";
      parameter Real q0S_2=q0S_1 "|Geometry|Strut|";
      
      //steering rack
      parameter Real[3] rUL5_1={-0.130,0.645,0.075} "|Geometry|Steering|";
      parameter Real[3] rRL5_1={-0.160,0.305,0.095} "|Geometry|Steering|";
      parameter Real[3] rUL5_2=Utilities.Functions.EWM({1,-1,1}, rUL5_1) 
        "|Geometry|Steering|";
      parameter Real[3] rRL5_2=Utilities.Functions.EWM({1,-1,1}, rRL5_1) 
        "|Geometry|Steering|";
      parameter Real ratioWheelToRack=0.03 
        "|Geometry|Steering| rotatational to translational ratio [m/rad]";
      parameter Real iSW=0.2 "|steering|masses and inertias|";
      
      //torsion anti roll
      
      parameter Real[3] rUL6_1={-0.055,0.540,0.220} "|Geometry|Anti roll|";
      parameter Real[3] rAL6_1={-0.045,0.510,0.020} "|Geometry|Anti roll|";
      parameter Real[3] rCA_1={-0.240,0.340,0.035} "|Geometry|Anti roll|";
      
      parameter Real[3] rUL6_2=Utilities.Functions.EWM({1,-1,1}, rUL6_1) 
        "|Geometry|Anti roll|";
      parameter Real[3] rAL6_2=Utilities.Functions.EWM({1,-1,1}, rAL6_1) 
        "|Geometry|Anti roll|";
      parameter Real[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1) 
        "|Geometry|Anti roll|";
      
      annotation (Images(Parameters(name="|Geometry|Double wish bone|", source=
                "images/DoubleWishBone_text.png")), Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[-1, -20; -0.5, 
             -6; 0, 0; 0.5, 6; 1, 20], fc=[-1, 300; -0.5, 110; 0, 0; 0.5, -110; 
             1, -300]) "|Forces|Anti roll|" annotation (extent=[8, 0; 28, 20]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_SW(fc=[-1, 40000; 
            0, 0; 1, -40000], fd=[-1, 2000; 0, 0; 1, -2000]) 
        "|Forces|Steering|Steering wheel axis compliance"
        annotation (extent=[-32, 18; -12, 38]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -20000; 
             1, 20000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.385, 1336; 0.518, 1455; 0.995, 1851; 
            1.455, 2255]) "|Forces|Left double wish bone|Strut data"
        annotation (extent=[-57.5, -35.5; -22.5, -0.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=[-1, -20000; 
             1, 20000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.385, 1336; 0.518, 1455; 0.995, 1851; 
            1.455, 2255]) "|Forces|Left double wish bone|Strut data"
        annotation (extent=[42.5, -37.5; 77.5, -2.5]);
    end DoubleWishBoneData;
    
    record DoubleWishBoneData2 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //double wishbone
      parameter Real[3] rCL1_1={0.223,0.167,0.121} 
        "|Geometry|Double wish bone|";
      //{0.22,.51,.21};
      parameter Real[3] rCL2_1={-0.211,0.177,0.121} 
        "|Geometry|Double wish bone|";
      //{-0.23,.5,.2};
      parameter Real[3] rCL3_1={0.210,0.165,-0.113} 
        "|Geometry|Double wish bone|";
      //{0.23,.5,-.2};
      parameter Real[3] rCL4_1={-0.209,0.171,-0.114} 
        "|Geometry|Double wish bone|";
      //{-0.231,.51,-.22};
      parameter Real[3] rUL1L2_1={0.002,0.750,0.128} 
        "|Geometry|Double wish bone|";
      //{-0.02,0.8,0.2};
      parameter Real[3] rUL3L4_1={0.002,0.753,-0.117} 
        "|Geometry|Double wish bone|";
      //{0.01,0.82,-0.2};
      parameter Real[3] rUW_1={0.013,0.763,0.001} "|Geometry|Double wish bone|";
      //{0,0.87,0};
      
      parameter Real[3] rCL1_2=Utilities.Functions.EWM({1,-1,1}, rCL1_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL2_2=Utilities.Functions.EWM({1,-1,1}, rCL2_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL3_2=Utilities.Functions.EWM({1,-1,1}, rCL3_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL4_2=Utilities.Functions.EWM({1,-1,1}, rCL4_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL1L2_2=Utilities.Functions.EWM({1,-1,1}, rUL1L2_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL3L4_2=Utilities.Functions.EWM({1,-1,1}, rUL3L4_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Double wish bone|";
      
      //steering rack
      parameter Real[3] rUL5_1={-0.130,0.755,0.075} "|Geometry|Steering|";
      parameter Real[3] rRL5_1={-0.160,0.305,0.095} "|Geometry|Steering|";
      //parameter Real[3] rUL5_1={0.180,0.761,0.101} "|Geometry|Steering|";
      //{0.1,0.82,0.1};
      //parameter Real[3] rRL5_1={0.211,0.171,0.110} "|Geometry|Steering|";
      //{0.11,0.34,0.19};
      parameter Real[3] rUL5_2=Utilities.Functions.EWM({1,-1,1}, rUL5_1) 
        "|Geometry|Steering|";
      parameter Real[3] rRL5_2=Utilities.Functions.EWM({1,-1,1}, rRL5_1) 
        "|Geometry|Steering|";
      parameter Real ratioWheelToRack=0.03 
        " |Geometry|Steering| rotatational to translational ratio [m/rad]";
      parameter Real iSW=0.2 "|steering|masses and inertias|";
      parameter Real[:, 2] tcSW=[-1, 60000; 0, 0; 1, -60000] 
        "|Forces|Steering|";
      parameter Real[:, 2] tdSW=[-1, 2000; 0, 0; 1, -2000] "|Forces|Steering|";
      
      //mono-damper belleville anti roll
      parameter Real[3] rCP={-0.080,0.000,0.141} "|Geometry|BelleVille|";
      parameter Real[3] rCS={-0.401,0.000,0.211} "|Geometry|BelleVille|";
      parameter Real[3] rPS={-0.062,0.000,0.201} "|Geometry|BelleVille|";
      parameter Real[3] rPL6_1={-0.021,0.05,0.169} "|Geometry|BelleVille|";
      parameter Real[3] rUL6_1={0.001,0.732,-0.167} "|Geometry|BelleVille|";
      //{0,0.8,-0.1};
      parameter Real[3] nCP={0,1,0} "|Geometry|BelleVille|";
      parameter Real q0S=0.02 "|Geometry|BelleVille|";
      
      parameter Real[3] rPL6_2=Utilities.Functions.EWM({1,-1,1}, rPL6_1) 
        "|Geometry|BelleVille|";
      parameter Real[3] rUL6_2=Utilities.Functions.EWM({1,-1,1}, rUL6_1) 
        "|Geometry|BelleVille|";
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_S(fc=[-1, -80000; 
            -0.5, -50000; 0, 0; 0.5, 50000; 1, 80000], fd=[-1, -10000; -0.5, -4000; 
             0, 0; 0.5, 3000; 1, 8000]) "|Forces|BelleVille|"
        annotation (extent=[-60, 20; -40, 40]);
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fc=[-1, -250000; 
             -0.5, -110000; 0, 0; 0.5, 110000; 1, 250000], fd=[-1, -20; -0.5, -6; 
             0, 0; 0.5, 6; 1, 20]) "|Forces|BelleVille|"
        annotation (extent=[-60, -20; -40, 0]);
      
      annotation (Images(Parameters(name="|Geometry|Double wish bone|", source=
                "images/BelleVille_text.png")), Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
    end DoubleWishBoneData2;
    
    record DoubleWishBoneData3 
      
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //double wishbone
      parameter Real[3] rCL1_1={-2.6 + 0.18,0.17,0.09} 
        "|Geometry|Double wish bone|";
      //{0.22,.51,.21};
      parameter Real[3] rCL2_1={-2.6 - 0.18,0.17,0.09} 
        "|Geometry|Double wish bone|";
      //{-0.23,.5,.2};
      parameter Real[3] rCL3_1={-2.6 + 0.18,0.17,-0.1} 
        "|Geometry|Double wish bone|";
      //{0.23,.5,-.2};
      parameter Real[3] rCL4_1={-2.6 - 0.18,0.17,-0.1} 
        "|Geometry|Double wish bone|";
      //{-0.231,.51,-.22};
      parameter Real[3] rUL1L2_1={-2.6 + 0,0.67,0.09} 
        "|Geometry|Double wish bone|";
      //{-0.02,0.8,0.2};
      parameter Real[3] rUL3L4_1={-2.6 + 0,0.67,-0.1} 
        "|Geometry|Double wish bone|";
      //{0.01,0.82,-0.2};
      parameter Real[3] rUW_1={-2.6 + 0,0.70,0} "|Geometry|Double wish bone|";
      //{0,0.87,0};
      
      parameter Real[3] rCL1_2=Utilities.Functions.EWM({1,-1,1}, rCL1_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL2_2=Utilities.Functions.EWM({1,-1,1}, rCL2_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL3_2=Utilities.Functions.EWM({1,-1,1}, rCL3_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rCL4_2=Utilities.Functions.EWM({1,-1,1}, rCL4_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL1L2_2=Utilities.Functions.EWM({1,-1,1}, rUL1L2_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUL3L4_2=Utilities.Functions.EWM({1,-1,1}, rUL3L4_1) 
        "|Geometry|Double wish bone|";
      parameter Real[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Double wish bone|";
      
      //steering rack
      parameter Real[3] rUL5_1={-2.6 - 0.18,0.67,0.09} "|Geometry|Steering|";
      //{0.1,0.82,0.1};
      parameter Real[3] rRL5_1={-2.6 - 0.18,0.17,0.09} "|Geometry|Steering|";
      //{0.11,0.34,0.19};
      parameter Real[3] rUL5_2=Utilities.Functions.EWM({1,-1,1}, rUL5_1) 
        "|Geometry|Steering|";
      parameter Real[3] rRL5_2=Utilities.Functions.EWM({1,-1,1}, rRL5_1) 
        "|Geometry|Steering|";
      parameter Real ratioWheelToRack=0.03 
        "|Geometry|Steering| rotatational to translational ratio [m/rad]";
      
      //twindamper torsion anti roll
      parameter Real[3] rCP_1={-2.6 + 0.04,0.17,0.15} "|Geometry|Twin damper|";
      //{0.1,0.3,0.2};
      parameter Real[3] rCS_1={-2.6 + 0.33,0.11,0.15} "|Geometry|Twin damper|";
      //{0.41,0.1,0.2};
      parameter Real[3] rPS_1={-2.6 + 0.045,0.05,0.185} 
        "|Geometry|Twin damper|";
      //{0.1,0.1,0.21};
      parameter Real[3] rPL6_1={-2.6 - 0.045,0.17,0.15} 
        "|Geometry|Twin damper|";
      //{0,0.3,0.22};
      parameter Real[3] rUL6_1={-2.6 + 0,0.67,-0.1} "|Geometry|Twin damper|";
      //{0,0.8,-0.1};
      parameter Real[3] rPL7_1={-2.6 + 0,0.13,0.125} "|Geometry|Twin damper|";
      //{0.1,0.2,0.2};
      parameter Real[3] rAL7_1={-2.6 + 0.25,0.17,0.13} "|Geometry|Twin damper|";
      //{.35,0.2,0.215};
      parameter Real[3] rCA_1={-2.6 + 0.25,0.17,0.05} "|Geometry|Twin damper|";
      //{.5,.2,0};
      parameter Real[3] nP_1=cross((rPL6_1 - rUL6_1), (rPL7_1 - rAL7_1)) 
        "|Geometry|Twin damper|";
      parameter Real q0S_1=0.04 "|Geometry|Twin damper|";
      
      parameter Real[3] rCP_2=Utilities.Functions.EWM({1,-1,1}, rCP_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] rPS_2=Utilities.Functions.EWM({1,-1,1}, rPS_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] rPL6_2=Utilities.Functions.EWM({1,-1,1}, rPL6_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] rUL6_2=Utilities.Functions.EWM({1,-1,1}, rUL6_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] rPL7_2=Utilities.Functions.EWM({1,-1,1}, rPL7_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] rAL7_2=Utilities.Functions.EWM({1,-1,1}, rAL7_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1) 
        "|Geometry|Twin damper|";
      parameter Real[3] nP_2=Utilities.Functions.EWM({1,-1,1}, nP_1) 
        "|Geometry|Twin damper|";
      parameter Real q0S_2=q0S_1 "|Geometry|Twin damper|";
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -180000; 
             -0.5, -50000; 0, 0; 0.5, 50000; 1, 180000], fd=[-1, -18000; -0.5, 
            -8000; 0, 0; 0.5, 10000; 1, 25000]) "|Forces|Twin damper|"
        annotation (extent=[-60, 20; -40, 40]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=[-1, -180000; 
             -0.5, -50000; 0, 0; 0.5, 50000; 1, 180000], fd=[-1, -18000; -0.5, 
            -8000; 0, 0; 0.5, 10000; 1, 25000]) "|Forces|Twin damper|"
        annotation (extent=[-62, -36; -42, -16]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fc=[-1, -4200; 1, 
             4200], fd=[-1, -42; 0, 0; 1, 42]) "|Forces|Twin damper|"
        annotation (extent=[-24, -14; -4, 6]);
      
      annotation (Images(Parameters(name="|Geometry|Double wish bone|", source=
                "images/TwinStrut_text.png")), Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
    end DoubleWishBoneData3;
    
    record RigidAxle4LinkData "Record with data for trailing arm suspensions" 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //BEAM AXLE
      parameter Real[3] rUW_1={-2.55 + 0.000,0.733,0.030} 
        "|Geometry|Beam axle|";
      parameter Real[3] rCL1_1={-2.55 + 0.470,0.500,0.010} 
        "|Geometry|Beam axle|";
      parameter Real[3] rUL1_1={-2.55 - 0.060,0.662,-0.057} 
        "|Geometry|Beam axle|";
      parameter Real[3] rCL2_1={-2.55 + 0.433,0.670,0.167} 
        "|Geometry|Beam axle|";
      parameter Real[3] rUL2_1={-2.55 + 0.032,0.668,0.160} 
        "|Geometry|Beam axle|";
      
      parameter Real[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Beam axle|";
      parameter Real[3] rCL1_2=Utilities.Functions.EWM({1,-1,1}, rCL1_1) 
        "|Geometry|Beam axle|";
      parameter Real[3] rUL1_2=Utilities.Functions.EWM({1,-1,1}, rUL1_1) 
        "|Geometry|Beam axle|";
      parameter Real[3] rCL2_2=Utilities.Functions.EWM({1,-1,1}, rCL2_1) 
        "|Geometry|Beam axle|";
      parameter Real[3] rUL2_2=Utilities.Functions.EWM({1,-1,1}, rUL2_1) 
        "|Geometry|Beam axle|";
      
      // strut
      parameter Real[3] rCS_1={-2.55 + 0.061,0.518,0.450} "|Geometry|Strut|";
      parameter Real[3] rUS_1={-2.55 + 0.0605,0.581,-0.046} "|Geometry|Strut|";
      parameter Real q0S_1=0.068 "|Geometry|Strut|";
      
      parameter Real[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Strut|";
      parameter Real[3] rUS_2=Utilities.Functions.EWM({1,-1,1}, rUS_1) 
        "|Geometry|Strut|";
      parameter Real q0S_2=q0S_1 "|Geometry|Strut|";
      
      //torsion anti roll
      parameter Real[3] rUL3_1={-2.55 - 0.0395,0.490,0.002} 
        "|Geometry|Anti roll|";
      parameter Real[3] rAL3_1={-2.55 - 0.040,0.491,0.0775} 
        "|Geometry|Anti roll|";
      parameter Real[3] rCA_1={-2.55 - 0.210,0.463,0.060} 
        "|Geometry|Anti roll|";
      
      parameter Real[3] rUL3_2=Utilities.Functions.EWM({1,-1,1}, rUL3_1) 
        "|Geometry|Anti roll|";
      parameter Real[3] rAL3_2=Utilities.Functions.EWM({1,-1,1}, rAL3_1) 
        "|Geometry|Anti roll|";
      parameter Real[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1) 
        "|Geometry|Anti roll|";
      
      annotation (Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -20000; 
             1, 20000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.385, 1336; 0.518, 1455; 0.995, 1851; 
            1.455, 2255]) "|Forces|Left MacPherson|Strut data"
        annotation (extent=[-77.5, -37.5; -42.5, -2.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=[-1, -20000; 
             1, 20000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.385, 1336; 0.518, 1455; 0.995, 1851; 
            1.455, 2255]) "|Forces|Left MacPherson|Strut data"
        annotation (extent=[42.5, -37.5; 77.5, -2.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[-1, -100; 0, 
             0; 1, 100], fc=[-1, -33626; 0, 0; 1, 33626]) 
        "|Forces|Anti Roll|Anti Roll data"
        annotation (extent=[-17.5, 2.5; 17.5, 37.5]);
    end RigidAxle4LinkData;
    
    record FixedAxleData "Record with data for trailing arm suspensions" 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //BEAM AXLE
      parameter Real[3] rUW_1={-2.55 + 0.000,0.733,0.030} 
        "|Geometry|Beam axle|";
      parameter Real[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Beam axle|";
      
      annotation (Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")));
      
    end FixedAxleData;
    
    record FiveLinkData "Data record for FiveLinkSuspension" 
      
      // GEOMETRIC SETTINGS
      parameter Real[3] scaleFactor={1,1,1} 
        "|Geometry|Scaling|Use this to rescale the geometry, does not affec inertia properties and force elements";
      //fivelink
      parameter SI.Position[3] rCL1_1={-2.6430,0.231,-0.022} 
        "|Geometry|Left fivelink|link 1 to chassis/body";
      parameter SI.Position[3] rCL2_1={-2.5039,0.4150,0.174} 
        "|Geometry|Left fivelink|link 2 to chassis/body";
      parameter SI.Position[3] rCL3_1={-2.4617,0.3750,0.0637} 
        "|Geometry|Left fivelink|link 3 to chassis/body";
      parameter SI.Position[3] rCL4_1={-2.6617,0.2950,0.1637} 
        "|Geometry|Left fivelink|link 4 to chassis/body";
      parameter SI.Position[3] rCL5_1={-2.1617,0.4750,-0.0637} 
        "|Geometry|Left fivelink|link 5 to chassis/body";
      parameter SI.Position[3] rUL1_1={-2.6170,0.6672,-0.05327} 
        "|Geometry|Left fivelink|link 1 to upright";
      parameter SI.Position[3] rUL2_1={-2.5178,0.6678,0.1607} 
        "|Geometry|Left fivelink|link 2 to upright";
      parameter SI.Position[3] rUL3_1={-2.4270,0.612,0.050} 
        "|Geometry|Left fivelink|link 3 to upright";
      parameter SI.Position[3] rUL4_1={-2.6270,0.612,0.150} 
        "|Geometry|Left fivelink|link 4 to upright";
      parameter SI.Position[3] rUL5_1={-2.4270,0.612,-0.010} 
        "|Geometry|Left fivelink|link 5 to upright";
      parameter SI.Position[3] rUW_1={-2.557,0.7373,0.0290} 
        "|Geometry|Left fivelink|Wheel centre";
      
      parameter SI.Position[3] rCL1_2=Utilities.Functions.EWM({1,-1,1}, rCL1_1)
         "|Geometry|Right fivelink|link 1 to chassis/body";
      parameter SI.Position[3] rCL2_2=Utilities.Functions.EWM({1,-1,1}, rCL2_1)
         "|Geometry|Right fivelink|link 2 to chassis/body";
      parameter SI.Position[3] rCL3_2=Utilities.Functions.EWM({1,-1,1}, rCL3_1)
         "|Geometry|Right fivelink|link 3 to chassis/body";
      parameter SI.Position[3] rCL4_2=Utilities.Functions.EWM({1,-1,1}, rCL4_1)
         "|Geometry|Right fivelink|link 4 to chassis/body";
      parameter SI.Position[3] rCL5_2=Utilities.Functions.EWM({1,-1,1}, rCL5_1)
         "|Geometry|Right fivelink|link 5 to chassis/body";
      parameter SI.Position[3] rUL1_2=Utilities.Functions.EWM({1,-1,1}, rUL1_1)
         "|Geometry|Right fivelink|link 1 to upright";
      parameter SI.Position[3] rUL2_2=Utilities.Functions.EWM({1,-1,1}, rUL2_1)
         "|Geometry|Right fivelink|link 2 to upright";
      parameter SI.Position[3] rUL3_2=Utilities.Functions.EWM({1,-1,1}, rUL3_1)
         "|Geometry|Right fivelink|link 3 to upright";
      parameter SI.Position[3] rUL4_2=Utilities.Functions.EWM({1,-1,1}, rUL4_1)
         "|Geometry|Right fivelink|link 4 to upright";
      parameter SI.Position[3] rUL5_2=Utilities.Functions.EWM({1,-1,1}, rUL5_1)
         "|Geometry|Right fivelink|link 5 to upright";
      parameter SI.Position[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "|Geometry|Right fivelink|Wheel centre";
      
      // strut
      parameter SI.Position[3] rCS_1={-2.55 - 0.0621,0.518,0.450} 
        "|Geometry|Left strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_1={-2.55 - 0.063305,0.581,-0.046} 
        "|Geometry|Left strut|Strut ot upright";
      parameter SI.Position q0S_1=0.116 
        "|Geometry|Left strut|Additional length for unloaded strut";
      
      parameter SI.Position[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1) 
        "|Geometry|Right strut|Strut to chassis/body";
      parameter SI.Position[3] rUS_2=Utilities.Functions.EWM({1,-1,1}, rUS_1) 
        "|Geometry|Right strut|Strut ot upright";
      parameter SI.Position q0S_2=q0S_1 
        "|Geometry|Right strut|Additional length for unloaded strut";
      
      //torsion anti roll
      parameter SI.Position[3] rUL6_1={-2.55 - 0.03595,0.4590,0.0021} 
        "|Geometry|Anti roll, left|Anti roll bar link to upright";
      parameter SI.Position[3] rAL6_1={-2.55 - 0.040,0.4951,0.07755} 
        "|Geometry|Anti roll, left|Anti roll bar link to anti roll bar";
      parameter SI.Position[3] rCA_1={-2.55 - 0.2510,0.4653,0.0630} 
        "|Geometry|Anti roll, left|Anti roll bar to chassis/body";
      
      parameter SI.Position[3] rUL6_2=Utilities.Functions.EWM({1,-1,1}, rUL6_1)
         "|Geometry|Anti roll, right|Anti roll bar link to upright";
      parameter SI.Position[3] rAL6_2=Utilities.Functions.EWM({1,-1,1}, rAL6_1)
         "|Geometry|Anti roll, right|Anti roll bar link to anti roll bar";
      parameter SI.Position[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1) 
        "|Geometry|Anti roll, right|Anti roll bar to chassis/body";
      parameter SI.Position[3] rCMU_1={0.118,0.028,0.0776} 
        "|Mass and Inertia|left upright|";
      parameter SI.Mass mU_1=11.0 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i11U_1=0.0929 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i22U_1=0.287 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i33U_1=0.293 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i21U_1=-0.03 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i31U_1=0.12 "|Mass and Inertia|left upright|";
      parameter SI.Inertia i32U_1=0.01 "|Mass and Inertia|left upright|";
      
      parameter SI.Position[3] rCMU_2=Utilities.Functions.EWM({1,-1,1}, rCMU_1)
         "|Mass and Inertia|right upright|";
      parameter SI.Mass mU_2=mU_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i11U_2=i11U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i22U_2=i22U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i33U_2=i33U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i21U_2=i21U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i31U_2=i31U_1 "|Mass and Inertia|right upright|";
      parameter SI.Inertia i32U_2=i32U_1 "|Mass and Inertia|right upright|";
      
      annotation (
        Images(Parameters(name="|Geometry|Right fivelink|", source=
                "images/FiveLink_text.png")), 
        Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)), 
             
          Rectangle(extent=[-82, 82; 78, 54], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 50; 78, 22], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-82, 16; 78, -12], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -54; 80, -82], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -18; -4, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -46; -4, -42], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-8, -30; -4, -34], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(
            extent=[-100, 172; 100, 88], 
            style(
              color=0, 
              fillColor=0, 
              fillPattern=1), 
            string="%name")), 
        Diagram);
      
      parameter Utilities.Forces.Utilities.ForceTable1D data_A(fd=[0, 0], fc=[-1, 
             -5432; 1, 5432]) "|Forces||Anti Roll data"
        annotation (extent=[-17.5, -57.5; 17.5, -22.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_2(fc=data_S_1.fc, 
           fd=data_S_1.fd) "|Forces||Left strut data"
        annotation (extent=[38.5, -99.5; 73.5, -64.5]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S_1(fc=[-1, -24000; 
             1, 24000], fd=[-1.4950, -719; -0.996, -531; -0.518, -413; -0.389, 
            -376; -0.260, -327; -0.130, -166; -0.052, -82; 0, 0; 0.052, 154; 
            0.130, 419; 0.260, 1122; 0.389, 1336; 0.518, 1466; 0.996, 1891; 
            1.495, 2263]) "|Forces||Right strut data"
        annotation (extent=[-69.5, -99.5; -34.5, -64.5]);
    end FiveLinkData;
  end Data;
  
  package Components "Vehicle components to build suspensions." 
    import Modelica.SIunits;
    
    model MacPherson "MacPherson strut linkage" 
      import SI = Modelica.SIunits;
      
      extends Chassis.Interfaces.Interface;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      parameter SI.Position[3] rCL1={0.1070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to front link mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rCL2={-0.3070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to rear link mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rCS={-0.0295,0.850,0.5670} 
        "|Geometry| Vector from origin of frame_C to strut mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUS={-0.0070,0.8733,0.1380} 
        "|Geometry| Vector from origin of frame_C to strut mount in uppright resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUL1L2={-0.0070,0.8733,-0.0380} 
        "|Geometry| Vector from origin of frame_C to low spindleUtilities.Joints.Joint resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUW={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to wheel centre resolved in frame_C (at initial time)";
      parameter SI.Position[3] rRL3={-0.12,0.85,0.08} 
        "|Geometry| Vector from origin of frame_C to origin of frame_S resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUL3={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to lower ball of steering rod resolved in frame_C (at initial time)";
      parameter SI.Length q0S=0.05 
        "|Geometry| Unstretched additional spring length of the strut, compared to the construction geometry";
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|Wheel carrier|";
      parameter SI.Mass mU=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i11U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i22U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i33U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i21U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i31U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i32U=0 "|Mass and Inertia|Wheel carrier|";
      
      parameter SI.Position[3] rCML1L2={0,0,0} "|Mass and Inertia|Wishbone|";
      parameter SI.Mass mL1L2=0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i11L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i22L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i33L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i21L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i31L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i32L1L2=.0 "|Mass and Inertia|Wishbone|";
      
      /*Scaled coordinates*/
    protected 
      parameter SI.Position[3] rCL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL1);
      parameter SI.Position[3] rCL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL2);
      parameter SI.Position[3] rCS_scaled=Utilities.Functions.EWM(scaleFactor, 
          rCS);
      parameter SI.Position[3] rUS_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUS);
      parameter SI.Position[3] rUL1L2_scaled=Utilities.Functions.EWM(
          scaleFactor, rUL1L2);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      parameter SI.Position[3] rRL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL3);
      parameter SI.Position[3] rUL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL3);
      parameter SI.Length q0Strut=sqrt((rCS_scaled - rUS_scaled)*(rCS_scaled - 
          rUS_scaled)) + q0S;
      parameter SI.Length L_frontBar=sqrt((rCL1_scaled - rUL1L2_scaled)*(
          rCL1_scaled - rUL1L2_scaled)) "length of frontBar";
      parameter SI.Length L_rearBar=sqrt((rCL2_scaled - rUL1L2_scaled)*(
          rCL2_scaled - rUL1L2_scaled)) "length of rearBar";
      parameter SI.Length L_springRod=sqrt((rUS_scaled - rUL1L2_scaled)*(
          rUS_scaled - rUL1L2_scaled));
      parameter SI.Length L_outerRod=sqrt((rUW_scaled - rUL1L2_scaled)*(
          rUW_scaled - rUL1L2_scaled));
      parameter SI.Position rS[3]=rCS_scaled - rUL1L2_scaled;
      parameter SI.Position rU[3]=rUW_scaled - rUL1L2_scaled 
        "Position vector from frame_L1L2 to frame_U, resolved in frame_C";
      parameter Real n_a[3]=cross(rS, rU) 
        "First rotation axis of universalUtilities.Joints.Joint in springJoint, resolved in frame_C";
      
    public 
      ModelicaAdditions.MultiBody.Joints.Revolute innerJoint(
        q(stateSelect=StateSelect.always), 
        qd(stateSelect=StateSelect.always), 
        n=rCL1_scaled - rCL2_scaled)
        annotation (extent=[-70, -70; -50, -50], rotation=0);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Polygon(points=[52, 96; 64, 80; 62, 86; 70, 62; -100, 106; 52, 96], 
              style(pattern=0)), 
          Line(points=[-44, -44; -18, -24], style(color=0, thickness=4)), 
          Line(points=[-58, -46; 30, -30; -58, -20], style(color=0, thickness=4)), 
             
          Ellipse(extent=[-58, -30; -38, -10], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, -28; -40, -12], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-62, -10; -52, -30], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-54, -14; -42, -26], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-78, -58; -58, -38], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, -56; -60, -40], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-81, -37; -71, -57], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-74, -42; -62, -54], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[42, -12; 60, -12; 64, 4; 90, 4; 90, -2; 68, -2; 64, -18; 
                 42, -18; 42, -12], style(color=8, fillColor=8)), 
          Polygon(points=[62, 18; 56, 18; 48, -14; 56, -14; 62, 18], style(
              pattern=0, 
              gradient=2, 
              fillColor=8)), 
          Ellipse(extent=[50, 14; 70, 34], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[52, 16; 68, 32], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[42, 38; 54, 14], style(color=7, fillColor=7)), 
          Ellipse(extent=[-50, 50; -30, 70], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-48, 52; -32, 68], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-36, 72; -22, 46], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-46, 66; -34, 54], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[54, 30; 66, 18], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-34, 60; 58, 24], style(color=0, thickness=4)), 
          Ellipse(extent=[28, -40; 48, -20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, -38; 46, -22], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, -20; 50, -26], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[34, -8; 42, -26], style(color=8, fillColor=8)), 
          Polygon(points=[38, 2; 50, -2; 50, 2; 38, 6; 38, 2], style(color=10, 
                fillColor=10)), 
          Polygon(points=[26, 6; 38, 2; 38, 6; 26, 10; 26, 6], style(color=10, 
                fillColor=10)), 
          Polygon(points=[38, 18; 50, 14; 50, 18; 38, 22; 38, 18], style(color=
                  10, fillColor=10)), 
          Polygon(points=[26, 22; 38, 18; 38, 22; 26, 26; 26, 22], style(color=
                  10, fillColor=10)), 
          Polygon(points=[38, 34; 50, 30; 50, 34; 38, 38; 38, 34], style(color=
                  10, fillColor=10)), 
          Polygon(points=[26, 38; 38, 34; 38, 38; 26, 42; 26, 38], style(color=
                  10, fillColor=10)), 
          Polygon(points=[38, 50; 50, 46; 50, 50; 38, 54; 38, 50], style(color=
                  10, fillColor=10)), 
          Rectangle(extent=[32, 56; 44, -8], style(gradient=1, fillColor=8)), 
          Rectangle(extent=[30, 52; 46, 14], style(gradient=1, fillColor=8)), 
          Polygon(points=[26, 38; 38, 42; 38, 46; 26, 42; 26, 38], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, 42; 50, 46; 50, 50; 38, 46; 38, 42], style(color=
                  8, fillColor=8)), 
          Polygon(points=[26, 22; 38, 26; 38, 30; 26, 26; 26, 22], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, 26; 50, 30; 50, 34; 38, 30; 38, 26], style(color=
                  8, fillColor=8)), 
          Polygon(points=[26, 6; 38, 10; 38, 14; 26, 10; 26, 6], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[38, 10; 50, 14; 50, 18; 38, 14; 38, 10], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, -6; 50, -2; 50, 2; 38, -2; 38, -6], style(color=8, 
                 fillColor=8)), 
          Ellipse(extent=[28, 56; 48, 76], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, 58; 46, 74], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, 62; 50, 54], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[34, 66; 42, 54], style(gradient=1, fillColor=8)), 
          Ellipse(extent=[32, 72; 44, 60], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[32, -24; 44, -36], style(
              color=0, 
              fillColor=0, 
              fillPattern=1))), 
        Diagram(
          Line(points=[-87, -1; -75, -61], style(color=1, arrow=1)), 
          Text(
            extent=[-95, -47; -75, -54], 
            style(color=1, arrow=1), 
            string="rCL1"), 
          Line(points=[-87, -1; 3, -41], style(color=1, arrow=1)), 
          Text(
            extent=[-13, -28; 2, -35], 
            style(color=1, arrow=1), 
            string="rUL1L2"), 
          Text(
            extent=[-96, 35; -69, 29], 
            style(color=1, arrow=1), 
            string="rCS"), 
          Line(points=[-87, -1; -74, 39], style(color=1, arrow=1)), 
          Text(
            extent=[-9, 54; 15, 47], 
            style(color=1, arrow=1), 
            string="rRL3"), 
          Line(points=[-87, -1; 68, 79], style(color=1, arrow=1)), 
          Line(points=[-87, -1; 64, 53], style(color=1, arrow=1)), 
          Text(
            extent=[24, 50; 47, 43], 
            style(color=1, arrow=1), 
            string="rUL3"), 
          Text(
            extent=[67, 21; 94, 15], 
            style(color=1, arrow=1), 
            string="rUW"), 
          Line(points=[-87, -1; 93, 14], style(color=1, arrow=1)), 
          Text(
            extent=[18, -5; 35, -11], 
            style(color=1, arrow=1), 
            string="rUS"), 
          Line(points=[-87, -1; 43, -5], style(color=1, arrow=1))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUL1L2: location of outer ballUtilities.Joints.Joint, resolved in frame_C
  rCL1:   location of inner frontUtilities.Joints.Joint, resolved in frame_C
  rCL2:   location of inner rearUtilities.Joints.Joint, resolved in frame_C
  rCS:    location of strut mount in chassis, resolved in frame_C
  rUS:    location of strut mount in upright, resolved in frame_C
  rUW:    location of frame_W, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_L12
        annotation (extent=[-13, -119; 17, -89], rotation=270);
      Utilities.Forces.Utilities.ForceTable1D data_S
        annotation (extent=[94, 68; 124, 98]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation upper(r=rCS_scaled)
        annotation (extent=[-64, 9; -84, 29], rotation=90);
      Utilities.Joints.JointUPS MacPherson(n_a=n_a)
        annotation (extent=[22, -21; -17, 19], rotation=90);
      Utilities.Parts.Body springRod(r=rUS_scaled - rUL1L2_scaled, mass=false)
        annotation (extent=[33, -31; 53, -11], rotation=90);
      Utilities.Forces.SpringDamperTableLine3D strut(s_rel0=q0Strut, data=
            data_S) annotation (extent=[32, -1; 52, 19], rotation=90);
      Utilities.Parts.Body outerRod(r=rUW_scaled - rUL1L2_scaled, mass=false)
        annotation (extent=[68, -11; 88, 9], rotation=0);
      ModelicaAdditions.MultiBody.Parts.Body U(
        rCM=rCMU, 
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U) annotation (extent=[69, -61; 89, -41]);
      Utilities.Parts.Body steeringRod(r=rUL3_scaled - rRL3_scaled, shape=
            "cylinder") annotation (extent=[13, 79; 33, 59], rotation=180);
      Utilities.Joints.JointRSU steeringJoint(
        n_b={1,0,0}, 
        n_a=rCS_scaled - rUL1L2_scaled, 
        r_a=rUL3_scaled - rUL1L2_scaled)
        annotation (extent=[54, 39; 84, 69], rotation=90);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation lower(r=rCL1_scaled)
        annotation (extent=[-84, -31; -64, -11], rotation=270);
      Utilities.Parts.Body frontBar(r=rUL1L2_scaled - rCL1_scaled)
        annotation (extent=[-37, -71; -17, -51]);
      Utilities.Parts.Body rearBar(r=rCL2_scaled - rUL1L2_scaled, 
          widthDirection=cross(rCL2_scaled - rUL1L2_scaled, {0,0,1}))
        annotation (extent=[-37, -31; -17, -51], rotation=180);
    protected 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_ta1
        annotation (extent=[62, -11; 64, -9]);
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_S
        annotation (extent=[-70, 50; -100, 80], rotation=180);
      Utilities.Parts.Body steeringLever(r=rUL3_scaled - rUL1L2_scaled)
        annotation (extent=[98, 66; 118, 46], rotation=270);
    equation 
      connect(lower.frame_a, frame_C)
        annotation (points=[-74, -10.5; -74, -1; -105, -1]);
      connect(upper.frame_a, frame_C)
        annotation (points=[-74, 8.5; -74, -1; -105, -1]);
      connect(MacPherson.frame_b, upper.frame_b)
        annotation (points=[2.5, 20; 3, 39; -74, 39; -74, 29.5]);
      connect(steeringJoint.frame_b, frame_S)
        annotation (points=[69, 69.75; 69, 65; -85, 65]);
      connect(outerRod.frame_b, frame_U) annotation (points=[88.5, -1; 105, 1]);
      connect(MacPherson.frame_pb, strut.frame_b)
        annotation (points=[24.925, 11; 27, 3; 27, 29; 42, 29; 42, 19.5]);
      connect(springRod.frame_b, strut.frame_a)
        annotation (points=[43, -10.5; 43, -1.5; 42, -1.5]);
      connect(frame_ta1, outerRod.frame_a)
        annotation (points=[63, -10; 67.5, -1]);
      connect(U.frame_a, frame_ta1) annotation (points=[68.5, -51; 63, -10]);
      connect(frame_ta1, steeringJoint.frame_ta)
        annotation (points=[63, -10; 51.75, 42.15]);
      connect(MacPherson.frame_pa, steeringJoint.frame_a) annotation (points=[
            24.925, -13; 29, -15; 29, -51; 57, -51; 57, 29; 69, 29; 69, 38.25]);
      connect(springRod.frame_a, frame_ta1)
        annotation (points=[43, -31.5; 63, -10]);
      connect(rearBar.frame_a, frontBar.frame_b) annotation (points=[-16.5, -41; 
             -16.5, -45.75; -16, -45.75; -16, -50.5; -17, -50.5; -17, -62]);
      connect(frontBar.frame_b, MacPherson.frame_a)
        annotation (points=[-16.5, -61; 2.5, -61; 2.5, -22]);
      connect(steeringJoint.frame_c, steeringRod.frame_a)
        annotation (points=[51.75, 66; 33.5, 69]);
      connect(frame_L12, frontBar.frame_b)
        annotation (points=[2, -104; -16.5, -61]);
      connect(steeringLever.frame_a, steeringJoint.frame_ta)
        annotation (points=[108, 45.5; 51.75, 42.15]);
      connect(innerJoint.frame_a, lower.frame_b)
        annotation (points=[-70.5, -60; -74, -31.5]);
      connect(innerJoint.frame_b, frontBar.frame_a)
        annotation (points=[-49.5, -60; -43.5, -60; -43.5, -61; -37.5, -61]);
    end MacPherson;
    
    model RackSteering "RackSteering - standard steering system " 
      
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      parameter SI.Position[3] rXL_1={-0.12,0.85,0.08} 
        "Vector to . resolved in innerFrame";
      //{-0.1280,0.6430,0.0744} 
      parameter SI.Position[3] rRL_1={-0.16,0.54,0.085};
      //{-0.1600,0.3070,0.0950};
      parameter SI.Position[3] rX_1={0.0,0.9,0};
      //{0.0059,0.6905,-0.0710} + {-0.0060,0.0365,0.1110};
      parameter SI.Position[3] rXL_2=Utilities.Functions.EWM({1,-1,1}, rXL_1);
      parameter SI.Position[3] rRL_2=Utilities.Functions.EWM({1,-1,1}, rRL_1);
      parameter SI.Position[3] rX_2=Utilities.Functions.EWM({1,-1,1}, rX_1);
      
      parameter Real ratioWheelToRack=22 
        "rotatational to translational ratio [m/rad]";
      
    protected 
      parameter SI.Position[3] rXL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_1);
      parameter SI.Position[3] rRL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL_1);
      parameter SI.Position[3] rX_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_1);
      parameter SI.Position[3] rXL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_2);
      parameter SI.Position[3] rRL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL_2);
      parameter SI.Position[3] rX_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_2);
      parameter SI.Position[3] rr=rRL_2_scaled - (rRL_1_scaled + rRL_2_scaled)/
          2;
      parameter SI.Position[3] rl=rRL_1_scaled - (rRL_1_scaled + rRL_2_scaled)/
          2;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Text(
            extent=[-76, -44; -46, -84], 
            string="L", 
            style(color=73, fillColor=58)), 
          Text(
            extent=[-76, 84; -44, 44], 
            string="R", 
            style(color=41, fillColor=41)), 
          Rectangle(extent=[-76, 90; -4, 80], style(color=10, fillColor=8)), 
          Rectangle(extent=[-76, -80; -4, -90], style(color=10, fillColor=8)), 
          Rectangle(extent=[-2, 6; 62, -4], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-16, 80; -4, -80], style(
              color=9, 
              fillColor=8, 
              fillPattern=7)), 
          Rectangle(extent=[-20, 8; -2, -6], style(
              color=9, 
              fillColor=8, 
              fillPattern=8)), 
          Rectangle(extent=[52, 40; 62, -40], style(gradient=2, fillColor=8)), 
          Line(points=[88, 40; 62, 4], style(color=10)), 
          Line(points=[62, 4; 90, -40], style(color=10))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.55, 
          height=0.95), 
        Diagram, 
        Documentation(info="<HTML>
<p>
A geometrical approach shows that the outer wheel should turn less when the car turns.
This effect is often reffered to as Ackermann steering but was devided by Lankensperger.
The steering rack is connected to one upright/spindle/kingping at each end via a rod allows the oter wheel to turn less than the inner wheel.
</p>
<p>
Advantages:
Allows the wheels to rotate accorting to the Ackermann steering geometry.
</p>
<p>
Disadvantages:
-
</p>
<p>
Usage:
Most vehicles.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
The right hand side parameters, ending with _2, are by default symmetric to the left hand dito ending with _1.
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} mirrors it around the xz-plane.
  ratioWheelToRack=22: rack-pinion gear ratio [m/rad]
  rX_1:   location of frame X_1, resolved in frame_C
  rXL1_1: location of left rod end, resolved in frame_C
  rRL_1:  location of left rack end, resolved in frame_C
  rX_2:   location of frame X_2, resolved in frame_C,
  rXL1_2: location of right rod end, resolved in frame_C
  rRL_2:  location of right rack end, resolved in frame_C
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[88, 26; 118, 56], rotation=180);
    public 
      Modelica.Mechanics.Rotational.IdealGearR2T wheelToRack(ratio=1/
            ratioWheelToRack)
        annotation (extent=[60, -50; 80, -30], rotation=180);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a
        annotation (extent=[90, -56; 120, -24]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_X_1
        annotation (extent=[-76, -120; -46, -90], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_X_2
        annotation (extent=[-76, 90; -46, 120], rotation=90);
      ModelicaAdditions.MultiBody.Joints.Prismatic Prismatic(n=-(rRL_1_scaled
             - rRL_2_scaled)) annotation (extent=[20, 2; 40, 22], rotation=270);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation(r=(
            rRL_1_scaled + rRL_2_scaled)/2)
        annotation (extent=[50, 32; 70, 52], rotation=180);
      Utilities.Parts.Body leftRack(r=rl)
        annotation (extent=[-50, -10; -30, -30], rotation=90);
      Utilities.Parts.Body rightRack(r=rr)
        annotation (extent=[-50, 30; -30, 50], rotation=90);
    equation 
      connect(wheelToRack.flange_a, flange_a)
        annotation (points=[80, -40; 106, -40], style(color=0));
      connect(Prismatic.frame_a, frameTranslation.frame_b)
        annotation (points=[30, 22.5; 30, 42; 50, 42]);
      connect(frameTranslation.frame_a, frame_C)
        annotation (points=[72, 42; 96, 42]);
      connect(wheelToRack.flange_b, Prismatic.axis)
        annotation (points=[60, -40; 36, -40; 36, 5], style(color=58));
      connect(Prismatic.frame_b, leftRack.frame_a)
        annotation (points=[30, 0; -40, 0; -40, -8]);
      connect(rightRack.frame_a, Prismatic.frame_b)
        annotation (points=[-40, 28; -40, 0; 30, 0]);
      connect(leftRack.frame_b, frame_X_1)
        annotation (points=[-40, -32; -60, -100]);
      connect(rightRack.frame_b, frame_X_2)
        annotation (points=[-40, 50; -62, 96]);
    end RackSteering;
    
    model MasslessCutPushRod "Two bushings connected by a massless rod" 
      
      import SI = Modelica.SIunits;
      parameter SI.Position[3] rA={0,0,0} 
        "Position vector from frame_a to center of bushingA, resolved in frame_a";
      parameter SI.Position[3] rB={0,0,0} 
        "Position vector from center of bushingb to frame_b, resolved in frame_b";
      parameter SI.Position[3] rRod={1,0,0} 
        "Position vector from rod.frame_a to rod.frame_b, resolved in frame_a at the initial configuration";
      /*
  parameter Boolean autoInit=false 
    "|Initialization| = true: Compute nxRod, nyRod such that bushingB rotations are zero."
    ;
  parameter Real nxRod[3]={1,0,0} 
    "|Initialization| if autoInit = false, x-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
  parameter Real nyRod[3]={0,1,0} 
    "|Initialization| if autoInit = false, y-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
*/
      final parameter SI.Position rRod2[3]=rotA.S_rel*rRod;
      final parameter Real nxRod[3]=rotA.S_rel*dataB.nx;
      final parameter Real nyRod[3]=rotA.S_rel*dataB.ny;
      
      annotation (Diagram, Icon(
          Rectangle(extent=[-80, 14; -60, -12], style(color=0, fillColor=10)), 
          Rectangle(extent=[60, 14; 80, -12], style(color=0, fillColor=10)), 
          Rectangle(extent=[-60, 8; 60, -8], style(gradient=2, fillColor=8)), 
          Text(extent=[-100, 80; 100, 20], string="%name"), 
          Text(extent=[-80, -24; -38, -62], string="A"), 
          Text(extent=[-28, 8; 28, -8], string="a=0, z=0")));
      
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-110, -10; -90, 10]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_b
        annotation (extent=[90, 10; 110, -10], rotation=180);
      Utilities.Parts.Body rodShape(lengthDirection=rRod2, r=rRod2)
        annotation (extent=[-10, -40; 10, -20]);
      Chassis.Data.BumpBushingData dataA annotation (extent=[-60, 40; -40, 60]);
      Chassis.Data.BumpBushingData dataB annotation (extent=[40, 40; 60, 60]);
      Utilities.Parts.FrameAxesNoAcceleration rotA(
        nx=dataA.nx, 
        ny=dataA.ny, 
        r=rA) annotation (extent=[-80, -10; -60, 10]);
      Utilities.Forces.SpringDamperLinearBump3D bushingB(
        r_rela0=dataB.r_rela0, 
        r_min=dataB.r_min, 
        r_max=dataB.r_max, 
        bumpFactor=dataB.bumpFactor, 
        phi_rela0=dataB.phi_rela0, 
        C=dataB.C, 
        D=dataB.D, 
        linear=dataB.linear, 
        eps=dataB.eps, 
        animation=false) annotation (extent=[28, -10; 48, 10]);
      Utilities.Parts.FrameAxesNoAcceleration rotB(
        nx=dataB.nx, 
        ny=dataB.ny, 
        r=-rB) annotation (extent=[80, -10; 60, 10]);
      Utilities.Parts.FrameAxesNoAcceleration rod(
        r=rRod2, 
        nx=nxRod, 
        ny=nyRod) annotation (extent=[-10, -10; 10, 10]);
      Utilities.Forces.SpringDamperLinearBump3D_freeMotion bushingA(
        C=dataB.C, 
        D=dataB.D, 
        r_rela0=dataA.r_rela0, 
        r_min=dataA.r_min, 
        r_max=dataA.r_max, 
        bumpFactor=dataA.bumpFactor, 
        phi_rela0=dataA.phi_rela0, 
        linear=dataA.linear, 
        eps=dataA.eps, 
        animation=false) annotation (extent=[-50, -10; -30, 10]);
    initial equation 
      /*
  if autoInit then
    [rod.nx, rod.ny] = transpose(frame_b.S)*frame_a.S[:, 1:2];
  else
    [rod.nx, rod.ny] = [nxRod2, nyRod2];
  end if;
*/
    equation 
      connect(rotA.frame_a, frame_a) annotation (points=[-80, 0; -92, 0]);
      connect(frame_b, rotB.frame_a) annotation (points=[92, 0; 79, 0]);
      connect(rod.frame_a, rodShape.frame_a)
        annotation (points=[-10, 0; -16, 0; -16, -30; -10.5, -30]);
      connect(rod.frame_b, bushingB.frame_a) annotation (points=[10, 0; 28, 0]);
      connect(rotB.frame_b, bushingB.frame_b)
        annotation (points=[60, 0; 48, 0]);
      connect(rotA.frame_b, bushingA.frame_a)
        annotation (points=[-60, 0; -50.5, 0]);
      connect(bushingA.frame_b, rod.frame_a)
        annotation (points=[-30, 0; -12, 0]);
    end MasslessCutPushRod;
    
    model MultiLink4 
      "MultiLink4 - Linkage with four massless links, one more than neccessary"
       
      
      extends Chassis.Interfaces.Interface;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      
      parameter SI.Position[3] rCL1={-0.5550,-0.3690,-0.0320};
      parameter SI.Position[3] rCL2={-0.4240,-0.1900,0.1640};
      parameter SI.Position[3] rCL3={-0.3320,-0.1300,0.0570};
      parameter SI.Position[3] rCU={0,0,0};
      parameter SI.Position[3] rUL1={-0.5250,0.0620,-0.0670};
      parameter SI.Position[3] rUL2={-0.4330,0.0680,0.1500};
      parameter SI.Position[3] rUL3={-0.3350,0.0116,0.0400};
      parameter SI.Position[3] rUW={-0.4650,0.1330,0.0200};
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|";
      parameter SI.Mass mU=1 "|Mass and Inertia|";
      parameter SI.Inertia i11U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i22U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i33U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i21U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i31U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i32U=.01 "|Mass and Inertia|";
      
      /*Scaled coordinates*/
    protected 
      parameter SI.Position[3] rCL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL1);
      parameter SI.Position[3] rCL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL2);
      parameter SI.Position[3] rCL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL3);
      parameter SI.Position[3] rCU_scaled=Utilities.Functions.EWM(scaleFactor, 
          rCU);
      parameter SI.Position[3] rUL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL1);
      parameter SI.Position[3] rUL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL2);
      parameter SI.Position[3] rUL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL3);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Diagram, 
        Icon(
          Polygon(points=[10, 80; 24, 8; 42, -28; 44, -42; 40, -52; 22, -52; 22, 
                 -20; 14, 6; 0, 80; 10, 80], style(color=9, fillColor=8)), 
          Rectangle(extent=[-62, -44; 16, -48], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-46, -12; 16, -16], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-46, 22; 4, 18], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[4, 24; 14, 16], style(color=0, fillColor=10)), 
          Rectangle(extent=[14, -10; 24, -18], style(color=0, fillColor=10)), 
          Rectangle(extent=[14, -42; 24, -50], style(color=0, fillColor=10)), 
          Rectangle(extent=[-68, -42; -58, -50], style(color=0, fillColor=10)), 
             
          Rectangle(extent=[-54, -10; -44, -18], style(color=0, fillColor=10)), 
             
          Rectangle(extent=[-54, 24; -44, 16], style(color=0, fillColor=10)), 
          Line(points=[44, -36; 60, 0; 88, 0], style(color=9)), 
          Line(points=[-66, -42; -80, 0; -90, 0], style(color=9, fillColor=9)), 
             
          Line(points=[-54, -12; -80, 0; -90, 0], style(color=9, fillColor=9)), 
             
          Line(points=[-54, 20; -80, 0; -88, 0], style(color=9, fillColor=9)), 
          Rectangle(extent=[-2, 84; 14, 76], style(color=0, fillColor=10)), 
          Line(points=[-2, 80; -54, 80; -80, 0], style(color=10)), 
          Line(points=[0, -72; 0, -60; 14, -48], style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The essence of a multilink is that there are more links than necessary to constrain a motion. This is done because the bushings used to reduce vibrations and nocie to the chassis also give a less controlled motion.</p>
<p>
Usage:
Premium cars and sportier cars that do not use double wishbones.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUL1:   location of lower outer joint, resolved in frame_C
  rCL1:   location of lower inner joint, resolved in frame_C
  rUL2:   location of upper outer joint, resolved in frame_C
  rCL2:   location of upper inner front joint, resolved in frame_C
  rUL3:   location of outer steering joint, resolved in frame_C
  rCL3:   location of inner steering joint, resolved in frame_C
  rCU:    location of inner (front) trailing arm joint, resolved in frame_C
  rUW:    location of wheel centre, resolved in frame_C
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslationUW(r=
            rUW_scaled - rCU_scaled) annotation (extent=[42, -10; 62, 10]);
      Utilities.Parts.Shape shape1(
        r0=rUW_scaled - rCU_scaled, 
        lengthDirection={-1,-0.05,0}, 
        widthDirection={0,0,1}, 
        length=0.14, 
        width=0.07, 
        height=0.03, 
        material={0.1,0.2,0.2,0.5}) annotation (extent=[0, -100; -20, -80]);
      
      /*
  Chassis.Components.MasslessCutPushRod Link2(
    rB=rCU_scaled - rUL2_scaled, 
    rA=rCL2_scaled, 
    rRod=rUL2_scaled - rCL2_scaled, 
    dataA=data_CL2, 
    dataB=data_UL2) annotation (extent=[-40, -50; 0, -10]);
  Chassis.Components.MasslessCutPushRod Link1(
    rB=rCU_scaled - rUL1_scaled, 
    rA=rCL1_scaled, 
    rRod=rUL1_scaled - rCL1_scaled, 
    dataA=data_CL1, 
    dataB=data_UL1) annotation (extent=[-40, -82; 0, -42]);
  Chassis.Components.MasslessCutPushRod Link3(
    rB=rCU_scaled - rUL3_scaled, 
    rA=rCL3_scaled, 
    rRod=rUL3_scaled - rCL3_scaled, 
    dataA=data_CL3, 
    dataB=data_UL3) annotation (extent=[-40, -20; 0, 20]);
*/
      
      MasslessCutPushRod Link2(
        rB=rCU_scaled - rUL2_scaled, 
        rA=rCL2_scaled, 
        rRod=rUL2_scaled - rCL2_scaled, 
        dataA=data_CL2, 
        dataB=data_UL2) annotation (extent=[-40, -50; 0, -10]);
      MasslessCutPushRod Link1(
        rB=rCU_scaled - rUL1_scaled, 
        rA=rCL1_scaled, 
        rRod=rUL1_scaled - rCL1_scaled, 
        dataA=data_CL1, 
        dataB=data_UL1) annotation (extent=[-40, -82; 0, -42]);
      MasslessCutPushRod Link3(
        rB=rCU_scaled - rUL3_scaled, 
        rA=rCL3_scaled, 
        rRod=rUL3_scaled - rCL3_scaled, 
        dataA=data_CL3, 
        dataB=data_UL3) annotation (extent=[-40, -20; 0, 20]);
      
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslationCU(r=
            rCU_scaled) annotation (extent=[-42, 68; -22, 88]);
      parameter Chassis.Data.BumpBushingData data_CL3
        annotation (extent=[50, 70; 70, 90]);
      Chassis.Data.BumpBushingData data_UL3 annotation (extent=[70, 70; 90, 90]);
      Chassis.Data.BumpBushingData data_CL2 annotation (extent=[50, 50; 70, 70]);
      Chassis.Data.BumpBushingData data_UL2 annotation (extent=[70, 50; 90, 70]);
      Chassis.Data.BumpBushingData data_CL1 annotation (extent=[50, 30; 70, 50]);
      parameter Chassis.Data.BumpBushingData data_UL1
        annotation (extent=[70, 30; 90, 50]);
      ModelicaAdditions.MultiBody.Parts.Body U(
        rCM=rCMU, 
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U) annotation (extent=[40, -40; 60, -20]);
      parameter Chassis.Data.BumpBushingData data_CU
        annotation (extent=[60, 80; 80, 100]);
      Chassis.Components.Bushing Bushing1(data=data_CU)
        annotation (extent=[12, 40; 32, 60], rotation=270);
      Utilities.Parts.Shape shape(
        widthDirection={0,0,1}, 
        width=0.07, 
        height=0.03, 
        material={0.1,0.2,0.2,0.5}, 
        lengthDirection=rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0}, 
        length=sqrt((rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0})*(
            rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0})), 
        r0=-{0,rCU_scaled[2]*0.25,0}) annotation (extent=[40, -100; 60, -80]);
    equation 
      connect(Link2.frame_a, frame_C) annotation (points=[-40, -30; -69, -30; -69, 
             -4; -98, -4], style(thickness=4));
      connect(frameTranslationUW.frame_b, frame_U)
        annotation (points=[64, 0; 90, 0], style(thickness=4));
      connect(Link3.frame_a, frame_C) annotation (points=[-40, 0; -69, 0; -69, 
            -2; -98, -2], style(thickness=4));
      connect(Link1.frame_a, frame_C) annotation (points=[-40, -62; -69, -62; -69, 
             -4; -98, -4], style(thickness=4));
      connect(frameTranslationCU.frame_a, frame_C)
        annotation (points=[-44, 78; -47, 78; -47, 2; -96, 2]);
      connect(Bushing1.frame_a, frameTranslationCU.frame_b)
        annotation (points=[22.1, 60.5; -22, 78]);
      connect(Bushing1.frame_b, Link3.frame_b)
        annotation (points=[22, 40; 0, 0]);
      connect(Link2.frame_b, Bushing1.frame_b)
        annotation (points=[0, -30; 22, 40]);
      connect(Link1.frame_b, Bushing1.frame_b)
        annotation (points=[-2, -62; 22, 40]);
      connect(shape1.frame_a, Bushing1.frame_b)
        annotation (points=[0, -90; 22, 40]);
      connect(U.frame_a, Bushing1.frame_b)
        annotation (points=[38, -30; 22, 40]);
      connect(frameTranslationUW.frame_a, Bushing1.frame_b)
        annotation (points=[42, 0; 22, 40]);
      connect(shape.frame_a, Bushing1.frame_b) annotation (points=[40, -90; 22, 
             40], style(fillColor=76, fillPattern=1));
    end MultiLink4;
    
    annotation (uses(DLR_LightWeightRobot2(version="0.7")));
    
    model MacPherson4 "MacPherson strut linkage" 
      import SI = Modelica.SIunits;
      
      extends Chassis.Interfaces.Interface;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      parameter SI.Position[3] rCL1={0.1070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to front link mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rCL2={-0.3070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to rear link mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rCS={-0.0295,0.850,0.5670} 
        "|Geometry| Vector from origin of frame_C to strut mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUS={-0.0070,0.8733,0.1380} 
        "|Geometry| Vector from origin of frame_C to strut mount in uppright resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUL1L2={-0.0070,0.8733,-0.0380} 
        "|Geometry| Vector from origin of frame_C to low spindleUtilities.Joints.Joint resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUW={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to wheel centre resolved in frame_C (at initial time)";
      parameter SI.Position[3] rRL3={-0.12,0.85,0.08} 
        "|Geometry| Vector from origin of frame_C to origin of frame_S resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUL3={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to lower ball of steering rod resolved in frame_C (at initial time)";
      parameter SI.Length q0S=0.05 
        "|Geometry| Unstretched additional spring length of the strut, compared to the construction geometry";
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|Wheel carrier|";
      parameter SI.Mass mU=1 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i11U=0.1 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i22U=0.1 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i33U=0.1 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i21U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i31U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i32U=0 "|Mass and Inertia|Wheel carrier|";
      
      parameter SI.Position[3] rCML1L2={0,0,0} "|Mass and Inertia|Wishbone|";
      parameter SI.Mass mL1L2=1 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i11L1L2=0.1 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i22L1L2=0.1 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i33L1L2=0.1 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i21L1L2=0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i31L1L2=0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i32L1L2=0 "|Mass and Inertia|Wishbone|";
      
      /*Scaled coordinates*/
    protected 
      parameter SI.Position[3] rCL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL1);
      parameter SI.Position[3] rCL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL2);
      parameter SI.Position[3] rCS_scaled=Utilities.Functions.EWM(scaleFactor, 
          rCS);
      parameter SI.Position[3] rUS_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUS);
      parameter SI.Position[3] rUL1L2_scaled=Utilities.Functions.EWM(
          scaleFactor, rUL1L2);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      parameter SI.Position[3] rRL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL3);
      parameter SI.Position[3] rUL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL3);
      parameter SI.Length q0Strut=sqrt((rCS_scaled - rUS_scaled)*(rCS_scaled - 
          rUS_scaled)) + q0S;
      parameter SI.Length L_frontBar=sqrt((rCL1_scaled - rUL1L2_scaled)*(
          rCL1_scaled - rUL1L2_scaled)) "length of frontBar";
      parameter SI.Length L_rearBar=sqrt((rCL2_scaled - rUL1L2_scaled)*(
          rCL2_scaled - rUL1L2_scaled)) "length of rearBar";
      parameter SI.Length L_springRod=sqrt((rUS_scaled - rUL1L2_scaled)*(
          rUS_scaled - rUL1L2_scaled));
      parameter SI.Length L_outerRod=sqrt((rUW_scaled - rUL1L2_scaled)*(
          rUW_scaled - rUL1L2_scaled));
      parameter SI.Position rS[3]=rCS_scaled - rUL1L2_scaled;
      parameter SI.Position rU[3]=rUW_scaled - rUL1L2_scaled 
        "Position vector from frame_L1L2 to frame_U, resolved in frame_C";
      parameter Real n_a[3]=cross(rS, rU) 
        "First rotation axis of universalUtilities.Joints.Joint in springJoint, resolved in frame_C";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Polygon(points=[52, 96; 64, 80; 62, 86; 70, 62; -100, 106; 52, 96], 
              style(pattern=0)), 
          Line(points=[-44, -44; -18, -24], style(color=0, thickness=4)), 
          Line(points=[-58, -46; 30, -30; -58, -20], style(color=0, thickness=4)), 
             
          Polygon(points=[42, -12; 60, -12; 64, 4; 90, 4; 90, -2; 68, -2; 64, -18; 
                 42, -18; 42, -12], style(color=8, fillColor=8)), 
          Polygon(points=[62, 18; 56, 18; 48, -14; 56, -14; 62, 18], style(
              pattern=0, 
              gradient=2, 
              fillColor=8)), 
          Ellipse(extent=[50, 14; 70, 34], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[52, 16; 68, 32], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[42, 38; 54, 14], style(color=7, fillColor=7)), 
          Ellipse(extent=[-50, 50; -30, 70], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-48, 52; -32, 68], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-36, 72; -22, 46], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-46, 66; -34, 54], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[54, 30; 66, 18], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-34, 60; 58, 24], style(color=0, thickness=4)), 
          Ellipse(extent=[28, -40; 48, -20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, -38; 46, -22], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, -20; 50, -26], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[34, -8; 42, -26], style(color=8, fillColor=8)), 
          Polygon(points=[38, 2; 50, -2; 50, 2; 38, 6; 38, 2], style(color=10, 
                fillColor=10)), 
          Polygon(points=[26, 6; 38, 2; 38, 6; 26, 10; 26, 6], style(color=10, 
                fillColor=10)), 
          Polygon(points=[38, 18; 50, 14; 50, 18; 38, 22; 38, 18], style(color=
                  10, fillColor=10)), 
          Polygon(points=[26, 22; 38, 18; 38, 22; 26, 26; 26, 22], style(color=
                  10, fillColor=10)), 
          Polygon(points=[38, 34; 50, 30; 50, 34; 38, 38; 38, 34], style(color=
                  10, fillColor=10)), 
          Polygon(points=[26, 38; 38, 34; 38, 38; 26, 42; 26, 38], style(color=
                  10, fillColor=10)), 
          Polygon(points=[38, 50; 50, 46; 50, 50; 38, 54; 38, 50], style(color=
                  10, fillColor=10)), 
          Rectangle(extent=[32, 56; 44, -8], style(gradient=1, fillColor=8)), 
          Rectangle(extent=[30, 52; 46, 14], style(gradient=1, fillColor=8)), 
          Polygon(points=[26, 38; 38, 42; 38, 46; 26, 42; 26, 38], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, 42; 50, 46; 50, 50; 38, 46; 38, 42], style(color=
                  8, fillColor=8)), 
          Polygon(points=[26, 22; 38, 26; 38, 30; 26, 26; 26, 22], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, 26; 50, 30; 50, 34; 38, 30; 38, 26], style(color=
                  8, fillColor=8)), 
          Polygon(points=[26, 6; 38, 10; 38, 14; 26, 10; 26, 6], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[38, 10; 50, 14; 50, 18; 38, 14; 38, 10], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, -6; 50, -2; 50, 2; 38, -2; 38, -6], style(color=8, 
                 fillColor=8)), 
          Ellipse(extent=[28, 56; 48, 76], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, 58; 46, 74], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, 62; 50, 54], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[34, 66; 42, 54], style(gradient=1, fillColor=8)), 
          Ellipse(extent=[32, 72; 44, 60], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[32, -24; 44, -36], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[-58, -13; -46, -26], style(fillColor=10, 
                fillPattern=1)), 
          Rectangle(extent=[-70, -38; -58, -51], style(fillColor=10, 
                fillPattern=1))), 
        Diagram(
          Line(points=[-90, 0; -78, -60], style(color=1, arrow=1)), 
          Text(
            extent=[-98, -46; -78, -53], 
            style(color=1, arrow=1), 
            string="rCL1"), 
          Line(points=[-90, 0; 0, -40], style(color=1, arrow=1)), 
          Text(
            extent=[-16, -27; -1, -34], 
            style(color=1, arrow=1), 
            string="rUL1L2"), 
          Text(
            extent=[-99, 36; -72, 30], 
            style(color=1, arrow=1), 
            string="rCS"), 
          Line(points=[-90, 0; -77, 40], style(color=1, arrow=1)), 
          Text(
            extent=[-12, 55; 12, 48], 
            style(color=1, arrow=1), 
            string="rRL3"), 
          Line(points=[-90, 0; 65, 80], style(color=1, arrow=1)), 
          Line(points=[-90, 0; 61, 54], style(color=1, arrow=1)), 
          Text(
            extent=[21, 51; 44, 44], 
            style(color=1, arrow=1), 
            string="rUL3"), 
          Text(
            extent=[64, 22; 91, 16], 
            style(color=1, arrow=1), 
            string="rUW"), 
          Line(points=[-90, 0; 90, 15], style(color=1, arrow=1)), 
          Text(
            extent=[15, -4; 32, -10], 
            style(color=1, arrow=1), 
            string="rUS"), 
          Line(points=[-90, 0; 40, -4], style(color=1, arrow=1))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUL1L2: location of outer ballUtilities.Joints.Joint, resolved in frame_C
  rCL1:   location of inner frontUtilities.Joints.Joint, resolved in frame_C
  rCL2:   location of inner rearUtilities.Joints.Joint, resolved in frame_C
  rCS:    location of strut mount in chassis, resolved in frame_C
  rUS:    location of strut mount in upright, resolved in frame_C
  rUW:    location of frame_W, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    protected 
      parameter Chassis.Data.BumpBushingData data_CL1
        annotation (extent=[-66, 71; -44, 93]);
      parameter Chassis.Data.BumpBushingData data_CL2
        annotation (extent=[-36, 71; -14, 93]);
      Chassis.Components.PairBushing pairBushing(
        rB1={0,0,0}, 
        data2=data_CL2, 
        data1=data_CL1, 
        rB2=rCL2_scaled - rCL1_scaled) annotation (extent=[-71, -70; -51, -50]);
    public 
      ModelicaAdditions.MultiBody.Parts.Body body(
        m=mL1L2, 
        rCM=rCML1L2, 
        I11=i11L1L2, 
        I22=i22L1L2, 
        I33=i33L1L2, 
        I21=i21L1L2, 
        I31=i31L1L2, 
        I32=i32L1L2) annotation (extent=[-75, -88; -95, -68]);
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_L12
        annotation (extent=[-16, -118; 14, -88], rotation=270);
      Utilities.Forces.Utilities.ForceTable1D data_S
        annotation (extent=[91, 69; 121, 99]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation upper(r=rCS_scaled)
        annotation (extent=[-67, 10; -87, 30], rotation=90);
      Utilities.Joints.JointUPS MacPherson(n_a=n_a)
        annotation (extent=[19, -20; -20, 20], rotation=90);
      Utilities.Parts.Body springRod(r=rUS_scaled - rUL1L2_scaled, mass=false)
        annotation (extent=[30, -30; 50, -10], rotation=90);
      Utilities.Forces.SpringDamperTableLine3D strut(s_rel0=q0Strut, data=
            data_S) annotation (extent=[29, 0; 49, 20], rotation=90);
      Utilities.Parts.Body outerRod(r=rUW_scaled - rUL1L2_scaled, mass=false)
        annotation (extent=[65, -10; 85, 10], rotation=0);
      ModelicaAdditions.MultiBody.Parts.Body U(
        rCM=rCMU, 
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U) annotation (extent=[66, -60; 86, -40]);
      Utilities.Parts.Body steeringRod(r=rUL3_scaled - rRL3_scaled, shape=
            "cylinder") annotation (extent=[10, 80; 30, 60], rotation=180);
      Utilities.Joints.JointRSU steeringJoint(
        n_b={1,0,0}, 
        n_a=rCS_scaled - rUL1L2_scaled, 
        r_a=rUL3_scaled - rUL1L2_scaled)
        annotation (extent=[51, 40; 81, 70], rotation=90);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation lower(r=rCL1_scaled)
        annotation (extent=[-87, -30; -67, -10], rotation=270);
      Utilities.Parts.Body frontBar(r=rUL1L2_scaled - rCL1_scaled)
        annotation (extent=[-40, -70; -20, -50]);
      Utilities.Parts.Body rearBar(r=rCL2_scaled - rUL1L2_scaled, 
          widthDirection=cross(rCL2_scaled - rUL1L2_scaled, {0,0,1}))
        annotation (extent=[-40, -30; -20, -50], rotation=180);
    protected 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_ta1
        annotation (extent=[59, -10; 61, -8]);
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_S
        annotation (extent=[-70, 50; -100, 80], rotation=180);
      Utilities.Parts.Body steeringLever(r=rUL3_scaled - rUL1L2_scaled)
        annotation (extent=[95, 67; 115, 47], rotation=270);
    equation 
      connect(lower.frame_a, frame_C)
        annotation (points=[-77, -9; -77, 0; -115, 0]);
      connect(upper.frame_a, frame_C)
        annotation (points=[-77, 10; -77, 0; -115, 0]);
      connect(MacPherson.frame_b, upper.frame_b)
        annotation (points=[1, 22.05; 0, 40; -77, 40; -77, 31]);
      connect(steeringJoint.frame_b, frame_S)
        annotation (points=[65, 71; 65, 80; -85, 80]);
      connect(outerRod.frame_b, frame_U) annotation (points=[85.5, 0; 90, -1]);
      connect(MacPherson.frame_pb, strut.frame_b)
        annotation (points=[23, 12.6; 24, 4; 24, 30; 40, 30; 40, 20]);
      connect(springRod.frame_b, strut.frame_a)
        annotation (points=[40, -9.5; 40, 0]);
      connect(frame_ta1, outerRod.frame_a) annotation (points=[60, -9; 64, 0]);
      connect(U.frame_a, frame_ta1) annotation (points=[66, -51; 60, -8]);
      connect(frame_ta1, steeringJoint.frame_ta)
        annotation (points=[60, -9; 48, 43]);
      connect(MacPherson.frame_pa, steeringJoint.frame_a) annotation (points=[
            23, -11.2; 26, -14; 26, -50; 54, -50; 54, 30; 65, 30; 65, 39]);
      connect(springRod.frame_a, frame_ta1)
        annotation (points=[40, -30.5; 60, -9]);
      connect(rearBar.frame_a, frontBar.frame_b)
        annotation (points=[-20, -40; -20, -61]);
      connect(frontBar.frame_b, MacPherson.frame_a)
        annotation (points=[-20, -59; 0, -60; -0.5, -19.975]);
      connect(steeringJoint.frame_c, steeringRod.frame_a)
        annotation (points=[49, 68; 30, 70]);
      connect(pairBushing.frame_b, frontBar.frame_a)
        annotation (points=[-50, -60; -41, -60]);
      connect(pairBushing.frame_a, lower.frame_b)
        annotation (points=[-71, -59; -76, -59; -77, -31]);
      connect(pairBushing.frame_b, body.frame_a)
        annotation (points=[-50, -60; -50, -80; -75, -79]);
      connect(frame_L12, frontBar.frame_b)
        annotation (points=[-1, -96; -20, -60]);
      connect(steeringLever.frame_a, steeringJoint.frame_ta)
        annotation (points=[104, 46; 48.75, 43.15]);
    end MacPherson4;
    
    model RackSteering4 "RackSteering - standard steering system " 
      import SI = Modelica.SIunits;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      parameter SI.Position[3] rXL_1={-0.12,0.85,0.08} 
        "Vector to . resolved in innerFrame";
      //{-0.1280,0.6430,0.0744} 
      parameter SI.Position[3] rRL_1={-0.16,0.54,0.085};
      //{-0.1600,0.3070,0.0950};
      parameter SI.Position[3] rX_1={0.0,0.9,0};
      //{0.0059,0.6905,-0.0710} + {-0.0060,0.0365,0.1110};
      parameter SI.Position[3] rXL_2=Utilities.Functions.EWM({1,-1,1}, rXL_1);
      parameter SI.Position[3] rRL_2=Utilities.Functions.EWM({1,-1,1}, rRL_1);
      parameter SI.Position[3] rX_2=Utilities.Functions.EWM({1,-1,1}, rX_1);
      parameter SI.Inertia iSW=0.2 "Steering inertia";
      parameter Real ratioWheelToRack=22 
        "rotatational to translational ratio [m/rad]";
      
    protected 
      parameter SI.Position[3] rXL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_1);
      parameter SI.Position[3] rRL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL_1);
      parameter SI.Position[3] rX_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_1);
      parameter SI.Position[3] rXL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_2);
      parameter SI.Position[3] rRL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL_2);
      parameter SI.Position[3] rX_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_2);
      parameter SI.Position[3] rr=rRL_2_scaled - (rRL_1_scaled + rRL_2_scaled)/
          2;
      parameter SI.Position[3] rl=rRL_1_scaled - (rRL_1_scaled + rRL_2_scaled)/
          2;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Text(
            extent=[-76, -44; -46, -84], 
            string="L", 
            style(color=73, fillColor=58)), 
          Text(
            extent=[-76, 84; -44, 44], 
            string="R", 
            style(color=41, fillColor=41)), 
          Rectangle(extent=[-76, 90; -4, 80], style(color=10, fillColor=8)), 
          Rectangle(extent=[-76, -80; -4, -90], style(color=10, fillColor=8)), 
          Rectangle(extent=[-2, 6; 62, -4], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-16, 80; -4, -80], style(
              color=9, 
              fillColor=8, 
              fillPattern=7)), 
          Rectangle(extent=[-20, 8; -2, -6], style(
              color=9, 
              fillColor=8, 
              fillPattern=8)), 
          Rectangle(extent=[52, 40; 62, -40], style(gradient=2, fillColor=8)), 
          Line(points=[88, 40; 62, 4], style(color=10)), 
          Line(points=[62, 4; 90, -40], style(color=10))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.55, 
          height=0.95), 
        Diagram, 
        Documentation(info="<HTML>
<p>
A geometrical approach shows that the outer wheel should turn less when the car turns.
This effect is often reffered to as Ackermann steering but was devided by Lankensperger.
The steering rack is connected to one upright/spindle/kingping at each end via a rod allows the oter wheel to turn less than the inner wheel.
</p>
<p>
Advantages:
Allows the wheels to rotate accorting to the Ackermann steering geometry.
</p>
<p>
Disadvantages:
-
</p>
<p>
Usage:
Most vehicles.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
The right hand side parameters, ending with _2, are by default symmetric to the left hand dito ending with _1.
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} mirrors it around the xz-plane.
  ratioWheelToRack=22: rack-pinion gear ratio [m/rad]
  rX_1:   location of frame X_1, resolved in frame_C
  rXL1_1: location of left rod end, resolved in frame_C
  rRL_1:  location of left rack end, resolved in frame_C
  rX_2:   location of frame X_2, resolved in frame_C,
  rXL1_2: location of right rod end, resolved in frame_C
  rRL_2:  location of right rack end, resolved in frame_C
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[88, 26; 118, 56], rotation=180);
    public 
      Modelica.Mechanics.Rotational.IdealGearR2T wheelToRack(ratio=1/
            ratioWheelToRack)
        annotation (extent=[-3, -30; 17, -50], rotation=180);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a
        annotation (extent=[90, -56; 120, -24]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_X_1
        annotation (extent=[-76, -120; -46, -90], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_X_2
        annotation (extent=[-76, 90; -46, 120], rotation=90);
      ModelicaAdditions.MultiBody.Joints.Prismatic Prismatic(
        n=-(rRL_1_scaled - rRL_2_scaled), 
        q(stateSelect=StateSelect.always), 
        qd(stateSelect=StateSelect.always))
        annotation (extent=[-30, 0; -10, 20], rotation=270);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation(r=(
            rRL_1_scaled + rRL_2_scaled)/2)
        annotation (extent=[40, 30; 60, 50], rotation=180);
      Utilities.Parts.Body rightRack(r=rr)
        annotation (extent=[-60, 20; -40, 40], rotation=90);
      Utilities.Parts.Body leftRack(r=rl)
        annotation (extent=[-60, -20; -40, -40], rotation=90);
      Modelica.Mechanics.Rotational.Inertia inertia(J=iSW)
        annotation (extent=[62, -50; 82, -30], rotation=180);
      Utilities.Forces.SpringDamperTableRot1D springDamper(data=data_R)
        annotation (extent=[31, -51; 51, -31], rotation=180);
      parameter Utilities.Forces.Utilities.ForceTable1D data_R
        annotation (extent=[23, -2; 43, 18]);
    equation 
      connect(Prismatic.frame_a, frameTranslation.frame_b)
        annotation (points=[-20, 20.5; -20, 40; 39.5, 40]);
      connect(frameTranslation.frame_a, frame_C)
        annotation (points=[60.5, 40; 88, 40]);
      connect(Prismatic.frame_b, leftRack.frame_a)
        annotation (points=[-20, 0; -20, -12; -50, -12; -50, -19.5]);
      connect(rightRack.frame_a, leftRack.frame_a)
        annotation (points=[-50, 19.5; -50, -19.5]);
      connect(rightRack.frame_b, frame_X_2)
        annotation (points=[-50, 40.5; -50, 70; -60, 70; -60, 96]);
      connect(leftRack.frame_b, frame_X_1)
        annotation (points=[-50, -40.5; -50, -70; -60, -70; -60, -94]);
      connect(inertia.flange_a, flange_a)
        annotation (points=[82, -40; 92, -40], style(color=0));
      connect(wheelToRack.flange_b, Prismatic.axis) annotation (points=[-4, -40; 
             -12, -40; -12, -12; 0, -12; 0, 3; -13, 3], style(color=58));
      connect(springDamper.flange_b, wheelToRack.flange_a)
        annotation (points=[31, -42; 18, -40], style(color=0));
      connect(springDamper.flange_a, inertia.flange_b)
        annotation (points=[51, -41; 62, -40], style(color=0));
    end RackSteering4;
    
    model MultiLink42 
      "MultiLink4 - Linkage with four links, one more than neccessary" 
      
      extends Chassis.Interfaces.Interface;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      
      parameter SI.Position[3] rCL1={-0.5550,-0.3690,-0.0320};
      parameter SI.Position[3] rCL2={-0.4240,-0.1900,0.1640};
      parameter SI.Position[3] rCL3={-0.3320,-0.1300,0.0570};
      parameter SI.Position[3] rCU={0,0,0};
      parameter SI.Position[3] rUL1={-0.5250,0.0620,-0.0670};
      parameter SI.Position[3] rUL2={-0.4330,0.0680,0.1500};
      parameter SI.Position[3] rUL3={-0.3350,0.0116,0.0400};
      parameter SI.Position[3] rUW={-0.4650,0.1330,0.0200};
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|";
      parameter SI.Mass mU=1 "|Mass and Inertia|";
      parameter SI.Inertia i11U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i22U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i33U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i21U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i31U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i32U=.01 "|Mass and Inertia|";
      
      /*Scaled coordinates*/
    protected 
      parameter SI.Position[3] rCL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL1);
      parameter SI.Position[3] rCL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL2);
      parameter SI.Position[3] rCL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL3);
      parameter SI.Position[3] rCU_scaled=Utilities.Functions.EWM(scaleFactor, 
          rCU);
      parameter SI.Position[3] rUL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL1);
      parameter SI.Position[3] rUL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL2);
      parameter SI.Position[3] rUL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL3);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Diagram, 
        Icon(
          Polygon(points=[10, 80; 24, 8; 42, -28; 44, -42; 40, -52; 22, -52; 22, 
                 -20; 14, 6; 0, 80; 10, 80], style(color=9, fillColor=8)), 
          Rectangle(extent=[-62, -44; 16, -48], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-46, -12; 16, -16], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-46, 22; 4, 18], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[4, 24; 14, 16], style(color=0, fillColor=10)), 
          Rectangle(extent=[14, -10; 24, -18], style(color=0, fillColor=10)), 
          Rectangle(extent=[14, -42; 24, -50], style(color=0, fillColor=10)), 
          Rectangle(extent=[-68, -42; -58, -50], style(color=0, fillColor=10)), 
             
          Rectangle(extent=[-54, -10; -44, -18], style(color=0, fillColor=10)), 
             
          Rectangle(extent=[-54, 24; -44, 16], style(color=0, fillColor=10)), 
          Line(points=[44, -36; 60, 0; 88, 0], style(color=9)), 
          Line(points=[-66, -42; -80, 0; -90, 0], style(color=9, fillColor=9)), 
             
          Line(points=[-54, -12; -80, 0; -90, 0], style(color=9, fillColor=9)), 
             
          Line(points=[-54, 20; -80, 0; -88, 0], style(color=9, fillColor=9)), 
          Rectangle(extent=[-2, 84; 14, 76], style(color=0, fillColor=10)), 
          Line(points=[-2, 80; -54, 80; -80, 0], style(color=10)), 
          Line(points=[0, -72; 0, -60; 14, -48], style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The essence of a multilink is that there are more links than necessary to constrain a motion. This is done because the bushings used to reduce vibrations and nocie to the chassis also give a less controlled motion.</p>
<p>
Usage:
Premium cars and sportier cars that do not use double wishbones.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUL1:   location of lower outer joint, resolved in frame_C
  rCL1:   location of lower inner joint, resolved in frame_C
  rUL2:   location of upper outer joint, resolved in frame_C
  rCL2:   location of upper inner front joint, resolved in frame_C
  rUL3:   location of outer steering joint, resolved in frame_C
  rCL3:   location of inner steering joint, resolved in frame_C
  rCU:    location of inner (front) trailing arm joint, resolved in frame_C
  rUW:    location of wheel centre, resolved in frame_C
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslationUW(r=
            rUW_scaled - rCU_scaled) annotation (extent=[42, -10; 62, 10]);
      Utilities.Parts.Shape shape(
        widthDirection={0,0,1}, 
        width=0.07, 
        height=0.03, 
        material={0.1,0.2,0.2,0.5}, 
        lengthDirection=rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0}, 
        length=sqrt((rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0})*(
            rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0})), 
        r0=-{0,rCU_scaled[2]*0.25,0}) annotation (extent=[40, -100; 60, -80]);
      Utilities.Parts.Shape shape1(
        r0=rUW_scaled - rCU_scaled, 
        lengthDirection={-1,-0.05,0}, 
        widthDirection={0,0,1}, 
        length=0.14, 
        width=0.07, 
        height=0.03, 
        material={0.1,0.2,0.2,0.5}) annotation (extent=[0, -100; -20, -80]);
      
      /*
  Chassis.Components.MasslessCutPushRod Link2(
    rB=rCU_scaled - rUL2_scaled, 
    rA=rCL2_scaled, 
    rRod=rUL2_scaled - rCL2_scaled, 
    dataA=data_CL2, 
    dataB=data_UL2) annotation (extent=[-40, -50; 0, -10]);
  Chassis.Components.MasslessCutPushRod Link1(
    rB=rCU_scaled - rUL1_scaled, 
    rA=rCL1_scaled, 
    rRod=rUL1_scaled - rCL1_scaled, 
    dataA=data_CL1, 
    dataB=data_UL1) annotation (extent=[-40, -82; 0, -42]);
  Chassis.Components.MasslessCutPushRod Link3(
    rB=rCU_scaled - rUL3_scaled, 
    rA=rCL3_scaled, 
    rRod=rUL3_scaled - rCL3_scaled, 
    dataA=data_CL3, 
    dataB=data_UL3) annotation (extent=[-40, -20; 0, 20]);
*/
      
      CutPushRod Link2(
        rB=rCU_scaled - rUL2_scaled, 
        rA=rCL2_scaled, 
        rRod=rUL2_scaled - rCL2_scaled, 
        dataA=data_CL2, 
        dataB=data_UL2) annotation (extent=[-40, -50; 0, -10]);
      CutPushRod Link1(
        rB=rCU_scaled - rUL1_scaled, 
        rA=rCL1_scaled, 
        rRod=rUL1_scaled - rCL1_scaled, 
        dataA=data_CL1, 
        dataB=data_UL1) annotation (extent=[-40, -82; 0, -42]);
      CutPushRod Link3(
        rB=rCU_scaled - rUL3_scaled, 
        rA=rCL3_scaled, 
        rRod=rUL3_scaled - rCL3_scaled, 
        dataA=data_CL3, 
        dataB=data_UL3) annotation (extent=[-40, -20; 0, 20]);
      
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslationCU(r=
            rCU_scaled) annotation (extent=[-42, 68; -22, 88]);
      parameter Chassis.Data.BumpBushingData data_CL3
        annotation (extent=[50, 70; 70, 90]);
      Chassis.Data.BumpBushingData data_UL3 annotation (extent=[70, 70; 90, 90]);
      Chassis.Data.BumpBushingData data_CL2 annotation (extent=[50, 50; 70, 70]);
      Chassis.Data.BumpBushingData data_UL2 annotation (extent=[70, 50; 90, 70]);
      Chassis.Data.BumpBushingData data_CL1 annotation (extent=[50, 30; 70, 50]);
      parameter Chassis.Data.BumpBushingData data_UL1
        annotation (extent=[70, 30; 90, 50]);
      ModelicaAdditions.MultiBody.Parts.Body U(
        rCM=rCMU, 
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U) annotation (extent=[40, -40; 60, -20]);
      parameter Chassis.Data.BumpBushingData data_CU
        annotation (extent=[60, 80; 80, 100]);
      Chassis.Components.Bushing Bushing1(data=data_CU)
        annotation (extent=[12, 40; 32, 60], rotation=270);
    equation 
      connect(Link2.frame_a, frame_C) annotation (points=[-40, -30; -69, -30; -69, 
             -4; -98, -4], style(thickness=4));
      connect(frameTranslationUW.frame_b, frame_U)
        annotation (points=[64, 0; 90, 0], style(thickness=4));
      connect(Link3.frame_a, frame_C) annotation (points=[-40, 0; -69, 0; -69, 
            -2; -98, -2], style(thickness=4));
      connect(Link1.frame_a, frame_C) annotation (points=[-40, -62; -69, -62; -69, 
             -4; -98, -4], style(thickness=4));
      connect(frameTranslationCU.frame_a, frame_C)
        annotation (points=[-44, 78; -47, 78; -47, 2; -96, 2]);
      connect(Bushing1.frame_a, frameTranslationCU.frame_b)
        annotation (points=[22.1, 60.5; -22, 78]);
      connect(Bushing1.frame_b, Link3.frame_b)
        annotation (points=[22, 40; 0, 0]);
      connect(Link2.frame_b, Bushing1.frame_b)
        annotation (points=[0, -30; 22, 40]);
      connect(Link1.frame_b, Bushing1.frame_b)
        annotation (points=[-2, -62; 22, 40]);
      connect(shape1.frame_a, Bushing1.frame_b)
        annotation (points=[0, -90; 22, 40]);
      connect(shape.frame_a, Bushing1.frame_b)
        annotation (points=[40, -90; 22, 39.7]);
      connect(U.frame_a, Bushing1.frame_b)
        annotation (points=[38, -30; 22, 40]);
      connect(frameTranslationUW.frame_a, Bushing1.frame_b)
        annotation (points=[42, 0; 22, 40]);
    end MultiLink42;
    
    model CutPushRod "Two bushings connected by a massless rod" 
      import SI = Modelica.SIunits;
      parameter SI.Position[3] rA={0,0,0} 
        "Position vector from frame_a to center of bushingA, resolved in frame_a";
      parameter SI.Position[3] rB={0,0,0} 
        "Position vector from center of bushingb to frame_b, resolved in frame_b";
      parameter SI.Position[3] rRod={1,0,0} 
        "Position vector from rod.frame_a to rod.frame_b, resolved in frame_a at the initial configuration";
      /*
  parameter Boolean autoInit=false 
    "|Initialization| = true: Compute nxRod, nyRod such that bushingB rotations are zero."
    ;
  parameter Real nxRod[3]={1,0,0} 
    "|Initialization| if autoInit = false, x-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
  parameter Real nyRod[3]={0,1,0} 
    "|Initialization| if autoInit = false, y-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
*/
      final parameter SI.Position rRod2[3]=rotA.S_rel*rRod;
      final parameter Real nxRod[3]=rotA.S_rel*dataB.nx;
      final parameter Real nyRod[3]=rotA.S_rel*dataB.ny;
      
      annotation (Diagram, Icon(
          Rectangle(extent=[-80, 14; -60, -12], style(color=0, fillColor=10)), 
          Rectangle(extent=[60, 14; 80, -12], style(color=0, fillColor=10)), 
          Rectangle(extent=[-60, 8; 60, -8], style(gradient=2, fillColor=8)), 
          Text(extent=[-100, 80; 100, 20], string="%name"), 
          Text(extent=[-80, -24; -38, -62], string="A")));
      
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-110, -10; -90, 10]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_b
        annotation (extent=[90, 10; 110, -10], rotation=180);
      Utilities.Parts.Body rodShape(r=rRod2)
        annotation (extent=[-10, -40; 10, -20]);
      Chassis.Data.BumpBushingData dataA(
        r_min={-1,-1,-1}, 
        r_max={1,1,1}, 
        phi_min={-1,-1,-1}, 
        phi_max={1,1,1}) annotation (extent=[-60, 40; -40, 60]);
      Chassis.Data.BumpBushingData dataB(
        r_min={-1,-1,-1}, 
        r_max={1,1,1}, 
        phi_min={-1,-1,-1}, 
        phi_max={1,1,1}) annotation (extent=[40, 40; 60, 60]);
      Utilities.Parts.FrameAxesNoAcceleration rotA(
        nx=dataA.nx, 
        ny=dataA.ny, 
        r=rA) annotation (extent=[-80, -10; -60, 10]);
      Utilities.Forces.SpringDamperLinearBump3D bushingB(
        r_rela0=dataB.r_rela0, 
        r_min=dataB.r_min, 
        r_max=dataB.r_max, 
        bumpFactor=dataB.bumpFactor, 
        phi_rela0=dataB.phi_rela0, 
        C=dataB.C, 
        D=dataB.D, 
        linear=dataB.linear, 
        eps=dataB.eps, 
        width=dataB.width, 
        length=dataB.length, 
        height=dataB.height) annotation (extent=[28, -10; 48, 10]);
      Utilities.Parts.FrameAxesNoAcceleration rotB(
        nx=dataB.nx, 
        ny=dataB.ny, 
        r=-rB) annotation (extent=[80, -10; 60, 10]);
      Utilities.Parts.FrameAxesNoAcceleration rod(
        r=rRod2, 
        nx=nxRod, 
        ny=nyRod) annotation (extent=[-10, -10; 10, 10]);
      Utilities.Forces.SpringDamperLinearBump3D bushingA(
        r_rela0=dataA.r_rela0, 
        r_min=dataA.r_min, 
        r_max=dataA.r_max, 
        bumpFactor=dataA.bumpFactor, 
        phi_rela0=dataA.phi_rela0, 
        linear=dataA.linear, 
        eps=dataA.eps, 
        animation=true, 
        C=dataA.C, 
        D=dataA.D, 
        width=dataA.width, 
        length=dataA.length, 
        height=dataA.height) annotation (extent=[-50, -10; -30, 10]);
      Utilities.Joints.FreeMotion FreeMotion1
        annotation (extent=[-52, -30; -32, -10]);
    initial equation 
    equation 
      connect(rotA.frame_a, frame_a) annotation (points=[-80, 0; -92, 0]);
      connect(frame_b, rotB.frame_a) annotation (points=[92, 0; 79, 0]);
      connect(rod.frame_a, rodShape.frame_a)
        annotation (points=[-10, 0; -16, 0; -16, -30; -10.5, -30]);
      connect(rod.frame_b, bushingB.frame_a) annotation (points=[10, 0; 28, 0]);
      connect(rotB.frame_b, bushingB.frame_b)
        annotation (points=[60, 0; 48, 0]);
      connect(rotA.frame_b, bushingA.frame_a)
        annotation (points=[-60, 0; -50.5, 0]);
      connect(bushingA.frame_b, rod.frame_a)
        annotation (points=[-30, 0; -12, 0]);
      connect(FreeMotion1.frame_a, rotA.frame_b)
        annotation (points=[-52, -20; -60, 0]);
      connect(FreeMotion1.frame_b, bushingA.frame_b)
        annotation (points=[-32, -20; -30, 0]);
    end CutPushRod;
    
    model Simple 
      "Simple - Suspension linkage model whithout geometry. Camber and Steering are specified as functions of bounce. This should be extended so that track width changes could be modelled as well."
       
      
      import SI = Modelica.SIunits;
      parameter SI.Position[3] rUW={0.0,0.9,0} "|Geometry|";
      parameter SI.Position[3] rCU={0.0,0.85,0} "|Geometry|";
      parameter SI.Position q0Strut=0.2 "|Geometry|";
      parameter Boolean leftLinkage=(rUW_scaled[2] > 0) "|Geometry|";
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|";
      parameter SI.Mass mU=1 "|Mass and Inertia|";
      parameter SI.Inertia i11U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i22U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i33U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i21U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i31U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i32U=.01 "|Mass and Inertia|";
      
      //SCALED COORDINATES
    protected 
      parameter Real left=if leftLinkage then 1 else -1;
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      parameter SI.Position[3] rCU_scaled=Utilities.Functions.EWM(scaleFactor, 
          rCU);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-35, 79; 81, -79], style(color=0, fillColor=51)), 
          Rectangle(extent=[-25, -67; 69, 71], style(color=7, fillColor=7)), 
          Line(points=[-10, 71; -10, -67]), 
          Line(points=[-25, -67; -25, 71], style(
              color=0, 
              thickness=2, 
              arrow=1)), 
          Line(points=[-25, -67; 69, -67], style(
              color=0, 
              thickness=2, 
              arrow=1)), 
          Line(points=[-25, -67; -13, -41; -3, -10; 20, 15; 62, 40], style(
                color=0, thickness=2)), 
          Line(points=[-25, -67; 5, -45; 34, -14; 60, 54], style(color=0, 
                thickness=2)), 
          Line(points=[57, -55; 63, -55; 57, -63; 63, -63], style(color=0)), 
          Line(points=[-22, 68; -17, 59; -16, 56; -16, 53; -18, 51; -20, 51; -21, 
                 53; -21, 57; -19, 61; -14, 69], style(color=0)), 
          Line(points=[-15, 60; -14, 58; -15, 54], style(color=0)), 
          Line(points=[-7, 67; -9, 68; -11, 66; -11, 65; -12, 62; -10, 59; -5, 
                60; -5, 67; -9, 72; -9, 76; -6, 77], style(color=0)), 
          Line(points=[-25, 71; 69, 71]), 
          Polygon(points=[-63, 15; -49, 11; -49, 15; -63, 19; -63, 15], style(
                color=10, fillColor=10)), 
          Polygon(points=[-77, 19; -63, 15; -63, 19; -77, 23; -77, 19], style(
                color=10, fillColor=10)), 
          Polygon(points=[-63, 31; -49, 27; -49, 31; -63, 35; -63, 31], style(
                color=10, fillColor=10)), 
          Polygon(points=[-77, 35; -63, 31; -63, 35; -77, 39; -77, 35], style(
                color=10, fillColor=10)), 
          Polygon(points=[-63, 47; -48, 43; -48, 47; -63, 51; -63, 47], style(
                color=10, fillColor=10)), 
          Polygon(points=[-77, 51; -63, 47; -63, 51; -77, 55; -77, 51], style(
                color=10, fillColor=10)), 
          Polygon(points=[-63, 63; -49, 59; -49, 63; -63, 67; -63, 63], style(
                color=10, fillColor=10)), 
          Rectangle(extent=[-69, 69; -57, -2], style(gradient=1, fillColor=8)), 
             
          Rectangle(extent=[-74, 65; -52, -3], style(gradient=1, fillColor=8)), 
             
          Polygon(points=[-77, 51; -63, 55; -63, 59; -77, 55; -77, 51], style(
                color=8, fillColor=8)), 
          Polygon(points=[-63, 55; -49, 59; -49, 63; -63, 59; -63, 55], style(
                color=8, fillColor=8)), 
          Polygon(points=[-77, 35; -63, 39; -63, 43; -77, 39; -77, 35], style(
                color=8, fillColor=8)), 
          Polygon(points=[-62, 39; -48, 43; -48, 47; -62, 43; -62, 39], style(
                color=8, fillColor=8)), 
          Polygon(points=[-77, 19; -63, 23; -63, 27; -77, 23; -77, 19], style(
                color=8, fillColor=8)), 
          Polygon(points=[-63, 23; -48, 27; -48, 31; -63, 27; -63, 23], style(
                color=8, fillColor=8)), 
          Polygon(points=[-63, 7; -48, 11; -48, 15; -63, 11; -63, 7], style(
                color=8, fillColor=8)), 
          Line(points=[-80, 0; -80, 90; -63, 90; -63, 69], style(color=0)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[-64, -72; -64, -90; -41, -90], style(color=0)), 
          Line(points=[81, 0; 90, 0], style(color=0)), 
          Polygon(points=[-77, -59; -65, -63; -65, -59; -77, -55; -77, -59], 
              style(color=10, fillColor=10)), 
          Polygon(points=[-77, -45; -48, -52; -48, -47; -77, -41; -77, -45], 
              style(color=10, fillColor=10)), 
          Polygon(points=[-77, -29; -48, -38; -48, -33; -77, -25; -77, -29], 
              style(color=10, fillColor=10)), 
          Polygon(points=[-77, -13; -48, -21; -48, -17; -77, -9; -77, -13], 
              style(color=10, fillColor=10)), 
          Polygon(points=[-52, -3; -50, -5; -50, -1; -52, -1; -52, -3], style(
                color=10, fillColor=10)), 
          Rectangle(extent=[-69, -3; -57, -72], style(gradient=1, fillColor=8)), 
             
          Polygon(points=[-77, -13; -62, -9; -62, -5; -77, -9; -77, -13], style(
                color=8, fillColor=8)), 
          Polygon(points=[-62, -9; -48, -5; -48, -1; -62, -5; -62, -9], style(
                color=8, fillColor=8)), 
          Polygon(points=[-77, -29; -62, -25; -62, -21; -77, -25; -77, -29], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-62, -25; -48, -21; -48, -17; -62, -21; -62, -25], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-77, -45; -48, -37; -48, -33; -77, -40; -77, -45], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-77, -60; -48, -52; -48, -47; -77, -54; -77, -60], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-77, 3; -74, 2; -74, 4; -77, 7; -77, 3], style(color=
                  10, fillColor=10)), 
          Polygon(points=[-77, 3; -63, 7; -63, 11; -77, 7; -77, 3], style(color
                =8, fillColor=8)), 
          Line(points=[-41, -90; -41, -1; -35, -1], style(color=0)), 
          Line(points=[6, 71; 6, -67]), 
          Line(points=[23, 71; 23, -67]), 
          Line(points=[38, 71; 38, -67]), 
          Line(points=[54, 71; 54, -67]), 
          Line(points=[69, 71; 69, -67]), 
          Line(points=[-25, -52; 69, -52]), 
          Line(points=[-25, -37; 69, -37]), 
          Line(points=[-24, -22; 70, -22]), 
          Line(points=[-26, -5; 68, -5]), 
          Line(points=[-25, 11; 69, 11]), 
          Line(points=[-25, 27; 69, 27]), 
          Line(points=[-25, 43; 69, 43]), 
          Line(points=[-25, 57; 69, 57])), 
        Diagram(Text(extent=[-47, 2; -22, -9], string="obs vnd tecken")), 
        Window(
          x=0.4, 
          y=0.17, 
          width=0.53, 
          height=0.79), 
        Documentation(info="<HTML>
<p>This model has no linkage defined. Instead, the model applies camber and additional steering angles to the wheel depending on its bounce motion. For handling studies, this model provides an convienient way to examine different linkage characteristics. From a simulation point of view, it may also be advatageous when simulation speed is critical due to its lack of closed kinematic loops. 
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUW:         location of wheel centre, resolved in frame_C
  q0Strut:     The additional unstretched strut length, compared to the referecne geometry.
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
 
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-120, -14; -90, 16]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_U
        annotation (extent=[90, -14; 120, 16]);
      parameter Utilities.Forces.Utilities.ForceTable1D data_S
        annotation (extent=[52, -95; 92, -55]);
      ModelicaAdditions.MultiBody.Joints.Prismatic bounce(
        n={0,0,1}, 
        q0=0.4, 
        q(stateSelect=StateSelect.always), 
        qd(stateSelect=StateSelect.always))
        annotation (extent=[50, 70; 30, 90], rotation=270);
      ModelicaAdditions.MultiBody.Joints.Revolute steering
        annotation (extent=[61, 18; 31, 48], rotation=270);
      ModelicaAdditions.MultiBody.Joints.Revolute camber(n={left,0,0})
        annotation (extent=[50, 15; 80, -15]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a steerAxis
        annotation (extent=[-20, -110; 0, -90]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation upper(r=rCU_scaled + {
            0,0,0.4}) annotation (extent=[-80, 86; -60, 106]);
      Modelica.Mechanics.Translational.Sensors.PositionSensor sensor
        annotation (extent=[-40, 41; -20, 61], rotation=180);
      Modelica.Mechanics.Rotational.Position camberAngle(exact=true)
        annotation (extent=[-40, -60; -20, -40]);
      Modelica.Mechanics.Rotational.Position addSteeringAngle(exact=true)
        annotation (extent=[-28, -30; -8, -10]);
      Utilities.Polynomial.Evaluation camberTable(poly=camberPoly) 
        "Camber angle[rad] as function of bounce [m] as scaled polynomial"
        annotation (extent=[-70, -60; -50, -40]);
      Utilities.Polynomial.Evaluation steeringTable(poly=steeringPoly) 
        "Steering angle[rad] as function of bounce [m] as scaled polynomial"
        annotation (extent=[-70, -30; -50, -10]);
      Utilities.RotationalSum sum(ratio_a1=scaleFactor[2], ratio_a2=scaleFactor[
            2]) annotation (extent=[-9, 0; 11, 20], rotation=90);
      Utilities.Forces.SpringDamperTableLine3D SpringDamperTableLine3D1(
        data=data_S, 
        s_rel0=q0Strut + 0.4, 
        n1={1,0,0}) annotation (extent=[75, 69; 95, 89], rotation=90);
      Utilities.Parts.Body upright(
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U, 
        widthDirection={1,0,0}, 
        r=rUW_scaled - rCU_scaled, 
        rCM=rCMU) annotation (extent=[71, -50; 91, -30]);
      Modelica.Mechanics.Translational.Interfaces.Flange_a bounceAxis
        annotation (extent=[-19, 89; 1, 109]);
      parameter Utilities.Polynomial.ScaledPolynomial steeringPoly(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.082, 
        u_max=0.078, 
        y_min=-0.010297, 
        y_max=0.0012217, 
        c={6.30814489981295e+000,-1.63208206468340e+001,1.59971846025538e+001,-9.43372508316803e+000,
            2.72106634745229e+000,7.28435761181664e-001}, 
        original=[-8.20000000000000e-002, -1.91986217719376e-003; -6.20000000000000e-002, 
             -4.20000000000000e-002; -2.20000000000000e-002, -1.60000000000000e-002; 
             -2.00000000000000e-003, 0.00000000000000e+000; 
            1.80000000000000e-002, 3.80000000000000e-002; 5.80000000000000e-002, 
             7.80000000000000e-002; 6.98131700797732e-004, 
            1.22173047639603e-003; 1.22173047639603e-003, 8.72664625997165e-004; 
             1.74532925199433e-004, 0.00000000000000e+000; -1.57079632679490e-003, 
             -4.01425727958696e-003; -7.15584993317675e-003, -1.02974425867665e-002])
        annotation (extent=[-90, -90; -70, -70]);
      parameter Utilities.Polynomial.ScaledPolynomial camberPoly(
        redeclare type uType = SI.Position, 
        redeclare type yType = SI.Angle, 
        u_min=-0.082, 
        u_max=0.078, 
        y_min=-0.0055851, 
        y_max=0.027576, 
        c={-4.98260440195846e+000,1.28215982847983e+001,-1.17916202644075e+001,
            5.83708966179418e+000,-2.88532206648880e+000,9.99807679818011e-001}, 
           
        original=[-8.20000000000000e-002, 2.75762021815104e-002; -6.20000000000000e-002, 
             -4.20000000000000e-002; -2.20000000000000e-002, -1.60000000000000e-002; 
             -2.00000000000000e-003, 0.00000000000000e+000; 
            1.80000000000000e-002, 3.80000000000000e-002; 5.80000000000000e-002, 
             7.80000000000000e-002; 1.79768912955416e-002, 
            1.09955742875643e-002; 5.75958653158129e-003, 4.01425727958696e-003; 
             5.23598775598299e-004, 0.00000000000000e+000; -2.96705972839036e-003, 
             -4.71238898038469e-003; -5.58505360638185e-003, -5.58505360638185e-003])
        annotation (extent=[-60, -90; -40, -70]);
      
      Modelica.Blocks.Math.Gain Gain1(k={left})
        annotation (extent=[-43, -22; -35, -16]);
    equation 
      connect(camber.frame_a, steering.frame_b)
        annotation (points=[47.25, 1; 45, 1; 45, 19.25], style(thickness=4));
      connect(upper.frame_a, frame_C)
        annotation (points=[-82, 96; -98, 96; -98, 0], style(thickness=4));
      connect(upper.frame_b, bounce.frame_a)
        annotation (points=[-60, 96; 45, 96; 39.5, 90.5], style(thickness=4));
      connect(camberAngle.flange_b, camber.axis)
        annotation (points=[-20, -50; 63, -50; 65, -9.5], style(color=0));
      connect(bounce.axis, sensor.flange_a) annotation (points=[34.2, 73.45; 33, 
             51; -19.75, 51; -19.75, 50; -20, 50], style(color=58));
      connect(bounce.frame_b, steering.frame_a)
        annotation (points=[39.5, 70.375; 45, 48]);
      connect(camberTable.outPort, camberAngle.inPort)
        annotation (points=[-49, -50; -42, -50], style(color=3));
      connect(sensor.outPort, steeringTable.inPort) annotation (points=[-41, 51; 
             -77, 51; -77, -20; -72, -20], style(color=3));
      connect(camberTable.inPort, sensor.outPort) annotation (points=[-72, -49; 
             -77, -49; -77, 51; -41, 51], style(color=3));
      connect(addSteeringAngle.flange_b, sum.flange_a1)
        annotation (points=[-8, -20; -3, -20; -3, -1], style(color=0));
      connect(steerAxis, sum.flange_a2)
        annotation (points=[-10, -96; -10, -50; 5, -50; 5, 0], style(color=0));
      connect(sum.flange_b, steering.axis)
        annotation (points=[2, 21; 2, 33; 35, 33], style(color=0));
      connect(bounce.frame_b, SpringDamperTableLine3D1.frame_a)
        annotation (points=[40, 69.35; 86, 69]);
      connect(SpringDamperTableLine3D1.frame_b, bounce.frame_a)
        annotation (points=[85, 89; 40.5, 91.525]);
      connect(upright.frame_a, camber.frame_b)
        annotation (points=[70.5, -40; 78.75, 1]);
      connect(upright.frame_b, frame_U) annotation (points=[92, -39; 90, -1]);
      connect(bounce.axis, bounceAxis)
        annotation (points=[34.2, 73.45; -9, 65; -9, 99], style(color=58));
      connect(Gain1.outPort, addSteeringAngle.inPort)
        annotation (points=[-35, -19; -31, -21], style(color=3));
      connect(Gain1.inPort, steeringTable.outPort)
        annotation (points=[-44, -19; -49, -20], style(color=3));
    end Simple;
    
    //>>>>>>> 1.17
    
    model Bushing "CutBusing - to be used as a cut in a closed loop." 
      
      parameter SI.Position[3] rA=data.rA 
        "|General geometry|Position|vector from frame_A to bushing centre";
      parameter SI.Position[3] rB=data.rB 
        "|General geometry|Position|vector from frame_B to bushing centre";
      parameter Real[3] nx=data.nx 
        "|General geometry|Orientation|x direction of bushing, resolved in frame_A";
      parameter Real[3] ny=data.ny 
        "|General geometry|Orientation|y direction of bushing, resolved in frame_A";
      
      parameter SI.Position[3] r_rela0=data.r_rela0 
        "|General geometry|Pretention|Unstretched translational spring vector";
      parameter SI.Position[3] r_min=data.r_min 
        "|Forces|Bump|vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Position[3] r_max=data.r_max 
        "|Forces|Bump|vector to bump stop from reference point in positive direction, resolved in frame_A";
      
      parameter SI.Angle[3] phi_rela0=data.phi_rela0 
        "|General geometry|Pretention|Unstretched rotational spring vector";
      parameter SI.Angle[3] phi_min=data.phi_min 
        "|Forces|Bump|rotational vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Angle[3] phi_max=data.phi_max 
        "|Forces|Bump|rotational vector to bump stop from reference point in positive direction, resolved in frame_A";
      parameter Real bumpFactor=data.bumpFactor 
        "|Forces|Bump|scaleFactor of spring force at after bump";
      
      parameter Real[6, 6] C=data.C "|Forces|Linear|Stiffness matrix";
      parameter Real[6, 6] D=data.D "|Forces|Linear|Damping matrix";
      
      parameter Boolean linear=data.linear 
        "|Advanced||turn on/off linearisation of cosine and sine in rotation vector equation";
      
      parameter Real eps=data.eps "|Advanced||Guard against division by zero";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-78, 74; 82, -86], style(color=0, fillColor=9)), 
          Line(points=[0, 0; 0, -58], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Line(points=[0, 0; 58, 0], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Polygon(points=[0, -60; 6, -50; -6, -50; 0, -60], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Polygon(points=[60, 0; 52, -6; 52, 6; 60, 0], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Ellipse(extent=[-22, 20; 20, -20], style(color=7, thickness=2)), 
          Rectangle(extent=[-24, 20; -2, 0], style(color=9, fillColor=9)), 
          Polygon(points=[-4, 20; 2, 14; 2, 26; -4, 20], style(
              color=7, 
              thickness=2, 
              fillColor=7))), 
        Window(
          x=0.16, 
          y=0.13, 
          width=0.69, 
          height=0.71), 
        Diagram, 
        Documentation(info="<HTML>
<p>
The bushing is linear within a specified range. Outside this range, bumpstops are modellled as a much stiffer spring-dampers.
The range is defined with parameters as well as the ratio between stiffness inside outside range. The stiffness inside the range is read from record like for a normal linear SpringDamper3D.
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  r_rela0:    Unstretched translational spring vector.
  r_min:      Vector to bump stop from reference point in negative direction of coordinate system.
  r_max:      Vector to bump stop from reference point in positive direction of coordinate system.
  bumpFactor: Scale-factor of spring force at after bump.
  phi_rela0:  Unstretched rotational spring vector.
  
</pre>
</HTML>
"));
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_b
        annotation (extent=[118, -16; 88, 14], rotation=180);
      Utilities.Joints.FreeMotion freeMotion(r_rela(start=rA - rB))
        annotation (extent=[-26, -80; -6, -60]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -14; -90, 16]);
      Utilities.Forces.SpringDamperLinearBump3D springDamper(
        C=C, 
        D=D, 
        r_rela0=data.r_rela0, 
        r_min=data.r_min, 
        r_max=data.r_max, 
        bumpFactor=data.bumpFactor, 
        phi_rela0=data.phi_rela0, 
        linear=data.linear, 
        eps=data.eps) annotation (extent=[-40, -40; 40, 40]);
      ModelicaAdditions.MultiBody.Parts.FrameAxes r_A(
        r=data.rA, 
        nx=data.nx, 
        ny=data.ny) annotation (extent=[-80, -10; -60, 10]);
      ModelicaAdditions.MultiBody.Parts.FrameAxes r_B(
        r=data.rB, 
        nx=data.nx, 
        ny=data.ny) annotation (extent=[60, -10; 80, 10], rotation=180);
      parameter Chassis.Data.BumpBushingData data
        annotation (extent=[40, 40; 80, 80]);
    equation 
      connect(r_A.frame_a, frame_a)
        annotation (points=[-80, 0; -94, 0], style(thickness=4));
      connect(r_A.frame_b, springDamper.frame_a)
        annotation (points=[-60, 0; -40, 0], style(thickness=4));
      connect(r_B.frame_b, springDamper.frame_b)
        annotation (points=[58, 0; 42, 0], style(thickness=4));
      connect(r_B.frame_a, frame_b) annotation (points=[80, 0; 98, 0]);
      connect(freeMotion.frame_a, r_A.frame_b)
        annotation (points=[-26, -70; -60, 0]);
      connect(freeMotion.frame_b, springDamper.frame_b)
        annotation (points=[-6, -70; 42, 0]);
    end Bushing;
    
    model BushingAntiRollBar "AntiRollBar - standard anti-roll linkage." 
      
      parameter Real[3] scaleFactor={1,1,1};
      parameter SI.Position[3] rCA_1={-0.2405,0.3420,0.0330} 
        "vector to left torsion beam end resolved in frame_C";
      parameter SI.Position[3] rAL_1={-0.0439,0.5115,0.0203} 
        "vector to left inner anti roll rod end/left anti roll lever resolved in frame_C";
      parameter SI.Position[3] rXL_1={-0.0564,0.5401,0.2183} 
        "vector to left outer anti roll rod end resolved in frame_C";
      parameter SI.Position[3] rX_1={0.0059,0.6905,-0.0710} + {-0.0060,0.0365,
          0.1110} "vector to left anti roll mount resolved in frame_C";
      
      parameter SI.Position[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1);
      parameter SI.Position[3] rAL_2=Utilities.Functions.EWM({1,-1,1}, rAL_1);
      parameter SI.Position[3] rXL_2=Utilities.Functions.EWM({1,-1,1}, rXL_1);
      parameter SI.Position[3] rX_2=Utilities.Functions.EWM({1,-1,1}, rX_1);
      
    protected 
      parameter SI.Position[3] rCA_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCA_1);
      parameter SI.Position[3] rAL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rAL_1);
      parameter SI.Position[3] rXL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_1);
      parameter SI.Position[3] rX_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_1);
      
      parameter SI.Position[3] rCA_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCA_2);
      parameter SI.Position[3] rAL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rAL_2);
      parameter SI.Position[3] rXL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_2);
      parameter SI.Position[3] rX_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_2);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-90, -36; -78, -44], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[76, -36; 88, -44], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-54, 60; 52, 52], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-12, 60; 12, 52], style(gradient=2, fillColor=41)), 
             
          Rectangle(extent=[-54, 60; -46, 12], style(color=8, fillColor=8)), 
          Ellipse(extent=[-60, 0; -40, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-58, 2; -42, 18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-82, -48; -62, -28], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-80, -46; -64, -30], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-64, 16; -34, 0; -40, -12; -70, 2; -64, 16], style(
                color=7, fillColor=7)), 
          Polygon(points=[-80, -16; -50, -32; -56, -44; -84, -28; -80, -16], 
              style(color=7, fillColor=7)), 
          Polygon(points=[-54, 10; -74, -36; -68, -40; -48, 6; -54, 10], style(
                color=8, fillColor=8)), 
          Ellipse(extent=[-78, -32; -66, -44], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 16; -44, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 16; -44, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[44, 60; 52, 12], style(color=8, fillColor=8)), 
          Ellipse(extent=[38, 0; 58, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[40, 2; 56, 18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[60, -50; 80, -30], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[62, -48; 78, -32], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[34, 4; 68, 16; 66, 2; 42, -12; 34, 4], style(color=7, 
                 fillColor=7)), 
          Polygon(points=[54, -32; 76, -20; 82, -28; 58, -46; 54, -32], style(
                color=7, fillColor=7)), 
          Ellipse(extent=[42, 16; 54, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[42, 8; 50, 12; 72, -36; 66, -40; 44, 8; 42, 8], style(
                color=8, fillColor=8)), 
          Ellipse(extent=[42, 16; 54, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[64, -34; 76, -46], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[0, 86; 0, 62], style(color=10)), 
          Text(extent=[34, -18; 66, -66], string="L")), 
        Diagram, 
        Window(
          x=0.05, 
          y=0.02, 
          width=0.51, 
          height=0.91), 
        Documentation(info="<HTML>
<p>
To a torsion bar with levers in each end, two connecting rods are attached, these are normally attached at the left and right suspension linkage to reduce roll motion. This component is modelled with rigid elements, except for a torsion bar. The force generation fue to deflection is modelled from table values.
</p>
<p>
Usage:
Most vehicles with independent suspensions, both front and rear.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
The right hand side parameters, ending with _2, are by default symmetric to the left hand dito ending with _1.
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} mirrors it around the xz-plane.
  ratioWheelToRack=22: rack-pinion gear ratio [m/rad]
  rX_1:   location of frame X_1, resolved in frame_C
  rXL1_1: location of left link end, resolved in frame_C
  rAL_1:  location of left torsion bar end, resolved in frame_C
  rX_2:   location of frame X_2, resolved in frame_C,
  rXL1_2: location of right link end, resolved in frame_C
  rAL_2:  location of right torsion bar end, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Joints.Revolute rightRevolute(
        n=rAL_1_scaled - rAL_2_scaled, 
        q(stateSelect=StateSelect.never), 
        qd(stateSelect=StateSelect.never))
        annotation (extent=[-30, 30; -10, 50], rotation=180);
      ModelicaAdditions.MultiBody.Joints.Revolute leftRevolute(
        n=rAL_1_scaled - rAL_2_scaled, 
        q(stateSelect=StateSelect.never), 
        qd(stateSelect=StateSelect.never)) annotation (extent=[10, 50; 30, 30]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation(r=(
            rCA_1_scaled + rCA_2_scaled)/2)
        annotation (extent=[-10, 60; 10, 80], rotation=270);
      Utilities.Parts.Body leftBar(r=rCA_1_scaled - (rCA_1_scaled + 
            rCA_2_scaled)/2) annotation (extent=[40, 30; 60, 50]);
      Utilities.Parts.Body rightBar(r=rCA_2_scaled - (rCA_1_scaled + 
            rCA_2_scaled)/2)
        annotation (extent=[-60, 30; -40, 50], rotation=180);
      Utilities.Parts.Body rightLever(r=rAL_2_scaled - rCA_2_scaled)
        annotation (extent=[-80, 10; -60, 30], rotation=270);
      Utilities.Parts.Body leftLever(r=rAL_1_scaled - rCA_1_scaled)
        annotation (extent=[60, 10; 80, 30], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_2
        annotation (extent=[-120, -56; -90, -26]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_1
        annotation (extent=[88, -54; 118, -24], rotation=180);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-14, 88; 16, 118], rotation=270);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation rightFrameTranslation(
          r=rXL_2_scaled - rX_2_scaled) annotation (extent=[-80, -50; -60, -30]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation leftFrameTranslation(r
          =rXL_1_scaled - rX_1_scaled)
        annotation (extent=[60, -50; 80, -30], rotation=180);
      Utilities.Forces.SpringDamperTableRot1D springDamper(data=data, fc=data.
            fc*[1, 0; 0, 1.2]) annotation (extent=[-10, 0; 10, 20]);
      parameter Utilities.Forces.Utilities.ForceTable1D data
        annotation (extent=[-20, -80; 20, -40]);
      Chassis.Components.CutPushRod rightPushRod(rRod=rXL_2_scaled - 
            rAL_2_scaled) annotation (extent=[-80, -20; -60, 0], rotation=270);
      Chassis.Components.CutPushRod leftPushRod(rRod=rXL_1_scaled - 
            rAL_1_scaled) annotation (extent=[60, -18; 80, 2], rotation=270);
    equation 
      connect(frameTranslation.frame_b, leftRevolute.frame_a)
        annotation (points=[-1.88738e-015, 59.5; -1.88738e-015, 40; 9.5, 40]);
      connect(rightRevolute.frame_a, frameTranslation.frame_b)
        annotation (points=[-9.5, 40; -1.88738e-015, 40; -1.88738e-015, 59.5]);
      connect(rightBar.frame_a, rightRevolute.frame_b)
        annotation (points=[-38, 40; -30, 40]);
      connect(leftRevolute.frame_b, leftBar.frame_a)
        annotation (points=[30, 40; 40, 40]);
      connect(leftLever.frame_a, leftBar.frame_b)
        annotation (points=[70, 30.5; 70, 40; 62, 40]);
      connect(rightLever.frame_a, rightBar.frame_b)
        annotation (points=[-70, 30.5; -70, 40; -62, 40]);
      connect(frameTranslation.frame_a, frame_C)
        annotation (points=[0, 82; 0, 103; 1, 103]);
      connect(rightFrameTranslation.frame_a, frame_X_2)
        annotation (points=[-80, -40; -89, -40; -89, -38; -98, -38]);
      connect(leftFrameTranslation.frame_a, frame_X_1)
        annotation (points=[80, -40; 87, -40; 87, -38; 94, -38]);
      connect(springDamper.flange_a, rightRevolute.axis)
        annotation (points=[-10, 10; -20, 10; -20, 32], style(color=0));
      connect(springDamper.flange_b, leftRevolute.axis)
        annotation (points=[10, 10; 20, 10; 20, 32], style(color=0));
      connect(rightPushRod.frame_a, rightLever.frame_b)
        annotation (points=[-70, 0; -70, 10]);
      connect(rightPushRod.frame_b, rightFrameTranslation.frame_b)
        annotation (points=[-70, -20; -60, -40]);
      connect(leftPushRod.frame_a, leftLever.frame_b)
        annotation (points=[70, 2; 70, 10]);
      connect(leftPushRod.frame_b, leftFrameTranslation.frame_b)
        annotation (points=[70, -18; 60, -40]);
    end BushingAntiRollBar;
    
    model SimpleAntiRollBar "AntiRollBar - standard anti-roll linkage." 
      
      SI.Position[3] r_rel "location difference of frame_X_1 and frame_X_2,
resolved in inertial frame";
      SI.Position[3] r_rel2 "location difference of frame_X_1 and frame_X_2,
resolved in frame_C";
      //Real s "offset to use in force calculation";
      
      parameter SI.Position r2t=1/abs(rAL_1_scaled[1] - rCA_1_scaled[1]);
      
      Real f;
      Real tau;
      SI.Angle phi_rel;
      Real w_rel;
      
      parameter Real[3] scaleFactor={1,1,1};
      parameter SI.Position[3] rCA_1={-0.2405,0.3420,0.0330} 
        "vector to left torsion beam end resolved in frame_C";
      parameter SI.Position[3] rAL_1={-0.0439,0.5115,0.0203} 
        "vector to left inner anti roll rod end/left anti roll lever resolved in frame_C";
      parameter SI.Position[3] rXL_1={-0.0564,0.5401,0.2183} 
        "vector to left outer anti roll rod end resolved in frame_C";
      parameter SI.Position[3] rX_1={0.0059,0.6905,-0.0710} + {-0.0060,0.0365,
          0.1110} "vector to left anti roll mount resolved in frame_C";
      
      parameter SI.Position[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1);
      parameter SI.Position[3] rAL_2=Utilities.Functions.EWM({1,-1,1}, rAL_1);
      parameter SI.Position[3] rXL_2=Utilities.Functions.EWM({1,-1,1}, rXL_1);
      parameter SI.Position[3] rX_2=Utilities.Functions.EWM({1,-1,1}, rX_1);
      
    protected 
      parameter SI.Position[3] rCA_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCA_1);
      parameter SI.Position[3] rAL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rAL_1);
      parameter SI.Position[3] rXL_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_1);
      parameter SI.Position[3] rX_1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_1);
      
      parameter SI.Position[3] rCA_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCA_2);
      parameter SI.Position[3] rAL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rAL_2);
      parameter SI.Position[3] rXL_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rXL_2);
      parameter SI.Position[3] rX_2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rX_2);
    public 
      parameter Utilities.Forces.Utilities.ForceTable1D data
        annotation (extent=[-20, -92; 20, -52]);
    public 
      ModelicaAdditions.Tables.CombiTable1D Dr(
        table=data.fd, 
        tableName=data.tableName_D, 
        fileName=data.fileName_D, 
        icol=data.icol_D) annotation (extent=[-20, 30; 20, 70]);
      ModelicaAdditions.Tables.CombiTable1D Cr(
        table=data.fc, 
        tableName=data.tableName_C, 
        fileName=data.fileName_C, 
        icol=data.icol_C) annotation (extent=[-20, -30; 20, 10]);
      
    public 
      Real f_C[3];
    equation 
      
      w_rel = der(phi_rel);
      
      Cr.inPort.signal[1] = phi_rel;
      Dr.inPort.signal[1] = w_rel;
      
      tau = Cr.outPort.signal[1] + Dr.outPort.signal[1];
      
      r_rel = frame_X_1.r0 - frame_X_2.r0;
      r_rel2 = transpose(frame_C.S)*r_rel;
      //frame_C.S*r_rel2 = r_rel;
      phi_rel = r2t*r_rel2[3];
      
      f = tau*r2t;
      
      f_C = frame_C.S*{0,0,f};
      frame_X_1.f = transpose(frame_X_1.S)*f_C;
      frame_X_2.f = -transpose(frame_X_2.S)*f_C;
      
      //{0,0,f} = transpose(frame_C.S)*frame_X_1.S*frame_X_1.f;
      //{0,0,-f} = transpose(frame_C.S)*frame_X_2.S*frame_X_2.f;
      
      frame_X_1.t = zeros(3);
      frame_X_2.t = zeros(3);
      
      frame_C.t = cross({0,0,f}, r_rel2);
      frame_C.f = zeros(3);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-90, -36; -78, -44], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[76, -36; 88, -44], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-54, 60; 52, 52], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-12, 60; 12, 52], style(gradient=2, fillColor=41)), 
             
          Rectangle(extent=[-54, 60; -46, 12], style(color=8, fillColor=8)), 
          Ellipse(extent=[-60, 0; -40, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-58, 2; -42, 18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-82, -48; -62, -28], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-80, -46; -64, -30], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-64, 16; -34, 0; -40, -12; -70, 2; -64, 16], style(
                color=7, fillColor=7)), 
          Polygon(points=[-80, -16; -50, -32; -56, -44; -84, -28; -80, -16], 
              style(color=7, fillColor=7)), 
          Polygon(points=[-54, 10; -74, -36; -68, -40; -48, 6; -54, 10], style(
                color=8, fillColor=8)), 
          Ellipse(extent=[-78, -32; -66, -44], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 16; -44, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 16; -44, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[44, 60; 52, 12], style(color=8, fillColor=8)), 
          Ellipse(extent=[38, 0; 58, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[40, 2; 56, 18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[60, -50; 80, -30], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[62, -48; 78, -32], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[34, 4; 68, 16; 66, 2; 42, -12; 34, 4], style(color=7, 
                 fillColor=7)), 
          Polygon(points=[54, -32; 76, -20; 82, -28; 58, -46; 54, -32], style(
                color=7, fillColor=7)), 
          Ellipse(extent=[42, 16; 54, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[42, 8; 50, 12; 72, -36; 66, -40; 44, 8; 42, 8], style(
                color=8, fillColor=8)), 
          Ellipse(extent=[42, 16; 54, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[64, -34; 76, -46], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[0, 86; 0, 62], style(color=10)), 
          Text(extent=[34, -18; 66, -66], string="L")), 
        Diagram, 
        Window(
          x=0.05, 
          y=0.02, 
          width=0.51, 
          height=0.91), 
        Documentation(info="<HTML>
<p>
To a torsion bar with levers in each end, two connecting rods are attached, these are normally attached at the left and right suspension linkage to reduce roll motion. This component is modelled with rigid elements, except for a torsion bar. The force generation fue to deflection is modelled from table values.
</p>
<p>
Usage:
Most vehicles with independent suspensions, both front and rear.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
The right hand side parameters, ending with _2, are by default symmetric to the left hand dito ending with _1.
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} mirrors it around the xz-plane.
  ratioWheelToRack=22: rack-pinion gear ratio [m/rad]
  rX_1:   location of frame X_1, resolved in frame_C
  rXL1_1: location of left link end, resolved in frame_C
  rAL_1:  location of left torsion bar end, resolved in frame_C
  rX_2:   location of frame X_2, resolved in frame_C,
  rXL1_2: location of right link end, resolved in frame_C
  rAL_2:  location of right torsion bar end, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_2
        annotation (extent=[-120, -56; -90, -26]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_1
        annotation (extent=[88, -54; 118, -24], rotation=180);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-14, 88; 16, 118], rotation=270);
    end SimpleAntiRollBar;
    
    model PairBushing "A pair of bushings, to be used with e.g. WishBones." 
      
      parameter SI.Position rB1[3]={0.3,0,0} "vector to bushing 1";
      parameter SI.Position rB2[3]={0.3,0,0} "vector to bushing 2";
      
      parameter Real[3] nx1=data1.nx 
        "|General geometry|Orientation|x direction of bushing, resolved in frame_A";
      parameter Real[3] ny1=data1.ny 
        "|General geometry|Orientation|y direction of bushing, resolved in frame_A";
      
      parameter SI.Position[3] r_rela01=data1.r_rela0 
        "|General geometry|Pretention|Unstretched translational spring vector";
      parameter SI.Position[3] r_min1=data1.r_min 
        "|Forces|Bump|vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Position[3] r_max1=data1.r_max 
        "|Forces|Bump|vector to bump stop from reference point in positive direction, resolved in frame_A";
      
      parameter SI.Angle[3] phi_rela01=data1.phi_rela0 
        "|General geometry|Pretention|Unstretched rotational spring vector";
      parameter SI.Angle[3] phi_min1=data1.phi_min 
        "|Forces|Bump|rotational vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Angle[3] phi_max1=data1.phi_max 
        "|Forces|Bump|rotational vector to bump stop from reference point in positive direction, resolved in frame_A";
      parameter Real bumpFactor1=data1.bumpFactor 
        "|Forces|Bump|scaleFactor of spring force at after bump";
      
      parameter Real[6, 6] C1=data1.C "|Forces|Linear|Stiffness matrix";
      parameter Real[6, 6] D1=data1.D "|Forces|Linear|Damping matrix";
      
      parameter Boolean linear1=data1.linear 
        "|Advanced||turn on/off linearisation of cosine and sine in rotation vector equation";
      
      parameter Real eps1=data1.eps "|Advanced||Guard against division by zero";
      parameter Real[3] ny2=data2.ny 
        "|General geometry|Orientation|y direction of bushing, resolved in frame_A";
      
      parameter SI.Position[3] r_rela02=data2.r_rela0 
        "|General geometry|Pretention|Unstretched translational spring vector";
      parameter SI.Position[3] r_min2=data2.r_min 
        "|Forces|Bump|vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Position[3] r_max2=data2.r_max 
        "|Forces|Bump|vector to bump stop from reference point in positive direction, resolved in frame_A";
      
      parameter SI.Angle[3] phi_rela02=data2.phi_rela0 
        "|General geometry|Pretention|Unstretched rotational spring vector";
      parameter SI.Angle[3] phi_min2=data2.phi_min 
        "|Forces|Bump|rotational vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Angle[3] phi_max2=data2.phi_max 
        "|Forces|Bump|rotational vector to bump stop from reference point in positive direction, resolved in frame_A";
      parameter Real bumpFactor2=data2.bumpFactor 
        "|Forces|Bump|scaleFactor of spring force at after bump";
      
      parameter Real[6, 6] C2=data2.C "|Forces|Linear|Stiffness matrix";
      parameter Real[6, 6] D2=data2.D "|Forces|Linear|Damping matrix";
      
      parameter Boolean linear2=data2.linear 
        "|Advanced||turn on/off linearisation of cosine and sine in rotation vector equation";
      
      parameter Real eps2=data2.eps "|Advanced||Guard against division by zero";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-40, -10; 40, -90], style(color=0, fillColor=10)), 
          Rectangle(extent=[-40, 88; 40, 8], style(color=0, fillColor=10)), 
          Text(extent=[-130, 138; 138, 96], string="%name")), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram, 
        Documentation(info="<HTML>
<p>
The bushings are linear within a specified range. Outside this range, bumpstops are modellled as a much stiffer spring-dampers.
The range is defined with parameters as well as the ratio between stiffness inside outside range. The stiffness inside the range is read from record like for a normal linear SpringDamper3D.
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  rB1:         Vector to bushing 1.
  rB2:         Vector to bushing 2.
  r_rela01:    Unstretched translational spring vector.
  r_min1:      Vector to bump stop from reference point in negative direction of coordinate system.
  r_max1:      Vector to bump stop from reference point in positive direction of coordinate system.
  bumpFactor1: Scale-factor of spring force at after bump.
  phi_rela01:  Unstretched rotational spring vector.
  r_rela02:    Unstretched translational spring vector.
  r_min2:      Vector to bump stop from reference point in negative direction of coordinate system.
  r_max2:      Vector to bump stop from reference point in positive direction of coordinate system.
  bumpFactor2: Scale-factor of spring force at after bump.
  phi_rela02:  Unstretched rotational spring vector.
  
</pre>
</HTML>
"));
      
      Utilities.Joints.FreeMotion freeMotion
        annotation (extent=[-30, -30; 30, 30]);
      ModelicaAdditions.MultiBody.Parts.FrameAxes FrameAxes2_1(r=rB2)
        annotation (extent=[-80, -50; -40, -10], rotation=270);
      ModelicaAdditions.MultiBody.Parts.FrameAxes FrameAxes2_2(r=rB2)
        annotation (extent=[40, -50; 80, -10], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -14; -90, 16]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_b
        annotation (extent=[90, -14; 120, 16]);
      Utilities.Forces.SpringDamperLinearBump3D bushing1(
        r_rela0=r_rela01, 
        r_min=r_min1, 
        r_max=r_max1, 
        bumpFactor=bumpFactor1, 
        phi_rela0=phi_rela01, 
        C=[1000000, 0, 0, 0, 0, 0; 0, 1000000, 0, 0, 0, 0; 0, 0, 1000000, 0, 0, 
             0; 0, 0, 0, 100, 0, 0; 0, 0, 0, 0, 1, 0; 0, 0, 0, 0, 0, 1], 
        linear=linear1, 
        eps=eps1) annotation (extent=[-30, 28; 30, 92]);
      Utilities.Forces.SpringDamperLinearBump3D bushing2(
        r_rela0=r_rela02, 
        r_min=r_min2, 
        r_max=r_max2, 
        bumpFactor=bumpFactor2, 
        phi_rela0=phi_rela02, 
        C=bushing1.C, 
        D=bushing1.D, 
        linear=linear2, 
        eps=eps2) annotation (extent=[-30, -90; 30, -30]);
      ModelicaAdditions.MultiBody.Parts.FrameAxes FrameAxes1_1(r=rB1)
        annotation (extent=[-80, 10; -40, 50], rotation=90);
      ModelicaAdditions.MultiBody.Parts.FrameAxes FrameAxes1_2(r=rB1)
        annotation (extent=[40, 10; 80, 50], rotation=90);
      parameter Chassis.Data.BumpBushingData data2
        annotation (extent=[-120, -86; -80, -46]);
      parameter Chassis.Data.BumpBushingData data1
        annotation (extent=[-122, 44; -82, 84]);
    equation 
      connect(FrameAxes2_1.frame_a, frame_a)
        annotation (points=[-60, -10; -60, 0; -94, 0], style(thickness=4));
      connect(FrameAxes2_1.frame_b, bushing2.frame_a)
        annotation (points=[-60, -52; -60, -60; -30, -60], style(thickness=4));
      connect(bushing2.frame_b, FrameAxes2_2.frame_b)
        annotation (points=[30, -60; 60, -60; 60, -52], style(thickness=4));
      connect(freeMotion.frame_b, FrameAxes2_2.frame_a)
        annotation (points=[32, 0; 60, 0; 60, -10], style(thickness=4));
      connect(FrameAxes1_1.frame_a, frame_a)
        annotation (points=[-60, 8; -60, 0; -92, 0], style(thickness=4));
      connect(FrameAxes1_1.frame_b, bushing1.frame_a)
        annotation (points=[-60, 54; -60, 60; -28, 60], style(thickness=4));
      connect(FrameAxes1_2.frame_a, freeMotion.frame_b)
        annotation (points=[60, 10; 60, 0; 32, 0], style(thickness=4));
      connect(bushing1.frame_b, FrameAxes1_2.frame_b)
        annotation (points=[32, 60; 60, 60; 60, 52], style(thickness=4));
      connect(frame_b, freeMotion.frame_b)
        annotation (points=[90, 0; 34, 0], style(thickness=4));
      connect(freeMotion.frame_a, frame_a)
        annotation (points=[-30, 0; -100, 0]);
    end PairBushing;
    
    model CutBushing "CutBusing - to be used as a cut in a closed loop." 
      parameter SI.Position[3] rA=data.rA 
        "|General geometry|Position|vector from frame_A to bushing centre";
      parameter SI.Position[3] rB=data.rB 
        "|General geometry|Position|vector from frame_B to bushing centre";
      parameter Real[3] nx=data.nx 
        "|General geometry|Orientation|x direction of bushing, resolved in frame_A";
      parameter Real[3] ny=data.ny 
        "|General geometry|Orientation|y direction of bushing, resolved in frame_A";
      
      parameter SI.Position[3] r_rela0=data.r_rela0 
        "|General geometry|Pretention|Unstretched translational spring vector";
      parameter SI.Position[3] r_min=data.r_min 
        "|Forces|Bump|vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Position[3] r_max=data.r_max 
        "|Forces|Bump|vector to bump stop from reference point in positive direction, resolved in frame_A";
      
      parameter SI.Angle[3] phi_rela0=data.phi_rela0 
        "|General geometry|Pretention|Unstretched rotational spring vector";
      parameter SI.Angle[3] phi_min=data.phi_min 
        "|Forces|Bump|rotational vector to bump stop from reference point in negative direction, resolved in frame_A";
      parameter SI.Angle[3] phi_max=data.phi_max 
        "|Forces|Bump|rotational vector to bump stop from reference point in positive direction, resolved in frame_A";
      parameter Real bumpFactor=data.bumpFactor 
        "|Forces|Bump|scaleFactor of spring force at after bump";
      
      parameter Real[6, 6] C=data.C "|Forces|Linear|Stiffness matrix";
      parameter Real[6, 6] D=data.D "|Forces|Linear|Damping matrix";
      
      parameter Boolean linear=data.linear 
        "|Advanced||turn on/off linearisation of cosine and sine in rotation vector equation";
      
      parameter Real eps=data.eps "|Advanced||Guard against division by zero";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-78, 74; 82, -86], style(color=0, fillColor=9)), 
          Line(points=[0, 0; 0, -58], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Line(points=[0, 0; 58, 0], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Polygon(points=[0, -60; 6, -50; -6, -50; 0, -60], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Polygon(points=[60, 0; 52, -6; 52, 6; 60, 0], style(
              color=7, 
              thickness=2, 
              fillColor=7)), 
          Ellipse(extent=[-22, 20; 20, -20], style(color=7, thickness=2)), 
          Rectangle(extent=[-24, 20; -2, 0], style(color=9, fillColor=9)), 
          Polygon(points=[-4, 20; 2, 14; 2, 26; -4, 20], style(
              color=7, 
              thickness=2, 
              fillColor=7))), 
        Window(
          x=0.16, 
          y=0.13, 
          width=0.69, 
          height=0.71), 
        Diagram, 
        Documentation(info="<HTML>
<p>
The bushing is linear within a specified range. Outside this range, bumpstops are modellled as a much stiffer spring-dampers.
The range is defined with parameters as well as the ratio between stiffness inside outside range. The stiffness inside the range is read from record like for a normal linear SpringDamper3D.
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  r_rela0:    Unstretched translational spring vector.
  r_min:      Vector to bump stop from reference point in negative direction of coordinate system.
  r_max:      Vector to bump stop from reference point in positive direction of coordinate system.
  bumpFactor: Scale-factor of spring force at after bump.
  phi_rela0:  Unstretched rotational spring vector.
  
</pre>
</HTML>
"));
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -14; -90, 16]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_b
        annotation (extent=[118, -16; 88, 14]);
      Utilities.Forces.SpringDamperLinearBump3D springDamper(
        r_rela0=r_rela0, 
        r_min=r_min, 
        r_max=r_max, 
        bumpFactor=bumpFactor, 
        phi_rela0=phi_rela0, 
        C=C, 
        D=D, 
        linear=linear, 
        eps=eps) annotation (extent=[-40, -40; 40, 40]);
      ModelicaAdditions.MultiBody.Parts.FrameAxes r_A(
        r=rA, 
        nx=nx, 
        ny=ny) annotation (extent=[-80, -10; -60, 10]);
      ModelicaAdditions.MultiBody.Parts.FrameAxes r_B(
        nx=nx, 
        ny=ny, 
        r=rB) annotation (extent=[60, -10; 80, 10], rotation=180);
      parameter Chassis.Data.BumpBushingData data
        annotation (extent=[40, 40; 80, 80]);
    equation 
      connect(r_A.frame_a, frame_a)
        annotation (points=[-80, 0; -94, 0], style(thickness=4));
      connect(r_A.frame_b, springDamper.frame_a)
        annotation (points=[-60, 0; -40, 0], style(thickness=4));
      connect(r_B.frame_b, springDamper.frame_b)
        annotation (points=[58, 0; 42, 0], style(thickness=4));
      connect(r_B.frame_a, frame_b)
        annotation (points=[80, 0; 100, 0], style(thickness=4));
    end CutBushing;
    
    model SedanBody 
      "Body and body shape of a car. by default a middle sized sedan car with BMW 3-series shape."
       
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Icon(
          Rectangle(extent=[-78, 52; 86, -46], style(color=41, fillColor=41)), 
          Polygon(points=[-30, 38; -30, -36; 2, -22; 2, 32; -30, 38; -30, 38], 
              style(color=0, fillColor=8)), 
          Polygon(points=[-16, -36; 8, -24; 64, -24; 80, -40; -14, -36; -16, -36], 
               style(color=0, fillColor=8)), 
          Polygon(points=[-16, 42; 6, 34; 70, 34; 80, 44; 80, 44; -14, 42; -16, 
                 42], style(color=0, fillColor=8)), 
          Polygon(points=[68, 26; 68, -18; 82, -36; 82, 40; 68, 26], style(
                color=0, fillColor=8))), 
        Documentation(info="<HTML>
<p>
This component contains the visual information of a BMW 3 series sedan.
</p>
</HTML>
"), 
        Diagram);
      
      Chassis.Data.BodyData data(
        r0={-.72,0,0.35}, 
        lengthDirection={0,-1,0}, 
        widthDirection={1,0,0}, 
        length=0.002, 
        width=0.002, 
        height=0.002, 
        extra=1, 
        rcmB={-2.55*0.4,0,2.55*0.12}, 
        mB=1400, 
        i11B=500, 
        i22B=1500, 
        i33B=1800, 
        shape="2") "Uses a dxf file of a BMW 3 series sedan"
        annotation (extent=[-60, 40; -40, 60]);
      parameter SI.Position rcmB[3]=data.rcmB 
        "|Body|Vector from frame_a to center of mass, resolved in frame_a";
      parameter SI.Mass mB=data.mB "|Body|Mass of body";
      parameter SI.Inertia i11B=data.i11B 
        "|Body|(1,1) element of inertia tensor";
      parameter SI.Inertia i22B=data.i22B 
        "|Body|(2,2) element of inertia tensor";
      parameter SI.Inertia i33B=data.i33B 
        "|Body|(3,3) element of inertia tensor";
      parameter SI.Inertia i21B=data.i21B 
        "|Body|(2,1) element of inertia tensor";
      parameter SI.Inertia i31B=data.i31B 
        "|Body|(3,1) element of inertia tensor";
      parameter SI.Inertia i32B=data.i32B 
        "|Body|(3,2) element of inertia tensor";
      
      parameter String shape=data.shape "|Shape|Name of shape";
      parameter SI.Position r0[3]=data.r0 
        "|Shape|Vector from frame_a to shape origin. resolved in frame_a";
      parameter SI.Position lengthDirection[3]=data.lengthDirection 
        "|Shape|Vector in length direction, resolved in frame_a";
      parameter SI.Position widthDirection[3]=data.widthDirection 
        "|Shape|Vector in width direction, resolved in frame_a";
      parameter SI.Length length=data.length "|Shape|length of shape";
      parameter SI.Length width=data.width "width of shape|Shape|";
      parameter SI.Length height=data.height "height of shape|Shape|";
      parameter Real material[4]=data.material 
        "|Shape|Color and specular coefficient";
      parameter Real extra=data.extra 
        "|Shape|Additional parameter for cone and pipe";
      
      Utilities.Parts.Body shapeBody(
        m=mB, 
        I11=i11B, 
        I22=i22B, 
        I33=i33B, 
        I21=i21B, 
        I31=i31B, 
        I32=i32B, 
        rCM=rcmB, 
        shape=shape, 
        r0=r0, 
        widthDirection=widthDirection, 
        length=length, 
        width=width, 
        height=height, 
        material=material, 
        extra=extra, 
        lengthDirection=lengthDirection) annotation (extent=[-20, -20; 20, 20]);
      annotation (Diagram);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a1
        annotation (extent=[-112, -16; -80, 16]);
    equation 
      connect(shapeBody.frame_a, frame_a1) annotation (points=[-20, 0; -80, 0], 
           style(fillColor=1, fillPattern=1));
      
    end SedanBody;
    
    model AckermannSteering 
      "AckermannSteering - steers the right and left wheel according to the Ackermann geometry."
       
      
      parameter Real wheelbase=2.55 "Distance from front to rear axis";
      parameter Real trackwidth=1.6 "Distance between the front wheels";
      parameter Real ratio=1 
        "Gear ratio to be used to compensate for the ratio in a steering linkage, should be angle on steering wheel/steer angle";
    protected 
      parameter Real b=wheelbase;
      parameter Real tw=trackwidth;
      
      Real tand;
      Real delta;
      Real delta_1;
      Real delta_2;
      Real ratio_1;
      Real ratio_2;
      //  Real w_1;
      //  Real w_2;
      //  Real w;
      
      //SI.Position r1,r1d,r2,r2d;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[62, 40; 72, -40], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-80, 60; 40, -60], style(color=0, fillColor=51)), 
          Rectangle(extent=[4, -6; 22, -14], style(color=10, fillColor=10)), 
          Rectangle(extent=[4, 44; 22, 36], style(color=10, fillColor=10)), 
          Line(points=[12, -52; -56, 40]), 
          Polygon(points=[-62, 32; -48, 42; -52, 48; -66, 38; -62, 32], style(
                color=10, fillColor=10)), 
          Line(points=[12, -52; -56, -10]), 
          Polygon(points=[-64, -14; -54, 0; -48, -4; -58, -18; -64, -14], style(
                color=10, fillColor=10)), 
          Rectangle(extent=[40, 4; 62, -4], style(gradient=2, fillColor=8)), 
          Text(extent=[-76, -22; -44, -58], string="L")), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.55, 
          height=0.95), 
        Diagram(Line(points=[-50, -80; 0, -86; 22, -40; 22, 20; 2, 66; -50, 80])), 
           
        Documentation(info="<HTML>
<p>
A geometrical approach shows that the outer wheel should turn less when the car turns.
The inner wheel should turn with an angle delta_i=atan(L/R) while the outer wheel should turn with a smaller angle delta_o=atan(L/(b+R)) where R is the turning radius of the inner wheel, b is the trackwidth and L is the wheelbase.
This effect is often reffered to as Ackermann steering but was devided by Lankensperger.
This model assumes that the driver steers a wheel in the midle of the car and from this, the inner and outer turning angles are calculated.
</p>
</HTML>
"));
      
    public 
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a
        annotation (extent=[80, -16; 110, 16]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b1 annotation (
        extent=[-50, -90; -70, -70], 
        rotation=90, 
        layer="icon");
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b2 annotation (
        extent=[-50, 70; -70, 90], 
        rotation=90, 
        layer="icon");
    equation 
      // geometric equations for steering angles
      flange_b1.phi = delta_1;
      flange_b2.phi = delta_2;
      flange_a.phi = delta;
      
      tand = tan(ratio*delta);
      //delta_1 = atan(2*wheelbase*tand/(2*wheelbase - trackwidth*tand));
      //delta_2 = atan(2*wheelbase*tand/(2*wheelbase + trackwidth*tand));
      
      //  w_1 = der(delta_1);
      //  w_2 = der(delta_2);
      //  w = der(delta);
      
      //  w_1 = ratio_1*w;
      //  w_2 = ratio_2*w;
      
      delta_1 = (ratio*ratio_1)*delta;
      delta_2 = (ratio*ratio_2)*delta;
      
      // Torque equations  
      0 = flange_a.tau + ratio*ratio_1*flange_b1.tau + ratio*ratio_2*flange_b2.
        tau;
      
      // Derivatives delta_1/delta and delta_2/delta
      ratio_1 = 4*b*b*(1 + tand*tand)/(4*b^2 - 4*b*tw*tand + tw*tw*tand^2 + 4*b
        *b*tand^2);
      ratio_2 = 4*b*b*(1 + tand*tand)/(4*b^2 + 4*b*tw*tand + tw*tw*tand^2 + 4*b
        *b*tand^2);
      
    end AckermannSteering;
    
    model TrailingArm 
      parameter Real[3] scaleFactor={1,1,1} "|Geometry|";
      parameter SI.Position[3] rCU={0.34,0.72,0.01} "|Geometry|";
      parameter SI.Position[3] rUW={0.0,0.9,0} "|Geometry|";
      parameter Real[3] nCU={0.11,1,-0.001} "|Geometry|";
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|";
      parameter SI.Mass mU=1 "|Mass and Inertia|";
      parameter SI.Inertia i11U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i22U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i33U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i21U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i31U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i32U=.01 "|Mass and Inertia|";
      
    protected 
      parameter SI.Position[3] rCU_scaled=Utilities.Functions.EWM(scaleFactor, 
          rCU);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      parameter Real[3] nCU_scaled=Utilities.Functions.EWM(scaleFactor, nCU);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Line(points=[-80, 0; -90, 0], style(color=10)), 
          Line(points=[58, -42; 0, -42], style(color=10)), 
          Rectangle(extent=[-46, -20; -32, -60], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[40, 20; 54, -20], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[0, -90; 0, -42], style(color=10)), 
          Rectangle(extent=[-42, 42; 72, 14], style(pattern=0, fillColor=0)), 
          Polygon(points=[-42, 14; 42, 14; -26, 0; -42, -28; -42, 14], style(
                pattern=0, fillColor=0)), 
          Rectangle(extent=[-80, 48; -42, -46], style(
              pattern=0, 
              gradient=1, 
              fillColor=9)), 
          Line(points=[58, -42; 58, 14], style(color=10))), 
        Diagram, 
        Window(
          x=0.33, 
          y=0.14, 
          width=0.56, 
          height=0.83), 
        Documentation(info="A so called wishbone (A-arm) suspesnion component
- Left side shall be connected to chassis
- Right side shall be connected to wheel carrier, the so called
spindle
- Lower side intefrace might be used to attach spring,
shockabsorber or anti roll bar for example to the wishbone
Any bushings needed in model are to be attched outside the
wishbone. The chassis side needs a PairBushing to make a standard
wishbone configuration 
"));
      
    public 
      ModelicaAdditions.MultiBody.Joints.Revolute innerJoint(
        n=nCU_scaled, 
        q(start=0), 
        qd(start=0)) annotation (extent=[-60, -10; -40, 10]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation(r=
            rUW_scaled - rCU_scaled) annotation (extent=[-20, -10; 0, 10]);
      ModelicaAdditions.MultiBody.Parts.Body body(rCM={-0.1,0,0}, m=10)
        annotation (extent=[-20, 20; 0, 40]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_U
        annotation (extent=[-16, -118; 14, -88], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-120, -15; -90, 15], layer="icon");
      Utilities.Parts.Shape rearEnd(
        widthDirection=nCU_scaled, 
        length=0.17, 
        width=sqrt((rUW_scaled - rCU_scaled)*(rUW_scaled - rCU_scaled))/20, 
        height=sqrt((rUW_scaled - rCU_scaled)*(rUW_scaled - rCU_scaled))/10, 
        material={0.1,0.1,0.1,1}, 
        lengthDirection=Utilities.Functions.EWM({1,-1,1}, (rUW_scaled - 
            rCU_scaled))) annotation (extent=[22, 40; 42, 60]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation2(r=
            rCU_scaled) annotation (extent=[-88, -10; -68, 10]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis
        annotation (extent=[-74, 48; -48, 74]);
      Utilities.Parts.Shape shape(
        widthDirection={0,0,1}, 
        width=0.07, 
        height=0.03, 
        material={0.1,0.2,0.2,0.5}, 
        lengthDirection=rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0}, 
        length=sqrt((rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0})*(
            rUW_scaled - rCU_scaled + {0,rCU_scaled[2]*0.25,0})), 
        r0=-{0,rCU_scaled[2]*0.25,0}) annotation (extent=[20, -40; 40, -20]);
    equation 
      connect(innerJoint.frame_b, frameTranslation.frame_a)
        annotation (points=[-39.5, 0; -20.5, 0], style(thickness=4));
      connect(body.frame_a, innerJoint.frame_b) annotation (points=[-20.5, 30; 
            -34.5, 30; -34.5, 0; -39.5, 0], style(thickness=4));
      connect(frameTranslation.frame_b, frame_U) annotation (points=[0, 0; 74, 
            0; 74, -103; -1, -103], style(thickness=4));
      connect(rearEnd.frame_a, frameTranslation.frame_b)
        annotation (points=[21.5, 50; 10, 50; 10, 0; 0, 0]);
      connect(frameTranslation2.frame_b, innerJoint.frame_a)
        annotation (points=[-68, 0; -60, 0]);
      connect(frameTranslation2.frame_a, frame_C)
        annotation (points=[-88, 0; -100, 0]);
      connect(innerJoint.axis, axis)
        annotation (points=[-50, 6; -60, 48], style(color=0));
      connect(shape.frame_a, innerJoint.frame_b) annotation (points=[20, -30; -40, 
             0], style(fillColor=76, fillPattern=1));
    end TrailingArm;
    
    model Strut "Strut - Integrated spring-damper unit" 
      parameter Real[3] scaleFactor={1,1,1};
      parameter SI.Position[3] rA={.0,-.05,0.05};
      parameter SI.Position[3] rB={0.1,0.1,.3};
      
      parameter Real s0=0 "Unstreched strut length";
      
    protected 
      parameter SI.Position[3] rA_scaled=Utilities.Functions.EWM(scaleFactor, 
          rA);
      parameter SI.Position[3] rB_scaled=Utilities.Functions.EWM(scaleFactor, 
          rB);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Ellipse(extent=[50, 10; 68, -10], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[-90, 8; -70, -8], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[68, 8; 88, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[-80, -20; -40, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, -16; -44, 16], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-54, 20; -40, -20], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[40, -20; 80, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[44, -16; 76, 16], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[40, 20; 54, -20], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[36, 0; 32, -20; 28, -20; 32, 0; 36, 0], style(color=
                  10, fillColor=9)), 
          Polygon(points=[24, 20; 20, 0; 16, 0; 20, 20; 24, 20], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[20, 0; 16, -20; 12, -20; 16, 0; 20, 0], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[8, 20; 4, 0; 0, 0; 4, 20; 8, 20], style(color=9, 
                fillColor=9)), 
          Polygon(points=[4, 0; 0, -20; -4, -20; 0, 0; 4, 0], style(color=9, 
                fillColor=9)), 
          Polygon(points=[-8, 20; -12, 0; -16, 0; -12, 20; -8, 20], style(color
                =9, fillColor=9)), 
          Polygon(points=[-12, 0; -16, -20; -20, -20; -16, 0; -12, 0], style(
                color=9, fillColor=9)), 
          Polygon(points=[-24, 20; -28, 0; -32, 0; -28, 20; -24, 20], style(
                color=9, fillColor=9)), 
          Polygon(points=[-28, 0; -32, -20; -36, -20; -32, 0; -28, 0], style(
                color=9, fillColor=9)), 
          Rectangle(extent=[-4, 16; 40, -16], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-40, 10; -4, -10], style(gradient=2, fillColor=8)), 
             
          Polygon(points=[-24, 20; -20, 0; -24, 0; -28, 20; -24, 20], style(
                color=8, fillColor=8)), 
          Polygon(points=[-20, 0; -16, -20; -20, -20; -24, 0; -20, 0], style(
                color=8, fillColor=8)), 
          Polygon(points=[-8, 20; -4, 0; -8, 0; -12, 20; -8, 20], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[-4, 0; 0, -20; -4, -20; -8, 0; -4, 0], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[8, 20; 12, 0; 8, 0; 4, 20; 8, 20], style(color=8, 
                fillColor=8)), 
          Polygon(points=[12, 0; 16, -20; 12, -20; 8, 0; 12, 0], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[24, 20; 28, 0; 24, 0; 20, 20; 24, 20], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[28, 0; 32, -20; 28, -20; 24, 0; 28, 0], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[-36, 0; -32, -20; -36, -20; -40, 0; -36, 0], style(
                color=8, fillColor=8)), 
          Rectangle(extent=[-58, 8; -40, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[-70, 10; -52, -10], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[40, 8; 60, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[50, 10; 68, -10], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(extent=[-86, 74; -52, 30], string="A")), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram, 
        Documentation(info="<HTML>
<p>
The damper is mounted inside the coil spring which gives a compact component. The force generation is modelled from table values
</p>
<p>
Advantages:
Compact.
</p>
<p>
Disadvantages:
Mounting and Demounting is sometimes more timeconsuming than with separate components.
</p>
<p>
Usage:
Front suspension in many cars.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} mirrors it around the xz-plane.
  rA: location of strut end A, resolved in frame_A.
  rB: location of strut end B, resolved in frame_B.
  s0: unstretched strut length. Not affected by scaleFactor.
  forceTable: To set the force generation table add a component named [name] of class Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_b
        annotation (extent=[88, 14; 118, -16], rotation=180);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -14; -90, 16]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslationA(r=
            rA_scaled) annotation (extent=[-80, 10; -60, -10]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslationB(r=
            rB_scaled) annotation (extent=[60, -10; 80, 10], rotation=180);
      annotation (extent=[-20, -20; 20, 20]);
      parameter Utilities.Forces.Utilities.ForceTable1D data(fd=0.8*[-1, 10000; 
             -0.5, 3300; 0, 0; 0.5, -3300; 1, -10000], fc=8*[-1, 10000; -0.5, 
            3300; 0, 0; 0.5, -3300; 1, -10000])
        annotation (extent=[58, 56; 98, 96]);
      Utilities.Forces.SpringDamperTableLine3D springDamper(data=data, s_rel0=
            s0) annotation (extent=[-20, -22; 20, 20]);
      Utilities.Parts.Shape shapeA(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[-50, -44; -30, -24]);
      Utilities.Parts.Shape shapeB(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[54, -48; 74, -28]);
    equation 
      connect(frameTranslationA.frame_a, frame_a) annotation (points=[-80.5, 0; 
             -94, 0; -94, 2; -98, 2], style(thickness=4));
      connect(frameTranslationB.frame_a, frame_b)
        annotation (points=[80.5, -1.33227e-015; 100, 0], style(thickness=4));
      connect(springDamper.frame_a, frameTranslationA.frame_b)
        annotation (points=[-20, 0; -60, 0]);
      connect(springDamper.frame_b, frameTranslationB.frame_b)
        annotation (points=[20, 0; 60, 0]);
      connect(shapeA.frame_a, frameTranslationA.frame_b)
        annotation (points=[-50, -34; -58, 0]);
      connect(shapeB.frame_a, frameTranslationB.frame_b)
        annotation (points=[52, -38; 60, 0]);
    end Strut;
    
    model FormulaBody 
      "Essentially the same as the SedanBody but with default values from a formula car."
       
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Icon(
          Rectangle(extent=[-10, 20; 96, -20], style(color=41, fillColor=43)), 
          Polygon(points=[-10, -22; -2, -40; 20, -48; 58, -48; 70, -20; -10, -20; 
                 -10, -22], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Polygon(points=[-10, 20; 70, 20; 58, 46; 18, 46; -2, 40; -10, 22; -10, 
                 20], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Polygon(points=[-54, 48; -54, -46; -72, -42; -72, 44; -54, 48], style(
                color=41, fillColor=41)), 
          Polygon(points=[-8, 20; -78, 10; -78, -8; -8, -20; -8, 20], style(
                color=41, fillColor=43)), 
          Polygon(points=[-72, 6; -72, -4; 8, -12; 14, -16; 48, -16; 56, -12; 
                82, -8; 82, 6; 56, 10; 48, 16; 14, 16; 8, 12; -72, 6], style(
                color=41, fillColor=41)), 
          Ellipse(extent=[14, 12; 48, -12], style(color=8, fillColor=10)), 
          Polygon(points=[82, 32; 82, -32; 90, -40; 106, -40; 110, -20; 110, 20; 
                 106, 40; 106, 40; 90, 40; 82, 32; 82, 32], style(color=41, 
                fillColor=41))), 
        Documentation(info="<HTML>
<p>
This component contains the visual information of a BMW 3 series sedan.
</p>
</HTML>
"), 
        Diagram);
      
      Chassis.Data.BodyData data(
        extra=1, 
        mB=530, 
        i11B=45, 
        i22B=470, 
        i33B=514, 
        shape="1", 
        width=15/18, 
        height=15/18, 
        lengthDirection={1,0,-0.03}, 
        r0={-0.04,0,-0.2}, 
        length=-78/90, 
        rcmB={-1.4,0,0.07}) "Uses a dxf file of a formula car"
        annotation (extent=[-60, 40; -40, 60]);
      
      parameter SIunits.Position rcmB[3]=data.rcmB 
        "|Body|Vector from frame_a to center of mass, resolved in frame_a";
      parameter SIunits.Mass mB=data.mB "|Body|Mass of body";
      parameter SIunits.Inertia i11B=data.i11B 
        "|Body|(1,1) element of inertia tensor";
      parameter SIunits.Inertia i22B=data.i22B 
        "|Body|(2,2) element of inertia tensor";
      parameter SIunits.Inertia i33B=data.i33B 
        "|Body|(3,3) element of inertia tensor";
      parameter SIunits.Inertia i21B=data.i21B 
        "|Body|(2,1) element of inertia tensor";
      parameter SIunits.Inertia i31B=data.i31B 
        "|Body|(3,1) element of inertia tensor";
      parameter SIunits.Inertia i32B=data.i32B 
        "|Body|(3,2) element of inertia tensor";
      
      parameter String shape=data.shape "|Shape|Name of shape";
      parameter SIunits.Position r0[3]=data.r0 
        "|Shape|Vector from frame_a to shape origin. resolved in frame_a";
      parameter SIunits.Position lengthDirection[3]=data.lengthDirection 
        "|Shape|Vector in length direction, resolved in frame_a";
      parameter SIunits.Position widthDirection[3]=data.widthDirection 
        "|Shape|Vector in width direction, resolved in frame_a";
      parameter SIunits.Length length=data.length "|Shape|length of shape";
      parameter SIunits.Length width=data.width "width of shape|Shape|";
      parameter SIunits.Length height=data.height "height of shape|Shape|";
      parameter Real material[4]=data.material 
        "|Shape|Color and specular coefficient";
      parameter Real extra=data.extra 
        "|Shape|Additional parameter for cone and pipe";
      
      Utilities.Parts.Body shapeBody(
        m=mB, 
        I11=i11B, 
        I22=i22B, 
        I33=i33B, 
        I21=i21B, 
        I31=i31B, 
        I32=i32B, 
        rCM=rcmB, 
        shape=shape, 
        r0=r0, 
        widthDirection=widthDirection, 
        length=length, 
        width=width, 
        height=height, 
        material=material, 
        extra=extra, 
        lengthDirection=lengthDirection) annotation (extent=[-20, -20; 20, 20]);
      annotation (Diagram);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a1
        annotation (extent=[-112, -16; -80, 16]);
    equation 
      connect(shapeBody.frame_a, frame_a1) annotation (points=[-20, 0; -80, 0], 
           style(fillColor=1, fillPattern=1));
      
    end FormulaBody;
    
    model DoubleWishBone "MacPherson strut linkage" 
      import SI = Modelica.SIunits;
      
      extends Chassis.Interfaces.Interface;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      parameter SI.Position[3] rCL3={0.1070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to front link mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rCL4={-0.3070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to rear link mount in chassis resolved in frame_C (at initial time)";
      parameter Real[3] rCL1={0.12,0.55,-0.14} 
        "Vector to low front link abutment resolved in frame_C";
      parameter Real[3] rCL2={-0.125,0.55,-0.14} 
        "Vector to low rear link abutment resolved in frame_C";
      parameter Real[3] rUL1L2={0.0,0.85,-0.14} 
        "Vector to low spindle joint resolved in frame_C";
      parameter SI.Position[3] rUL3L4={-0.0070,0.8733,-0.0380} 
        "|Geometry| Vector from origin of frame_C to low spindleUtilities.Joints.Joint resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUW={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to wheel centre resolved in frame_C (at initial time)";
      parameter SI.Position[3] rRL5={-0.12,0.85,0.08} 
        "|Geometry| Vector from origin of frame_C to origin of frame_S resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUL5={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to lower ball of steering rod resolved in frame_C (at initial time)";
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|Wheel carrier|";
      parameter SI.Mass mU=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i11U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i22U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i33U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i21U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i31U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i32U=0 "|Mass and Inertia|Wheel carrier|";
      
      /*  parameter SI.Position[3] rCML3L4={0,0,0} "|Mass and Inertia|LowerWishbone|";
  parameter SI.Mass mL3L4=0 "|Mass and Inertia|Lower Wishbone|";
  parameter SI.Inertia i11L3L4=.0 "|Mass and Inertia|LowerWishbone|";
  parameter SI.Inertia i22L3L4=.0 "|Mass and Inertia|LowerWishbone|";
  parameter SI.Inertia i33L3L4=.0 "|Mass and Inertia|LowerWishbone|";
  parameter SI.Inertia i21L3L4=.0 "|Mass and Inertia|LowerWishbone|";
  parameter SI.Inertia i31L3L4=.0 "|Mass and Inertia|LowerWishbone|";
  parameter SI.Inertia i32L3L4=.0 "|Mass and Inertia|LowerWishbone|";
  
  parameter SI.Position[3] rCML1L2={0,0,0} "|Mass and Inertia|Wishbone|";
  parameter SI.Mass mL1L2=0 "|Mass and Inertia|Wishbone|";
  parameter SI.Inertia i11L1L2=.0 "|Mass and Inertia|Upper Wishbone|";
  parameter SI.Inertia i22L1L2=.0 "|Mass and Inertia|Upper Wishbone|";
  parameter SI.Inertia i33L1L2=.0 "|Mass and Inertia|Upper Wishbone|";
  parameter SI.Inertia i21L1L2=.0 "|Mass and Inertia|Upper Wishbone|";
  parameter SI.Inertia i31L1L2=.0 "|Mass and Inertia|Upper Wishbone|";
  parameter SI.Inertia i32L1L2=.0 "|Mass and Inertia|Upper Wishbone|";
*/
      /*Scaled coordinates*/
    protected 
      parameter SI.Position[3] rCL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL3);
      parameter SI.Position[3] rCL4_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL4);
      parameter Real[3] rCL1_scaled=Utilities.Functions.EWM(scaleFactor, rCL1);
      parameter Real[3] rCL2_scaled=Utilities.Functions.EWM(scaleFactor, rCL2);
      parameter Real[3] rUL1L2_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUL1L2);
      parameter SI.Position[3] rUL3L4_scaled=Utilities.Functions.EWM(
          scaleFactor, rUL3L4);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      parameter SI.Position[3] rRL5_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL5);
      parameter SI.Position[3] rUL5_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL5);
      parameter SI.Position rU[3]=rUW_scaled - rUL3L4_scaled 
        "Position vector from frame_L3L4 to frame_U, resolved in frame_C";
      parameter Real n_a[3]=cross(rCL1, rU) 
        "First rotation axis of universalUtilities.Joints.Joint in springJoint, resolved in frame_C";
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_L3L4
        annotation (extent=[-16, -118; 14, -88], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_S
        annotation (extent=[-70, 50; -100, 80], rotation=180);
      Utilities.Parts.Body uprightShape(r=rUL1L2_scaled - rUL3L4_scaled)
        annotation (extent=[15, -44; 35, -24], rotation=90);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation upper(r=rCL1_scaled)
        annotation (extent=[-67, 10; -87, 30], rotation=90);
      Utilities.Joints.JointUS uprightJoint(n_a=n_a, L=sqrt((rUL1L2_scaled - 
            rUL3L4_scaled)*(rUL1L2_scaled - rUL3L4_scaled)))
        annotation (extent=[14, -14; -16, 16], rotation=90);
      Utilities.Parts.Body outerRod(r=rUW_scaled - rUL3L4_scaled)
        annotation (extent=[65, -10; 85, 10], rotation=0);
      ModelicaAdditions.MultiBody.Parts.Body U(
        rCM=rCMU, 
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U) annotation (extent=[66, -60; 86, -40]);
      Utilities.Parts.Body steeringRod(
        material={0,0,1,1}, 
        r=rUL5_scaled - rRL5_scaled, 
        shape="cylinder", 
        height=0.03, 
        animation=true) 
        "{sqrt((rUL3L4_scaled - rRL5_scaled)*(rUL3L4_scaled - rRL5_scaled)),0,0}"
        annotation (extent=[-10, 91; 10, 71], rotation=180);
      Utilities.Joints.JointRSU steeringJoint(
        n_b={1,0,0}, 
        n_a=rUL1L2_scaled - rUL3L4_scaled, 
        r_a=rUL5_scaled - rUL3L4_scaled)
        annotation (extent=[50, 40; 80, 70], rotation=90);
      ModelicaAdditions.MultiBody.Joints.Revolute lowerInnerJoint(
        q(stateSelect=StateSelect.always), 
        qd(stateSelect=StateSelect.always), 
        n=rCL3_scaled - rCL4_scaled) annotation (extent=[-70, -50; -50, -30]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation lower(r=rCL3_scaled)
        annotation (extent=[-87, -30; -67, -10], rotation=270);
      Utilities.Parts.Body lowerFrontBar(r=rUL3L4_scaled - rCL3_scaled, height=
            0.03) annotation (extent=[-40, -50; -20, -30]);
      Utilities.Parts.Body lowerRearBar(
        r=rCL4_scaled - rUL3L4_scaled, 
        widthDirection=cross(rCL4_scaled - rUL3L4_scaled, {0,0,1}), 
        height=0.03) annotation (extent=[-49, -6; -29, -26], rotation=180);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-44, -42; -18, -22], style(color=0, thickness=4)), 
          Ellipse(extent=[28, -36; 48, -16], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, -34; 46, -18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, -16; 50, -24], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[28, 19; 48, 39], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, 21; 46, 37], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, 27; 50, 19], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-58, -44; 30, -28; -58, -18], style(color=0, thickness=4)), 
             
          Ellipse(extent=[-58, -28; -38, -8], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, -26; -40, -10], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-60, -8; -50, -28], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-54, -12; -42, -24], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-78, -56; -58, -36], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, -54; -60, -38], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-80, -36; -70, -56], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-74, -40; -62, -52], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-2, -88; -2, -64; 28, -64; 28, -28], style(color=0)), 
          Polygon(points=[42, -12; 60, -12; 64, 4; 90, 4; 90, -2; 68, -2; 64, -18; 
                 42, -18; 42, -12], style(color=8, fillColor=8)), 
          Polygon(points=[66, 34; 59, 36; 47, -12; 55, -12; 66, 34], style(
              pattern=0, 
              gradient=2, 
              fillColor=8)), 
          Polygon(points=[52, 96; 64, 80; 62, 86; 70, 62; -100, 106; 52, 96], 
              style(pattern=0)), 
          Rectangle(extent=[34, 27; 42, -27], style(color=8, fillColor=8)), 
          Ellipse(extent=[32, 35; 44, 23], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[32, -20; 44, -32], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-39, 16; -18, 33], style(color=0, thickness=4)), 
          Line(points=[-57, 13; 38, 29; -58, 36], style(color=0, thickness=4)), 
             
          Ellipse(extent=[-58, 26; -38, 46], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 28; -40, 44], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 46; -50, 26], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-54, 42; -42, 30], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-77, 4; -57, 24], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-75, 6; -59, 22], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-78, 25; -68, 3], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-73, 20; -61, 8], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-60, 56; -40, 76], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-58, 58; -42, 74], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-47, 72; -33, 46], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 72; -44, 60], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[55, 33; 75, 53], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[57, 35; 73, 51], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[47, 59; 59, 35], style(color=7, fillColor=7)), 
          Ellipse(extent=[59, 49; 71, 37], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[64, 43; 64, 43; -48, 66], style(color=0, thickness=4))), 
           
        Diagram(
          Line(points=[-90, 0; -77, -40], style(color=1, arrow=1)), 
          Text(
            extent=[-97, -35; -77, -42], 
            style(color=1, arrow=1), 
            string="rCL3"), 
          Line(points=[-90, 0; 0, -40], style(color=1, arrow=1)), 
          Text(
            extent=[-20, 36; -5, 29], 
            style(color=1, arrow=1), 
            string="rUL1L2"), 
          Text(
            extent=[-99, 36; -72, 30], 
            style(color=1, arrow=1), 
            string="rCL1"), 
          Line(points=[-90, 0; -77, 40], style(color=1, arrow=1)), 
          Text(
            extent=[-12, 55; 12, 48], 
            style(color=1, arrow=1), 
            string="rRL3"), 
          Line(points=[-90, 0; 65, 80], style(color=1, arrow=1)), 
          Line(points=[-90, 0; 61, 54], style(color=1, arrow=1)), 
          Text(
            extent=[21, 51; 44, 44], 
            style(color=1, arrow=1), 
            string="rUL3"), 
          Text(
            extent=[64, 22; 91, 16], 
            style(color=1, arrow=1), 
            string="rUW"), 
          Line(points=[-90, 0; 90, 15], style(color=1, arrow=1)), 
          Text(
            extent=[-17, -27; 0, -33], 
            style(color=1, arrow=1), 
            string="rUL3L4"), 
          Line(points=[-90, 0; 0, 39], style(color=1, arrow=1))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUL1L2: location of outer ballUtilities.Joints.Joint, resolved in frame_C
  rCL1:   location of inner frontUtilities.Joints.Joint, resolved in frame_C
  rCL4:   location of inner rearUtilities.Joints.Joint, resolved in frame_C
  rCS:    location of strut mount in chassis, resolved in frame_C
  rUS:    location of strut mount in upright, resolved in frame_C
  rUW:    location of frame_W, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    protected 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_ta1
        annotation (extent=[54, -11; 68, -24]);
    public 
      ModelicaAdditions.MultiBody.Joints.Revolute upperInnerJoint(n=rCL1_scaled
             - rCL2_scaled) annotation (extent=[-70, 30; -50, 50]);
      Utilities.Parts.Body upperFrontBar(r=rUL1L2_scaled - rCL1_scaled, height=
            0.03) annotation (extent=[-40, 30; -20, 50]);
      Utilities.Parts.Body upperRearBar(
        r=rCL2_scaled - rUL1L2_scaled, 
        widthDirection=cross(rCL2_scaled - rUL1L2_scaled, {0,0,1}), 
        height=0.03) annotation (extent=[-44, 73; -24, 53], rotation=180);
      Utilities.Parts.Body steeringLever(r=rUL5_scaled - rUL3L4_scaled)
        annotation (extent=[78, 61; 98, 41], rotation=270);
      Utilities.Parts.Shape shape1(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[-46, 19; -26, 39]);
      Utilities.Parts.Shape shape2(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[-42, 76; -22, 96]);
      Utilities.Parts.Shape shape3(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[-61, -75; -41, -55]);
      Utilities.Parts.Shape shape4(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[-50, -39; -30, -19]);
      Utilities.Parts.Shape shape5(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[19, 72; 39, 92]);
      Utilities.Parts.Shape shape6(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[8, 83; 28, 103]);
      Utilities.Parts.Shape shape7(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[6, -69; 26, -49]);
      Utilities.Parts.Shape shape8(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[14, 13; 34, 33]);
    equation 
      connect(lower.frame_a, frame_C)
        annotation (points=[-77, -9; -77, 0; -94, 0]);
      connect(lowerInnerJoint.frame_b, lowerFrontBar.frame_a)
        annotation (points=[-49.5, -40; -40, -40]);
      connect(lowerFrontBar.frame_b, uprightJoint.frame_a)
        annotation (points=[-19, -40; 0, -40; -1, -14.75]);
      connect(lowerRearBar.frame_a, lowerFrontBar.frame_b)
        annotation (points=[-28.5, -15; -20, -15; -20, -40]);
      connect(lowerFrontBar.frame_b, frame_L3L4)
        annotation (points=[-19.5, -40; 0, -40; 0, -90]);
      connect(upper.frame_a, frame_C)
        annotation (points=[-77, 10; -77, 0; -91, 0]);
      connect(steeringJoint.frame_b, frame_S)
        annotation (points=[65, 71; 65, 80; -82, 59]);
      connect(steeringJoint.frame_c, steeringRod.frame_a)
        annotation (points=[47, 67; 40, 67; 40, 97; 20, 97; 10.5, 82]);
      connect(outerRod.frame_b, frame_U) annotation (points=[85.5, 0; 90, 0]);
      connect(lower.frame_b, lowerInnerJoint.frame_a)
        annotation (points=[-77, -30; -77, -40; -69, -40]);
      connect(frame_ta1, outerRod.frame_a)
        annotation (points=[68, -20.5; 64, 0]);
      connect(U.frame_a, frame_ta1) annotation (points=[66, -51; 68, -20.5]);
      connect(frame_ta1, steeringJoint.frame_ta)
        annotation (points=[68, -20.5; 48, 43]);
      connect(uprightShape.frame_a, frame_ta1)
        annotation (points=[26, -44.5; 45, -41; 68, -18.5]);
      connect(upperInnerJoint.frame_a, upper.frame_b)
        annotation (points=[-70, 39; -77, 39; -77, 31]);
      connect(upperFrontBar.frame_b, uprightJoint.frame_b)
        annotation (points=[-19, 40; 0, 40; -1, 16.75]);
      connect(uprightJoint.frame_c, steeringJoint.frame_a)
        annotation (points=[16.25, -4.85; 47, -12; 44, 28; 65, 29; 65, 40]);
      connect(upperRearBar.frame_a, upperFrontBar.frame_b)
        annotation (points=[-24, 63; -19, 40]);
      connect(upperFrontBar.frame_a, upperInnerJoint.frame_b)
        annotation (points=[-40, 40; -49, 40]);
      connect(steeringLever.frame_a, steeringJoint.frame_ta)
        annotation (points=[88, 41; 48, 43]);
      connect(shape3.frame_a, lowerInnerJoint.frame_b)
        annotation (points=[-61, -66; -50, -40]);
      connect(shape4.frame_a, lowerRearBar.frame_b)
        annotation (points=[-50, -29; -50, -16]);
      connect(shape1.frame_a, upperInnerJoint.frame_b)
        annotation (points=[-46, 30; -49, 39]);
      connect(shape2.frame_a, upperRearBar.frame_b)
        annotation (points=[-42, 85; -44, 63]);
      connect(shape5.frame_a, steeringRod.frame_a)
        annotation (points=[18, 82; 11, 80]);
      connect(shape6.frame_a, steeringRod.frame_b)
        annotation (points=[9, 93; -11, 81]);
      connect(shape7.frame_a, uprightJoint.frame_a)
        annotation (points=[6, -60; -1, -14]);
      connect(shape8.frame_a, uprightJoint.frame_b)
        annotation (points=[14, 23; 0, 16]);
    end DoubleWishBone;
    
    model FiveLink 
      "Five links constrain the motion of the upright and reduces the degrees of freedom to one."
       
      
      extends Chassis.Interfaces.Interface;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      
      parameter SI.Position[3] rCL1={-0.5550,-0.3690,-0.0320};
      parameter SI.Position[3] rCL2={-0.4240,-0.1900,0.1640};
      parameter SI.Position[3] rCL3={-0.3320,-0.1300,0.0570};
      parameter SI.Position[3] rCL4={-0.4240,-0.1900,0.2640};
      parameter SI.Position[3] rCL5={-0.3320,-0.1300,0.1570};
      parameter SI.Position[3] rUL1={-0.5250,0.0620,-0.0670};
      parameter SI.Position[3] rUL2={-0.4330,0.0680,0.1500};
      parameter SI.Position[3] rUL3={-0.3350,0.0116,0.0400};
      parameter SI.Position[3] rUL4={-0.4330,0.0680,0.2800};
      parameter SI.Position[3] rUL5={-0.3350,0.0116,0.1900};
      parameter SI.Position[3] rUW={-0.4650,0.1330,0.0200};
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|";
      parameter SI.Mass mU=1 "|Mass and Inertia|";
      parameter SI.Inertia i11U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i22U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i33U=.1 "|Mass and Inertia|";
      parameter SI.Inertia i21U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i31U=.01 "|Mass and Inertia|";
      parameter SI.Inertia i32U=.01 "|Mass and Inertia|";
      
      /*Scaled coordinates*/
    protected 
      parameter SI.Position[3] rCL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL1);
      parameter SI.Position[3] rCL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL2);
      parameter SI.Position[3] rCL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL3);
      parameter SI.Position[3] rCL4_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL4);
      parameter SI.Position[3] rCL5_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL5);
      parameter SI.Position[3] rUL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL1);
      parameter SI.Position[3] rUL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL2);
      parameter SI.Position[3] rUL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL3);
      parameter SI.Position[3] rUL4_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL4);
      parameter SI.Position[3] rUL5_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL5);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Diagram, 
        Icon(
          Ellipse(extent=[34, 50; 54, 70], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[36, 52; 52, 68], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-76, 70; -56, 90], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-74, 72; -58, 88], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-64, 92; -54, 68], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[12, -88; 32, -68], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[14, -86; 30, -70], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[12, -68; 34, -76], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[54, -42; 74, -22], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[56, -40; 72, -24], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[48, -30; 80, -46], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[62, -32; 68, -34; 26, -76; 20, -76; 62, -32], style(
                color=10, fillColor=8)), 
          Ellipse(extent=[58, -26; 70, -38], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[16, -72; 28, -84], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[-62, 84; -64, 78; 42, 56; 44, 62; -62, 84], style(
                color=9, fillColor=8)), 
          Ellipse(extent=[38, 66; 50, 54], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-72, 86; -60, 74], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[52, 26; 72, 46], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[54, 28; 70, 44], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[52, 48; 58, 24], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-80, 32; -60, 52], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-78, 34; -62, 50], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-68, 54; -58, 30], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-66, 46; -68, 40; 60, 32; 60, 38; -66, 46], style(
                color=9, fillColor=8)), 
          Ellipse(extent=[56, 42; 68, 30], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, 48; -64, 36], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[24, -26; 44, -6], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[26, -24; 42, -8], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[24, -4; 30, -28], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-74, 2; -54, 22], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-72, 4; -56, 20], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-62, 24; -52, 0], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-62, 14; -64, 8; 34, -18; 34, -12; -62, 14], style(
                color=9, fillColor=8)), 
          Ellipse(extent=[28, -10; 40, -22], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-70, 18; -58, 6], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[48, 0; 68, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[50, 2; 66, 18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[48, 22; 54, -2], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-78, -44; -58, -24], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, -42; -60, -26], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-66, -22; -56, -46], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-64, -30; -66, -36; 56, 6; 54, 12; -64, -30], style(
                color=9, fillColor=8)), 
          Ellipse(extent=[52, 16; 64, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-74, -28; -62, -40], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-90, 2; -86, 2; -86, 80; -76, 80], style(color=0)), 
          Line(points=[-80, 42; -86, 42], style(color=0)), 
          Line(points=[-74, 12; -86, 12], style(color=0)), 
          Line(points=[-78, -34; -84, -34; -86, -34], style(color=0)), 
          Line(points=[-86, 2; -86, -92; 22, -92; 22, -88], style(color=0)), 
          Line(points=[54, 58; 80, 58; 80, -2; 90, -2], style(color=0)), 
          Line(points=[72, 36; 80, 36], style(color=0)), 
          Line(points=[68, 10; 80, 10], style(color=0)), 
          Line(points=[60, -18; 60, -16; 80, -16; 80, -2], style(color=0)), 
          Line(points=[60, -16; 44, -16], style(color=0))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
Suspension frame consisting of two links in a triangular, wishbone-like, shape.
The inner ends are normally mounted in the chassis at two points alowing the WishBone to revolute
around the mounting points. The outer ends meet and are attached to a ball joint,
providing a lateral-longitudinal location.
</p>
<p>
The Wishbone is also called A-arm and A-frame.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  rUL1L2: location of outer ball joint, resolved in frame_C
  rCL1:   location of inner front joint, resolved in frame_C
  rCL2:   location of inner rear joint, resolved in frame_C
  Mass and inertia properties not yet solved!  
</pre>
Thus rCL1-rCL2 gives the axis around which the wishbone revolutes.
</HTML>
"));
    protected 
      Link link1(
        rA=rCL1, 
        rB=rUW - rUL1, 
        rRod=rUL1 - rCL1) annotation (extent=[-20, 40; 20, 80]);
      CutLink link2(
        rA=rCL2, 
        rB=rUW - rUL2, 
        rRod=rUL2 - rCL2) annotation (extent=[-20, 10; 20, 50]);
      CutLink link3(
        rA=rCL3, 
        rB=rUW - rUL3, 
        rRod=rUL3 - rCL3) annotation (extent=[-20, -20; 20, 20]);
      CutLink link4(
        rA=rCL4, 
        rB=rUW - rUL4, 
        rRod=rUL4 - rCL4) annotation (extent=[-20, -50; 20, -10]);
      CutLink link5(
        rA=rCL5, 
        rB=rUW - rUL5, 
        rRod=rUL5 - rCL5) annotation (extent=[-20, -80; 20, -40]);
      ModelicaAdditions.MultiBody.Parts.Body U(
        rCM=rCMU, 
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U) annotation (extent=[62, -52; 82, -32]);
    public 
      Utilities.Parts.Body shape1(mass=false, r=rUL1 - rUW)
        annotation (extent=[60, 50; 80, 70]);
      Utilities.Parts.Body shape2(r=rUL2 - rUW, mass=false)
        annotation (extent=[60, 30; 80, 50]);
      Utilities.Parts.Body shape3(r=rUL3 - rUW, mass=false)
        annotation (extent=[60, 16; 80, 36]);
      Utilities.Parts.Body shape4(r=rUL4 - rUW, mass=false)
        annotation (extent=[58, -4; 78, 16]);
      Utilities.Parts.Body shape5(r=rUL5 - rUW, mass=false)
        annotation (extent=[58, -24; 78, -4]);
    equation 
      connect(link1.frame_a, frame_C) annotation (points=[-20, 60; -96, 2]);
      connect(link2.frame_a, frame_C) annotation (points=[-20, 30; -96, -2]);
      connect(link3.frame_a, frame_C) annotation (points=[-20, 0; -96, 0]);
      connect(link4.frame_a, frame_C) annotation (points=[-18, -30; -96, -2]);
      connect(link5.frame_a, frame_C) annotation (points=[-20, -60; -96, -2]);
      connect(link5.frame_b, frame_U) annotation (points=[20, -60; 100, 0]);
      connect(link4.frame_b, frame_U) annotation (points=[20, -30; 100, 0]);
      connect(link3.frame_b, frame_U) annotation (points=[20, 0; 100, 0]);
      connect(link2.frame_b, frame_U) annotation (points=[20, 30; 100, 0]);
      connect(link1.frame_b, frame_U) annotation (points=[20, 60; 100, 0]);
      connect(U.frame_a, frame_U) annotation (points=[60, -42; 100, 0]);
      connect(shape1.frame_a, link1.frame_b)
        annotation (points=[59.5, 60; 20, 60]);
      connect(shape2.frame_a, frame_U) annotation (points=[60, 40; 100, 4]);
      connect(shape3.frame_a, frame_U) annotation (points=[60, 26; 93, -10]);
      connect(shape4.frame_a, frame_U) annotation (points=[58, 6; 98, -9]);
      connect(shape5.frame_a, frame_U) annotation (points=[58, -14; 98, -13]);
    end FiveLink;
    
    model CutLink "Two bushings connected by a massless rod" 
      import SI = Modelica.SIunits;
      parameter Boolean kinematicConstraint=true 
        "|Advanced|| = false, if used together with DependentRevolute to solve loop analytically";
      parameter SI.Position[3] rA={0,0,0} 
        "Position vector from frame_a to center of bushingA, resolved in frame_a";
      parameter SI.Position[3] rB={0,0,0} 
        "Position vector from center of bushingb to frame_b, resolved in frame_b";
      parameter SI.Position[3] rRod={1,0,0} 
        "Position vector from rod.frame_a to rod.frame_b, resolved in frame_a at the initial configuration";
      /*
  parameter Boolean autoInit=false 
    "|Initialization| = true: Compute nxRod, nyRod such that bushingB rotations are zero."
    ;
  parameter Real nxRod[3]={1,0,0} 
    "|Initialization| if autoInit = false, x-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
  parameter Real nyRod[3]={0,1,0} 
    "|Initialization| if autoInit = false, y-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
*/
      annotation (Diagram, Icon(
          Text(extent=[-100, 100; 100, 40], string="%name"), 
          Text(extent=[-80, -24; -38, -62], string="A"), 
          Rectangle(extent=[-90, 8; -70, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[-80, -20; -40, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, -16; -44, 16], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-54, 20; -40, -20], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[72, 8; 90, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[44, -20; 84, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[48, -16; 80, 16], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[44, 20; 58, -20], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-62, 8; 68, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[54, 10; 74, -10], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-70, 10; -50, -10], style(
              color=0, 
              fillColor=0, 
              fillPattern=1))));
      
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-110, -10; -90, 10]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_b
        annotation (extent=[90, 10; 110, -10], rotation=180);
      Utilities.Parts.Body rod(
        r=rRod, 
        shape="cylinder", 
        height=0.02, 
        width=0.02) "{sqrt((rRod)*(rRod)),0,0}"
        annotation (extent=[0, 20; 20, 40]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation rotA(r=rA)
        annotation (extent=[-80, -10; -60, 10]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation rotB(r=-rB)
        annotation (extent=[80, -10; 60, 10]);
      Utilities.Joints.JointUS link(L=sqrt((rRod)*(rRod)), kinematicConstraint=
            kinematicConstraint) annotation (extent=[-8, -10; 12, 10]);
      Utilities.Parts.Shape shapeA(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[-38, -32; -18, -12]);
      Utilities.Parts.Shape shapeB(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[38, -36; 58, -16]);
    initial equation 
    equation 
      
      connect(rotA.frame_a, frame_a) annotation (points=[-80, 0; -92, 0]);
      connect(frame_b, rotB.frame_a) annotation (points=[92, 0; 79, 0]);
      connect(link.frame_a, rotA.frame_b) annotation (points=[-10, 0; -60, 0]);
      connect(link.frame_b, rotB.frame_b) annotation (points=[12.5, 0; 60, 0]);
      connect(rod.frame_a, link.frame_c)
        annotation (points=[0, 30; -4, 30; -1.9, 10]);
      connect(shapeA.frame_a, rotA.frame_b)
        annotation (points=[-38, -22; -59.5, 0]);
      connect(shapeB.frame_a, rod.frame_b)
        annotation (points=[38, -26; 20, 30]);
    end CutLink;
    
    model Link "Two bushings connected by a massless rod" 
      import SI = Modelica.SIunits;
      parameter SI.Position[3] rA={0,0,0} 
        "Position vector from frame_a to center of bushingA, resolved in frame_a";
      parameter SI.Position[3] rB={0,0,0} 
        "Position vector from center of bushingb to frame_b, resolved in frame_b";
      parameter SI.Position[3] rRod={1,0,0} 
        "Position vector from rod.frame_a to rod.frame_b, resolved in frame_a at the initial configuration";
      /*
  parameter Boolean autoInit=false 
    "|Initialization| = true: Compute nxRod, nyRod such that bushingB rotations are zero."
    ;
  parameter Real nxRod[3]={1,0,0} 
    "|Initialization| if autoInit = false, x-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
  parameter Real nyRod[3]={0,1,0} 
    "|Initialization| if autoInit = false, y-axis vector of rod.frame_b, resolved in frame_a at the initial configuration"
    ;
*/
      annotation (Diagram, Icon(
          Text(extent=[-80, -24; -38, -62], string="A"), 
          Rectangle(extent=[-90, 8; -70, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[-80, -20; -40, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, -16; -44, 16], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-54, 20; -40, -20], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[72, 8; 90, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[44, -20; 84, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[48, -16; 80, 16], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[44, 20; 58, -20], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-62, 8; 68, -8], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[54, 10; 74, -10], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-70, 10; -50, -10], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(extent=[-80, 88; 80, 28], string="%name")));
      
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-110, -10; -90, 10]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_b
        annotation (extent=[90, 10; 110, -10], rotation=0);
      Utilities.Parts.Body rod(
        r=rRod, 
        shape="cylinder", 
        material={0.5,0.5,0.5,0.5}) annotation (extent=[-10, -10; 10, 10]);
      Chassis.Data.BumpBushingData dataA(
        r_min={-1,-1,-1}, 
        r_max={1,1,1}, 
        phi_min={-1,-1,-1}, 
        phi_max={1,1,1}) annotation (extent=[-60, 40; -40, 60]);
      Chassis.Data.BumpBushingData dataB(
        r_min={-1,-1,-1}, 
        r_max={1,1,1}, 
        phi_min={-1,-1,-1}, 
        phi_max={1,1,1}) annotation (extent=[40, 40; 60, 60]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation rotA(r=rA)
        annotation (extent=[-80, -10; -60, 10]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation rotB(r=rB)
        annotation (extent=[80, -10; 60, 10], rotation=180);
      ModelicaAdditions.MultiBody.Joints.Universal jointB(nx=cross(rRod, {0,0,1}), 
           ny=cross(rRod, jointB.nx)) annotation (extent=[24, -10; 44, 10]);
      Utilities.Joints.Spherical jointA annotation (extent=[-50, -10; -30, 10]);
      Utilities.Parts.Shape shapeA(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[-48, -42; -28, -22]);
      Utilities.Parts.Shape shapeB(
        shape="sphere", 
        r0={-0.015,0,0}, 
        lengthDirection={1,0,0}, 
        material={0,0,0.7,0.5}) annotation (extent=[28, -46; 48, -26]);
    initial equation 
    equation 
      
      connect(rotB.frame_b, frame_b) annotation (points=[82, 0; 94, 0]);
      connect(rotA.frame_a, frame_a) annotation (points=[-80, 0; -94, 0]);
      connect(shapeA.frame_a, rotA.frame_b)
        annotation (points=[-48, -32; -60, 0]);
      connect(shapeB.frame_a, rod.frame_b) annotation (points=[28, -36; 10, 0]);
      connect(jointB.frame_b, rotB.frame_a) annotation (points=[46, 0; 58, 0]);
      connect(jointB.frame_a, rod.frame_b) annotation (points=[24, 0; 10, 0]);
      connect(jointA.frame_b, rod.frame_a) annotation (points=[-28, 0; -10, 0]);
      connect(jointA.frame_a, rotA.frame_b)
        annotation (points=[-50, 0; -60, 0]);
    end Link;
    
    model MonoDamperBelleVilleAntiRoll 
      "OnBoard linkage that decouples roll and bounce (pitch)." 
      parameter Real[3] scaleFactor={1,1,1};
      
      parameter Real[3] rCP={-0.2,0,0.3};
      parameter Real[3] rCS={-0.1,0,0.4};
      parameter Real[3] rPS={-1,0,0.4};
      parameter Real[3] nCP={0,1,0};
      parameter Real q0S=0.05;
      
      parameter Real[3] rPL_1={0,0.3,0.4};
      parameter Real[3] rXL_1={0.0129,0.3172,-0.0330};
      parameter Real[3] rX_1={0.0,0.85,-0.14} 
        "vector to frame_X_1 resolved in frame_C";
      parameter Real[3] rPL_2=Utilities.Functions.EWM({1,-1,1}, rPL_1);
      parameter Real[3] rXL_2=Utilities.Functions.EWM({1,-1,1}, rXL_1);
      parameter Real[3] rX_2=Utilities.Functions.EWM({1,-1,1}, rXL_1);
      
    protected 
      parameter Real[3] rCP_scaled=Utilities.Functions.EWM(scaleFactor, rCP);
      parameter Real[3] rCS_scaled=Utilities.Functions.EWM(scaleFactor, rCS);
      parameter Real[3] rPS_scaled=Utilities.Functions.EWM(scaleFactor, rPS);
      parameter Real[3] nCP_scaled=Utilities.Functions.EWM(scaleFactor, nCP);
      
      parameter Real[3] rPL_1_scaled=Utilities.Functions.EWM(scaleFactor, rPL_1);
      parameter Real[3] rXL_1_scaled=Utilities.Functions.EWM(scaleFactor, rXL_1);
      parameter Real[3] rX_1_scaled=Utilities.Functions.EWM(scaleFactor, rX_1);
      parameter Real[3] rPL_2_scaled=Utilities.Functions.EWM(scaleFactor, rPL_2);
      parameter Real[3] rXL_2_scaled=Utilities.Functions.EWM(scaleFactor, rXL_2);
      parameter Real[3] rX_2_scaled=Utilities.Functions.EWM(scaleFactor, rX_2);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Diagram, 
        Icon(
          Polygon(points=[56, 0; 52, -20; 48, -20; 52, 0; 56, 0], style(color=
                  10, fillColor=9)), 
          Polygon(points=[44, 20; 40, 0; 36, 0; 40, 20; 44, 20], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[40, 0; 36, -20; 32, -20; 36, 0; 40, 0], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[28, 20; 24, 0; 20, 0; 24, 20; 28, 20], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[24, 0; 20, -20; 16, -20; 20, 0; 24, 0], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[12, 20; 8, 0; 4, 0; 8, 20; 12, 20], style(color=9, 
                fillColor=9)), 
          Polygon(points=[8, 0; 4, -20; 0, -20; 4, 0; 8, 0], style(color=9, 
                fillColor=9)), 
          Polygon(points=[-4, 20; -8, 0; -12, 0; -8, 20; -4, 20], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[-8, 0; -12, -20; -16, -20; -12, 0; -8, 0], style(
                color=9, fillColor=9)), 
          Rectangle(extent=[16, 16; 56, -16], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-20, 10; 16, -10], style(gradient=2, fillColor=8)), 
             
          Polygon(points=[-4, 20; 0, 0; -4, 0; -8, 20; -4, 20], style(color=8, 
                fillColor=8)), 
          Polygon(points=[0, 0; 4, -20; 0, -20; -4, 0; 0, 0], style(color=8, 
                fillColor=8)), 
          Polygon(points=[12, 20; 16, 0; 12, 0; 8, 20; 12, 20], style(color=8, 
                fillColor=8)), 
          Polygon(points=[16, 0; 20, -20; 16, -20; 12, 0; 16, 0], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[28, 20; 32, 0; 28, 0; 24, 20; 28, 20], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[32, 0; 36, -20; 32, -20; 28, 0; 32, 0], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[44, 20; 48, 0; 44, 0; 40, 20; 44, 20], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[48, 0; 52, -20; 48, -20; 44, 0; 48, 0], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[-16, 0; -12, -20; -16, -20; -20, 0; -16, 0], style(
                color=8, fillColor=8)), 
          Rectangle(extent=[88, 4; 100, -4], style(
              color=8, 
              gradient=2, 
              fillColor=9)), 
          Rectangle(extent=[-54, 40; -46, -40], style(gradient=1, fillColor=8)), 
             
          Rectangle(extent=[-74, -28; -66, -100], style(gradient=1, fillColor=8)), 
             
          Text(
            extent=[-60, 100; -20, 60], 
            string="R", 
            style(color=41, fillColor=41)), 
          Text(
            extent=[-60, -60; -20, -100], 
            string="L", 
            style(color=73, fillColor=58)), 
          Rectangle(extent=[64, 20; 72, -20], style(gradient=1, fillColor=8)), 
          Rectangle(extent=[-36, 12; -28, -12], style(gradient=1, fillColor=8)), 
             
          Rectangle(extent=[60, 8; 74, -8], style(gradient=1, fillColor=8)), 
          Rectangle(extent=[56, 8; 60, -8], style(color=10, fillColor=10)), 
          Rectangle(extent=[-40, 8; -24, -8], style(gradient=1, fillColor=8)), 
          Rectangle(extent=[-20, 6; -24, -6], style(color=10, fillColor=10)), 
          Polygon(points=[60, 20; 80, 20; 80, -20; 60, -20; 60, -26; 88, -26; 
                88, 26; 60, 26; 60, 20], style(color=9, fillColor=8)), 
          Polygon(points=[-44, 12; -26, 12; -26, 20; -60, 20; -60, 30; -62, 30; 
                 -62, 14; -78, 14; -78, 20; -78, -14; -62, -14; -62, -20; -62, 
                -30; -60, -30; -60, -20; -24, -20; -24, -12; -44, -12; -44, 12], 
               style(color=9, fillColor=8)), 
          Rectangle(extent=[-78, 30; -82, -30], style(color=9, fillColor=8)), 
          Rectangle(extent=[-58, 30; -62, -30], style(color=9, fillColor=8)), 
          Rectangle(extent=[-78, 28; -62, 18], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-78, -18; -62, -28], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-74, 100; -66, 28], style(gradient=1, fillColor=8))), 
           
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="
<HTML>
<p>
The MonoStrut with BelleVilleAntiRoll is an attempt to decouple pitch and roll motion of cars. In racing this is desired while it makes tuning more convinient. 
The left and right links are coupled to the same pivot element and when it rotates, both wheels travels symmertrically. To allow assymetrick motion (roll), the pivot element also can travel along the revolute axis.
</p>
<p>
Usage:
In some formula race cars.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rCP:    location of pivot point, resolved in frame_C
  rCS:    location of strut mount in chassis, resolved in frame_C
  rPS:    location of strut mount in pivot element, resolved in frame_C
  nCP:    direction of pivot revolute and translation, resolved in frame_C
  rPL_1:  location of left link mount in pivot element, resolved in frame_C
  rPX_1:  location of left link outer mount, resolved in frame_C
  rX_1:   location of frame_X_1, resolved in frame_C
  rPL_2:  location of right link mount in pivot element, resolved in frame_C
  rPX_2:  location of right link outer mount, resolved in frame_C
  rX_2:   location of frame_X_2, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a FromChassis
        annotation (extent=[-80, -120; -60, -100], rotation=90);
      ModelicaAdditions.MultiBody.Joints.Prismatic Prismatic1(
        n=nCP, 
        startValueFixed=true, 
        q(stateSelect=StateSelect.prefer))
        annotation (extent=[-100, 2; -80, 22], rotation=270);
      Utilities.Parts.Body DamperLever(r=rPS_scaled - rCP_scaled)
        annotation (extent=[-70, -40; -50, -20]);
      ModelicaAdditions.MultiBody.Joints.Revolute Revolute3(
        n=nCP, 
        startValueFixed=true, 
        q(stateSelect=StateSelect.prefer))
        annotation (extent=[-100, -22; -80, -2], rotation=270);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation RCrank(r=rCP_scaled)
        annotation (extent=[40, 20; 60, 40], rotation=180);
      Chassis.Components.Strut rightStrut(
        rA={0,0,0}, 
        s0=sqrt((rCS_scaled - rPS_scaled)*(rCS_scaled - rPS_scaled)) + q0S, 
        data=data, 
        rB=rCS_scaled) annotation (extent=[-20, -50; 20, -10]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a FromChassis1 annotation (
        extent=[100, -10; 120, 10], 
        rotation=180, 
        layer="icon");
      ModelicaAdditions.MultiBody.Interfaces.Frame_a FromChassis2 annotation (
        extent=[-80, 100; -60, 120], 
        rotation=270, 
        layer="icon");
      Utilities.Forces.Utilities.ForceTable1D data
        annotation (extent=[60, 60; 100, 100]);
      Utilities.Forces.Utilities.ForceTable1D data_A
        annotation (extent=[60, -102; 100, -62]);
      Utilities.Forces.SpringDamperTable1D SpringDamperTable1D1(data=data_A)
        annotation (extent=[-60, -6; -40, 14], rotation=90);
      CutLink leftLink(
        rA=rPL_1_scaled - rCP_scaled, 
        rB=rXL_1_scaled - rX_1_scaled, 
        rRod=rXL_1_scaled - rPL_1_scaled)
        annotation (extent=[-80, -80; -60, -60], rotation=270);
      CutLink rightLink(
        rA=rPL_2_scaled - rCP_scaled, 
        rB=rXL_2_scaled - rX_2_scaled, 
        rRod=rXL_2_scaled - rPL_2_scaled)
        annotation (extent=[-80, 60; -60, 80], rotation=90);
      Utilities.Parts.Shape shape(
        material={0,0,0.7,0.5}, 
        shape="cylinder", 
        length=0.1, 
        r0={0,-0.05,0}, 
        lengthDirection={0,1,0})
        annotation (extent=[0, 38; 20, 58], rotation=180);
    equation 
      connect(Prismatic1.frame_b, Revolute3.frame_a)
        annotation (points=[-90, 1.5; -90, -1.5], style(thickness=4));
      connect(Revolute3.frame_b, DamperLever.frame_a) annotation (points=[-90, 
            -22.5; -90, -30; -70.5, -30], style(thickness=4));
      connect(RCrank.frame_b, Prismatic1.frame_a) annotation (points=[39.5, 30; 
             -25.25, 30; -90, 30; -90, 22.5], style(thickness=4));
      connect(DamperLever.frame_b, rightStrut.frame_a)
        annotation (points=[-48, -30; -22, -30], style(thickness=4));
      connect(RCrank.frame_a, FromChassis1) annotation (points=[60, 30; 82, 30; 
             82, 0; 104, 0], style(thickness=4));
      connect(SpringDamperTable1D1.flange_a, Prismatic1.axis) annotation (
          points=[-50, -6; -50, -11; -84, -11; -84, 4], style(color=58));
      connect(SpringDamperTable1D1.flange_b, Prismatic1.bearing)
        annotation (points=[-50, 14; -84, 14], style(color=58));
      connect(leftLink.frame_b, FromChassis)
        annotation (points=[-70, -80; -70, -102]);
      connect(leftLink.frame_a, Revolute3.frame_b)
        annotation (points=[-70, -60; -70, -46; -90, -46; -90, -22]);
      connect(rightLink.frame_b, FromChassis2)
        annotation (points=[-70, 80; -70, 102]);
      connect(rightLink.frame_a, Revolute3.frame_b) annotation (points=[-70, 60; 
             -70, 38; -108, 38; -108, -42; -90, -42; -90, -22]);
      connect(rightStrut.frame_b, FromChassis1)
        annotation (points=[20, -30; 80, -30; 82, 0; 104, 0]);
      connect(shape.frame_a, RCrank.frame_b)
        annotation (points=[20, 48; 40, 30]);
    end MonoDamperBelleVilleAntiRoll;
    
    model TwinDamperTorsionAntiRoll 
      "Onboard linkage with separate struts and anti-roll linkage." 
      
      import Car;
      parameter Real[3] scaleFactor={1,1,1};
      
      parameter Real[3] rCP_1={0.45,0.5,0.1} 
        "vector to left pivot's chassis mount point resolved in frame_C";
      parameter Real[3] rCS_1={0.45,0.5,0.1} 
        "vector to left strut's chassis mount point resolved in frame_C";
      parameter Real[3] rPS_1={0,0.50,0.15} 
        "vector to left strut's pivot element mount point resolved in frame_C";
      parameter Real[3] rPL1_1={0.04,0.60,0.15} 
        "vector to link1's pivot element mount point resolved in frame_C";
      parameter Real[3] rX_1={0.0,0.85,-0.14} 
        "vector to frame_X_1 resolved in frame_C";
      parameter Real[3] rXL1_1={0.0,0.8,-0.14} 
        "vector to link1's outer mount point resolved in frame_C";
      parameter Real[3] rPL2_1={0,0.5,0.18} 
        "vector to outer anti roll rod end resolved in frame_C";
      parameter Real[3] rAL2_1={.24,0.47,0.21} 
        "vector to inner anti roll rod end/left anti roll lever resolved in frame_C";
      parameter Real[3] rCA_1={.24,.45,0} 
        "vector to left torsion beam end resolved in frame_C";
      parameter Real[3] nP_1=cross((rXL1_1 - rPL1_1), (rPL2_1 - rAL2_1));
      
      
        //{0,.1,1} "direction of rotation of left pivot element resolved in frame_C";
      parameter Real q0S_1=0.048;
      
      parameter Real[3] rCP_2=Utilities.Functions.EWM({1,-1,1}, rCP_1);
      parameter Real[3] rCS_2=Utilities.Functions.EWM({1,-1,1}, rCS_1);
      parameter Real[3] rPS_2=Utilities.Functions.EWM({1,-1,1}, rPS_1);
      parameter Real[3] rPL1_2=Utilities.Functions.EWM({1,-1,1}, rPL1_1);
      parameter Real[3] rX_2=Utilities.Functions.EWM({1,-1,1}, rX_1);
      parameter Real[3] rXL1_2=Utilities.Functions.EWM({1,-1,1}, rXL1_1);
      parameter Real[3] rPL2_2=Utilities.Functions.EWM({1,-1,1}, rPL2_1);
      parameter Real[3] rAL2_2=Utilities.Functions.EWM({1,-1,1}, rAL2_1);
      parameter Real[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1);
      parameter Real[3] nP_2=Utilities.Functions.EWM({1,-1,1}, nP_1);
      parameter Real q0S_2=q0S_1;
      
    protected 
      parameter Real[3] rCP_1_scaled=Utilities.Functions.EWM(scaleFactor, rCP_1);
      parameter Real[3] rCS_1_scaled=Utilities.Functions.EWM(scaleFactor, rCS_1);
      parameter Real[3] rPS_1_scaled=Utilities.Functions.EWM(scaleFactor, rPS_1);
      parameter Real[3] rPL1_1_scaled=Utilities.Functions.EWM(scaleFactor, 
          rPL1_1);
      parameter Real[3] rX_1_scaled=Utilities.Functions.EWM(scaleFactor, rX_1);
      parameter Real[3] rXL1_1_scaled=Utilities.Functions.EWM(scaleFactor, 
          rXL1_1);
      parameter Real[3] rPL2_1_scaled=Utilities.Functions.EWM(scaleFactor, 
          rPL2_1);
      parameter Real[3] rAL2_1_scaled=Utilities.Functions.EWM(scaleFactor, 
          rAL2_1);
      parameter Real[3] rCA_1_scaled=Utilities.Functions.EWM(scaleFactor, rCA_1);
      parameter Real[3] nP_1_scaled=Utilities.Functions.EWM(scaleFactor, nP_1);
      
      parameter Real[3] rCP_2_scaled=Utilities.Functions.EWM(scaleFactor, rCP_2);
      parameter Real[3] rCS_2_scaled=Utilities.Functions.EWM(scaleFactor, rCS_2);
      parameter Real[3] rPS_2_scaled=Utilities.Functions.EWM(scaleFactor, rPS_2);
      parameter Real[3] rPL1_2_scaled=Utilities.Functions.EWM(scaleFactor, 
          rPL1_2);
      parameter Real[3] rX_2_scaled=Utilities.Functions.EWM(scaleFactor, rX_2);
      parameter Real[3] rXL1_2_scaled=Utilities.Functions.EWM(scaleFactor, 
          rXL1_2);
      parameter Real[3] rPL2_2_scaled=Utilities.Functions.EWM(scaleFactor, 
          rPL2_2);
      parameter Real[3] rAL2_2_scaled=Utilities.Functions.EWM(scaleFactor, 
          rAL2_2);
      parameter Real[3] rCA_2_scaled=Utilities.Functions.EWM(scaleFactor, rCA_2);
      parameter Real[3] nP_2_scaled=Utilities.Functions.EWM(scaleFactor, nP_2);
      
    public 
      Chassis.Components.AntiRollBar AntiRollBar(
        rCA_1=rCA_1, 
        rAL_1=rAL2_1, 
        rXL_1=rPL2_1, 
        rX_1=rPL2_1, 
        rCA_2=rCA_2, 
        rAL_2=rAL2_2, 
        rXL_2=rPL2_2, 
        rX_2=rPL2_2, 
        data=data_A) annotation (extent=[52, 20; 92, -20], rotation=90);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Polygon(points=[-10, -10; -14, -30; -18, -30; -14, -10; -10, -10], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-14, -30; -18, -50; -22, -50; -18, -30; -14, -30], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-26, -10; -30, -30; -34, -30; -30, -10; -26, -10], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-30, -30; -34, -50; -38, -50; -34, -30; -30, -30], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-38, -30; -34, -50; -38, -50; -42, -30; -38, -30], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-26, -10; -22, -30; -26, -30; -30, -10; -26, -10], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-22, -30; -18, -50; -22, -50; -26, -30; -22, -30], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-10, -10; -14, -30; -18, -30; -14, -10; -10, -10], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-14, -30; -18, -50; -22, -50; -18, -30; -14, -30], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-26, -10; -30, -30; -34, -30; -30, -10; -26, -10], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-30, -30; -34, -50; -38, -50; -34, -30; -30, -30], 
              style(color=9, fillColor=9)), 
          Polygon(points=[6, 50; 2, 30; -2, 30; 2, 50; 6, 50], style(color=9, 
                fillColor=9)), 
          Polygon(points=[2, 30; -2, 10; -6, 10; -2, 30; 2, 30], style(color=9, 
                 fillColor=9)), 
          Polygon(points=[-10, 50; -14, 30; -18, 30; -14, 50; -10, 50], style(
                color=9, fillColor=9)), 
          Polygon(points=[-14, 30; -18, 10; -22, 10; -18, 30; -14, 30], style(
                color=9, fillColor=9)), 
          Polygon(points=[-26, 50; -30, 30; -34, 30; -30, 50; -26, 50], style(
                color=9, fillColor=9)), 
          Polygon(points=[-30, 30; -34, 10; -38, 10; -34, 30; -30, 30], style(
                color=9, fillColor=9)), 
          Polygon(points=[-26, 50; -22, 30; -26, 30; -30, 50; -26, 50], style(
                color=8, fillColor=8)), 
          Polygon(points=[-22, 30; -18, 10; -22, 10; -26, 30; -22, 30], style(
                color=8, fillColor=8)), 
          Polygon(points=[28, -38; 44, -60; 62, -86; 80, -78; 78, -46; 64, -30; 
                 34, -20; 28, -38], style(color=9, fillColor=8)), 
          Polygon(points=[28, 38; 44, 62; 62, 84; 80, 74; 78, 44; 64, 30; 32, 
                20; 28, 38], style(color=9, fillColor=8)), 
          Rectangle(extent=[66, 30; 74, 12], style(gradient=1, fillColor=8)), 
          Rectangle(extent=[66, 100; 74, 80], style(gradient=1, fillColor=8)), 
          Ellipse(extent=[60, 88; 80, 68], style(color=9, fillColor=8)), 
          Ellipse(extent=[40, 66; 60, 46], style(color=9, fillColor=8)), 
          Ellipse(extent=[60, 48; 80, 28], style(color=9, fillColor=8)), 
          Ellipse(extent=[64, 72; 76, 84], style(color=8, fillColor=10)), 
          Ellipse(extent=[44, 50; 56, 62], style(color=8, fillColor=10)), 
          Ellipse(extent=[64, 32; 76, 44], style(color=8, fillColor=10)), 
          Text(
            extent=[20, 100; 60, 60], 
            string="R", 
            style(color=41)), 
          Polygon(points=[18, 30; 14, 10; 10, 10; 14, 30; 18, 30], style(color=
                  10, fillColor=9)), 
          Rectangle(extent=[18, 38; 28, 22], style(gradient=2, fillColor=8)), 
          Ellipse(extent=[24, 40; 44, 20], style(color=9, fillColor=8)), 
          Ellipse(extent=[28, 24; 40, 36], style(color=8, fillColor=10)), 
          Polygon(points=[18, -30; 14, -50; 10, -50; 14, -30; 18, -30], style(
                color=10, fillColor=9)), 
          Polygon(points=[6, -10; 2, -30; -2, -30; 2, -10; 6, -10], style(color
                =9, fillColor=9)), 
          Polygon(points=[2, -30; -2, -50; -6, -50; -2, -30; 2, -30], style(
                color=9, fillColor=9)), 
          Rectangle(extent=[-22, -14; 18, -46], style(gradient=2, fillColor=8)), 
             
          Polygon(points=[-10, -10; -6, -30; -10, -30; -14, -10; -10, -10], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-6, -30; -2, -50; -6, -50; -10, -30; -6, -30], style(
                color=8, fillColor=8)), 
          Polygon(points=[6, -10; 10, -30; 6, -30; 2, -10; 6, -10], style(color
                =8, fillColor=8)), 
          Polygon(points=[10, -30; 14, -50; 10, -50; 6, -30; 10, -30], style(
                color=8, fillColor=8)), 
          Rectangle(extent=[18, -22; 28, -38], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[66, -14; 74, -32], style(gradient=1, fillColor=8)), 
             
          Rectangle(extent=[66, -80; 74, -100], style(gradient=1, fillColor=8)), 
             
          Ellipse(extent=[24, -20; 44, -40], style(color=9, fillColor=8)), 
          Ellipse(extent=[60, -28; 80, -48], style(color=9, fillColor=8)), 
          Ellipse(extent=[40, -46; 60, -66], style(color=9, fillColor=8)), 
          Ellipse(extent=[60, -70; 80, -90], style(color=9, fillColor=8)), 
          Ellipse(extent=[28, -36; 40, -24], style(color=8, fillColor=10)), 
          Ellipse(extent=[64, -44; 76, -32], style(color=8, fillColor=10)), 
          Ellipse(extent=[44, -62; 56, -50], style(color=8, fillColor=10)), 
          Ellipse(extent=[64, -86; 76, -74], style(color=8, fillColor=10)), 
          Rectangle(extent=[66, 12; 74, -14], style(gradient=1, fillColor=41)), 
             
          Polygon(points=[18, -30; 14, -50; 10, -50; 14, -30; 18, -30], style(
                color=10, fillColor=9)), 
          Polygon(points=[6, -10; 2, -30; -2, -30; 2, -10; 6, -10], style(color
                =9, fillColor=9)), 
          Polygon(points=[2, -30; -2, -50; -6, -50; -2, -30; 2, -30], style(
                color=9, fillColor=9)), 
          Rectangle(extent=[18, -22; 28, -38], style(gradient=2, fillColor=8)), 
             
          Ellipse(extent=[24, -20; 44, -40], style(color=9, fillColor=8)), 
          Ellipse(extent=[28, -36; 40, -24], style(color=8, fillColor=10)), 
          Rectangle(extent=[-74, 32; -66, -32], style(gradient=1, fillColor=8)), 
             
          Polygon(points=[-42, -10; -46, -30; -50, -30; -46, -10; -42, -10], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-46, -30; -50, -50; -54, -50; -50, -30; -46, -30], 
              style(color=9, fillColor=9)), 
          Rectangle(extent=[-58, -20; -22, -40], style(gradient=2, fillColor=8)), 
             
          Polygon(points=[-42, -10; -38, -30; -42, -30; -46, -10; -42, -10], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-54, -30; -50, -50; -54, -50; -58, -30; -54, -30], 
              style(color=8, fillColor=8)), 
          Rectangle(extent=[-68, -22; -58, -38], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-100, 6; -74, -6], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-76, 8; -64, -8], style(color=10, fillColor=8)), 
          Ellipse(extent=[-80, -20; -60, -40], style(color=9, fillColor=8)), 
          Ellipse(extent=[-76, -36; -64, -24], style(color=8, fillColor=10)), 
          Polygon(points=[-42, -10; -46, -30; -50, -30; -46, -10; -42, -10], 
              style(color=9, fillColor=9)), 
          Polygon(points=[-46, -30; -50, -50; -54, -50; -50, -30; -46, -30], 
              style(color=9, fillColor=9)), 
          Rectangle(extent=[-58, -20; -22, -40], style(gradient=2, fillColor=8)), 
             
          Polygon(points=[-42, -10; -38, -30; -42, -30; -46, -10; -42, -10], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-54, -30; -50, -50; -54, -50; -58, -30; -54, -30], 
              style(color=8, fillColor=8)), 
          Rectangle(extent=[-68, -22; -58, -38], style(gradient=2, fillColor=8)), 
             
          Ellipse(extent=[-80, -20; -60, -40], style(color=9, fillColor=8)), 
          Ellipse(extent=[-76, -36; -64, -24], style(color=8, fillColor=10)), 
          Polygon(points=[-42, 50; -46, 30; -50, 30; -46, 50; -42, 50], style(
                color=9, fillColor=9)), 
          Polygon(points=[-46, 30; -50, 10; -54, 10; -50, 30; -46, 30], style(
                color=9, fillColor=9)), 
          Rectangle(extent=[-58, 40; -22, 20], style(gradient=2, fillColor=8)), 
             
          Polygon(points=[-42, 50; -38, 30; -42, 30; -46, 50; -42, 50], style(
                color=8, fillColor=8)), 
          Polygon(points=[-38, 30; -34, 10; -38, 10; -42, 30; -38, 30], style(
                color=8, fillColor=8)), 
          Polygon(points=[-54, 30; -50, 10; -54, 10; -58, 30; -54, 30], style(
                color=8, fillColor=8)), 
          Rectangle(extent=[-68, 38; -58, 22], style(gradient=2, fillColor=8)), 
             
          Ellipse(extent=[-80, 40; -60, 20], style(color=9, fillColor=8)), 
          Ellipse(extent=[-76, 24; -64, 36], style(color=8, fillColor=10)), 
          Text(
            extent=[20, -60; 60, -100], 
            string="L", 
            style(color=73)), 
          Polygon(points=[-38, -30; -34, -50; -38, -50; -42, -30; -38, -30], 
              style(color=8, fillColor=8)), 
          Rectangle(extent=[-22, -14; 18, -46], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-22, 46; 18, 14], style(gradient=2, fillColor=8)), 
          Polygon(points=[-10, 50; -6, 30; -10, 30; -14, 50; -10, 50], style(
                color=8, fillColor=8)), 
          Polygon(points=[-6, 30; -2, 10; -6, 10; -10, 30; -6, 30], style(color
                =8, fillColor=8)), 
          Polygon(points=[6, 50; 10, 30; 6, 30; 2, 50; 6, 50], style(color=8, 
                fillColor=8)), 
          Polygon(points=[10, 30; 14, 10; 10, 10; 6, 30; 10, 30], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[10, -30; 14, -50; 10, -50; 6, -30; 10, -30], style(
                color=8, fillColor=8)), 
          Polygon(points=[6, -10; 10, -30; 6, -30; 2, -10; 6, -10], style(color
                =8, fillColor=8)), 
          Polygon(points=[-10, -10; -6, -30; -10, -30; -14, -10; -10, -10], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-6, -30; -2, -50; -6, -50; -10, -30; -6, -30], style(
                color=8, fillColor=8)), 
          Polygon(points=[-22, -30; -18, -50; -22, -50; -26, -30; -22, -30], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-26, -10; -22, -30; -26, -30; -30, -10; -26, -10], 
              style(color=8, fillColor=8))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Diagram(
          Line(points=[58, 0; -110, 0]), 
          Line(points=[58, 0; -110, 0]), 
          Line(points=[58, 0; -110, 0]), 
          Line(points=[40, -56.5; 40, -59.5]), 
          Line(points=[40, 56; 40, 60]), 
          Line(points=[40, 56; 40, 60]), 
          Line(points=[40, 56; 40, 60]), 
          Line(points=[51.4, 0; -110, 0]), 
          Line(points=[70.5, -30; 80, -30; 80, -20]), 
          Line(points=[68, 30; 80, 30; 80, 22]), 
          Line(points=[40, 56; 40, 60]), 
          Line(points=[12, -56; 12, -60; 0, -60]), 
          Line(points=[12, 54; 12, 60; 0, 60])), 
        Documentation(info="
<HTML>
<p>
The Components is typically used to mount the struts inside a formula body. The upgoing motion from the suspension linkages is redirected via a pivot element on each side. to these, the struts and anti-roll bars are connected.
</p>
<p>
Usage:
In some formula race cars.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rCP_1:  location of left pivot point, resolved in frame_C
  rCS_1:  location of left strut mount in chassis, resolved in frame_C
  rPS_1:  location of left strut mount in pivot element, resolved in frame_C
  nCP_1:  direction of left pivot revolute, resolved in frame_C
  rPL_1:  location of left link mount in pivot element, resolved in frame_C
  rPX_1:  location of left link outer mount, resolved in frame_C
  rX_1:   location of frame_X_1, resolved in frame_C
  rCP_2:  location of right pivot point, resolved in frame_C
  rCS_2:  location of right strut mount in chassis, resolved in frame_C
  rPS_2:  location of right strut mount in pivot element, resolved in frame_C
  nCP_2:  direction of right pivot revolute, resolved in frame_C
  rPL_2:  location of right link mount in pivot element, resolved in frame_C
  rPX_2:  location of right link outer mount, resolved in frame_C
  rX_2:   location of frame_X_2, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
The class models two dampers, connected with a torsion anti roll.
 Parameterisation:
The parameterisation is by default made to generate a symmetric model. Therfore, all rRight* parameters are by default the rleft* parameters mirrored along the y-axis.
"));
      
      ModelicaAdditions.MultiBody.Parts.FrameTranslation RRightCrank(r=
            rCP_2_scaled) annotation (extent=[-80, 20; -60, 40]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation RLeftCrank(r=
            rCP_1_scaled) annotation (extent=[-80, -40; -60, -20]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a FromChassis
        annotation (extent=[-122, -14; -92, 16]);
      ModelicaAdditions.MultiBody.Joints.Revolute LeftCrank(
        n=nP_1_scaled, 
        startValueFixed=true, 
        q(stateSelect=StateSelect.prefer))
        annotation (extent=[-20, -40; 0, -20]);
      ModelicaAdditions.MultiBody.Joints.Revolute RightCrank(
        n=nP_2_scaled, 
        startValueFixed=true, 
        q(stateSelect=StateSelect.prefer)) annotation (extent=[-20, 20; 0, 40]);
      Chassis.Components.Strut Strut_2(
        rB={0,0,0}, 
        s0=sqrt((rCS_2_scaled - rPS_2_scaled)*(rCS_2_scaled - rPS_2_scaled)) + 
            q0S_2, 
        data=data_S_2, 
        rA=rCS_2_scaled) annotation (extent=[-40, 40; 0, 80]);
      Chassis.Components.Strut Strut_1(
        rB={0,0,0}, 
        s0=sqrt((rCS_1_scaled - rPS_1_scaled)*(rCS_1_scaled - rPS_1_scaled)) + 
            q0S_1, 
        data=data_S_1, 
        rA=rCS_1_scaled) annotation (extent=[-40, -80; 0, -40]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_2
        annotation (extent=[44, 88; 74, 118], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_1
        annotation (extent=[44, -118; 74, -88], rotation=90);
      Utilities.Parts.Body LeftStrutLever(r=rPS_1_scaled - rCP_1_scaled)
        annotation (extent=[2, -56; 22, -36], rotation=270);
      Utilities.Parts.Body LeftAntiRollLever(r=rPL2_1_scaled - rCP_1_scaled)
        annotation (extent=[50, -40; 70, -20]);
      Utilities.Parts.Body RightLeftStrutLever(r=rPS_2_scaled - rCP_2_scaled)
        annotation (extent=[2, 54; 22, 34], rotation=270);
      Utilities.Parts.Body RightAntiRollLever(r=rPL2_2_scaled - rCP_2_scaled)
        annotation (extent=[48, 40; 68, 20]);
      Utilities.Forces.Utilities.ForceTable1D data_S_2
        annotation (extent=[-100, 80; -80, 100]);
      Utilities.Forces.Utilities.ForceTable1D data_S_1
        annotation (extent=[-100, -100; -80, -80]);
      Utilities.Forces.Utilities.ForceTable1D data_A
        annotation (extent=[80, -8; 100, 12]);
      CutLink leftLink(rB=rXL1_1_scaled - rX_1_scaled, rRod=rXL1_1_scaled - 
            rPL1_1_scaled) annotation (extent=[38, -70; 58, -50]);
      CutLink rightLink(rB=rXL1_2_scaled - rX_2_scaled, rRod=rXL1_2_scaled - 
            rPL1_2_scaled) annotation (extent=[40, 52; 60, 72]);
      Utilities.Parts.Body LeftStrutLever1(r=rPL1_1_scaled - rCP_1_scaled)
        annotation (extent=[18, -54; 38, -34], rotation=270);
      Utilities.Parts.Body RightLeftStrutLever1(r=rPL1_2_scaled - rCP_2_scaled)
        annotation (extent=[22, 54; 42, 34], rotation=270);
    equation 
      connect(FromChassis, RLeftCrank.frame_a)
        annotation (points=[-110, 0; -95.25, 0; -95.25, -30; -80.5, -30]);
      connect(RRightCrank.frame_a, FromChassis)
        annotation (points=[-80.5, 30; -95.25, 30; -95.25, -8; -110, -8]);
      connect(RLeftCrank.frame_b, LeftCrank.frame_a)
        annotation (points=[-59.5, -30; -20.5, -30]);
      connect(RightCrank.frame_a, RRightCrank.frame_b)
        annotation (points=[-20.5, 30; -59.5, 30]);
      connect(LeftCrank.frame_b, LeftAntiRollLever.frame_a)
        annotation (points=[2, -30; 49.5, -30]);
      connect(LeftStrutLever.frame_a, LeftCrank.frame_b)
        annotation (points=[12, -36; 12, -30; 0, -30]);
      connect(LeftStrutLever.frame_b, Strut_1.frame_b)
        annotation (points=[12, -56; 12, -60; 0, -60]);
      connect(RightLeftStrutLever.frame_b, Strut_2.frame_b)
        annotation (points=[12, 54; 12, 60; 0, 60]);
      connect(RightCrank.frame_b, RightLeftStrutLever.frame_a)
        annotation (points=[2, 30; 12, 30; 12, 32]);
      connect(RightAntiRollLever.frame_a, RightCrank.frame_b)
        annotation (points=[46, 30; 2, 30]);
      connect(AntiRollBar.frame_C, FromChassis)
        annotation (points=[51.4, 0; -110, 0]);
      connect(LeftAntiRollLever.frame_b, AntiRollBar.frame_X_1)
        annotation (points=[70.5, -30; 80, -30; 80, -20]);
      connect(RightAntiRollLever.frame_b, AntiRollBar.frame_X_2)
        annotation (points=[68, 30; 80, 30; 80, 22]);
      connect(leftLink.frame_b, frame_X_1)
        annotation (points=[58, -60; 60, -60; 60, -94]);
      connect(Strut_1.frame_a, FromChassis)
        annotation (points=[-40, -60; -96, -60; -96, 0]);
      connect(Strut_2.frame_a, FromChassis)
        annotation (points=[-40, 60; -98, 60; -98, 4]);
      connect(rightLink.frame_b, frame_X_2)
        annotation (points=[60, 62; 60, 92]);
      connect(LeftStrutLever1.frame_a, LeftCrank.frame_b)
        annotation (points=[28, -32; 0, -30]);
      connect(LeftStrutLever1.frame_b, leftLink.frame_a)
        annotation (points=[28, -54; 38, -60]);
      connect(RightLeftStrutLever1.frame_a, RightCrank.frame_b)
        annotation (points=[32, 34; 2, 30]);
      connect(RightLeftStrutLever1.frame_b, rightLink.frame_a)
        annotation (points=[32, 54; 40, 62]);
    end TwinDamperTorsionAntiRoll;
    
    model AntiRollBar "AntiRollBar - standard anti-roll linkage." 
      
      parameter Real[3] scaleFactor={1,1,1};
      parameter Real[3] rCA_1={-0.2405,0.3420,0.0330} 
        "vector to left torsion beam end resolved in frame_C";
      parameter Real[3] rAL_1={-0.0439,0.5115,0.0203} 
        "vector to left inner anti roll rod end/left anti roll lever resolved in frame_C";
      parameter Real[3] rXL_1={-0.0564,0.5401,0.2183} 
        "vector to left outer anti roll rod end resolved in frame_C";
      parameter Real[3] rX_1={0.0059,0.6905,-0.0710} + {-0.0060,0.0365,0.1110} 
        "vector to left anti roll mount resolved in frame_C";
      
      parameter Real[3] rCA_2=Utilities.Functions.EWM({1,-1,1}, rCA_1);
      parameter Real[3] rAL_2=Utilities.Functions.EWM({1,-1,1}, rAL_1);
      parameter Real[3] rXL_2=Utilities.Functions.EWM({1,-1,1}, rXL_1);
      parameter Real[3] rX_2=Utilities.Functions.EWM({1,-1,1}, rX_1);
      
    protected 
      parameter Real[3] rCA_1_scaled=Utilities.Functions.EWM(scaleFactor, rCA_1);
      parameter Real[3] rAL_1_scaled=Utilities.Functions.EWM(scaleFactor, rAL_1);
      parameter Real[3] rXL_1_scaled=Utilities.Functions.EWM(scaleFactor, rXL_1);
      parameter Real[3] rX_1_scaled=Utilities.Functions.EWM(scaleFactor, rX_1);
      
      parameter Real[3] rCA_2_scaled=Utilities.Functions.EWM(scaleFactor, rCA_2);
      parameter Real[3] rAL_2_scaled=Utilities.Functions.EWM(scaleFactor, rAL_2);
      parameter Real[3] rXL_2_scaled=Utilities.Functions.EWM(scaleFactor, rXL_2);
      parameter Real[3] rX_2_scaled=Utilities.Functions.EWM(scaleFactor, rX_2);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-90, -36; -78, -44], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[76, -36; 88, -44], style(gradient=2, fillColor=8)), 
             
          Rectangle(extent=[-54, 60; 52, 52], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-12, 60; 12, 52], style(gradient=2, fillColor=41)), 
             
          Rectangle(extent=[-54, 60; -46, 12], style(color=8, fillColor=8)), 
          Ellipse(extent=[-60, 0; -40, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-58, 2; -42, 18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-82, -48; -62, -28], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-80, -46; -64, -30], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-64, 16; -34, 0; -40, -12; -70, 2; -64, 16], style(
                color=7, fillColor=7)), 
          Polygon(points=[-80, -16; -50, -32; -56, -44; -84, -28; -80, -16], 
              style(color=7, fillColor=7)), 
          Polygon(points=[-54, 10; -74, -36; -68, -40; -48, 6; -54, 10], style(
                color=8, fillColor=8)), 
          Ellipse(extent=[-78, -32; -66, -44], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 16; -44, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, 16; -44, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[44, 60; 52, 12], style(color=8, fillColor=8)), 
          Ellipse(extent=[38, 0; 58, 20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[40, 2; 56, 18], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[60, -50; 80, -30], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[62, -48; 78, -32], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[34, 4; 68, 16; 66, 2; 42, -12; 34, 4], style(color=7, 
                 fillColor=7)), 
          Polygon(points=[54, -32; 76, -20; 82, -28; 58, -46; 54, -32], style(
                color=7, fillColor=7)), 
          Ellipse(extent=[42, 16; 54, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[42, 8; 50, 12; 72, -36; 66, -40; 44, 8; 42, 8], style(
                color=8, fillColor=8)), 
          Ellipse(extent=[42, 16; 54, 4], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[64, -34; 76, -46], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[0, 86; 0, 62], style(color=10)), 
          Text(extent=[34, -18; 66, -66], string="L")), 
        Diagram, 
        Window(
          x=0.05, 
          y=0.02, 
          width=0.51, 
          height=0.91), 
        Documentation(info="<HTML>
<p>
To a torsion bar with levers in each end, two connecting rods are attached, these are normally attached at the left and right suspension linkage to reduce roll motion. This component is modelled with rigid elements, except for a torsion bar. The force generation fue to deflection is modelled from table values.
</p>
<p>
Usage:
Most vehicles with independent suspensions, both front and rear.</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
The right hand side parameters, ending with _2, are by default symmetric to the left hand dito ending with _1.
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} mirrors it around the xz-plane.
  ratioWheelToRack=22: rack-pinion gear ratio [m/rad]
  rX_1:   location of frame X_1, resolved in frame_C
  rXL1_1: location of left link end, resolved in frame_C
  rAL_1:  location of left torsion bar end, resolved in frame_C
  rX_2:   location of frame X_2, resolved in frame_C,
  rXL1_2: location of right link end, resolved in frame_C
  rAL_2:  location of right torsion bar end, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Joints.Revolute rightRevolute(
        q(start=0), 
        n=rAL_1_scaled - rAL_2_scaled, 
        qd(start=0)) annotation (extent=[-30, 30; -10, 50], rotation=180);
      ModelicaAdditions.MultiBody.Joints.Revolute leftRevolute(
        n=rAL_1_scaled - rAL_2_scaled, 
        q(start=0), 
        qd(start=0)) annotation (extent=[10, 50; 30, 30]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation(r=(
            rCA_1_scaled + rCA_2_scaled)/2)
        annotation (extent=[-10, 60; 10, 80], rotation=270);
      Utilities.Parts.Body leftBar(r=rCA_1_scaled - (rCA_1_scaled + 
            rCA_2_scaled)/2) annotation (extent=[40, 30; 60, 50]);
      Utilities.Parts.Body rightBar(r=rCA_2_scaled - (rCA_1_scaled + 
            rCA_2_scaled)/2)
        annotation (extent=[-60, 30; -40, 50], rotation=180);
      Utilities.Parts.Body rightLever(r=rAL_2_scaled - rCA_2_scaled)
        annotation (extent=[-80, 10; -60, 30], rotation=270);
      Utilities.Parts.Body leftLever(r=rAL_1_scaled - rCA_1_scaled)
        annotation (extent=[60, 10; 80, 30], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_2
        annotation (extent=[-120, -56; -90, -26]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_X_1
        annotation (extent=[88, -54; 118, -24], rotation=180);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-14, 88; 16, 118], rotation=270);
      Utilities.Forces.SpringDamperTableRot1D springDamper(data=data)
        annotation (extent=[-10, 0; 10, 20]);
      parameter Utilities.Forces.Utilities.ForceTable1D data
        annotation (extent=[-20, -80; 20, -40]);
      CutLink rightLink(rB=-rXL_2_scaled + rX_2_scaled, rRod=rXL_2_scaled - 
            rAL_2_scaled) annotation (extent=[-80, -20; -60, 0], rotation=270);
      CutLink leftLink(rB=-rXL_1_scaled + rX_1_scaled, rRod=rXL_1_scaled - 
            rAL_1_scaled) annotation (extent=[60, -20; 80, 0], rotation=270);
    equation 
      connect(frameTranslation.frame_b, leftRevolute.frame_a)
        annotation (points=[-1.88738e-015, 59.5; -1.88738e-015, 40; 9.5, 40]);
      connect(rightRevolute.frame_a, frameTranslation.frame_b)
        annotation (points=[-9.5, 40; -1.88738e-015, 40; -1.88738e-015, 59.5]);
      connect(rightBar.frame_a, rightRevolute.frame_b)
        annotation (points=[-38, 40; -30, 40]);
      connect(leftRevolute.frame_b, leftBar.frame_a)
        annotation (points=[30, 40; 40, 40]);
      connect(leftLever.frame_a, leftBar.frame_b)
        annotation (points=[70, 30.5; 70, 40; 62, 40]);
      connect(rightLever.frame_a, rightBar.frame_b)
        annotation (points=[-70, 30.5; -70, 40; -62, 40]);
      connect(frameTranslation.frame_a, frame_C)
        annotation (points=[0, 82; 0, 103; 1, 103]);
      connect(springDamper.flange_a, rightRevolute.axis)
        annotation (points=[-10, 10; -20, 10; -20, 32], style(color=0));
      connect(springDamper.flange_b, leftRevolute.axis)
        annotation (points=[10, 10; 20, 10; 20, 32], style(color=0));
      connect(rightLink.frame_b, frame_X_2)
        annotation (points=[-70, -20; -94, -38]);
      connect(rightLink.frame_a, rightLever.frame_b)
        annotation (points=[-70, 0; -70, 10]);
      connect(leftLink.frame_a, leftLever.frame_b)
        annotation (points=[70, 0; 70, 10]);
      connect(leftLink.frame_b, frame_X_1)
        annotation (points=[70, -20; 92, -40]);
    end AntiRollBar;
    
    model BeamAxle "BeamAxle, used in rigid axle suspensions" 
      parameter Real[3] scaleFactor={1,1,1};
      parameter Real[3] rUW_1={0,0.9,0} 
        "Vector to left wheel centre resolved in frame_C";
      parameter Real[3] rUW_2=Utilities.Functions.EWM({1,-1,1}, rUW_1) 
        "Vector to right wheel centre resolved in frame_C";
      
      /*Scaled coordinates*/
    protected 
      parameter Real[3] rUW_1_scaled=Utilities.Functions.EWM(scaleFactor, rUW_1);
      parameter Real[3] rUW_2_scaled=Utilities.Functions.EWM(scaleFactor, rUW_2);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-16, 90; 14, -90], style(gradient=1, fillColor=8)), 
             
          Rectangle(extent=[80, 6; 64, -6], style(color=0, fillColor=8)), 
          Rectangle(extent=[58, 6; 42, -6], style(color=0, fillColor=8)), 
          Rectangle(extent=[34, 6; 18, -6], style(color=0, fillColor=8)), 
          Text(extent=[-18, -32; 14, -80], string="L")), 
        Window(
          x=0.12, 
          y=0.17, 
          width=0.34, 
          height=0.61), 
        Documentation(info="
<HTML>
<p>
Until the late 70s, most cars still used this simple non-independent suspensions, especially at the rear axle.
 Basically, it is a rigid axle fixed between left and right wheels. The car body is suspended by leaf springs or 
coil springs on the axle / wheels unit. 
On a smoth road, the wheels remains uncambered as the car rolls but if a wheel hits a bump, the shock will be 
transfeered to the other wheel as well, giving a large unsprung mass and camber wffects on both wheels.
If the beam axle is also the driving axle, it is called live axle. A live axle consists of the final 
drive / differential, drive shafts and a strong tube enclosing these components. Since the live axle 
is rigidly connected to the wheels, the unsprung mass is often very high for this solution. Without drive, the axle is called dead axle, this in normally a lot lighter.
The BeamAxle is normally used toghether with a Watt linkage or a Panhard rod to contrain the motion of the axle.
  
Advantage:   
Disadvantage:    
Who use it ?     
</p>
<p>
Advantages:
Cheap. Body roll does not influence the camber of wheels.
</p>
<p>
Disadvantages:
Non-independent, bad ride quality, both wheels cambered on bump.
</p>
<p>
Usage:
Some American sedans, and most Trucks.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUW_1:    location of frame_W_1, resolved in frame_C
  rUW_2:    location of frame_W_2, resolved in frame_C
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"), 
        Diagram);
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[88, -14; 118, 16], rotation=180);
      Utilities.Parts.Body rightBeam(
        width=0.02, 
        height=0.05, 
        r=rUW_2_scaled - (rUW_1_scaled + rUW_2_scaled)/2)
        annotation (extent=[-8, 12; 8, 32], rotation=90);
      Utilities.Parts.Body leftBeam(
        width=0.02, 
        height=0.05, 
        r=rUW_1_scaled - (rUW_1_scaled + rUW_2_scaled)/2)
        annotation (extent=[-10, -30; 10, -10], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_U_1
        annotation (extent=[-16, -120; 14, -90], rotation=270);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_U_2
        annotation (extent=[-16, 90; 14, 120], rotation=90);
      Utilities.Joints.FreeMotion freeMotion(
        phi(start={0,0,0}), 
        w_rela(start={0,0,0}), 
        enforceStates=false, 
        r_rela(start={0,0,0}))
        annotation (extent=[8, -10; 28, 10], rotation=180);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation FrameTranslation1(r=(
            rUW_1_scaled + rUW_2_scaled)/2)
        annotation (extent=[50, -10; 70, 10], rotation=180);
      ModelicaAdditions.MultiBody.Parts.Shape Shape2(
        Shape="sphere", 
        Length=0.2, 
        Width=0.2, 
        Height=0.2, 
        Material={0,0,1,0.5}) annotation (extent=[22, -40; 42, -20]);
      ModelicaAdditions.MultiBody.Parts.Shape Shape1(
        Shape="sphere", 
        Length=0.2, 
        Width=0.2, 
        Height=0.2, 
        Material={0,1,0,0.5}) annotation (extent=[28, 20; 48, 40]);
    equation 
      
      connect(leftBeam.frame_a, freeMotion.frame_b)
        annotation (points=[-1, -9.3; 7.5, 1.33227e-015]);
      connect(rightBeam.frame_a, freeMotion.frame_b)
        annotation (points=[1, 11.5; 7.5, 1.33227e-015]);
      connect(rightBeam.frame_b, frame_U_2) annotation (points=[0, 32; 0, 94]);
      connect(leftBeam.frame_b, frame_U_1) annotation (points=[0, -32; 0, -92]);
      connect(FrameTranslation1.frame_a, frame_C)
        annotation (points=[70, -1.33227e-015; 92, 0]);
      connect(FrameTranslation1.frame_b, freeMotion.frame_a)
        annotation (points=[50, 0; 28, 0]);
      connect(Shape2.frame_a, FrameTranslation1.frame_b)
        annotation (points=[21.5, -30; 50, 0]);
      connect(Shape1.frame_a, freeMotion.frame_b)
        annotation (points=[28, 30; 8, 0]);
    end BeamAxle;
    
    model MacPherson2 "MacPherson strut linkage" 
      import SI = Modelica.SIunits;
      
      extends Chassis.Interfaces.Interface;
      parameter Real[3] scaleFactor={1,1,1} 
        "To make right hand side Linkage use {1,-1,1}";
      parameter SI.Position[3] rCL1={0.1070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to front link mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rCL2={-0.3070,0.55,-0.0380} 
        "|Geometry| Vector from origin of frame_C to rear link mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rCS={-0.0295,0.850,0.5670} 
        "|Geometry| Vector from origin of frame_C to strut mount in chassis resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUS={-0.0070,0.8733,0.1380} 
        "|Geometry| Vector from origin of frame_C to strut mount in uppright resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUL1L2={-0.0070,0.8733,-0.0380} 
        "|Geometry| Vector from origin of frame_C to low spindleUtilities.Joints.Joint resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUW={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to wheel centre resolved in frame_C (at initial time)";
      parameter SI.Position[3] rRL3={-0.12,0.85,0.08} 
        "|Geometry| Vector from origin of frame_C to origin of frame_S resolved in frame_C (at initial time)";
      parameter SI.Position[3] rUL3={0.0,0.9,0} 
        "|Geometry| Vector from origin of frame_C to lower ball of steering rod resolved in frame_C (at initial time)";
      parameter SI.Length q0S=0.05 
        "|Geometry| Unstretched additional spring length of the strut, compared to the construction geometry";
      
      parameter SI.Position[3] rCMU={0,0,0} "|Mass and Inertia|Wheel carrier|";
      parameter SI.Mass mU=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i11U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i22U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i33U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i21U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i31U=0 "|Mass and Inertia|Wheel carrier|";
      parameter SI.Inertia i32U=0 "|Mass and Inertia|Wheel carrier|";
      
      parameter SI.Position[3] rCML1L2={0,0,0} "|Mass and Inertia|Wishbone|";
      parameter SI.Mass mL1L2=0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i11L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i22L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i33L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i21L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i31L1L2=.0 "|Mass and Inertia|Wishbone|";
      parameter SI.Inertia i32L1L2=.0 "|Mass and Inertia|Wishbone|";
      
      /*Scaled coordinates*/
    protected 
      parameter SI.Position[3] rCL1_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL1);
      parameter SI.Position[3] rCL2_scaled=Utilities.Functions.EWM(scaleFactor, 
           rCL2);
      parameter SI.Position[3] rCS_scaled=Utilities.Functions.EWM(scaleFactor, 
          rCS);
      parameter SI.Position[3] rUS_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUS);
      parameter SI.Position[3] rUL1L2_scaled=Utilities.Functions.EWM(
          scaleFactor, rUL1L2);
      parameter SI.Position[3] rUW_scaled=Utilities.Functions.EWM(scaleFactor, 
          rUW);
      parameter SI.Position[3] rRL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rRL3);
      parameter SI.Position[3] rUL3_scaled=Utilities.Functions.EWM(scaleFactor, 
           rUL3);
      parameter SI.Length q0Strut=sqrt((rCS_scaled - rUS_scaled)*(rCS_scaled - 
          rUS_scaled)) + q0S;
      parameter SI.Length L_frontBar=sqrt((rCL1_scaled - rUL1L2_scaled)*(
          rCL1_scaled - rUL1L2_scaled)) "length of frontBar";
      parameter SI.Length L_rearBar=sqrt((rCL2_scaled - rUL1L2_scaled)*(
          rCL2_scaled - rUL1L2_scaled)) "length of rearBar";
      parameter SI.Length L_springRod=sqrt((rUS_scaled - rUL1L2_scaled)*(
          rUS_scaled - rUL1L2_scaled));
      parameter SI.Length L_outerRod=sqrt((rUW_scaled - rUL1L2_scaled)*(
          rUW_scaled - rUL1L2_scaled));
      parameter SI.Position rS[3]=rCS_scaled - rUL1L2_scaled;
      parameter SI.Position rU[3]=rUW_scaled - rUL1L2_scaled 
        "Position vector from frame_L1L2 to frame_U, resolved in frame_C";
      parameter Real n_a[3]=cross(rS, rU) 
        "First rotation axis of universalUtilities.Joints.Joint in springJoint, resolved in frame_C";
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_L12
        annotation (extent=[-16, -118; 14, -88], rotation=270);
      Utilities.Forces.Utilities.ForceTable1D data_S
        annotation (extent=[91, 69; 121, 99]);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation upper(r=rCS_scaled)
        annotation (extent=[-67, 10; -87, 30], rotation=90);
      Utilities.Joints.JointUPS MacPherson(n_a=n_a)
        annotation (extent=[19, -20; -20, 20], rotation=90);
      Utilities.Parts.Body springRod(r=rUS_scaled - rUL1L2_scaled, mass=false)
        annotation (extent=[30, -30; 50, -10], rotation=90);
      Utilities.Forces.SpringDamperTableLine3D strut(s_rel0=q0Strut, data=
            data_S) annotation (extent=[29, 0; 49, 20], rotation=90);
      Utilities.Parts.Body outerRod(r=rUW_scaled - rUL1L2_scaled, mass=false)
        annotation (extent=[65, -10; 85, 10], rotation=0);
      ModelicaAdditions.MultiBody.Parts.Body U(
        rCM=rCMU, 
        m=mU, 
        I11=i11U, 
        I22=i22U, 
        I33=i33U, 
        I21=i21U, 
        I31=i31U, 
        I32=i32U) annotation (extent=[66, -60; 86, -40]);
      Utilities.Parts.Body steeringRod(r=rUL3_scaled - rRL3_scaled, shape=
            "cylinder") annotation (extent=[10, 80; 30, 60], rotation=180);
      Utilities.Joints.JointRSU steeringJoint(
        n_b={1,0,0}, 
        n_a=rCS_scaled - rUL1L2_scaled, 
        r_a=rUL3_scaled - rUL1L2_scaled)
        annotation (extent=[51, 40; 81, 70], rotation=90);
      ModelicaAdditions.MultiBody.Joints.Revolute innerJoint(
        q(stateSelect=StateSelect.always), 
        qd(stateSelect=StateSelect.always), 
        n=rCL1_scaled - rCL2_scaled)
        annotation (extent=[-70, -70; -50, -50], rotation=0);
      ModelicaAdditions.MultiBody.Parts.FrameTranslation lower(r=rCL1_scaled)
        annotation (extent=[-87, -30; -67, -10], rotation=270);
      Utilities.Parts.Body frontBar(r=rUL1L2_scaled - rCL1_scaled)
        annotation (extent=[-40, -70; -20, -50]);
      Utilities.Parts.Body rearBar(r=rCL2_scaled - rUL1L2_scaled, 
          widthDirection=cross(rCL2_scaled - rUL1L2_scaled, {0,0,1}))
        annotation (extent=[-40, -30; -20, -50], rotation=180);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Polygon(points=[52, 96; 64, 80; 62, 86; 70, 62; -100, 106; 52, 96], 
              style(pattern=0)), 
          Line(points=[-44, -44; -18, -24], style(color=0, thickness=4)), 
          Line(points=[-58, -46; 30, -30; -58, -20], style(color=0, thickness=4)), 
             
          Ellipse(extent=[-58, -30; -38, -10], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-56, -28; -40, -12], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-62, -10; -52, -30], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-54, -14; -42, -26], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-78, -58; -58, -38], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-76, -56; -60, -40], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-81, -37; -71, -57], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-74, -42; -62, -54], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[42, -12; 60, -12; 64, 4; 90, 4; 90, -2; 68, -2; 64, -18; 
                 42, -18; 42, -12], style(color=8, fillColor=8)), 
          Polygon(points=[62, 18; 56, 18; 48, -14; 56, -14; 62, 18], style(
              pattern=0, 
              gradient=2, 
              fillColor=8)), 
          Ellipse(extent=[50, 14; 70, 34], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[52, 16; 68, 32], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[42, 38; 54, 14], style(color=7, fillColor=7)), 
          Ellipse(extent=[-50, 50; -30, 70], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-48, 52; -32, 68], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-36, 72; -22, 46], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-46, 66; -34, 54], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[54, 30; 66, 18], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-34, 60; 58, 24], style(color=0, thickness=4)), 
          Ellipse(extent=[28, -40; 48, -20], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, -38; 46, -22], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, -20; 50, -26], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[34, -8; 42, -26], style(color=8, fillColor=8)), 
          Polygon(points=[38, 2; 50, -2; 50, 2; 38, 6; 38, 2], style(color=10, 
                fillColor=10)), 
          Polygon(points=[26, 6; 38, 2; 38, 6; 26, 10; 26, 6], style(color=10, 
                fillColor=10)), 
          Polygon(points=[38, 18; 50, 14; 50, 18; 38, 22; 38, 18], style(color=
                  10, fillColor=10)), 
          Polygon(points=[26, 22; 38, 18; 38, 22; 26, 26; 26, 22], style(color=
                  10, fillColor=10)), 
          Polygon(points=[38, 34; 50, 30; 50, 34; 38, 38; 38, 34], style(color=
                  10, fillColor=10)), 
          Polygon(points=[26, 38; 38, 34; 38, 38; 26, 42; 26, 38], style(color=
                  10, fillColor=10)), 
          Polygon(points=[38, 50; 50, 46; 50, 50; 38, 54; 38, 50], style(color=
                  10, fillColor=10)), 
          Rectangle(extent=[32, 56; 44, -8], style(gradient=1, fillColor=8)), 
          Rectangle(extent=[30, 52; 46, 14], style(gradient=1, fillColor=8)), 
          Polygon(points=[26, 38; 38, 42; 38, 46; 26, 42; 26, 38], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, 42; 50, 46; 50, 50; 38, 46; 38, 42], style(color=
                  8, fillColor=8)), 
          Polygon(points=[26, 22; 38, 26; 38, 30; 26, 26; 26, 22], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, 26; 50, 30; 50, 34; 38, 30; 38, 26], style(color=
                  8, fillColor=8)), 
          Polygon(points=[26, 6; 38, 10; 38, 14; 26, 10; 26, 6], style(color=8, 
                 fillColor=8)), 
          Polygon(points=[38, 10; 50, 14; 50, 18; 38, 14; 38, 10], style(color=
                  8, fillColor=8)), 
          Polygon(points=[38, -6; 50, -2; 50, 2; 38, -2; 38, -6], style(color=8, 
                 fillColor=8)), 
          Ellipse(extent=[28, 56; 48, 76], style(
              color=0, 
              gradient=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[30, 58; 46, 74], style(
              color=0, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[26, 62; 50, 54], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[34, 66; 42, 54], style(gradient=1, fillColor=8)), 
          Ellipse(extent=[32, 72; 44, 60], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[32, -24; 44, -36], style(
              color=0, 
              fillColor=0, 
              fillPattern=1))), 
        Diagram(
          Line(points=[-90, 0; -78, -60], style(color=1, arrow=1)), 
          Text(
            extent=[-98, -46; -78, -53], 
            style(color=1, arrow=1), 
            string="rCL1"), 
          Line(points=[-90, 0; 0, -40], style(color=1, arrow=1)), 
          Text(
            extent=[-16, -27; -1, -34], 
            style(color=1, arrow=1), 
            string="rUL1L2"), 
          Text(
            extent=[-99, 36; -72, 30], 
            style(color=1, arrow=1), 
            string="rCS"), 
          Line(points=[-90, 0; -77, 40], style(color=1, arrow=1)), 
          Text(
            extent=[-12, 55; 12, 48], 
            style(color=1, arrow=1), 
            string="rRL3"), 
          Line(points=[-90, 0; 65, 80], style(color=1, arrow=1)), 
          Line(points=[-90, 0; 61, 54], style(color=1, arrow=1)), 
          Text(
            extent=[21, 51; 44, 44], 
            style(color=1, arrow=1), 
            string="rUL3"), 
          Text(
            extent=[64, 22; 91, 16], 
            style(color=1, arrow=1), 
            string="rUW"), 
          Line(points=[-90, 0; 90, 15], style(color=1, arrow=1)), 
          Text(
            extent=[15, -4; 32, -10], 
            style(color=1, arrow=1), 
            string="rUS"), 
          Line(points=[-90, 0; 40, -4], style(color=1, arrow=1))), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
The MacPherson strut suspension was invented in the 1940s by Earl S. MacPherson of Ford.
It was introduced on the 1950 English Ford and has since become one of the dominating suspensions 
systems of the world because of its compactness and low cost. 
Unlike other suspension designs, in MacPherson strut suspension, the telescopic shock absorber also 
serves as a link to control the position of the wheel and makes the upper control arm obsolete. Since 
the strut is vertically positioned, the whole suspension is very compact and suitable for front-wheel
 drive cars, whose engine and transmission are all located inside the front compartment and need front suspensions which engage very little width of the car.
</p>
<p>
Advantages:
Compact and cheap.
</p>
<p>
Disadvantages:
Body roll and wheel's movement lead to variation in camber, although not as severe as swing axle suspension. 
</p>
<p>
Usage:
The MacPherson strut is the dominatin front suspension for cars.
</p>
<p>
The following parameters are used to define the component, see nomenclature for further information:
</p>
<pre>
  scaleFactor: The component can be scaled assymetrically, e.g. {1,-1,1} makes a left-hand linkage into a right-hand dito.
  rUL1L2: location of outer ballUtilities.Joints.Joint, resolved in frame_C
  rCL1:   location of inner frontUtilities.Joints.Joint, resolved in frame_C
  rCL2:   location of inner rearUtilities.Joints.Joint, resolved in frame_C
  rCS:    location of strut mount in chassis, resolved in frame_C
  rUS:    location of strut mount in upright, resolved in frame_C
  rUW:    location of frame_W, resolved in frame_C
  forceTable: To set the force generation table add a component named [name] of class Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    protected 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_ta1
        annotation (extent=[59, -10; 61, -8]);
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_S
        annotation (extent=[-70, 50; -100, 80], rotation=180);
      Utilities.Parts.Body steeringLever(r=rUL3_scaled - rUL1L2_scaled)
        annotation (extent=[84, 57; 104, 37], rotation=270);
    equation 
      connect(lower.frame_a, frame_C)
        annotation (points=[-77, -9; -77, 0; -94, 0]);
      connect(upper.frame_a, frame_C)
        annotation (points=[-77, 10; -77, 0; -91, 0]);
      connect(MacPherson.frame_b, upper.frame_b)
        annotation (points=[1, 22.05; 0, 40; -77, 40; -77, 31]);
      connect(steeringJoint.frame_b, frame_S)
        annotation (points=[65, 71; 65, 80; -85, 80]);
      connect(outerRod.frame_b, frame_U) annotation (points=[85.5, 0; 90, 0]);
      connect(MacPherson.frame_pb, strut.frame_b)
        annotation (points=[23, 12.6; 24, 4; 24, 30; 40, 30; 40, 20]);
      connect(springRod.frame_b, strut.frame_a)
        annotation (points=[40, -9.5; 40, 0]);
      connect(frame_ta1, outerRod.frame_a) annotation (points=[60, -9; 64, 0]);
      connect(U.frame_a, frame_ta1) annotation (points=[66, -51; 60, -8]);
      connect(frame_ta1, steeringJoint.frame_ta)
        annotation (points=[60, -9; 48, 43]);
      connect(MacPherson.frame_pa, steeringJoint.frame_a) annotation (points=[
            23, -11.2; 26, -14; 26, -50; 54, -50; 54, 30; 65, 30; 65, 39]);
      connect(springRod.frame_a, frame_ta1)
        annotation (points=[40, -30.5; 60, -9]);
      connect(rearBar.frame_a, frontBar.frame_b)
        annotation (points=[-20, -40; -20, -61]);
      connect(innerJoint.frame_b, frontBar.frame_a)
        annotation (points=[-50, -60; -40, -60]);
      connect(lower.frame_b, innerJoint.frame_a)
        annotation (points=[-78, -30; -78, -60; -70, -60]);
      connect(frontBar.frame_b, MacPherson.frame_a)
        annotation (points=[-20, -59; 0, -60; -0.5, -19.975]);
      connect(steeringJoint.frame_c, steeringRod.frame_a)
        annotation (points=[49, 68; 30, 70]);
      connect(frame_L12, frontBar.frame_b)
        annotation (points=[3, -96; -19, -61]);
      connect(steeringLever.frame_a, steeringJoint.frame_ta)
        annotation (points=[94, 36; 50, 42]);
    end MacPherson2;
  end Components;
  
  model FormulaChassis 
    Chassis.Components.FormulaBody FormulaBody1
      annotation (extent=[-4, 10; 30, 46]);
    extends Chassis.Interfaces.ChassisInterface;
    
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Diagram, 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.28, 
        height=0.42), 
      Icon(
        Rectangle(extent=[52, 54; 90, 32], style(gradient=1, fillColor=9)), 
        Line(points=[-48, 42; -58, 20; -58, -20; -48, -44; -38, -20; -38, 20; -48, 
               42], style(
            color=0, 
            thickness=4, 
            fillColor=0)), 
        Line(points=[70, 44; 58, 18; 58, -16; 70, -44; 80, -20; 80, 20; 70, 44], 
             style(
            color=0, 
            thickness=4, 
            fillColor=0)), 
        Rectangle(extent=[-26, 20; 80, -20], style(color=41, fillColor=43)), 
        Polygon(points=[-26, -22; -18, -40; 4, -48; 42, -48; 54, -20; -26, -20; 
               -26, -22], style(
            color=41, 
            thickness=2, 
            fillColor=41)), 
        Polygon(points=[-26, 20; 54, 20; 42, 46; 2, 46; -18, 40; -26, 22; -26, 
              20], style(
            color=41, 
            thickness=2, 
            fillColor=41)), 
        Polygon(points=[-70, 48; -70, -46; -88, -42; -88, 44; -70, 48], style(
              color=41, fillColor=41)), 
        Polygon(points=[-24, 20; -94, 10; -94, -8; -24, -20; -24, 20], style(
              color=41, fillColor=43)), 
        Polygon(points=[-88, 6; -88, -4; -8, -12; -2, -16; 32, -16; 40, -12; 66, 
               -8; 66, 6; 40, 10; 32, 16; -2, 16; -8, 12; -88, 6], style(color=
                41, fillColor=41)), 
        Ellipse(extent=[-2, 12; 32, -12], style(color=8, fillColor=10)), 
        Ellipse(extent=[2, 10; 26, -10], style(gradient=3, fillColor=73)), 
        Rectangle(extent=[-66, 54; -30, 32], style(gradient=1, fillColor=10)), 
        Rectangle(extent=[-66, -30; -30, -52], style(gradient=1, fillColor=10)), 
           
        Rectangle(extent=[52, 54; 90, 32], style(gradient=1, fillColor=10)), 
        Rectangle(extent=[52, -32; 90, -56], style(gradient=1, fillColor=10)), 
        Polygon(points=[66, 32; 66, -32; 74, -40; 90, -40; 94, -20; 94, 20; 90, 
               40; 90, 40; 74, 40; 66, 32; 66, 32], style(color=41, fillColor=
                41)), 
        Polygon(points=[-24, 26; -18, 40; 0, 46; 42, 46; 42, 44; 0, 44; -16, 38; 
               -20, 28; -24, 26], style(color=41, fillColor=43)), 
        Polygon(points=[-20, -30; -16, -40; 0, -46; 42, -46; 42, -48; 0, -48; -18, 
               -42; -24, -28; -20, -30], style(color=41, fillColor=43)), 
        Text(extent=[0, -14; 40, -52], string="L")), 
      Documentation(info="A model of a Dallara F3 chassis. 
"));
    
    ModelicaAdditions.MultiBody.Parts.InertialSystem ground(ng={0,0,-1})
      annotation (extent=[-40, -100; -20, -80]);
    Utilities.Joints.FreeMotion freeMotion(
      r_rela(start=rInit, fixed=true), 
      v_rela(start=vInit, fixed=true), 
      w_rela(start={0,0,0}, fixed=true), 
      phi(start=phiInit, fixed=true))
      annotation (extent=[-20, -70; 0, -50], rotation=90);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_1(tyreData=
          frontWheelData, w(start=vInit[1]/0.3)) extends 
      Wheels.Utilities.BaseWheel "Front left wheel" annotation (
      choicesAllMatching, 
      extent=[-80, -60; -40, -30], 
      rotation=270);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_2(
      leftWheel=false, 
      tyreData=frontWheelData, 
      w(start=vInit[1]/0.3)) extends Wheels.Utilities.BaseWheel 
      "Front right wheel" annotation (
      choicesAllMatching, 
      extent=[-40, 30; -80, 60], 
      rotation=90);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_3(tyreData=
          rearWheelData, w(start=vInit[1]/0.3)) extends 
      Wheels.Utilities.BaseWheel "Rear left wheel" annotation (
      choicesAllMatching, 
      extent=[40, -60; 80, -30], 
      rotation=270);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_4(
      leftWheel=false, 
      tyreData=rearWheelData, 
      w(start=vInit[1]/0.3)) extends Wheels.Utilities.BaseWheel 
      "Rear right wheel" annotation (
      choicesAllMatching, 
      extent=[80, 32; 40, 62], 
      rotation=90);
    replaceable Chassis.Suspensions.DoubleWishBoneSuspension2 frontSuspension
       extends Chassis.Interfaces.SteerableSuspensionInterface 
      "Front suspension"
      annotation (choicesAllMatching, extent=[-78, -20; -38, 20]);
    replaceable Chassis.Suspensions.DoubleWishBoneSuspension3 rearSuspension
       extends Chassis.Interfaces.SteerableSuspensionInterface 
      "Rear suspension"
      annotation (choicesAllMatching, extent=[78, -20; 38, 20]);
    Modelica.Mechanics.Rotational.Fixed fixed
      annotation (extent=[10, -20; 30, 0]);
    Wheels.RillTyre.TyreDataDallaraFrontWheel frontWheelData(
      R0=0.2663, 
      Rrim=0.3302/2, 
      m=10, 
      Iyy=10*4/9*0.2663*0.2663/2, 
      Ixx=10*4/9*0.2663*0.2663, 
      c_x=190000, 
      c_y=170000, 
      c_z=165000, 
      d_x=3000, 
      d_y=3000, 
      d_z=3000, 
      width=0.223) "Pirelli 200/530-13" annotation (extent=[-100, 80; -80, 100]);
    Wheels.RillTyre.TyreDataDallaraRearWheel rearWheelData
      annotation (extent=[80, 80; 100, 100]);
  equation 
    connect(ground.frame_b, freeMotion.frame_a) annotation (points=[-19.5, -90; 
           -10, -90; -10, -70.5], style(thickness=4));
    connect(fixed.flange_b, rearSuspension.flange_SW)
      annotation (points=[20, -10; 43, -10], style(color=0));
    connect(freeMotion.frame_b, BGR) annotation (points=[-10, -50; -10, 28; -92.5, 
           28; -92.5, 38; -92, 38], style(thickness=4));
    connect(frontSuspension.frame_U_1, wheel_1.carrierFrame)
      annotation (points=[-60, -24; -60, -34], style(thickness=4));
    connect(frontSuspension.frame_U_2, wheel_2.carrierFrame)
      annotation (points=[-60, 24; -60, 34], style(thickness=4));
    connect(frontSuspension.frame_C, freeMotion.frame_b)
      annotation (points=[-40, 8; -10, 8; -10, -48], style(thickness=4));
    connect(rearSuspension.frame_U_2, wheel_4.carrierFrame)
      annotation (points=[60, 24; 60, 36.5], style(thickness=4));
    connect(rearSuspension.frame_U_1, wheel_3.carrierFrame)
      annotation (points=[60, -24; 60, -34], style(thickness=4));
    connect(wheel_1.driveShaft1D, flange_1)
      annotation (points=[-60, -56; -60, -80], style(color=0));
    connect(wheel_3.driveShaft1D, flange_3)
      annotation (points=[60, -56; 60, -78], style(color=0));
    connect(wheel_4.driveShaft1D, flange_4)
      annotation (points=[60, 56; 60, 80], style(color=0));
    connect(wheel_2.driveShaft1D, flange_2)
      annotation (points=[-60, 56; -60, 78], style(color=0));
    connect(frontSuspension.flange_SW, flange_SW) annotation (points=[-42.8, -10; 
           -32, -10; -32, -20; -100, -20; -100, -44], style(color=0));
    connect(freeMotion.frame_b, rearSuspension.frame_C) annotation (points=[-10, 
           -50; -10, 8; 40, 8], style(
        thickness=4, 
        fillColor=1, 
        fillPattern=1));
    connect(FormulaBody1.frame_a1, freeMotion.frame_b) annotation (points=[-2, 
          28; -10, 28; -10, -50], style(
        thickness=4, 
        fillColor=1, 
        fillPattern=1));
  end FormulaChassis;
  
  model Trailer 
    
    parameter SI.Position rCP[3]={-3.5,0,0.1} 
      "Vector from car's BRG to trailer pivot point which also is trailer's BGR";
    parameter SI.Position[3] rCMBody={-lambda*L,0,kappa*L};
    parameter SI.Distance L=2.85;
    parameter Real lambda=0.95;
    parameter Real kappa=0.2;
    parameter SI.Mass mBody=600;
    parameter SI.Inertia i11Body=500;
    parameter SI.Inertia i22Body=500;
    parameter SI.Inertia i33Body=800;
    
    replaceable Wheels.RillTyre.WheelWithForceAnimation Wheel_1(hub(qd(start=20
              /0.3))) extends Wheels.Utilities.BaseWheel "Left wheel" 
      annotation (
      choicesAllMatching, 
      extent=[50, -60; 90, -30], 
      rotation=270);
    replaceable Wheels.RillTyre.WheelWithForceAnimation Wheel_2(leftWheel=false, 
         hub(qd(start=20/0.3))) extends Wheels.Utilities.BaseWheel 
      "Right wheel" annotation (
      choicesAllMatching, 
      extent=[90, 30; 50, 60], 
      rotation=90);
    
    replaceable Chassis.Suspensions.FixedAxleSuspension suspension extends 
      Chassis.Interfaces.SuspensionInterface "Suspension" annotation (
      choicesAllMatching, 
      extent=[48, 20; 88, -20], 
      rotation=180);
    Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_1
      annotation (extent=[60, -80; 80, -60]);
    Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_2
      annotation (extent=[60, 60; 80, 80], rotation=180);
    annotation (Diagram, Icon(
        Rectangle(extent=[-40, 50; 100, -48], style(
            color=0, 
            fillColor=0, 
            fillPattern=1)), 
        Rectangle(extent=[-38, 48; 98, -46], style(
            color=0, 
            fillColor=8, 
            fillPattern=1)), 
        Rectangle(extent=[-80, 4; -40, -4], style(
            color=0, 
            gradient=2, 
            fillColor=8, 
            fillPattern=1))));
    VehicleDynamics.Utilities.Joints.Spherical spherical
      annotation (extent=[-20, -10; 0, 10]);
    ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation(r=rCP)
      annotation (extent=[-60, -10; -40, 10]);
    ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
      annotation (extent=[-100, -10; -80, 10]);
    ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_b
      annotation (extent=[102, -10; 122, 10]);
    Utilities.Parts.Body body(
      lengthDirection={-1,0,0}, 
      height=0.1, 
      material={0.4,0.4,0.4,0.2}, 
      rCM=rCMBody, 
      m=mBody, 
      I11=i11Body, 
      I22=i22Body, 
      I33=i33Body, 
      r0={-1,0,0.2}, 
      length=L - 0.5, 
      width=1.2, 
      widthDirection={0,1,0}) annotation (extent=[12, 22; 32, 42]);
    Utilities.Parts.Body rod(
      material={0.4,0.4,0.4,0.2}, 
      r={-1,0,0}, 
      shape="cylinder") annotation (extent=[12, 50; 32, 70]);
  equation 
    connect(suspension.frame_U_2, Wheel_2.carrierFrame) annotation (points=[
          69.8, 21; 69.8, 28; 70, 28; 70, 34.5], style(fillColor=0, fillPattern
          =1));
    connect(Wheel_1.carrierFrame, suspension.frame_U_1) annotation (points=[70, 
           -34.5; 70, -21; 69.8, -21], style(fillColor=0, fillPattern=1));
    connect(Wheel_1.driveShaft1D, flange_1) annotation (points=[70, -55.5; 70, 
          -70], style(
        color=0, 
        fillColor=0, 
        fillPattern=1));
    connect(Wheel_2.driveShaft1D, flange_2) annotation (points=[70, 55.5; 70, 
          70], style(
        color=0, 
        fillColor=0, 
        fillPattern=1));
    connect(frameTranslation.frame_a, frame_a) annotation (points=[-60.5, 0; -90, 
           0], style(fillColor=0, fillPattern=1));
    connect(frameTranslation.frame_b, spherical.frame_a) annotation (points=[-39.5, 
           0; -20.5, 0], style(fillColor=0, fillPattern=1));
    connect(spherical.frame_b, suspension.frame_C) annotation (points=[0.5, 0; 
          20, 0; 20, 7.8; 51, 7.8], style(fillColor=0, fillPattern=1));
    connect(spherical.frame_b, frame_b)
      annotation (points=[0.5, 0; 22, -26; 100, -26; 112, 0]);
    connect(body.frame_a, spherical.frame_b)
      annotation (points=[11.5, 32; 0.5, 0]);
    connect(rod.frame_a, spherical.frame_b)
      annotation (points=[11.5, 60; 0.5, 0]);
  end Trailer;
  
  model StandardChassis 
    Chassis.Components.SedanBody body "Body of chassis"
      annotation (choicesAllMatching, extent=[0, -40; 40, 0]);
    extends Chassis.Interfaces.ChassisInterface;
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Diagram, 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.28, 
        height=0.42), 
      Icon(
        Rectangle(extent=[-80, 52; 84, -46], style(color=41, fillColor=41)), 
        Polygon(points=[-46, 38; -46, -36; -14, -22; -14, 32; -46, 38; -46, 38], 
             style(color=0, fillColor=8)), 
        Polygon(points=[-32, -36; -8, -24; 48, -24; 64, -40; -30, -36; -32, -36], 
             style(color=0, fillColor=8)), 
        Polygon(points=[-32, 42; -10, 34; 54, 34; 64, 44; 64, 44; -30, 42; -32, 
               42], style(color=0, fillColor=8)), 
        Polygon(points=[52, 26; 52, -18; 66, -36; 66, 40; 52, 26], style(color=
                0, fillColor=8)), 
        Ellipse(extent=[-74, 46; -68, 26], style(fillColor=7)), 
        Ellipse(extent=[-74, -20; -68, -40], style(fillColor=7))), 
      Documentation(info=
            "<HTML>This is a standard chassis assuming a rigid body, a front steerable suspension and a rear suspension without steering. All compoents such as wheels and suspensions can easilly be replaced wiht others based on the same interfaces.</HTML>"));
    ModelicaAdditions.MultiBody.Parts.InertialSystem ground(ng={0,0,-1})
      annotation (choicesAllMatching, extent=[-40, -100; -20, -80]);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_1(hub(qd(start=
              vInit[1]/0.3)), tyreData=frontWheelData) extends 
      Wheels.Utilities.BaseWheel "Front left wheel" annotation (
      choicesAllMatching, 
      extent=[-80, -58; -40, -28], 
      rotation=270);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_2(
      leftWheel=false, 
      hub(qd(start=vInit[1]/0.3)), 
      tyreData=frontWheelData) extends Wheels.Utilities.BaseWheel(leftWheel=
          false) "Front right wheel" annotation (
      choicesAllMatching, 
      extent=[-40, 30; -80, 60], 
      rotation=90);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_3(hub(qd(start=
              vInit[1]/0.3)), tyreData=rearWheelData) extends 
      Wheels.Utilities.BaseWheel "Rear left wheel" annotation (
      choicesAllMatching, 
      extent=[40, -60; 80, -30], 
      rotation=270);
    replaceable Wheels.RillTyre.WheelWithForceAnimation wheel_4(
      leftWheel=false, 
      hub(qd(start=vInit[1]/0.3)), 
      tyreData=rearWheelData) extends Wheels.Utilities.BaseWheel 
      "Rear right wheel" annotation (
      choicesAllMatching, 
      extent=[80, 30; 40, 60], 
      rotation=90);
    Utilities.Joints.FreeMotion freeMotion(
      r_rela(start=rInit, fixed=true), 
      v_rela(start=vInit, fixed=true), 
      w_rela(start={0,0,0}, fixed=true), 
      phi(start=phiInit, fixed=true)) annotation (
      choicesAllMatching, 
      extent=[-20, -72; 0, -52], 
      rotation=90);
    replaceable Suspensions.MacPhersonSuspension2 frontSuspension extends 
      Chassis.Interfaces.SteerableSuspensionInterface 
      "Front suspension with steering"
      annotation (choicesAllMatching, extent=[-78, -20; -38, 20]);
    replaceable Suspensions.MultiLink4Suspension5 rearSuspension extends 
      Chassis.Interfaces.SuspensionInterface "Rear suspension without steering"
      annotation (choicesAllMatching, extent=[78, -20; 38, 20]);
    Wheels.RillTyre.TyreData frontWheelData
      annotation (choicesAllMatching, extent=[-100, 60; -80, 80]);
    Wheels.RillTyre.TyreData rearWheelData
      annotation (choicesAllMatching, extent=[14, 64; 34, 84]);
  equation 
    connect(ground.frame_b, freeMotion.frame_a) annotation (points=[-19.5, -90; 
           -10, -90; -10, -72.5], style(thickness=4));
    connect(BGR, freeMotion.frame_b) annotation (points=[-100, 40; -100, 28; -10, 
           28; -10, -51.5], style(thickness=4));
    connect(wheel_4.driveShaft1D, flange_4)
      annotation (points=[60, 55.5; 60, 70], style(color=0));
    connect(wheel_2.driveShaft1D, flange_2)
      annotation (points=[-60, 55.5; -60, 70], style(color=0));
    connect(frontSuspension.flange_SW, flange_SW) annotation (points=[-41, -7.8; 
           -30, -7.8; -30, -26; -98, -26; -98, -40], style(color=0, thickness=2));
    connect(wheel_1.driveShaft1D, flange_1)
      annotation (points=[-60, -53.5; -60, -70], style(color=0));
    connect(wheel_3.driveShaft1D, flange_3)
      annotation (points=[60, -55.5; 60, -70], style(color=0));
    connect(rearSuspension.frame_U_2, wheel_4.carrierFrame) annotation (points=
          [59.8, 21; 59.8, 28; 60, 28; 60, 34], style(thickness=4));
    connect(rearSuspension.frame_U_1, wheel_3.carrierFrame) annotation (points=
          [59.8, -21; 59.8, -28; 60, -28; 60, -34], style(thickness=4));
    connect(frontSuspension.frame_U_1, wheel_1.carrierFrame) annotation (points
        =[-59.8, -21; -59.8, -27; -60, -27; -60, -32], style(thickness=4));
    connect(frontSuspension.frame_U_2, wheel_2.carrierFrame) annotation (points
        =[-59.8, 21; -59.8, 28; -60, 28; -60, 34], style(thickness=4));
    connect(frontSuspension.frame_C, freeMotion.frame_b)
      annotation (points=[-41, 7.8; -10, 7.8; -10, -51.5], style(thickness=4));
    connect(rearSuspension.frame_C, freeMotion.frame_b)
      annotation (points=[41, 7.8; -10, 7.8; -10, -51.5], style(thickness=4));
    connect(freeMotion.frame_b, body.frame_a1)
      annotation (points=[-10, -51.5; -10, -20; 0.8, -20], style(thickness=4));
  end StandardChassis;
end Chassis;


package Wheels "Wheel, tyre and road models" 
  extends Modelica.Icons.Library;
  
  annotation (Coordsys(
      extent=[0, 0; 632, 426], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.05, 
      y=0.1, 
      width=0.4, 
      height=0.4, 
      library=1, 
      autolayout=1));
  package RillTyre "Data and functions for tyre model according to Rill" 
    import SI = Modelica.SIunits;
    extends Modelica.Icons.Library;
    
    record TyreData "Tyre data" 
      extends Utilities.WheelData;
      
      // Spring and damping characteristic of tyre
      parameter Real c_x(unit="N/m") = 100000 
        "tyre spring constant in x-direction";
      parameter Real c_y(unit="N/m") = 150000 
        "tyre spring constant in y-direction";
      parameter Real c_z(unit="N/m") = 200000 
        "tyre spring constant in z-direction";
      parameter Real d_x(unit="N.s/m") = 1500 
        "tyre damping constant in x-direction";
      parameter Real d_y(unit="N.s/m") = 300 
        "tyre damping constant in y-direction";
      parameter Real d_z(unit="N.s/m") = 300 
        "tyre damping constant in z-direction";
      
      // Data for nominal vertical load 1
      parameter Real mue_nom=0.7 
        "friction coefficient of road for which tyre data is valid";
      
      NominalLoad load1 annotation (extent=[-85, 15; -15, 85]);
      NominalLoad load2(
        Fz_nom=6000, 
        Fds_x=75000, 
        s_max_x=0.18, 
        F_max_x=4500, 
        s_slide_x=0.5, 
        F_slide_x=4200, 
        Fds_y=60000, 
        s_max_y=0.24, 
        F_max_y=4125, 
        s_slide_y=0.8, 
        F_slide_y=3750, 
        dL0=0.1, 
        s0=0.275, 
        sE=0.8) annotation (extent=[15, 15; 85, 85]);
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
    end TyreData;
    
    record NominalLoad "Tyre force data for nominal load" 
      /* Definition of tyre force characteristic
       Note, that the following requirements must be fulfilled:
             Fds_x >= 2*Fmax_x / s_Fmax_x
             Fds_y >= 2*Fmax_y / s_Fmax_y
       in order that there is no turning point for 0 < s <= s_Fmax
       Also: 0.1 <= dL0 <= 1/6
    */
      parameter SI.Force Fz_nom=3000 "Nominal normal force";
      
      parameter SI.Force Fds_x=50000 "Slope at s_x=0 in longitudinal direction";
      parameter Real s_max_x=0.15 "Slip of maximum longitudinal tyre force";
      parameter SI.Force F_max_x=3000 "maximum longitudinal tyre force";
      parameter Real s_slide_x=0.4 "Slip of sliding begin";
      parameter SI.Force F_slide_x=2800 
        "Longitudinal tyre force at sliding begin";
      
      parameter SI.Force Fds_y=40000 "Slope at s_y=0 in lateral direction";
      parameter Real s_max_y=0.21 "Slip of maximum lateral tyre force";
      parameter SI.Force F_max_y=2750 "maximum lateral tyre force";
      parameter Real s_slide_y=0.6 "Slip of sliding begin";
      parameter SI.Force F_slide_y=2500 "Lateral tyre force at sliding begin";
      
      parameter Real dL0=0.133 
        "(distance contact point to f_y) / (length of contact area) for zero lateral slip";
      parameter Real s0=0.25 
        "Lateral slip where dL=0 for the first time (usually, a bit higher as s_Fmax_y)";
      parameter Real sE=0.6 
        "Lateral slip where dL remains zero for higher slips";
    end NominalLoad;
    
    model Wheel "Wheel model according to Rill" 
      import Car;
      import Modelica.Math;
      extends Utilities.BaseWheel(final wheelData(
          R0=tyreData.R0, 
          width=tyreData.width, 
          m=tyreData.m, 
          Iyy=tyreData.Iyy, 
          Ixx=tyreData.Ixx));
      
      //SLIP DEFINITION
      
      SI.Angle slipAngle 
        "Angle between wheel axis velocity and the x-direction of the wheel";
      Real slip "slip at contact point computed with contact velocities";
      Real slip_limited "slip limited in the range -1.1 for plotting";
      Real sinphi 
        "sin(phi), where phi is the angle between -Wv_x and -Wv_y, where Wv_x, Wv_y are the contact point velocities";
      Real cosphi "cos(phi)";
      SI.Velocity Wv_abs "Absolute value of contact point velocity";
      SI.Velocity Rw "denominator of slip quotient";
      
      //TYRE DEFLECTION
    protected 
      parameter Real delta_x_nom=tyreData.load1.F_slide_x/tyreData.c_x 
        "nominal tyre deflection in x-direction";
      parameter Real delta_y_nom=tyreData.load1.F_slide_y/tyreData.c_y 
        "nominal tyre deflection in y-direction";
    public 
      SI.Position delta_x 
        "Longitudinal tyre deflection (= wheel deformation in x-direction of W-frame)";
      SI.Position delta_y 
        "Lateral tyre deflection (= wheel deformation in y-direction of W-frame)";
      SI.Position delta_z 
        "Radial tyre deflection (= wheel deformation in z-direction of W-frame)";
      parameter SI.Position delta_z_offset=1e-3 
        "In order to make the z-computations less non-linear";
      
      //TYRE CONTACT AND FORCE GENERATION
      
      Boolean contact "true, if wheel has contact with road";
      Real mueRoad;
      parameter Real delta_eps=0.1 
        "should be about 0.1 ... 1; see explanation in model";
      SI.Force f_x "Tyre/contact force at origin of W-frame in x-direction";
      SI.Force f_y "Tyre/contact force at origin of W-frame in y-direction";
      SI.Force f_z "Tyre/contact force at origin of W-frame in z-direction";
      SI.Force f_x0 "f_x at steady state conditions";
      SI.Force f_y0 "f_y at steady state conditions";
      SI.Force f0 
        "Absolute value of tyre/contact force in contact plane at steady state conditions";
      SI.Force df_ds 
        "Derivative of steady state contact force in contact plane with respect to slip";
      SI.Force t_z "Tyre/contact torque at W-frame in z-direction";
      Real dL 
        "(distance contact point to f_y) / (length of contact area); used to compute t_z";
      Real dfx_dvx1 "Partial derivative of f_x with respect to Wv_x";
      Real dfy_dvy1 "Partial derivative of f_y with respect to Wv_y";
      Real dfx_dvx "dfx_dvx1 where potential positive values are set to zero";
      Real dfy_dvy "dfy_dvy1 where potential positive values are set to zero";
      
      Real Lcontact "length of contact area in x-direction";
      Real delta_diff;
      Real delta_diff_pos;
      
      Real delta_x_scaled "delta_x scaled on nominal value";
      Real delta_y_scaled "delta_y scaled on nominal value";
      Real S_rel[3, 3];
      
    public 
      parameter TyreData tyreData annotation (extent=[-80, 60; -60, 80]);
      tyreForces tyre(data=tyreData) annotation (extent=[14, -68; 34, -48]);
      annotation (
        Documentation(info="This element contains the inertia properties of a
rotating wheel and the tyre forces acting at the
contact point of the wheel with the road. The tyre forces
are computed accorded to the equations of Rill (see reference below).
The wheel data is defined via the record WheelData and the tire
data is defined via the record RillTyre.TyreData.
Therefore, it suffices to define one instance of
these records and use them for all 4 wheels of a vehicle.
The frames of this object have the following meaning:
Frame carrierFrame is fixed in the carrier of the wheel
and is located on the spin axis of the wheel in the
middle point of the wheel. This point is also approximately
used as the center-of-mass of the wheel.
It is temporarily assumed that the road lies in the
x-y plane of the inertial frame! This restriction can
be removed by defining the road properties via inner/outer
functions.
Flange driveShaft is used to drive the wheel with
a 1D-drive train.
This element is realized according to the description
  Georg Rill: Simulation von Kraftfahrzeugen,
              Vieweg, 1994
"), 
        Icon, 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.03, 
          y=0.14, 
          width=0.45, 
          height=0.81), 
        Diagram);
      
      VisualShape shape_1(
        r0=-nShape*0.7*abs(wheelData.width), 
        Length=abs(wheelData.width), 
        Width=2*wheelData.R0, 
        Height=2*wheelData.R0, 
        LengthDirection=nShape, 
        WidthDirection={1,0,0}, 
        Material={0,0,0,.3}, 
        Extra=wheelData.Rrim/wheelData.R0, 
        Shape="tire") annotation (extent=[0, -40; 20, -20]);
      VisualShape shape0(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        WidthDirection=-nShape, 
        Material={0.2,0,0,0.7}) annotation (extent=[-44, -96; -24, -76]);
      VisualShape shape1(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={0.6235,0,0.7818}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[-16, -96; 4, -76]);
      VisualShape shape2(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.2225,0,0.9749}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[16, -94; 36, -74]);
      VisualShape shape3(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.9010,0,0.4339}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[48, -92; 68, -72]);
      VisualShape shape4(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.9010,0,-0.4339}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[74, -90; 94, -70]);
      VisualShape shape5(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.2225,0,-0.9749}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[104, -88; 124, -68]);
      VisualShape shape6(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={0.6235,0,-0.7818}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[118, -86; 138, -66]);
    equation 
      
      //SLIP DEFINITION
      //slipAngle = Modelica.Math.atan2(Cv[2], noEvent(abs(Cv[1])));
      slipAngle = Modelica.Math.atan2(Wv_y, Rw);
      //also possible slipAngle=Modelica.Math.atan2(Wv_y,Wv_x);
      /* Determine slip
       Wv_abs: Absolute value of contact point velocity (= sqrt(Wv*Wv))
       phi   : Angle between longitudinal and lateral contact point velocity
               Assumption: For Wv_abs small -> Wv_x = Wv_abs, Wv_y = 0, i.e.,
                           velocity only in longitudinal direction.
       sinphi: sin(phi) (= 0 for Wv_abs = 0)
       cosphi: cos(phi) (= -1 for Wv_abs = 0)
    */
      Wv_abs = noEvent(max([sqrt(Wv_x*Wv_x + Wv_y*Wv_y); 1.e-10]));
      Rw = wheelData.R0*abs(w) + 0.1;
      slip = Wv_abs/Rw;
      slip_limited = noEvent(if slip > 1 then 1 else if slip < -1 then -1 else 
        slip);
      if noEvent(Wv_abs > v_eps) then
        sinphi = -Wv_y/Wv_abs;
        cosphi = -Wv_x/Wv_abs;
      else
        sinphi = 0;
        cosphi = -1;
      end if;
      
      //TYRE FORCES AND TORQUES  
      
      delta_z = if delta_r >= 0 then delta_r else 0;
      
      // Compute  force perpendicular to contact plane.
      f_z = if delta_r >= 0 then tyreData.c_z*delta_z + tyreData.d_z*ddelta_r
         else 0;
      
      // Compute steady state tyre forces in x- and y-direction
      
      // (f0,f_x0,f_y0,df_ds,dL) = RillTyre.tyreForces(tyreData, slip, sinphi,
      //  cosphi, f_z, mueRoad, camberAngle);
      tyre.slip = slip;
      tyre.sinphi = sinphi;
      tyre.cosphi = cosphi;
      tyre.f_z = f_z;
      tyre.mueRoad = mueRoad;
      tyre.camberAngle = camberAngle;
      tyre.f0 = f0;
      tyre.f_x0 = f_x0;
      tyre.f_y0 = f_y0;
      tyre.df_ds = df_ds;
      tyre.dL = dL;
      
      // Compute data for tyre deflection differential equations
      if noEvent(Wv_abs > v_eps) then
        dfx_dvx1 = -(df_ds*slip*cosphi*cosphi + f0*sinphi*sinphi)/Wv_abs;
        dfy_dvy1 = -(df_ds*slip*sinphi*sinphi + f0*cosphi*cosphi)/Wv_abs;
      else
        dfx_dvx1 = -df_ds/Rw;
        dfy_dvy1 = dfx_dvx1;
      end if;
      
      dfx_dvx = if dfx_dvx1 <= 0 then dfx_dvx1 else 0;
      dfy_dvy = if dfy_dvy1 <= 0 then dfy_dvy1 else 0;
      
      /* Differential equations for tyre deflections in x- and y-direction
         If the wheel angular velocity is too small, the prerequisites for the
         validity of the tyre force calculation are violated and the equations
         give wrong results. In such a case it is assumed that the profil
         particles do no longer get their maximum deflection. If only
         longitudinal slip is present, the deflection delta_x is calculated
         by "der(delta_x) = -Wv_x" (which is the definition of the deflection delta_x).
         However, delta_x has a maximum which is reached when the particle leaves
         the contact area. If the velocity is constant, a particle remains
         "T = Lcontact / (R0*abs(w))" seconds in the contact area, where Lcontact
         is the length of the contact area, R0 is the undeformed wheel radius
         and w is the angular velocity of the wheel. Therefore, the maximum
         deflection is
               delta_x_max = -Wv_x*T
                           = -Wv_x*Lcontact/(R0*abs(w))
                           = slip*Lcontact.
         Therefore, the estimation is used that the simplified differential
         equation is used as long as deflection is less then "delta_eps*slip*Lcontact"
         where delta_eps is choosen appropriately (e.g., delta_eps=0.1;
         note that delta_eps=1 gives the theoretical switching condition
         derived by simplified assumptions).
      */
      contact = delta_r >= 0;
      Lcontact = noEvent(sqrt(8*tyreData.R0*(delta_z + delta_z_offset)) - sqrt(
        8*tyreData.R0*delta_z_offset));
      delta_diff = noEvent(sqrt(delta_x*delta_x + delta_y*delta_y) - abs(
        delta_eps*slip*Lcontact));
      delta_diff_pos = noEvent(if delta_diff >= 0 or Rw >= 1 then 1 else 0);
      
      der(delta_x_scaled) = (if not contact then 0 else if noEvent(
        delta_diff_pos > 0.5) then (f_x0 - tyreData.c_x*delta_x)/(tyreData.d_x
         - dfx_dvx) else -Wv_x)/delta_x_nom;
      der(delta_y_scaled) = (if not contact then 0 else if noEvent(
        delta_diff_pos > 0.5) then (f_y0 - tyreData.c_y*delta_y)/(tyreData.d_y
         - dfy_dvy) else -Wv_y)/delta_y_nom;
      
      delta_x = delta_x_scaled*delta_x_nom;
      delta_y = delta_y_scaled*delta_y_nom;
      
      when change(contact) then
        // Tyre looses contact to ground or gets contact to ground
        reinit(delta_x_scaled, 0);
        reinit(delta_y_scaled, 0);
      end when;
      
      
        // Compute dynamic tyre forces in x- and y-direction and tyre torque in z-direction
      f_x = tyreData.c_x*delta_x + tyreData.d_x*der(delta_x);
      f_y = tyreData.c_y*delta_y + tyreData.d_y*der(delta_y);
      t_z = -Lcontact*dL*f_y;
      
      //FORCES AND MOMENTS REQUIRED BY THE BaseWheel
      F_x = f_x;
      F_y = f_y;
      F_z = f_z;
      M_x = 0;
      //this model is yet unable to generate overturning moment 
      M_y = 0;
      //and roll resistance torque.
      M_z = t_z;
      
      //VISUAL REPRESENTATION OF TYRE AND RIM
      S_rel = [nn]*transpose([nn]) + (identity(3) - [nn]*transpose([nn]))*cos(
        phi) - skew(nn)*sin(phi);
      //tyre
      shape_1.S = wheel_body.frame_b.S*transpose(S_rel);
      shape_1.r = wheel_body.frame_b.r0;
      //rim
      shape0.S = shape_1.S;
      shape0.r = shape_1.r;
      shape1.S = shape0.S;
      shape1.r = shape0.r;
      shape2.S = shape1.S;
      shape2.r = shape1.r;
      shape3.S = shape2.S;
      shape3.r = shape2.r;
      shape4.S = shape3.S;
      shape4.r = shape3.r;
      shape5.S = shape4.S;
      shape5.r = shape4.r;
      shape6.S = shape5.S;
      shape6.r = shape5.r;
    end Wheel;
    
    model WheelWithForceAnimation 
      "Wheel model according to Rill, with animation of tyre forces" 
      
      extends Wheel(hub(qd, q(stateSelect=StateSelect.always)));
      parameter Real scale_F_x=1 "scaling of visual appearance of F_x";
      parameter Real scale_F_y=1 "scaling of visual appearance of F_y";
      parameter Real scale_F_z=1/tyreData.load1.Fz_nom 
        "scaling of visual appearance of F_z";
      parameter Real scale_M_z=1 "scaling of visual appearance of M_z";
      VisualVector force_x(
        category="force", 
        r=Wr, 
        Size=We_x*(scale_F_x*F_x), 
        S=Sroad) annotation (extent=[-94, -78; -74, -98]);
      VisualVector force_y(
        Material={0,1,0,0.5}, 
        category="force", 
        r=Wr, 
        Size=We_y*(scale_F_y*F_y), 
        S=Sroad) annotation (extent=[-64, -78; -44, -98]);
      VisualVector force_z(
        Material={0,0,1,0.5}, 
        category="force", 
        r=Wr, 
        Size=We_z*(scale_F_z*F_z), 
        S=Sroad) annotation (extent=[-34, -78; -14, -98]);
      VisualVector torque(
        Material={1,0,0,0.5}, 
        category="torque", 
        r=Wr, 
        Size=Wt*scale_M_z*M_z, 
        S=Sroad) annotation (extent=[-2, -78; 18, -98]);
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.45, 
          y=0.01, 
          width=0.49, 
          height=0.51));
    end WheelWithForceAnimation;
    
    block tyreForces 
      "Compute tyre forces and torques for steady state conditions" 
      
      // Compute tire force in contact plane
      
      // Compute longitudinal and lateral tire forces
      
      // Compute attachment point of f_y with respect to contact length (dL)
      input Real slip;
      input Real sinphi 
        "sin(phi), where phi is the angle between -Wv_x and -Wv_y, where Wv_x, Wv_y are the contact point velocities";
      input Real cosphi "cos(phi)";
      input SI.Force f_z "Load in normal direction";
      input Real mueRoad "Coefficient of friction of road";
      input SI.Angle camberAngle;
      output SI.Force f0 
        "Tyre force in contact plane for steady state conditions";
      output SI.Force f_x0 
        "Tyre force in x-direction for steady state conditions";
      output SI.Force f_y0 
        "Tyre force in y-direction for steady state conditions";
      output SI.Force df_ds 
        "Derivative of steady state contact force in contact plane with respect to slip";
      output Real dL 
        "(distance contact point to f_y) / (length of contact area); used to compute t_z";
    protected 
      SI.Force f_z1;
      SI.Force f_z2;
      SI.Force Fds_x;
      SI.Force Fds_y;
      SI.Force F_max_x;
      SI.Force F_max_y;
      SI.Force F_slide_x;
      SI.Force F_slide_y;
      SI.Force Fds;
      SI.Force F_max;
      SI.Force F_slide;
      Real s_max_x;
      Real s_max_y;
      Real s_slide_x;
      Real s_slide_y;
      Real s_max;
      Real s_slide;
      Real k_mue;
      Real sigma;
      Real dL0;
      Real s0;
      Real sE;
      Real s_y;
      Real c;
    public 
      parameter TyreData data "Data of tyre"
        annotation (extent=[-80, 40; -40, 80]);
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.11, 
          y=0, 
          width=0.88, 
          height=0.84));
    equation 
      
      // (f0,f_x0,f_y0,df_ds,dL) = RillTyre.tyreForces2(data, slip, sinphi,
      //  cosphi, f_z, mueRoad, camberAngle);
      k_mue = mueRoad/data.mue_nom;
      
      // Compute data of force slip curve, according to Rill
      f_z1 = data.load1.Fz_nom;
      f_z2 = data.load2.Fz_nom;
      
      Fds_x = Utilities.interpolate2(f_z, f_z1, data.load1.Fds_x, f_z2, data.
        load2.Fds_x);
      Fds_y = Utilities.interpolate2(f_z, f_z1, data.load1.Fds_y, f_z2, data.
        load2.Fds_y);
      
      F_max_x = k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_max_x, 
        f_z2, data.load2.F_max_x);
      F_max_y = k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_max_y, 
        f_z2, data.load2.F_max_y);
      
      F_slide_x = k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_slide_x, 
         f_z2, data.load2.F_slide_x);
      F_slide_y = k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_slide_y, 
         f_z2, data.load2.F_slide_y);
      
      s_max_x = k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_max_x, 
        f_z2, data.load2.s_max_x);
      s_max_y = k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_max_y, 
        f_z2, data.load2.s_max_y);
      
      s_slide_x = k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_slide_x, 
         f_z2, data.load2.s_slide_x);
      s_slide_y = k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_slide_y, 
         f_z2, data.load2.s_slide_y);
      
      Fds = noEvent(max([sqrt((Fds_x*cosphi)^2 + (Fds_y*sinphi)^2); 1.e-10]));
      s_max = noEvent(max([sqrt((s_max_x*cosphi)^2 + (s_max_y*sinphi)^2); 
        1.e-10]));
      F_max = noEvent(max([sqrt((F_max_x*cosphi)^2 + (F_max_y*sinphi)^2); 
        1.e-10]));
      s_slide = noEvent(max([sqrt((s_slide_x*cosphi)^2 + (s_slide_y*sinphi)^2); 
         1.0001*s_max]));
      F_slide = noEvent(max([sqrt((F_slide_x*cosphi)^2 + (F_slide_y*sinphi)^2); 
         1.e-10]));
      
      // Compute tire force in contact plane
      if noEvent(slip <= s_max) then
        sigma = slip/s_max;
        c = noEvent(sigma*(s_max/max([F_max, 1.e-10])*Fds - 2 + sigma) + 1);
        f0 = s_max*Fds*sigma/c;
        df_ds = Fds*(1 - sigma)/(c*c);
      elseif noEvent(slip <= s_slide) then
        sigma = (slip - s_max)/(s_slide - s_max);
        c = 0;
        f0 = F_max - (F_max - F_slide)*sigma*sigma*(3 - 2*sigma);
        df_ds = -(F_max - F_slide)*6*sigma*(1 - sigma)/(s_slide - s_max);
      else
        sigma = 0;
        c = 0;
        f0 = F_slide;
        df_ds = 0;
      end if;
      
      // Compute longitudinal and lateral tire forces
      f_x0 = f0*cosphi;
      f_y0 = f0*sinphi + f_z*tan(camberAngle);
      
      // Compute attachment point of f_y with respect to contact length (dL)
      dL0 = Utilities.interpolate1(f_z, f_z1, data.load1.dL0, f_z2, data.load2.
        dL0);
      s0 = Utilities.interpolate1(f_z, f_z1, data.load1.s0, f_z2, data.load2.s0);
      sE = Utilities.interpolate1(f_z, f_z1, data.load1.sE, f_z2, data.load2.sE);
      s_y = slip*sinphi;
      dL = noEvent(if abs(s_y) <= s0 then dL0*(1 - abs(s_y)/s0) else if abs(s_y)
         <= sE then -dL0*(abs(s_y) - s0)/s0*((sE - abs(s_y))/(sE - s0))^2 else 
        0);
    end tyreForces;
    
    function tyreForces2 
      "Compute tyre forces and torques for steady state conditions" 
      
      input TyreData data "Data of tyre";
      input Real slip;
      input Real sinphi 
        "sin(phi), where phi is the angle between -Wv_x and -Wv_y, where Wv_x, Wv_y are the contact point velocities";
      input Real cosphi "cos(phi)";
      input SI.Force f_z "Load in normal direction";
      input Real mueRoad "Coefficient of friction of road";
      input SI.Angle camberAngle;
      output SI.Force f0 
        "Tyre force in contact plane for steady state conditions";
      output SI.Force f_x0 
        "Tyre force in x-direction for steady state conditions";
      output SI.Force f_y0 
        "Tyre force in y-direction for steady state conditions";
      output SI.Force df_ds 
        "Derivative of steady state contact force in contact plane with respect to slip";
      output Real dL 
        "(distance contact point to f_y) / (length of contact area); used to compute t_z";
    protected 
      SI.Force f_z1;
      SI.Force f_z2;
      SI.Force Fds_x;
      SI.Force Fds_y;
      SI.Force F_max_x;
      SI.Force F_max_y;
      SI.Force F_slide_x;
      SI.Force F_slide_y;
      SI.Force Fds;
      SI.Force F_max;
      SI.Force F_slide;
      SI.Force f;
      Real s_max_x;
      Real s_max_y;
      Real s_slide_x;
      Real s_slide_y;
      Real s_max;
      Real s_slide;
      Real k_mue;
      Real sigma;
      Real dL0;
      Real s0;
      Real sE;
      Real s_y;
      Real c;
    algorithm 
      k_mue := mueRoad/data.mue_nom;
      
      // Compute data of force slip curve, according to Rill
      f_z1 := data.load1.Fz_nom;
      f_z2 := data.load2.Fz_nom;
      
      Fds_x := Utilities.interpolate2(f_z, f_z1, data.load1.Fds_x, f_z2, data.
        load2.Fds_x);
      Fds_y := Utilities.interpolate2(f_z, f_z1, data.load1.Fds_y, f_z2, data.
        load2.Fds_y);
      
      F_max_x := k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_max_x, 
        f_z2, data.load2.F_max_x);
      F_max_y := k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_max_y, 
        f_z2, data.load2.F_max_y);
      
      F_slide_x := k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_slide_x, 
         f_z2, data.load2.F_slide_x);
      F_slide_y := k_mue*Utilities.interpolate2(f_z, f_z1, data.load1.F_slide_y, 
         f_z2, data.load2.F_slide_y);
      
      s_max_x := k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_max_x, 
        f_z2, data.load2.s_max_x);
      s_max_y := k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_max_y, 
        f_z2, data.load2.s_max_y);
      
      s_slide_x := k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_slide_x, 
         f_z2, data.load2.s_slide_x);
      s_slide_y := k_mue*Utilities.interpolate1(f_z, f_z1, data.load1.s_slide_y, 
         f_z2, data.load2.s_slide_y);
      
      Fds := sqrt((Fds_x*cosphi)^2 + (Fds_y*sinphi)^2);
      s_max := sqrt((s_max_x*cosphi)^2 + (s_max_y*sinphi)^2);
      F_max := sqrt((F_max_x*cosphi)^2 + (F_max_y*sinphi)^2);
      s_slide := sqrt((s_slide_x*cosphi)^2 + (s_slide_y*sinphi)^2);
      F_slide := sqrt((F_slide_x*cosphi)^2 + (F_slide_y*sinphi)^2);
      
      // Compute tire force in contact plane
      if slip <= s_max then
        sigma := slip/s_max;
        c := sigma*(s_max/max([F_max, 1.e-10])*Fds - 2 + sigma) + 1;
        f0 := s_max*Fds*sigma/c;
        df_ds := Fds*(1 - sigma)/(c*c);
      elseif slip <= s_slide then
        sigma := (slip - s_max)/(s_slide - s_max);
        f0 := F_max - (F_max - F_slide)*sigma*sigma*(3 - 2*sigma);
        df_ds := -(F_max - F_slide)*6*sigma*(1 - sigma)/(s_slide - s_max);
      else
        f0 := F_slide;
        df_ds := 0;
      end if;
      
      // Compute longitudinal and lateral tire forces
      f_x0 := f0*cosphi;
      f_y0 := f0*sinphi + f_z*tan(camberAngle);
      
      // Compute attachment point of f_y with respect to contact length (dL)
      dL0 := Utilities.interpolate1(f_z, f_z1, data.load1.dL0, f_z2, data.load2
        .dL0);
      s0 := Utilities.interpolate1(f_z, f_z1, data.load1.s0, f_z2, data.load2.
        s0);
      sE := Utilities.interpolate1(f_z, f_z1, data.load1.sE, f_z2, data.load2.
        sE);
      s_y := slip*sinphi;
      dL := if abs(s_y) <= s0 then dL0*(1 - abs(s_y)/s0) else if abs(s_y) <= sE
         then -dL0*(abs(s_y) - s0)/s0*((sE - abs(s_y))/(sE - s0))^2 else 0;
    end tyreForces2;
    
    model TyreProperties1 "Compute characteristic values for tyre" 
      import SI = Modelica.SIunits;
      import Modelica.Math.*;
      import Modelica.Constants.*;
      TyreData tyreData;
      parameter Real f_z[:]=1000:1000:6000 "Load of wheel";
      parameter Real mue[:]={0.1,0.3,0.5,0.7} 
        "road friction coefficient variation";
      
      parameter Real camberAngleDeg=0 "Camber angle in [deg]";
      parameter Real mueRoad=tyreData.mue_nom;
      final parameter Integer nload=size(f_z, 1);
      final parameter Integer nmue=size(mue, 1);
      
      tyreForces tyre1[nload](each data=tyreData)
        annotation (extent=[-20, -40; 0, -20]);
      tyreForces tyre2[nload](each data=tyreData)
        annotation (extent=[-20, -40; 0, -20]);
      tyreForces tyre3[nload](each data=tyreData)
        annotation (extent=[-20, -40; 0, -20]);
      tyreForces tyre4[nmue](each data=tyreData)
        annotation (extent=[-20, -40; 0, -20]);
      
      Real slip "at all slip=time values the tyre equations are evaluated";
      Real f_x[nload] 
        "Tyre force in x-direction for pure longitudinal slip and different f_z values";
      Real df_ds[nload] 
        "Derivative of steady state contact force in contact plane with respect to slip for pure longitudinal slip";
      Real f_y[nload] 
        "Tyre force in y-direction for pure lateral slip and different f_z values";
      Real dL[nload] 
        "distance of f_y0 attachment point from contact point with respect to length of contact area for pure lateral slip";
      Real L[nload] "length of contact aread for pure lateral slip";
      Real t_z[nload] "Tyre torque in z-direction for pure lateral slip";
      Real f[nload] "Tyre force in contact plane for phi=45 deg slip";
      Real f_x_mue[nmue] 
        "Tyre force in x-direction for longitudinal slip, f_z=load1.f_z and different mue-values";
    protected 
      SI.Angle camberAngle;
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.04, 
          y=0.01, 
          width=0.77, 
          height=0.94));
    equation 
      camberAngle = camberAngleDeg*pi/180;
      slip = time;
      for i in 1:nload loop
        // characteristic 1
        tyre1[i].slip = slip;
        tyre1[i].sinphi = 0;
        tyre1[i].cosphi = 1;
        tyre1[i].f_z = f_z[i];
        tyre1[i].mueRoad = mueRoad;
        tyre1[i].camberAngle = camberAngle;
        f_x[i] = tyre1[i].f_x0;
        df_ds[i] = tyre1[i].df_ds;
        
        // characteristic 2
        tyre2[i].slip = slip;
        tyre2[i].sinphi = 1;
        tyre2[i].cosphi = 0;
        tyre2[i].f_z = f_z[i];
        tyre2[i].mueRoad = mueRoad;
        tyre2[i].camberAngle = camberAngle;
        f_y[i] = tyre2[i].f_y0;
        dL[i] = tyre2[i].dL;
        
        L[i] = noEvent(sqrt(8*tyreData.R0*f_z[i]/tyreData.c_z));
        t_z[i] = -L[i]*dL[i]*f_y[i];
        
        // characteristic 3
        tyre3[i].slip = slip;
        tyre3[i].sinphi = sin(pi/4);
        tyre3[i].cosphi = cos(pi/4);
        tyre3[i].f_z = f_z[i];
        tyre3[i].mueRoad = mueRoad;
        tyre3[i].camberAngle = camberAngle;
        f[i] = tyre3[i].f0;
      end for;
      
      for i in 1:nmue loop
        // characteristic 4
        tyre4[i].slip = slip;
        tyre4[i].sinphi = 0;
        tyre4[i].cosphi = 1;
        tyre4[i].f_z = tyreData.load1.Fz_nom;
        tyre4[i].mueRoad = mue[i];
        tyre4[i].camberAngle = camberAngle;
        
        f_x_mue[i] = tyre4[i].f_x0;
      end for;
    end TyreProperties1;
    
    model TyreProperties2 "Compute characteristic values for tyre" 
      import SI = Modelica.SIunits;
      import Modelica.Math.*;
      import Modelica.Constants.*;
      TyreData tyreData;
      parameter Real f_z[:]=1000:1000:6000 "Load of wheel";
      parameter Real mue[:]={0.1,0.3,0.5,0.7} 
        "road friction coefficient variation";
      parameter Real camberAngleDeg=0 "Camber angle in [deg]";
      parameter Real mueRoad=tyreData.mue_nom;
      final parameter Integer nload=size(f_z, 1);
      final parameter Integer nmue=size(mue, 1);
      Real slip "at all slip=time values the tyre equations are evaluated";
      Real f_x[nload] 
        "Tyre force in x-direction for pure longitudinal slip and different f_z values";
      Real df_ds[nload] 
        "Derivative of steady state contact force in contact plane with respect to slip for pure longitudinal slip";
      Real f_y[nload] 
        "Tyre force in y-direction for pure lateral slip and different f_z values";
      Real dL[nload] 
        "distance of f_y0 attachment point from contact point with respect to length of contact area for pure lateral slip";
      Real L[nload] "length of contact aread for pure lateral slip";
      Real t_z[nload] "Tyre torque in z-direction for pure lateral slip";
      Real f[nload] "Tyre force in contact plane for phi=45 deg slip";
      Real f_x_mue[nmue] 
        "Tyre force in x-direction for longitudinal slip, f_z=load1.f_z and different mue-values";
    protected 
      SI.Angle camberAngle;
      Real o1;
      Real o2;
      Real o3;
      Real o4;
      Real o5;
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.09, 
          y=0.12, 
          width=0.6, 
          height=0.6));
    algorithm 
      camberAngle := camberAngleDeg*pi/180;
      slip := time;
      for i in 1:nload loop
        (o1,o2,o3,o4,o5) := tyreForces2(tyreData, slip, 0, 1, f_z[i], mueRoad, 
          camberAngle);
        f_x[i] := o2;
        df_ds[i] := o4;
        
        (o1,o2,o3,o4,o5) := tyreForces2(tyreData, slip, 1, 0, f_z[i], mueRoad, 
          camberAngle);
        f_y[i] := o3;
        dL[i] := o5;
        L[i] := noEvent(sqrt(8*tyreData.R0*f_z[i]/tyreData.c_z));
        t_z[i] := -L[i]*dL[i]*f_y[i];
        
        (o1,o2,o3,o4,o5) := tyreForces2(tyreData, slip, sin(pi/4), cos(pi/4), 
          f_z[i], mueRoad, camberAngle);
        f[i] := o1;
      end for;
      
      for i in 1:nmue loop
        (o1,o2,o3,o4,o5) := tyreForces2(tyreData, slip, 0, 1, tyreData.load1.
          Fz_nom, mue[i], camberAngle);
        f_x_mue[i] := o2;
      end for;
    end TyreProperties2;
    annotation (Coordsys(
        extent=[0, 0; 632, 426], 
        grid=[1, 1], 
        component=[20, 20]), Window(
        x=0.07, 
        y=0.13, 
        width=0.4, 
        height=0.4, 
        library=1, 
        autolayout=1));
    record TyreDataDallaraRearWheel "Tyre data" 
      extends Utilities.WheelData(
        R0=0.31, 
        width=0.3, 
        Rrim=0.05);
      
      // Spring and damping characteristic of tyre
      parameter Real c_x(unit="N/m") = 100000 
        "tyre spring constant in x-direction";
      parameter Real c_y(unit="N/m") = 150000 
        "tyre spring constant in y-direction";
      parameter Real c_z(unit="N/m") = 200000 
        "tyre spring constant in z-direction";
      parameter Real d_x(unit="N.s/m") = 1500 
        "tyre damping constant in x-direction";
      parameter Real d_y(unit="N.s/m") = 300 
        "tyre damping constant in y-direction";
      parameter Real d_z(unit="N.s/m") = 300 
        "tyre damping constant in z-direction";
      
      // Data for nominal vertical load 1
      parameter Real mue_nom=0.7 
        "friction coefficient of road for which tyre data is valid";
      
      NominalLoad load1(
        Fz_nom=2.5*3000, 
        Fds_x=2.5*50000, 
        Fds_y=2.5*40000, 
        F_max_x=2.5*3000, 
        F_slide_x=2.5*2800, 
        F_max_y=2.5*2750, 
        F_slide_y=2.5*2500) annotation (extent=[-85, 15; -15, 85]);
      NominalLoad load2(
        Fz_nom=6000, 
        s_max_x=0.18, 
        s_slide_x=0.5, 
        s_max_y=0.24, 
        s_slide_y=0.8, 
        dL0=0.1, 
        s0=0.275, 
        sE=0.8, 
        Fds_x=2.5*75000, 
        F_max_x=2.5*4500, 
        F_slide_x=2.5*4200, 
        Fds_y=2.5*60000, 
        F_max_y=2.5*4125, 
        F_slide_y=2.5*3750) annotation (extent=[15, 15; 85, 85]);
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
    end TyreDataDallaraRearWheel;
    
    record TyreDataDallaraFrontWheel "Tyre data" 
      extends Utilities.WheelData(
        R0=0.28, 
        width=0.28, 
        Rrim=0.05);
      
      // Spring and damping characteristic of tyre
      parameter Real c_x(unit="N/m") = 100000 
        "tyre spring constant in x-direction";
      parameter Real c_y(unit="N/m") = 150000 
        "tyre spring constant in y-direction";
      parameter Real c_z(unit="N/m") = 200000 
        "tyre spring constant in z-direction";
      parameter Real d_x(unit="N.s/m") = 1500 
        "tyre damping constant in x-direction";
      parameter Real d_y(unit="N.s/m") = 300 
        "tyre damping constant in y-direction";
      parameter Real d_z(unit="N.s/m") = 300 
        "tyre damping constant in z-direction";
      
      // Data for nominal vertical load 1
      parameter Real mue_nom=0.7 
        "friction coefficient of road for which tyre data is valid";
      
      NominalLoad load1(
        Fz_nom=1.5*3000, 
        Fds_x=1.5*50000, 
        Fds_y=1.5*40000, 
        F_max_x=1.5*3000, 
        F_slide_x=1.5*2800, 
        F_max_y=1.5*2750, 
        F_slide_y=1.5*2500) annotation (extent=[-85, 15; -15, 85]);
      NominalLoad load2(
        Fz_nom=6000, 
        s_max_x=0.18, 
        s_slide_x=0.5, 
        s_max_y=0.24, 
        s_slide_y=0.8, 
        dL0=0.1, 
        s0=0.275, 
        sE=0.8, 
        Fds_x=1.5*75000, 
        F_max_x=1.5*4500, 
        F_slide_x=1.5*4200, 
        Fds_y=1.5*60000, 
        F_max_y=1.5*4125, 
        F_slide_y=1.5*3750) annotation (extent=[15, 15; 85, 85]);
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
    end TyreDataDallaraFrontWheel;
    
    record TyreDataWithSteeringFlex "Tyre data" 
      extends TyreData(
        load1(
          Fds_y=0.9*40000, 
          s_max_y=0.21/0.9, 
          F_max_y=2750, 
          s_slide_y=0.6/0.9), 
        load2(
          Fz_nom=6000, 
          Fds_x=75000, 
          s_max_x=0.18, 
          F_max_x=4500, 
          s_slide_x=0.5, 
          F_slide_x=4200, 
          dL0=0.1, 
          s0=0.275, 
          sE=0.8, 
          Fds_y=60000*0.9, 
          s_max_y=0.21/0.9, 
          F_max_y=4125, 
          s_slide_y=0.6/0.9, 
          F_slide_y=3750), 
        c_y=100000*0.9);
      //parameter Real 0.9=0.9 
      //  "coefficient that scales the side stiffness of the tyre.";
    end TyreDataWithSteeringFlex;
  end RillTyre;
  
  package Utilities 
    extends Modelica.Icons.Library;
    
    record WheelData "Data needed for all wheel types" 
      import SI = Modelica.SIunits;
      parameter SI.Radius R0=0.3 "undeformed radius of wheel";
      parameter SI.Radius Rrim=0.2 "radius of rim";
      parameter SI.Length width=0.19 "width of wheel (>0)";
      parameter SI.Mass m=10 "mass of wheel";
      parameter SI.Inertia Iyy=1 "wheel inertia with respect to wheel axis";
      parameter SI.Inertia Ixx=1 
        "wheel inertia with respect to axis perpendicular to wheel axis";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.06, 
          y=0.03, 
          width=0.61, 
          height=0.67), 
        Icon(
          Rectangle(extent=[-50, 70; 0, -50], style(color=7, fillPattern=0)), 
          Line(points=[-100, -50; -100, 70; 100, 70; 100, -50; -100, -50; -100, 
                 -20; 100, -20; 100, 10; -100, 10; -100, 40; 100, 40; 100, 70; 
                0, 70; 0, -51], style(color=0, fillPattern=0)), 
          Text(
            extent=[-94, 65; -5, 43], 
            string="R0", 
            style(color=10, fillColor=10)), 
          Text(
            extent=[-96, 36; -4, 12], 
            string="m", 
            style(color=10, fillColor=10)), 
          Text(
            extent=[-94, 5; -5, -15], 
            string="Ixx", 
            style(color=10, fillPattern=0)), 
          Text(
            extent=[-96, -25; -5, -43], 
            string="...", 
            style(color=10)), 
          Text(
            extent=[-100, -60; 100, -100], 
            string="wheel data", 
            style(color=0)), 
          Text(extent=[-122, 140; 116, 80], string="%name")));
      
    end WheelData;
    
    partial model BaseWheel 
      "Properties common to all types of wheels and tyres" 
      import SI = Modelica.SIunits;
      import Car;
      // PARAMETERS COMMON FOR ALL WHEELS
      parameter Real n[3]={0,1,0} 
        "Unit vector in direction of spin axis (same direction for left and right wheel)";
      parameter Boolean leftWheel=true 
        "true, if left wheel, otherwise right wheel";
      parameter Boolean exact=false 
        "false if influence of bearing acceleration is neglected";
      
      final parameter Real nShape[3]=if leftWheel then n else -n;
      
      constant SI.Velocity v_eps=1.e-10;
      
      
        // Modelica.Constants.eps "1/v_eps should be representable on the machine";
      
      // GEOMETRIC VARIABLES  
      SI.Angle camberAngle "Angle between wheel plane and contact normal";
      Real cosCamberAngle "Cosine of camber angle";
      
      SI.Radius Rdym "Distance between wheel center point and contact point";
      
      SI.Length delta_r 
        "= R0 - Rdym, where R0 is the undeformed wheel radius and Rdym is the distance between wheel center point and contact point";
      Real Sroad[3, 3] 
        "Transformation matrix from road frame to inertial frame";
      Real S[3, 3] 
        "Transformation matrix from wheel carrier frame to road frame";
      Real S2[3, 3] " S2: Transformation matrix from road frame to frame_b";
      
      SI.Position rRoad0[3] 
        "Position vector from inertial to road frame, resolved in inertial frame";
      SI.Position Cr[3] 
        "Position vector from road frame to C-Frame, resolved in road frame";
      SI.Position Wr[3] 
        "Position vector from road frame to W-Frame, resolved in road frame";
      SI.Position rP0[3] 
        "Position vector from road frame to point P0, resolved in road frame";
      SI.Position rP1[3] 
        "Position vector from road frame to point P1, resolved in road frame";
      SI.Position rC_P1[3] 
        "Position vector from C-frame to point P1, resolved in road frame";
      SI.Position rC_W[3] 
        "Position vector from C-frame to W-frame, resolved in road frame";
      
      Real nn[3] 
        "Normalized vector of wheel spin axis, resoved in wheel carrier frame";
      Real Ce_x[3] 
        "Unit vector in x-direction of C-Frame, resolved in road frame";
      Real Ce_y[3] 
        "Unit vector in y-direction of C-Frame, resolved in road frame";
      Real Ce_z[3] 
        "Unit vector in z-direction of C-Frame, resolved in road frame";
      Real Ce_z_0[3](start={0,0,1}) "Initial guess for Ce_z";
      
      Real We_x[3] 
        "Unit vector in x-direction of W-Frame, resolved in road frame";
      Real We_y[3] 
        "Unit vector in y-direction of W-Frame, resolved in road frame";
      Real We_z[3] 
        "Unit vector in z-direction of W-Frame, resolved in road frame";
      
      SI.AngularVelocity w "Rotational speed of wheel around wheel axis";
      SI.Angle phi "Rotational position of wheel around wheel axis";
      SI.AngularVelocity wWheel[3] 
        "Absolute angular velocity of the rotating wheel, resolved in frame road";
      SI.Velocity Cv[3] 
        "Absolute velocity of wheel center point, resolved in frame road";
      SI.Velocity ddelta_r "Derivative of delta_r (= -der(Rdym))";
      SI.Velocity Wv_x 
        "Absolute velocity of contact point in longitudinal direction We_x";
      SI.Velocity Wv_y 
        "Absolute velocity of contact point in lateral direction We_y";
      SI.Velocity v_x 
        "Absolute velocity of wheel in longitudinal direction We_x";
      SI.Velocity v_y "Absolute velocity of wheel in lateral direction We_y";
      
    protected 
      Real aux1[3] "Auxiliary vector 1 (vector perpendicular to Ce_y and We_z)";
      Real aux2[3] "Auxiliary vector 2";
      
      // THESE FORCES AND TORQUES SHOULD BE GENERATED BY THE TOTAL TYRE MODEL
    public 
      SI.Force F_x "Tyre/contact force at origin of W-frame in x-direction";
      SI.Force F_y "Tyre/contact force at origin of W-frame in y-direction";
      SI.Force F_z "Tyre/contact force at origin of W-frame in z-direction";
      SI.Force M_x "Tyre/contact torque at origin of W-frame in x-direction";
      SI.Force M_y "Tyre/contact torque at origin of W-frame in y-direction";
      SI.Force M_z "Tyre/contact torque at origin of W-frame in z-direction";
      // ********************************************************************
      
      // RESULTANT FORCES ANS TORQUES
      SI.Force Wf[3] 
        "Tyre/contact force at origin of W-frame, resolved in road frame";
      SI.Torque Wt[3] "Tyre/contact torque at W-frame, resolved in road frame";
      SI.Torque Ct[3] 
        "Tyre torque acting at wheel center, resolved in wheel_body frame";
      SI.Torque Ct_tyre 
        "Tyre torque acting at wheel center in direction of spin axis";
      
      
        // *********************************************************************  
      
      Road road;
      Road road2;
      Real mueRoad;
      Real altitudeRoad;
      Real normalRoad[3];
      
    public 
      outer block Road = Environments.Interfaces.TyreRoadInterface;
      
    public 
      replaceable Wheels.Utilities.WheelData wheelData
        annotation (extent=[0, 37; 20, 57]);
      annotation (
        Documentation(info="This element is a superclass of wheel/tyre
systems. It contains all the equations which are independent of the
tyre model. Especially, the contact point of wheel and road, as well
as the contact velocities are computed.
The frames of this object have the following meaning:
Frame carrierFrame is fixed in the carrier of the wheel
and is located on the spin axis of the wheel in the
middle point of the wheel. This point is also approximately
used as the center-of-mass of the wheel.
It is temporarily assumed that the road lies in the
x-y plane of the inertial frame! This restriction can
be removed by defining the road properties via inner/outer
functions.
Flange XXX is used to drive the wheel with a 1D-drive train.
This element is realized according to the description:
  Georg Rill: Simulation von Kraftfahrzeugen, Vieweg, 1994
"), 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.04, 
          y=0.05, 
          width=0.52, 
          height=0.89), 
        Icon(
          Text(extent=[-113, -102; 111, -139], string="%name"), 
          Polygon(points=[-60, -40; -50, -40; -50, 90; 40, 90; 40, 100; -60, 
                100; -60, -40], style(color=0, fillColor=8)), 
          Rectangle(extent=[-100, -79; 100, -100], style(
              color=0, 
              fillColor=7, 
              fillPattern=8)), 
          Rectangle(extent=[40, 10; 60, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[-35, 76; 40, -80], style(gradient=2, fillColor=10))), 
           
        Diagram(
          Text(
            extent=[-5, 98; 25, 92], 
            string="Tyre-Force", 
            style(color=10)), 
          Text(extent=[-55, 23; -55, 18], string=
                "(forward moving direction in wheel plane)"), 
          Text(extent=[-68, 31; -68, 26], string="(perpendicular to road)"), 
          Line(points=[29, 41; 30.51, 55.2; 31.71, 65.2; 32.92, 73.7; 34.13, 
                80.8; 35.63, 87.8; 37.14, 93; 38.65, 96.6; 40.5, 99.3; 42.6, 
                100.8; 46.2, 100.8; 51.9, 98.6; 63.7, 92.2; 68.5, 91; 89, 91], 
              style(color=0, thickness=2)), 
          Line(points=[-100, -1; -100, 31], style(arrow=1)), 
          Line(points=[-100, 0; -124, 0], style(arrow=1)), 
          Line(points=[-100, 0; -80, 16], style(arrow=1)), 
          Text(
            extent=[-99, 41; -41, 31], 
            string="usual selection of reference system", 
            style(color=41)), 
          Text(extent=[-162, -12; -88, -18], string=
                "(spin axis, towards left vehicle side)"), 
          Text(extent=[-79, 21; -76, 13], string="x"), 
          Text(extent=[-123, 0; -126, -9], string="y"), 
          Text(extent=[-97, 32; -94, 24], string="z"), 
          Line(points=[29, 41; 89, 41], style(color=10, arrow=1)), 
          Line(points=[29, 41; 29, 100], style(color=10, arrow=1)), 
          Text(
            extent=[77, 51; 91, 44], 
            string="slip", 
            style(color=10)), 
          Line(points=[45, 101; 45, 41], style(color=10, pattern=2)), 
          Line(points=[69, 91; 69, 41], style(color=10, pattern=2)), 
          Text(
            extent=[59, 39; 77, 34], 
            string="s_Fslide", 
            style(color=10)), 
          Text(
            extent=[35, 39; 54, 34], 
            string="s_Fmax", 
            style(color=10)), 
          Rectangle(extent=[-4, 105; 100, 33], style(color=10))));
      
      ModelicaAdditions.MultiBody.Forces.ExtForce tyreForceWheelCenter
        annotation (extent=[-37, -56; -57, -36], rotation=180);
      ModelicaAdditions.MultiBody.Forces.ExtTorque tyreTorqueWheelCenter
        annotation (extent=[-37, -56; -57, -76], rotation=180);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a carrierFrame
        annotation (extent=[-80, -10; -60, 10]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a driveShaft1D
        annotation (extent=[80, 10; 60, -10], rotation=180);
      Modelica.Mechanics.Rotational.Torque drivingTyreTorque
        annotation (extent=[-34, -5; -14, 15]);
      ModelicaAdditions.MultiBody.Joints.Revolute hub(n={0,1,0})
        annotation (extent=[-21, -28; -1, -8]);
      ModelicaAdditions.MultiBody.Parts.Body2 wheel_body(
        m=wheelData.m, 
        I11=wheelData.Ixx, 
        I22=wheelData.Iyy, 
        I33=wheelData.Ixx) annotation (extent=[11, -27; 31, -7]);
    equation 
      road.x = Wr[1];
      road.y = Wr[2];
      //road.v = 0;
      //temporarilly
      road.mue = mueRoad;
      road.z = altitudeRoad;
      road.n = normalRoad;
      
      connect(wheel_body.frame_a, hub.frame_b)
        annotation (points=[10.5, -17; 5, -29; 5, -18; -0.5, -18]);
      connect(hub.frame_a, carrierFrame)
        annotation (points=[-21.5, -18; -42, -30; -42, 0; -70, 0]);
      connect(drivingTyreTorque.flange_b, hub.axis)
        annotation (points=[-14, 5; -11, 5; -11, -11], style(color=0));
      connect(driveShaft1D, hub.axis)
        annotation (points=[70, 0; -11, 0; -11, -11], style(color=0));
      
      /* Definition of road:
      1. A road frame is defined which is fixed in the inertial frame and
         which is defined relatively to the inertial frame. This frame is
         defined as:
           Sroad : Transformation matrix from road frame to inertial frame.
           rroad0: Position vector from inertial frame origin to the
                   origin of the road frame, resolved in the inertial frame.
      2. The z-axis of the road frame points upwards. The road surface
         is defined as (Wr_z, We_z) = f(Wr_x, Wr_y), where
            Wr_x, rWy, rWz are the x-, y-, z-coordinates of the wheel/road
            contact point with respect to the road frame and
            We_z is the unit vector at the contact point which is
            perpendicular to the surface and points upwards.
      Temporarily it is assumed that the road is a plane at z=0 of the inertial
      frame. Still, the equations below are written for a general road profile.
      This assumption should be replaced later.
      The friction coefficient mue of the road is temporarily set to the
      nominal friction coefficient data.mue_max of the tyre.
      The determination of the contact point of wheel and road requires
      the solution of a nonlinear system of equations. However, since the
      contact area of the tyre of a car is a rectangle with about 10 cm x 20 cm
      area and the road is approximated in the order of centimeters, it does not
      make much sense to determine the "virtual" contact point very exactly.
      Below one iteration of a fixed point iteration scheme is used to solve
      this nonlinear system of equations. This works well, provided a good
      initial guess Ce_z_0 for the unit vector Ce_z (normalized vector from
      wheel center point to contact point), is available. A good guess is
      - a unit vector which is fixed in the main car body and points
        into the z-direction of the road frame when the car is at rest.
      - a unit vector which is fixed in the wheel carrier and points
        into the z-direction of the road frame when the car is at rest.
      - the value of Ce_z at the previous integration step.
      Temporarily, this initial guess is set for the case that
      the road is a plane at z=0.
      Note, that the exact nonlinear system of equations is constructed
      by replacing the equation for Ce_z_0 below by the equation:
          Ce_z = Ce_z_0;
      and by using an appropriate initial guess for Ce_z_0.
   */
      Sroad = identity(3);
      rRoad0 = zeros(3);
      //Ce_z_0 = Ce_z;
      Ce_z_0 = {0,0,1};
      //mueRoad = 0.7;
      //(mueRoad,altitude) = Road(Wr[1], Wr[2], sqrt(Wv_x*Wv_x + Wv_y*Wv_y));
      /* Definition of the ISO coordinate systems of the wheel:
      Frame C (ISO-C, TYDEX C axis system) is fixed to the wheel carrier.
        - The origin is described by vector Cr, from the origin of the
          road frame to the wheel center, resolved in the road frame.
        - The x-axis (= unit vector Ce_x) is parallel to the road tangent plane
          within the wheel plane
        - The y-axis (= unit vector Ce_y) is normal to the wheel plane and
          therefore parallel to the wheels spin axis.
        - The z-axis (= unit vector Ce_z) is orthogonal to the x- and y-axis, i.e.,
          it is a unit vector along the line from the contact point to
          the wheel center point.
      Frame W (ISO-W, TYDEX W axis system) is fixed at the contact point
      of the wheel and the road.
        - The origin is described by vector Wr, from the origin of the
          road frame to the tyre contact point, resolved in the road frame.
        - The x-axis (= unit vector We_x) lies in the local road tangent plane
          along the intersection of the wheel plane and the local road plane,
          i.e., it is parallel to the x-axis of frame C.
        - The z-axis (= unit vector We_z) is perpendicular to the local
          road tangent plane and points upward.
        - The y-axis (= unit vector We_y) lies in the local road plane and
          is perpendicular to the x- and z-axis.
      All vectors describing frame C and frame W are resolved in the
      road frame. Note, that We_x = Ce_x (but We_y <> Ce_y)
   */
      
      /* Determine C-frame, where all vectors are resolved in the road frame.
       n: spin-axis in wheel carrier frame;
       S: transformation matrix from wheel carrier frame to road frame).
    */
      S = transpose(Sroad)*carrierFrame.S;
      nn = n/sqrt(n*n);
      Ce_x = We_x;
      Ce_y = S*n;
      Ce_z = cross(Ce_x, Ce_y);
      Cr = transpose(Sroad)*(carrierFrame.r0 - rRoad0);
      
      /* Determine W-frame, where all vectors are resolved in the road frame.
       As shortly discussed above, this is performed by one iteration of a
       fixed point iteration scheme, constructing three points P0, P1, P2:
       Point P0: Defined by position vector Wr_0 which is computed using
                 the initial guess We_z_0 for We_z.
       Point P1: P0 is usually not on the road. The x-,y-coordinates of P0
                 are used for P1. The z-coordinate is computed by using
                 the road surface function. In point P1 the contact frame W
                 is constructed. This defines especially the contact plane.
       Point P2: P1 is moved in the constructed contact plane, such that
                 it is the intersection of the contact plane with the line
                 along Ce_z, i.e., along the line from the wheel-center point
                 to the contact point (which is computed above as cross
                 product of the wheel spin axis Ce_y and of We_x computed
                 for point P1).
       rP0     : Position vector from road frame origin to point P0.
       rP1     : Position vector from road frame origin to point P1.
                 Temporarily, rP1[3] is set to 0 and We_z is set to {0,0,1}.
                 Later the road surface function should be used to calculate
                 these values.
       Rdym    : Distance between wheel center point and contact point.
       rC_W    : Position vector from C-frame origin to W-frame origin.
                 During the contact point calculation (iteration), this is
                 the vector from C-frame origin to point P2.
                   rC_W = -Rdym*Ce_z
                        = rC_P1 + a*We_x + b*We_y;
                      multiply rC_W with We_z
                   We_z*rC_W = -Rdym*We_z*Ce_z
                             = We_z*rC_P1  (We_z is perpendicular to We_x and We_y)
                      solve for Rdym
                   Rdym = -(We_z*rC_P1)/(We_z*Ce_z)
    */
      rP0 = Cr - wheelData.R0*Ce_z_0;
      
      //road has to be called twice
      road2.x = rP0[1];
      road2.y = rP0[2];
      //road2.v = 0;
      rP1 = {rP0[1],rP0[2],road2.z};
      // general road: rP1[3] = f(rP1[1], rP1[2]);
      We_z = road2.n;
      //{0,0,1};
      // general road: We_z = f(rP1[1], rP1[2]);
      aux1 = cross(Ce_y, We_z);
      We_x = aux1/noEvent(sqrt(aux1*aux1));
      We_y = cross(We_z, We_x);
      
      rC_P1 = rP1 - Cr;
      cosCamberAngle = We_z*Ce_z;
      Rdym = -(We_z*rC_P1)/cosCamberAngle;
      rC_W = -Rdym*Ce_z;
      Wr = Cr + rC_W;
      
      /* Compute characteristic values of the contact:
       delta_r    : Distance between undeformed wheel point and contact point
                    R0 = Rdym + delta_r
       delta_z    : Radial tyre deflection
                    (= wheel deformation in z-direction of C-frame)
                    delta_z = delta_r if the wheel is in contact with the road
       camberAngle: Angle between wheel plane and We_z
                    (cos(pi/2 - camberAngle) = Ce_y*We_z)
    */
      delta_r = wheelData.R0 - Rdym;
      camberAngle = Modelica.Math.asin(Ce_y*We_z);
      
      /* Determine contact point velocities
       wWheel  : Absolute angular velocity of the rotating wheel, resolved in frame road
       ddelta_r: derivative of delta_r (= -der(Rdym))
       Cv      : Absolute velocity of wheel center point, resolved in frame road
       Wv      : Absolute velocity of contact point, resolved in frame road
                   Wv = der(Wr)
                      = der(Cr) + cross(wWheel, rC_W) + der(abs(rCW))*(-Ce_z)
                      = Cv + cross(wWheel, rC_W) + der(R0 - delta_r)*(-Ce_z)
                      = Cv + cross(wWheel, rC_W) + der(delta_r)*Ce_z
                   Wv has to be in the road plane. Therefore We_z*Wv = 0
                     0 = We_z*Wv = We_z*(Cv + cross(wWheel, rC_W)) + We_z*Ce_z*ddelta_r
                     ddelta_r = -We_z*(Cv + cross(wWheel, rC_W)) / (We_z*Ce_z)
       Wv_x  : Absolute velocity of contact point in longitudinal direction
               = We_x*Wv
               = We_x*(Cv + cross(wWheel, rC_W) + ddelta_r*Ce_z)
               = We_x*(Cv + cross(wWheel, rC_W))
               since We_x*Ce_z = Ce_x*Ce_z = 0
       Wv_y  : Absolute velocity of contact point in lateral direction
               = We_y*Wv
               = We_y*(Cv + cross(wWheel, rC_W) + ddelta_r*Ce_z)
       Wv_z  : Absolute velocity of contact point in normal direction
               = 0
    */
      wWheel = S*(carrierFrame.w + n*w);
      Cv = S*carrierFrame.v;
      aux2 = Cv + cross(wWheel, rC_W);
      ddelta_r = -We_z*aux2/cosCamberAngle;
      Wv_x = We_x*aux2;
      Wv_y = We_y*(aux2 + ddelta_r*We_z);
      
      //DETERMINE THE WHEEL CONTACT POINT VELOCITY
      v_x = Cv*We_x;
      v_y = Cv*We_y;
      
      //STATES OF WHEEL ROTATION
      w = hub.qd;
      phi = hub.q;
      
      //EXPRESSIONS FOR F_x, F_y, F_z, M_x, M_y, M_z AT EACH WHEEL MODEL
      Wf = We_x*F_x + We_y*F_y + We_z*F_z;
      Wt = We_x*M_x + We_y*M_y + We_z*M_z;
      
      // tyre forces shall act at wheel center point
      // S2: Transformation matrix from road frame to frame_b
      S2 = transpose(hub.frame_a.S)*Sroad;
      Ct = S2*(cross(rC_W, Wf) + Wt);
      Ct_tyre = nn*Ct;
      tyreForceWheelCenter.inPort.signal = S2*Wf;
      tyreTorqueWheelCenter.inPort.signal = Ct - nn*Ct_tyre;
      drivingTyreTorque.inPort.signal = {Ct_tyre};
      
      connect(tyreForceWheelCenter.frame_b, carrierFrame)
        annotation (points=[-36.5, -46; -70, 0]);
      connect(tyreTorqueWheelCenter.frame_b, carrierFrame)
        annotation (points=[-36.5, -66; -27, -38; -70, 0]);
    end BaseWheel;
    
    function interpolate1 "Interpolate linearly between two points" 
      
      input Real x "abszissa value to be interpolated";
      input Real x1 "abszissa value of point 1";
      input Real y1 "ordinate value of point 1";
      input Real x2 "abszissa value of point 2";
      input Real y2 "ordinate value of point 2";
      output Real y "ordinate value of x, i.e., y(x)";
    algorithm 
      y := y1 + (y2 - y1)*((x - x1)/(x2 - x1));
    end interpolate1;
    
    function interpolate2 
      "Interpolate quadratically between two points, such that y(0)=0" 
      
      input Real x "abszissa value to be interpolated";
      input Real x1 "abszissa value of point 1 (<> 0 required)";
      input Real y1 "ordinate value of point 1";
      input Real x2 "abszissa value of point 2 (x2 <> x1 required)";
      input Real y2 "ordinate value of point 2";
      output Real y "ordinate value of x, i.e., y(x)";
    algorithm 
      y := ((y2*x1/x2 - y1*x2/x1)*(x1 - x)/(x1 - x2) + y1*x/x1)*x/x1;
    end interpolate2;
    
    function interpolate2b 
      "Interpolate quadratically between two points, such that y(0)=0" 
      
      input Real x "abszissa value to be interpolated";
      input Real x1 "abszissa value of point 1 (<> 0 required)";
      input Real y1 "ordinate value of point 1";
      input Real x2 "abszissa value of point 2 (x2 <> x1 required)";
      input Real y2 "ordinate value of point 2";
      output Real y "ordinate value of x, i.e., y(x)";
    algorithm 
      y := (((y2 - y1*(x2/x1)*(x2/x1))/((x2/x1)*(1 - (x2/x1)))) + (x/x1)*(y1 - 
        ((y2 - y1*(x2/x1)*(x2/x1))/((x2/x1)*(1 - (x2/x1))))))*(x/x1);
    end interpolate2b;
    
    function interpolate2c 
      "Interpolate quadratically between two points, such that y(0)=0" 
      
      input Real x "abszissa value to be interpolated";
      input Real x1 "abszissa value of point 1 (<> 0 required)";
      input Real y1 "ordinate value of point 1";
      input Real x2 "abszissa value of point 2 (x2 <> x1 required)";
      input Real y2 "ordinate value of point 2";
      output Real y "ordinate value of x, i.e., y(x)";
    protected 
      Real k;
      Real a;
      Real b;
      Real xx;
    algorithm 
      k := x2/x1;
      a := (y2 - y1*k*k)/(k*(1 - k));
      b := y1 - a;
      xx := x/x1;
      y := (a + xx*b)*xx;
    end interpolate2c;
    annotation (Coordsys(
        extent=[0, 0; 632, 426], 
        grid=[1, 1], 
        component=[20, 20]), Window(
        x=0.07, 
        y=0.13, 
        width=0.4, 
        height=0.4, 
        library=1, 
        autolayout=1));
    
  end Utilities;
  annotation (Coordsys(
      extent=[0, 0; 504, 364], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.03, 
      y=0.02, 
      width=0.4, 
      height=0.4, 
      library=1, 
      autolayout=1));
  package MFTyre 
    extends Modelica.Icons.Library;
    annotation (Coordsys(
        extent=[0, 0; 442, 386], 
        grid=[2, 2], 
        component=[20, 20]), Window(
        x=0.45, 
        y=0.01, 
        width=0.28, 
        height=0.42, 
        library=1, 
        autolayout=1));
    block TyreForces 
      "Compute tyre forces and torques for steady state conditions" 
      
      input SI.Velocity v_x "Contact point longitudinal speed";
      input SI.Velocity v_y "Contact point lateral speed";
      input Boolean contact "true if wheel has contact to ground";
      input SI.AngularVelocity w "Angular welocity of rim";
      input SI.Angle gamma "Camber angle";
      input SI.Force Fz "normal force";
      
      /*
  input SI.Angle alpha1 "slip angle";
  input Real kappa1 "kappa";
  input SI.Angle gamma1 "Camber angle";
  input SI.Force Fz "normal force";
*/
      output SI.Force Fx "Tyre force in x-direction";
      output SI.Force Fy "Tyre force in y-direction";
      output SI.Torque Mz "Tyre torque in z-direction";
      output SI.Position u "Longitudinal tyre deflection";
      output SI.Position v "Lateral tyre deflection";
      
      //  output SI.Force My 
      //    "Rolling resistance force for steady state conditions";
      
      
        /**************************************************************************/
      //USER PARAMETERS
      parameter Boolean transient=true 
        "true if transient behaviour should be considered.";
      parameter Boolean combined=true 
        "true if combined slip should be considered.";
      parameter Boolean leftWheel=true "TRUE if tyre is mounted left.";
      
      parameter SI.Position radius0=0.3;
      
      constant Real pi=Modelica.Constants.PI;
      parameter Real F_z0=3000 "nominal load";
      parameter SI.Velocity v_eps=0.1;
      
      Real F_z=Fz;
      
      //SLIP DEFINITION
      SI.Velocity v_sx "Longitudinal slip speed";
      SI.Velocity v_xNonZero "Nonzero longitudinal speed";
      SI.Angle alpha1 
        "Angle between wheel axis velocity and the x-direction of the wheel";
      Real kappa1 "slip at contact point computed with contact velocities";
      SI.Angle ssAlpha 
        "Angle between wheel axis velocity and the x-direction of the wheel";
      Real ssKappa "slip at contact point computed with contact velocities";
      SI.Angle ttAlpha 
        "Angle between wheel axis velocity and the x-direction of the wheel";
      Real ttKappa "slip at contact point computed with contact velocities";
      
      //ROLLING RADIUS
      SI.Position rho "radial tyre deflection";
      SI.Position rho_0 "nominal tyre deflection";
      Real B_roll;
      Real D_roll;
      Real F_roll;
      Real R_e;
      
      
        /******************PURE SLIP***********************************************/
      //longitudinal
    protected 
      Real p_Cx1 "shape factor C";
      Real p_Dx1 "peak factor D";
      Real p_Dx2 "peak factor D";
      Real p_Ex1 "curvature factor E";
      Real p_Ex2 "curvature factor E";
      Real p_Ex3 "curvature factor E";
      Real p_Ex4 "curvature factor E";
      Real p_Kx1 "slip stiffness BCD";
      Real p_Kx2 "slip stiffness BCD";
      Real p_Kx3 "slip stiffness BCD";
      Real p_Hx1 "horisontal shift H";
      Real p_Hx2 "horisontal shift H";
      Real p_Vx1 "vertical shift V";
      Real p_Vx2 "vertical shift V";
      //lateral
      Real p_Cy1 "shape factor C";
      Real p_Dy1 "peak factor D";
      Real p_Dy2 "peak factor D";
      Real p_Dy3 "peak factor D";
      Real p_Ey1 "curvature factor E";
      Real p_Ey2 "curvature factor E";
      Real p_Ey3 "curvature factor E";
      Real p_Ey4 "curvature factor E";
      Real p_Ky1 "slip stiffness BCD";
      Real p_Ky2 "slip stiffness BCD";
      Real p_Ky3 "slip stiffness BCD";
      Real p_Hy1 "horisontal shift H";
      Real p_Hy2 "horisontal shift H";
      Real p_Hy3 "horisontal shift H";
      Real p_Vy1 "vertical shift V";
      Real p_Vy2 "vertical shift V";
      Real p_Vy3 "vertical shift V";
      Real p_Vy4 "vertical shift V";
      //aligning torque
      Real q_Bz1 "stiffness factor B";
      Real q_Bz2 "stiffness factor B";
      Real q_Bz3 "stiffness factor B";
      Real q_Bz4 "stiffness factor B";
      Real q_Bz5 "stiffness factor B";
      Real q_Bz9 "stiffness factor B";
      Real q_Cz1 "shape factor C";
      Real q_Dz1 "peak factor D";
      Real q_Dz2 "peak factor D";
      Real q_Dz3 "peak factor D";
      Real q_Dz4 "peak factor D";
      Real q_Dz6 "peak factor D";
      Real q_Dz7 "peak factor D";
      Real q_Dz8 "peak factor D";
      Real q_Dz9 "peak factor D";
      Real q_Ez1 "curvature factor E";
      Real q_Ez2 "curvature factor E";
      Real q_Ez3 "curvature factor E";
      Real q_Ez4 "curvature factor E";
      Real q_Ez5 "curvature factor E";
      Real q_Hz1 "horisontal shift H";
      Real q_Hz2 "horisontal shift H";
      Real q_Hz3 "horisontal shift H";
      Real q_Hz4 "horisontal shift H";
      // User scaling Factors
      Real lambda_F_z0 "nominal load";
      Real lambda_muex "peak friction coefficient";
      Real lambda_muey "peak friction coefficient";
      Real lambda_Kx "slip stiffness";
      Real lambda_Ky "slip stiffness";
      Real lambda_Cx "shape factor";
      Real lambda_Cy "shape factor";
      Real lambda_Ex "curvature factor";
      Real lambda_Ey "curvature factor";
      Real lambda_Hx "horisontal shift";
      Real lambda_Hy "horisontal shift";
      Real lambda_Vx "vertical shift";
      Real lambda_Vy "vertical shift";
      Real lambda_gammay "camber force stiffness";
      Real lambda_gammaz "camber torque stiffness";
      Real lambda_t "pneumatic trail";
      Real lambda_r;
      Real lambda_Mr "residual torque";
      
      Real alpha_y;
      Real alpha_t;
      Real alpha_r;
      
      Real kappa_x;
      
      Real S_Vx;
      Real S_Hx;
      Real S_Hy;
      Real S_Vy;
      Real S_Ht;
      Real S_Hf;
      
      Real M_y;
      Real M_z;
      Real F_x;
      Real F_y;
      Real M_zr0;
      Real M_zr;
      Real F_y0;
      Real M_z0;
      Real F_x0;
      
      Real t;
      Real t_0;
      
      Real df_z;
      Real K_x;
      Real K_y;
      Real K_z;
      
      Real B_r;
      Real D_r;
      
      Real B_t;
      Real C_t;
      Real D_t;
      Real E_t;
      
      Real B_x;
      Real C_x;
      Real D_x;
      Real E_x;
      
      Real B_y;
      Real C_y;
      Real D_y;
      Real E_y;
      
      Real mue_x;
      Real mue_y;
      
      Real gamma_y;
      Real gamma_z;
      
      
        /**************** COMBINED SLIP ******************************************/
      
      //longitudinal
      SI.Position r_Bx1 "stiffness factor B";
      SI.Position r_Bx2 "stiffness factor B";
      SI.Position r_Cx1 "shape factor C";
      SI.Position r_Hx1 "horisontal shift H";
      
      //lateral slip
      SI.Position r_By1 "stiffness factor B";
      SI.Position r_By2 "stiffness factor B";
      SI.Position r_By3 "stiffness factor B";
      SI.Position r_Cy1 "shape factor C";
      SI.Position r_Hy1 "horisontal shift H";
      SI.Position r_Vy1 "vertical shift V";
      SI.Position r_Vy2 "vertical shift V";
      SI.Position r_Vy3 "vertical shift V";
      SI.Position r_Vy4 "vertical shift V";
      SI.Position r_Vy5 "vertical shift V";
      SI.Position r_Vy6 "vertical shift V";
      
      //aligning torque
      Real s_sz1;
      Real s_sz2;
      Real s_sz3;
      Real s_sz4;
      
      //user scaling factors
      Real lambda_xalpha;
      Real lambda_ykappa;
      Real lambda_Vykappa;
      Real lambda_s;
      
      Real B_xalpha;
      Real C_xalpha;
      Real S_Hxalpha;
      Real G_xalpha;
      
      Real B_ykappa;
      Real C_ykappa;
      Real S_Hykappa;
      Real S_Vykappa;
      Real D_Vykappa;
      Real G_ykappa;
      
      Real alpha_req;
      Real alpha_teq;
      Real s;
      
      /********TRANSIENT BEHAVIOUR********/
      
      Real p_Tx1;
      Real p_Tx2;
      Real p_Tx3;
      Real p_Ty1;
      Real p_Ty2;
      Real lambda_sigmaalpha;
      Real lambda_sigmakappa;
      
      Real sigma_kappa;
      Real sigma_alpha;
      
      Real zeta_x;
      Real zeta_y;
      
      SI.Velocity v_sy;
      SI.Velocity v_r;
      Real sign_v_r;
      Real abs_v_r;
      Real denNonZero;
      
    protected 
      parameter Real left=if leftWheel then 1 else -1;
      Real gamma1=left*gamma;
      Real F_zNonZero;
    public 
      parameter Wheels.RillTyre.TyreData data "Data of tyre"
        annotation (extent=[-100, 40; -60, 80]);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.28, 
          y=0.03, 
          width=0.66, 
          height=0.9), 
        Icon(
          Rectangle(extent=[-100, 100; 100, -100]), 
          Line(points=[-74, 78; -72, 68; -68, 58; -68, 54; -70, 52; -74, 52; -76, 
                 54; -76, 58; -72, 62; -62, 80]), 
          Text(extent=[-72, 16; -52, -4], string="z"), 
          Text(extent=[-92, 36; -52, -4], string="f"), 
          Line(points=[-56, -24; -60, -32; -66, -38; -72, -40; -78, -38; -80, -32; 
                 -78, -26; -70, -22; -64, -24; -58, -40]), 
          Text(extent=[-90, -54; -50, -94], string="slip"), 
          Polygon(points=[14, -26; 32, 0; 14, 26; 14, 6; -34, 6; -34, -6; 14, -6; 
                 14, -26], style(color=0, fillColor=0)), 
          Text(extent=[60, 60; 80, 40], string="x"), 
          Text(extent=[60, 0; 80, -20], string="y"), 
          Text(extent=[64, -60; 84, -80], string="z"), 
          Text(extent=[44, -40; 68, -80], string="M"), 
          Text(extent=[44, 20; 68, -20], string="F"), 
          Text(extent=[46, 80; 70, 40], string="F")));
      Wheels.MFTyre.MFpureSlip pureSlip annotation (extent=[-46, 40; -6, 80]);
      Wheels.MFTyre.MFsine longSlipPure annotation (extent=[40, -80; 80, -40]);
      Wheels.MFTyre.MFsine latSlipPure annotation (extent=[40, -40; 80, 0]);
      Wheels.MFTyre.MFcosine pneuTrail annotation (extent=[-20, -40; 20, 0]);
      Wheels.MFTyre.MFcosine resTorque annotation (extent=[-80, -40; -40, 0]);
      Wheels.MFTyre.MFcosine pneuTrail2 annotation (extent=[-20, -80; 20, -40]);
      Wheels.MFTyre.MFcosine resTorque2 annotation (extent=[-80, -80; -40, -40]);
      Wheels.MFTyre.MFcombinedSlip combinedSlip
        annotation (extent=[8, 40; 48, 80]);
      MFtransient trans annotation (extent=[60, 40; 100, 80]);
    equation 
      /************EFFECTIVE ROLLING RADIUS, section 3.4**************/
      rho = Fz/data.c_z;
      rho_0 = F_z0/data.c_z;
      //R_e = data.R0;
      R_e = data.R0 - rho_0*(D_r*atan(B_r*rho/rho_0) + F_roll*rho/rho_0);
      B_roll = 8.4;
      D_roll = 0.27;
      F_roll = 0.045;
      F_zNonZero = if (F_z > 0 and contact) then F_z else v_eps;
      //if (contact) then F_z else v_eps;
      //Model cannot handle zero F_z
      
      /*******************SLIP DEFINITION*****************************/
      //-alpha1 = if (transient) then ttAlpha*left else ssAlpha*left;
      
      
        //kappa1 = if (ttKappa > 1) then 1 else if (ttKappa > -1) then ttKappa else -1;
      //kappa1 = if (transient) then ttKappa else ssKappa;
      kappa1 = -v_sx/(v_r*sign_v_r + 0.1);
      alpha1 = left*v_sy/(v_r*sign_v_r + 0.1);
      /*******************SLIP, TRANISENT, section 4.5****************/
      sigma_kappa*der(u) + abs_v_r*u = -sigma_kappa*v_sx;
      sigma_alpha*der(v) + abs_v_r*v = -sigma_alpha*v_sy;
      
      abs_v_r = v_r*sign_v_r + 0.1;
      sign_v_r = sign(v_r);
      
      ttAlpha = atan2(zeta_y, noEvent(abs(denNonZero)));
      ttKappa = zeta_x;
      // /(denNonZero);
      denNonZero = if ((abs(sign_v_r - zeta_x) > v_eps)) then sign_v_r - zeta_x
         else v_eps;
      
      zeta_x = if (noEvent(abs(sigma_kappa) > v_eps)) then u/sigma_kappa else u
        /v_eps;
      zeta_y = if (noEvent(abs(sigma_alpha) > v_eps)) then v/sigma_alpha else v
        /v_eps;
      
      sigma_alpha = p_Ty1*sin(2*atan(F_zNonZero/(p_Ty2*F_z0*lambda_F_z0)))*(1
         - p_Ky3*noEvent(abs(gamma1)))*data.R0*lambda_F_z0*lambda_sigmaalpha;
      sigma_kappa = F_zNonZero*(p_Tx1 + p_Tx2*df_z)*exp(-p_Tx3*df_z)*(data.R0/
        F_z0)*lambda_sigmakappa;
      
      trans.p_Tx1 = p_Tx1;
      trans.p_Tx2 = p_Tx2;
      trans.p_Tx3 = p_Tx3;
      trans.p_Ty1 = p_Ty1;
      trans.p_Ty2 = p_Ty2;
      trans.lambda_sigmaalpha = lambda_sigmaalpha;
      trans.lambda_sigmakappa = lambda_sigmakappa;
      
      //assumptions
      v_sy = v_y;
      
      v_r = R_e*w;
      //tangential speed
      
      when change(contact) then
        //Tyre looses contact to ground or gets contact to ground
        reinit(u, 0);
        reinit(v, 0);
      end when;
      
      /*******************SLIP, STEADY STATE, section 3.5*************/
      if (noEvent(abs(v_x) < 1)) then
        ssAlpha = Modelica.Math.atan2(v_y, noEvent(abs(v_xNonZero)));
        ssKappa = if (noEvent(abs(v_r) < v_eps)) then 0 else -v_sx/v_xNonZero;
        v_xNonZero = v_eps;
      else
        ssAlpha = Modelica.Math.atan2(v_y, noEvent(abs(v_x)));
        ssKappa = -v_sx/v_xNonZero;
        v_xNonZero = v_x;
      end if;
      /*
  if (noEvent(abs(v_x) > abs(v_r))) then
    ssAlpha = Modelica.Math.atan2(v_y, noEvent(abs(v_x)));
    ssKappa = -v_sx/v_x;
  elseif (noEvent(abs(v_r) > v_eps)) then
    ssAlpha = Modelica.Math.atan2(v_y, noEvent(abs(v_r)));
    ssKappa = -v_sx/v_r;
  else  
    ssAlpha = 0;
    ssKappa = -v_sx/v_eps;
  end if;
*/
      v_sx = v_x - v_r;
      //  v_xNonZero=0.1;//dummy
      
      /*******************READ TYRE DATA FOR PURE SLIP****************/
      //longitudinal
      pureSlip.p_Cx1 = p_Cx1;
      pureSlip.p_Dx1 = p_Dx1;
      pureSlip.p_Dx2 = p_Dx2;
      pureSlip.p_Ex1 = p_Ex1;
      pureSlip.p_Ex2 = p_Ex2;
      pureSlip.p_Ex3 = p_Ex3;
      pureSlip.p_Ex4 = p_Ex4;
      pureSlip.p_Kx1 = p_Kx1;
      pureSlip.p_Kx2 = p_Kx2;
      pureSlip.p_Kx3 = p_Kx3;
      pureSlip.p_Vx1 = p_Vx1;
      pureSlip.p_Vx2 = p_Vx2;
      pureSlip.p_Hx1 = p_Hx1;
      pureSlip.p_Hx2 = p_Hx2;
      //lateral
      pureSlip.p_Cy1 = p_Cy1;
      pureSlip.p_Dy1 = p_Dy1;
      pureSlip.p_Dy2 = p_Dy2;
      pureSlip.p_Dy3 = p_Dy3;
      pureSlip.p_Ey1 = p_Ey1;
      pureSlip.p_Ey2 = p_Ey2;
      pureSlip.p_Ey3 = p_Ey3;
      pureSlip.p_Ey4 = p_Ey4;
      pureSlip.p_Ky1 = p_Ky1;
      pureSlip.p_Ky2 = p_Ky2;
      pureSlip.p_Ky3 = p_Ky3;
      pureSlip.p_Hy1 = p_Hy1;
      pureSlip.p_Hy2 = p_Hy2;
      pureSlip.p_Hy3 = p_Hy3;
      pureSlip.p_Vy1 = p_Vy1;
      pureSlip.p_Vy2 = p_Vy2;
      pureSlip.p_Vy3 = p_Vy3;
      pureSlip.p_Vy4 = p_Vy4;
      
      //aligning torque
      pureSlip.q_Bz1 = q_Bz1;
      pureSlip.q_Bz2 = q_Bz2;
      pureSlip.q_Bz3 = q_Bz3;
      pureSlip.q_Bz4 = q_Bz4;
      pureSlip.q_Bz5 = q_Bz5;
      pureSlip.q_Bz9 = q_Bz9;
      pureSlip.q_Cz1 = q_Cz1;
      pureSlip.q_Dz1 = q_Dz1;
      pureSlip.q_Dz2 = q_Dz2;
      pureSlip.q_Dz3 = q_Dz3;
      pureSlip.q_Dz4 = q_Dz4;
      pureSlip.q_Dz6 = q_Dz6;
      pureSlip.q_Dz7 = q_Dz7;
      pureSlip.q_Dz8 = q_Dz8;
      pureSlip.q_Dz9 = q_Dz9;
      pureSlip.q_Ez1 = q_Ez1;
      pureSlip.q_Ez2 = q_Ez2;
      pureSlip.q_Ez3 = q_Ez3;
      pureSlip.q_Ez4 = q_Ez4;
      pureSlip.q_Ez5 = q_Ez5;
      pureSlip.q_Hz1 = q_Hz1;
      pureSlip.q_Hz2 = q_Hz2;
      pureSlip.q_Hz3 = q_Hz3;
      pureSlip.q_Hz4 = q_Hz4;
      
      //user scaling factors
      pureSlip.lambda_F_z0 = lambda_F_z0;
      pureSlip.lambda_muex = lambda_muex;
      pureSlip.lambda_muey = lambda_muey;
      pureSlip.lambda_Kx = lambda_Kx;
      pureSlip.lambda_Ky = lambda_Ky;
      pureSlip.lambda_Cx = lambda_Cx;
      pureSlip.lambda_Cy = lambda_Cy;
      pureSlip.lambda_Ex = lambda_Ex;
      pureSlip.lambda_Ey = lambda_Ey;
      pureSlip.lambda_Hx = lambda_Hx;
      pureSlip.lambda_Hy = lambda_Hy;
      pureSlip.lambda_Vx = lambda_Vx;
      pureSlip.lambda_Vy = lambda_Vy;
      pureSlip.lambda_gammay = lambda_gammay;
      pureSlip.lambda_gammaz = lambda_gammaz;
      pureSlip.lambda_t = lambda_t;
      pureSlip.lambda_r = lambda_r;
      pureSlip.lambda_Mr = lambda_Mr;
      /********************************************************/
      
      df_z = F_zNonZero*(1 - lambda_F_z0)/lambda_F_z0;
      
      // pure longitudinal slip 
      kappa_x = kappa1 + S_Hx;
      M_y = (S_Vx + K_x*S_Hx);
      C_x = p_Cx1*lambda_Cx;
      D_x = mue_x*F_zNonZero;
      mue_x = (p_Dx1 + p_Dx2*df_z)*lambda_muex;
      E_x = (p_Ex1 + p_Ex2*df_z + p_Ex3*df_z*df_z)*(1 - p_Ex4*sign(kappa_x))*
        lambda_Ex;
      K_x = F_zNonZero*(p_Kx1 + p_Kx2*df_z)*Modelica.Math.exp(-p_Kx3*df_z)*
        lambda_Kx;
      B_x = noEvent(if abs(D_x*C_x) > 0 then K_x/(C_x*D_x) else 0);
      //C_x*D_x*B_x = K_x;
      //B_x = if D_x*C_x>0 then K_x/(C_x*D_x) else 0;
      S_Hx = (p_Hx1 + p_Hx2*df_z)*lambda_Hx;
      S_Vx = F_zNonZero*(p_Vx1 + p_Vx2*df_z)*lambda_Vx*lambda_muex;
      longSlipPure.x = kappa_x;
      longSlipPure.B = B_x;
      longSlipPure.C = C_x;
      longSlipPure.D = D_x;
      longSlipPure.E = E_x;
      longSlipPure.y + S_Vx = F_x0;
      
      // pure lateral slip 
      alpha_y = alpha1 + S_Hy;
      gamma_y = gamma1*lambda_gammay;
      
      C_y = p_Cy1*lambda_Cy;
      D_y = mue_y*F_zNonZero;
      mue_y = (p_Dy1 + p_Dy2*df_z)*(1 - p_Dy3*gamma_y*gamma_y)*lambda_muey;
      E_y = (p_Ey1 + p_Ey2*df_z)*(1 - (p_Ey3 + p_Ey4*gamma_y)*noEvent(if 
        alpha_y < 0 then -1 else 1))*lambda_Ey;
      K_y = p_Ky1*F_z0*Modelica.Math.sin(2*Modelica.Math.atan(F_zNonZero/(p_Ky2
        *F_z0*lambda_F_z0)))*(1 - p_Ky3*noEvent(abs(gamma_y)))*lambda_F_z0*
        lambda_Ky;
      //consider algortihm
      //  B_y = K_y/(C_y*D_y);
      B_y = noEvent(if abs(D_y*C_y) > 0 then K_y/(C_y*D_y) else 0);
      //C_y*D_y*B_y = K_y;
      S_Hy = (p_Hy1 + p_Hy2*df_z + p_Hy3*gamma_y)*lambda_Hy;
      S_Vy = F_zNonZero*(p_Vy1 + p_Vy2*df_z + (p_Vy3 + p_Vy4*df_z)*gamma_y)*
        lambda_Vy*lambda_muey;
      
      latSlipPure.x = alpha_y;
      latSlipPure.B = B_y;
      latSlipPure.C = C_y;
      latSlipPure.D = D_y;
      latSlipPure.E = E_y;
      F_y0 = latSlipPure.y + S_Vy;
      
      //aligning torque
      M_z0 = -t*F_y0 + M_zr;
      t_0 = pneuTrail.y*Modelica.Math.cos(alpha1);
      pneuTrail.B = B_t;
      pneuTrail.C = C_t;
      pneuTrail.D = D_t;
      pneuTrail.E = E_t;
      pneuTrail.x = alpha_t;
      alpha_t = alpha1 + S_Ht;
      
      M_zr0 = resTorque.y*Modelica.Math.cos(alpha1);
      resTorque.B = B_r;
      resTorque.C = 1;
      resTorque.D = D_r;
      resTorque.E = 0;
      resTorque.x = alpha_r;
      alpha_r = alpha1 + S_Hf;
      
      S_Hf = (S_Hy + noEvent(if abs(K_y) > 0 then S_Vy/K_y else 0));
      gamma_z = gamma1*lambda_gammaz;
      B_t = (q_Bz1 + q_Bz2*df_z + q_Bz3*df_z*df_z)*(1 + q_Bz4*gamma_z + q_Bz5*
        noEvent(abs(gamma_z)))*lambda_Ky/lambda_muey;
      C_t = q_Cz1;
      D_t = F_zNonZero*(q_Dz1 + q_Dz2*df_z)*(1 + q_Dz3*gamma_z + q_Dz4*gamma_z*
        gamma_z)*(radius0/F_z0)*lambda_t;
      E_t = (q_Ez1*q_Ez2*df_z + q_Ez3*gamma_z*gamma_z)*(1 + (q_Ez4 + q_Ez5*
        gamma_z)*(2/pi)*Modelica.Math.atan(B_t*C_t*alpha_t));
      S_Ht = (q_Hz1 + q_Hz2*df_z + (q_Hz3*q_Hz4*df_z)*gamma_z);
      B_r = q_Bz9*lambda_Ky/lambda_muey;
      D_r = F_zNonZero*(q_Dz6 + q_Dz7*df_z + (q_Dz8 + q_Dz9*df_z)*gamma_z)*
        radius0*lambda_r*lambda_muey;
      K_z = t_0*K_y;
      
      //longitudinal slip
      combinedSlip.r_Bx1 = r_Bx1;
      combinedSlip.r_Bx2 = r_Bx2;
      combinedSlip.r_Cx1 = r_Cx1;
      combinedSlip.r_Hx1 = r_Hx1;
      
      //lateral slip
      combinedSlip.r_By1 = r_By1;
      combinedSlip.r_By2 = r_By2;
      combinedSlip.r_By3 = r_By3;
      combinedSlip.r_Cy1 = r_Cy1;
      combinedSlip.r_Hy1 = r_Hy1;
      combinedSlip.r_Vy1 = r_Vy1;
      combinedSlip.r_Vy2 = r_Vy2;
      combinedSlip.r_Vy3 = r_Vy3;
      combinedSlip.r_Vy4 = r_Vy4;
      combinedSlip.r_Vy5 = r_Vy5;
      combinedSlip.r_Vy6 = r_Vy6;
      
      //aligning torque
      combinedSlip.s_sz1 = s_sz1;
      combinedSlip.s_sz2 = s_sz2;
      combinedSlip.s_sz3 = s_sz3;
      combinedSlip.s_sz4 = s_sz4;
      
      //user scaling data
      combinedSlip.lambda_xalpha = lambda_xalpha;
      combinedSlip.lambda_ykappa = lambda_ykappa;
      combinedSlip.lambda_Vykappa = lambda_Vykappa;
      combinedSlip.lambda_s = lambda_s;
      
      //longitudinal slip
      B_xalpha = r_Bx1*Modelica.Math.cos(Modelica.Math.atan(r_Bx2*lambda_xalpha))
        *lambda_xalpha;
      C_xalpha = r_Cx1;
      S_Hxalpha = r_Hx1;
      
      G_xalpha = Modelica.Math.cos(C_xalpha*Modelica.Math.atan(B_xalpha*(alpha1
         + S_Hxalpha)))/Modelica.Math.cos(C_xalpha*Modelica.Math.atan(B_xalpha*
        S_Hxalpha));
      
      F_x = if (combined) then F_x0*G_xalpha else F_x0;
      
      //lateral slip
      B_ykappa = r_By1*Modelica.Math.cos(Modelica.Math.atan(r_By2*(alpha1 - 
        r_By3)))*lambda_ykappa;
      C_ykappa = r_Cy1;
      S_Hykappa = r_Hy1;
      S_Vykappa = D_Vykappa*Modelica.Math.sin(r_Vy5*Modelica.Math.atan(r_Vy6*
        kappa1))*lambda_Vykappa;
      D_Vykappa = mue_y*F_zNonZero*(r_Vy1*r_Vy2*df_z + r_Vy3*gamma1)*
        Modelica.Math.cos(Modelica.Math.atan(r_Vy4*alpha1));
      G_ykappa = Modelica.Math.cos(C_ykappa*Modelica.Math.atan(B_ykappa*(kappa1
         + S_Hykappa)))/Modelica.Math.cos(C_ykappa*Modelica.Math.atan(B_ykappa*
        S_Hykappa));
      
      F_y = if (combined) then F_y0*G_ykappa + S_Vykappa else F_y0;
      
      //aligning torque
      
      t = pneuTrail2.y*Modelica.Math.cos(alpha1);
      pneuTrail2.B = B_t;
      pneuTrail2.C = C_t;
      pneuTrail2.D = D_t;
      pneuTrail2.E = E_t;
      pneuTrail2.x = alpha_teq;
      alpha_teq = Modelica.Math.atan(sqrt(Modelica.Math.tan(alpha_t)*
        Modelica.Math.tan(alpha_t) + (K_x*K_x/(K_y*K_y))))*sign(alpha_t);
      
      M_zr = resTorque2.y*Modelica.Math.cos(alpha1);
      resTorque2.B = B_r;
      resTorque2.C = 1;
      resTorque2.D = D_r;
      resTorque2.E = 0;
      resTorque2.x = alpha_req;
      alpha_req = Modelica.Math.atan(sqrt(Modelica.Math.tan(alpha_r)*
        Modelica.Math.tan(alpha_r) + (K_x*K_x/(K_y*K_y))))*sign(alpha_r);
      
      s = (s_sz1 + s_sz2*(F_y/F_z0) + (s_sz3 + s_sz4*df_z)*gamma1)*radius0*
        lambda_s;
      
      //should be modifyed so that parts can be shut of  
      M_z = -t*(F_y - S_Vykappa) + s*F_x;
      //+ M_zgyr;
      
      
        /******************OUTPUT VARIABLES*****************************************/
      Fx = if not contact then 0 else if (noEvent(abs(v_r)) > v_eps) then F_x
         else 0;
      Fy = if contact then F_y*left else 0;
      Mz = if contact then M_z*left else 0;
      
      
        /**************************************************************************/
      
    end TyreForces;
    
    model Wheel "Wheel model according to the Magic formula" 
      
      import Car;
      import Modelica.Math;
      extends Utilities.BaseWheel(final wheelData(
          R0=tyreData.R0, 
          width=tyreData.width, 
          m=tyreData.m, 
          Iyy=tyreData.Iyy, 
          Ixx=tyreData.Ixx));
      
      //USER PARAMETERS
      Boolean transient=true 
        "true if transient behaviour should be considered.";
      Boolean combined=true "true if combined slip should be considered.";
      
      //TYRE CONTACT 
      Boolean contact "true, if wheel has contact with road";
      SI.Position delta_z "vertical deflection of tyre";
      
      //FORCE GENERATION
      SI.Force f_x "Tyre/contact force at origin of W-frame in x-direction";
      SI.Force f_y "Tyre/contact force at origin of W-frame in y-direction";
      SI.Force f_z "Tyre/contact force at origin of W-frame in z-direction";
      SI.Force t_z "Tyre/contact torque at origin of W-frame in z-direction";
      
      //VISUAL REPRESENTATION
      Real S_rel[3, 3];
      
    protected 
      parameter Real left=if (leftWheel) then 1 else -1;
    public 
      Wheels.RillTyre.TyreData tyreData annotation (extent=[-80, 60; -60, 80]);
      annotation (
        Documentation(info="This element contains the inertia properties of a
rotating wheel and the tyre forces acting at the
contact point of the wheel with the road. The tyre forces
are computed accorded to the equations of Rill (see reference below).
The wheel data is defined via the record WheelData and the tire
data is defined via the record RillTyre.TyreData.
Therefore, it suffices to define one instance of
these records and use them for all 4 wheels of a vehicle.
The frames of this object have the following meaning:
Frame carrierFrame is fixed in the carrier of the wheel
and is located on the spin axis of the wheel in the
middle point of the wheel. This point is also approximately
used as the center-of-mass of the wheel.
It is temporarily assumed that the road lies in the
x-y plane of the inertial frame! This restriction can
be removed by defining the road properties via inner/outer
functions.
Flange driveShaft is used to drive the wheel with
a 1D-drive train.
This element is realized according to the description
  Georg Rill: Simulation von Kraftfahrzeugen,
              Vieweg, 1994
"), 
        Icon, 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.46, 
          y=0.13, 
          width=0.51, 
          height=0.93));
      
      VisualShape shape_1(
        r0=-nShape*0.7*abs(wheelData.width), 
        Length=abs(wheelData.width), 
        Width=2*wheelData.R0, 
        Height=2*wheelData.R0, 
        LengthDirection=nShape, 
        WidthDirection={1,0,0}, 
        Shape="pipe", 
        Material={0,0,0,.3}, 
        Extra=0.7) annotation (extent=[-20, -120; 0, -100]);
      VisualShape shape0(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        WidthDirection=-nShape, 
        Material={0.2,0,0,0.7}) annotation (extent=[0, -120; 20, -100]);
      VisualShape shape1(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={0.6235,0,0.7818}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[20, -120; 40, -100]);
      VisualShape shape2(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.2225,0,0.9749}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[40, -120; 60, -100]);
      VisualShape shape3(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.9010,0,0.4339}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[60, -120; 80, -100]);
      VisualShape shape4(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.9010,0,-0.4339}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[80, -120; 100, -100]);
      VisualShape shape5(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={-0.2225,0,-0.9749}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[104, -88; 124, -68]);
      VisualShape shape6(
        r0=abs(wheelData.width)*nShape*0.2, 
        Length=wheelData.R0 - 0.01, 
        Width=0.02, 
        Height=0.05, 
        LengthDirection={0.6235,0,-0.7818}, 
        WidthDirection=-nShape, 
        Material={0.2,0.2,0.2,0.7}) annotation (extent=[118, -86; 138, -66]);
      Wheels.MFTyre.TyreForces tyreForces(
        leftWheel=leftWheel, 
        radius0=tyreData.R0, 
        data=tyreData) annotation (extent=[-64, -90; -44, -70]);
    equation 
      
      //NORMAL FORCE
      //according to Rill:
      delta_z = if delta_r >= 0 then delta_r else 0;
      contact = delta_r >= 0;
      //end when;
      // Compute  force perpendicular to contact plane.
      f_z = if (delta_r >= 0 and tyreData.c_z*delta_z + tyreData.d_z*ddelta_r
         > 0) then tyreData.c_z*delta_z + tyreData.d_z*ddelta_r else 0;
      
      //TYRE FORCE GENERATION
      tyreForces.v_x = v_x;
      tyreForces.v_y = v_y;
      tyreForces.w = w;
      tyreForces.contact = contact;
      tyreForces.gamma1 = camberAngle;
      tyreForces.Fz = f_z;
      
      tyreForces.Fx = f_x;
      tyreForces.Fy = f_y;
      tyreForces.Mz = t_z;
      
      //FORCES AND MOMENTS REQUIRED BY THE BaseWheel
      F_x = f_x;
      F_y = f_y;
      F_z = f_z;
      M_x = 0;
      //this model is yet unable to generate overturning moment 
      M_y = 0;
      //and roll resistance torque.
      M_z = t_z;
      
      //VISUAL REPRESENTATION OF TYRE AND RIM
      S_rel = [nn]*transpose([nn]) + (identity(3) - [nn]*transpose([nn]))*cos(
        phi) - skew(nn)*sin(phi);
      //tyre
      shape_1.S = wheel_body.frame_b.S*transpose(S_rel);
      shape_1.r = wheel_body.frame_b.r0;
      //rim
      shape0.S = shape_1.S;
      shape0.r = shape_1.r;
      shape1.S = shape0.S;
      shape1.r = shape0.r;
      shape2.S = shape1.S;
      shape2.r = shape1.r;
      shape3.S = shape2.S;
      shape3.r = shape2.r;
      shape4.S = shape3.S;
      shape4.r = shape3.r;
      shape5.S = shape4.S;
      shape5.r = shape4.r;
      shape6.S = shape5.S;
      shape6.r = shape5.r;
      
    end Wheel;
    
    model WheelWithForceAnimation 
      "Wheel model according to the Magic Formula, with animated tyre forces" 
      
      extends Wheel;
      parameter Real scale_F_x=1 "scaling of visual appearance of F_x";
      parameter Real scale_F_y=1 "scaling of visual appearance of F_y";
      parameter Real scale_F_z=1/tyreData.load1.Fz_nom 
        "scaling of visual appearance of F_z";
      parameter Real scale_M_z=1 "scaling of visual appearance of M_z";
      VisualVector force_x(
        category="force", 
        r=Wr, 
        Size=We_x*(scale_F_x*F_x), 
        S=Sroad) annotation (extent=[-94, -78; -74, -98]);
      VisualVector force_y(
        Material={0,1,0,0.5}, 
        category="force", 
        r=Wr, 
        Size=We_y*(scale_F_y*F_y), 
        S=Sroad) annotation (extent=[-64, -78; -44, -98]);
      VisualVector force_z(
        Material={0,0,1,0.5}, 
        category="force", 
        r=Wr, 
        Size=We_z*(scale_F_z*F_z), 
        S=Sroad) annotation (extent=[-34, -78; -14, -98]);
      VisualVector torque(
        Material={1,0,0,0.5}, 
        category="torque", 
        r=Wr, 
        Size=Wt*scale_M_z*M_z, 
        S=Sroad) annotation (extent=[-2, -78; 18, -98]);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.49, 
          height=0.51), 
        Diagram);
    end WheelWithForceAnimation;
    
    record TyreData "Tyre data" 
      extends Wheels.Utilities.WheelData;
      
      parameter Real lambda_muey=1;
      // Spring and damping characteristic of tyre
      parameter Real c_x(unit="N/m") = 100000 
        "tyre spring constant in x-direction";
      parameter Real c_y(unit="N/m") = 150000 
        "tyre spring constant in y-direction";
      parameter Real c_z(unit="N/m") = 200000 
        "tyre spring constant in z-direction";
      parameter Real d_x(unit="N.s/m") = 1500 
        "tyre damping constant in x-direction";
      parameter Real d_y(unit="N.s/m") = 300 
        "tyre damping constant in y-direction";
      parameter Real d_z(unit="N.s/m") = 300 
        "tyre damping constant in z-direction";
      
      // Data for nominal vertical load 1
      parameter Real mue_nom=0.7 
        "friction coefficient of road for which tyre data is valid";
      
      Wheels.RillTyre.NominalLoad load1(F_max_y=lambda_muey)
        annotation (extent=[-85, 15; -15, 85]);
      Wheels.RillTyre.NominalLoad load2(
        Fz_nom=6000, 
        Fds_x=75000, 
        s_max_x=0.18, 
        F_max_x=4500, 
        s_slide_x=0.5, 
        F_slide_x=4200, 
        Fds_y=60000, 
        s_max_y=0.24, 
        F_max_y=4125*lambda_muey, 
        s_slide_y=0.8, 
        F_slide_y=3750, 
        dL0=0.1, 
        s0=0.275, 
        sE=0.8) annotation (extent=[15, 15; 85, 85]);
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Window(
          x=0.3, 
          y=0.29, 
          width=0.6, 
          height=0.6));
    end TyreData;
    
    record MFpureSlip 
      //longitudinal
      parameter Real p_Cx1=1.75757 "shape factor C";
      parameter Real p_Dx1=1.08851 "peak factor D";
      parameter Real p_Dx2=-0.0312973 "peak factor D";
      parameter Real p_Ex1=0.551769 "curvature factor E";
      parameter Real p_Ex2=-0.049158;
      //0.370176 "curvature factor E";
      parameter Real p_Ex3=0.05;
      //-0.17014 "curvature factor E";
      parameter Real p_Ex4=0 "curvature factor E";
      parameter Real p_Kx1=19.0610 "slip stiffness BCD";
      parameter Real p_Kx2=0.0017;
      //-0.466168 "slip stiffness BCD";
      parameter Real p_Kx3=0.483251 "slip stiffness BCD";
      parameter Real p_Hx1=-.000545*0 "horisontal shift H";
      parameter Real p_Hx2=0*0.000209 "horisontal shift H";
      parameter Real p_Vx1=0 "vertical shift V";
      parameter Real p_Vx2=0 "vertical shift V";
      //lateral
      parameter Real p_Cy1=1.20328 "shape factor C";
      parameter Real p_Dy1=-1.07146 "peak factor D";
      parameter Real p_Dy2=0.178566 "peak factor D";
      parameter Real p_Dy3=2.78722 "peak factor D";
      parameter Real p_Ey1=-1.56726 "curvature factor E";
      parameter Real p_Ey2=-.548012 "curvature factor E";
      parameter Real p_Ey3=-.188264 "curvature factor E";
      parameter Real p_Ey4=5.14031 "curvature factor E";
      parameter Real p_Ky1=-15.9592 "slip stiffness BCD";
      parameter Real p_Ky2=1.161200 "slip stiffness BCD";
      parameter Real p_Ky3=0.623354 "slip stiffness BCD";
      parameter Real p_Hy1=-0.00716805 "horisontal shift H";
      parameter Real p_Hy2=0.00319597 "horisontal shift H";
      parameter Real p_Hy3=-0.0994834 "horisontal shift H";
      parameter Real p_Vy1=-0.0122314 "vertical shift V";
      parameter Real p_Vy2=-0.000606665 "vertical shift V";
      parameter Real p_Vy3=-0.753073 "vertical shift V";
      parameter Real p_Vy4=0.558202 "vertical shift V";
      //aligning torque
      parameter Real q_Bz1=10.7807 "stiffness factor B";
      parameter Real q_Bz2=-4.0056 "stiffness factor B";
      parameter Real q_Bz3=1.84837 "stiffness factor B";
      parameter Real q_Bz4=0.294563 "stiffness factor B";
      parameter Real q_Bz5=0 "stiffness factor B";
      parameter Real q_Bz9=31.8193 "stiffness factor B";
      parameter Real q_Cz1=1.23342 "shape factor C";
      parameter Real q_Dz1=0.0916462 "peak factor D";
      parameter Real q_Dz2=0.00317208 "peak factor D";
      parameter Real q_Dz3=-0.458539 "peak factor D";
      parameter Real q_Dz4=0 "peak factor D";
      parameter Real q_Dz6=0.00559840 "peak factor D";
      parameter Real q_Dz7=-0.00915360 "peak factor D";
      parameter Real q_Dz8=0.332951 "peak factor D";
      parameter Real q_Dz9=-0.174946 "peak factor D";
      parameter Real q_Ez1=-2.51151 "curvature factor E";
      parameter Real q_Ez2=1.14149 "curvature factor E";
      parameter Real q_Ez3=0 "curvature factor E";
      parameter Real q_Ez4=-0.474070 "curvature factor E";
      parameter Real q_Ez5=-3.88634 "curvature factor E";
      parameter Real q_Hz1=-0.00397677 "horisontal shift H";
      parameter Real q_Hz2=-0.0036033 "horisontal shift H";
      parameter Real q_Hz3=-0.215809 "horisontal shift H";
      parameter Real q_Hz4=-0.168349 "horisontal shift H";
      // User scaling Factors
      parameter Real lambda_F_z0=1 "nominal load";
      parameter Real lambda_muex=1 "peak friction coefficient";
      parameter Real lambda_muey=1 "peak friction coefficient";
      parameter Real lambda_Kx=1 "slip stiffness";
      parameter Real lambda_Ky=1 "slip stiffness";
      parameter Real lambda_Cx=1 "shape factor";
      parameter Real lambda_Cy=1 "shape factor";
      parameter Real lambda_Ex=1 "curvature factor";
      parameter Real lambda_Ey=1 "curvature factor";
      parameter Real lambda_Hx=1 "horisontal shift";
      parameter Real lambda_Hy=1 "horisontal shift";
      parameter Real lambda_Vx=1 "vertical shift";
      parameter Real lambda_Vy=1 "vertical shift";
      parameter Real lambda_gammay=1 "camber force stiffness";
      parameter Real lambda_gammaz=1 "camber torque stiffness";
      parameter Real lambda_t=1 "pneumatic trail";
      parameter Real lambda_r=1;
      parameter Real lambda_Mr=1 "residual torque";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.01, 
          y=0.12, 
          width=0.6, 
          height=0.75), 
        Icon(
          Text(
            extent=[-100, -54; 100, -94], 
            string="pure slip", 
            style(color=0)), 
          Text(extent=[-122, 140; 116, 80], string="%name"), 
          Text(
            extent=[-98, 33; -7, 15], 
            string="...", 
            style(color=10)), 
          Line(points=[-100, 10; -100, 70; 100, 70; 100, 10; -100, 10; -100, 40; 
                 100, 40; 100, 70; 0, 70; 0, 10], style(color=0, fillPattern=0)), 
             
          Text(
            extent=[-98, 65; -7, 47], 
            string="p_Cx1", 
            style(color=10)), 
          Text(
            extent=[-100, 0; 100, -40], 
            string="tyre data", 
            style(color=0)), 
          Text(
            extent=[-98, 33; -7, 15], 
            string="...", 
            style(color=10)), 
          Text(
            extent=[-100, -54; 100, -94], 
            string="pure slip", 
            style(color=0)), 
          Line(points=[-100, 10; -100, 70; 100, 70; 100, 10; -100, 10; -100, 40; 
                 100, 40; 100, 70; 0, 70; 0, 10], style(color=0, fillPattern=0)), 
             
          Text(
            extent=[-98, 65; -7, 47], 
            string="p_Cx1", 
            style(color=10)), 
          Text(
            extent=[-100, 0; 100, -40], 
            string="tyre data", 
            style(color=0)), 
          Text(
            extent=[-98, 33; -7, 15], 
            string="...", 
            style(color=10)), 
          Text(
            extent=[-100, -54; 100, -94], 
            string="pure slip", 
            style(color=0)), 
          Line(points=[-100, 10; -100, 70; 100, 70; 100, 10; -100, 10; -100, 40; 
                 100, 40; 100, 70; 0, 70; 0, 10], style(color=0, fillPattern=0)), 
             
          Text(
            extent=[-98, 65; -7, 47], 
            string="p_Cx1", 
            style(color=10)), 
          Text(
            extent=[-100, 0; 100, -40], 
            string="tyre data", 
            style(color=0)), 
          Text(extent=[-122, 140; 116, 80], string="%name")));
      
    end MFpureSlip;
    
    record MFcombinedSlip 
      //longitudinal
      parameter SI.Position r_Bx1=12.2428 "stiffness factor B";
      parameter SI.Position r_Bx2=-9.97521 "stiffness factor B";
      parameter SI.Position r_Cx1=1.06866 "shape factor C";
      parameter SI.Position r_Hx1=-0.00618958 "horisontal shift H";
      //lateral
      parameter SI.Position r_By1=5.80206 "stiffness factor B";
      parameter SI.Position r_By2=5.87349 "stiffness factor B";
      parameter SI.Position r_By3=1.08851 "stiffness factor B";
      parameter SI.Position r_Cy1=1.10446 "shape factor C";
      parameter SI.Position r_Hy1=-0.0311875 "horisontal shift H";
      parameter SI.Position r_Vy1=0.0620825 "vertical shift V";
      parameter SI.Position r_Vy2=-0.0363073 "vertical shift V";
      parameter SI.Position r_Vy3=-0.313050 "vertical shift V";
      parameter SI.Position r_Vy4=23.5060 "vertical shift V";
      parameter SI.Position r_Vy5=1.9 "vertical shift V";
      parameter SI.Position r_Vy6=-5.20498 "vertical shift V";
      
      //aligning torque
      parameter Real s_sz1=-0.0313609;
      parameter Real s_sz2=0.0143442;
      parameter Real s_sz3=-0.655596;
      parameter Real s_sz4=0.5;
      
      // User scaling Factors
      parameter Real lambda_xalpha=1;
      parameter Real lambda_ykappa=1;
      parameter Real lambda_Vykappa=1;
      parameter Real lambda_s=1;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.32, 
          y=0.13, 
          width=0.6, 
          height=0.75), 
        Icon(
          Text(extent=[-122, 140; 116, 80], string="%name"), 
          Text(
            extent=[-98, 65; -7, 47], 
            string="p_Cx1", 
            style(color=10)), 
          Line(points=[-100, 10; -100, 70; 100, 70; 100, 10; -100, 10; -100, 40; 
                 100, 40; 100, 70; 0, 70; 0, 10], style(color=0, fillPattern=0)), 
             
          Text(
            extent=[-98, 33; -7, 15], 
            string="...", 
            style(color=10)), 
          Text(
            extent=[-100, 0; 100, -40], 
            string="tyre data", 
            style(color=0)), 
          Text(
            extent=[-100, -54; 100, -94], 
            string="combined slip", 
            style(color=0))));
      
    end MFcombinedSlip;
    
    block MFcosine 
      
      input Real x;
      input Real B;
      input Real C;
      input Real D;
      input Real E;
      
      output Real y;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-100, 20; 100, -100]), 
          Line(points=[-80, -78; -60, -78; -46, -76; -36, -70; -26, -60; -20, -50; 
                 -14, -42; -4, -36; 0, -36]), 
          Line(points=[78, -78; 56, -78; 46, -76; 30, -70; 22, -60; 16, -48; 12, 
                 -42; -2, -36; -4, -36]), 
          Text(extent=[-100, 100; 100, 20], string="%name")), 
        Window(
          x=0.23, 
          y=0.23, 
          width=0.6, 
          height=0.6));
    equation 
      
      y = D*Modelica.Math.cos(C*Modelica.Math.atan(B*x - E*(B*x - 
        Modelica.Math.atan(B*x))));
    end MFcosine;
    
    block MFsine 
      input Real x;
      input Real B;
      input Real C;
      input Real D;
      input Real E;
      
      output Real y;
      
    protected 
      Real test1;
      Real test2;
      Real bx;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-100, 20; 100, -100]), 
          Line(points=[-40, -62; -24, -38; -18, -34; -8, -32; 0, -32; 8, -36; 
                24, -38; 60, -38]), 
          Text(extent=[-100, 100; 100, 20], string="%name")), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6));
    equation 
      test1 = C*Modelica.Math.atan(B*x - E*(B*x - Modelica.Math.atan(B*x)));
      test2 = B*x - E*(B*x - Modelica.Math.atan(B*x));
      bx = B*x;
      
      y = D*Modelica.Math.sin(C*Modelica.Math.atan(B*x - E*(B*x - 
        Modelica.Math.atan(B*x))));
    end MFsine;
    
    record MFtransient 
      //longitudinal
      parameter Real p_Tx1=1.9624;
      parameter Real p_Tx2=-0.0053684;
      parameter Real p_Tx3=-0.31037;
      parameter Real lambda_sigmakappa=1;
      //lateral
      parameter Real p_Ty1=2.3935;
      parameter Real p_Ty2=2.6895;
      parameter Real lambda_sigmaalpha=1;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.32, 
          y=0.13, 
          width=0.6, 
          height=0.75), 
        Icon(
          Text(extent=[-122, 140; 116, 80], string="%name"), 
          Text(
            extent=[-98, 65; -7, 47], 
            string="p_Cx1", 
            style(color=10)), 
          Line(points=[-100, 10; -100, 70; 100, 70; 100, 10; -100, 10; -100, 40; 
                 100, 40; 100, 70; 0, 70; 0, 10], style(color=0, fillPattern=0)), 
             
          Text(
            extent=[-98, 33; -7, 15], 
            string="...", 
            style(color=10)), 
          Text(
            extent=[-100, 0; 100, -40], 
            string="tyre data", 
            style(color=0)), 
          Text(
            extent=[-100, -54; 100, -94], 
            string="transient", 
            style(color=0))));
      
    end MFtransient;
    
    model TyreForceTest 
      
      Wheels.MFTyre.TyreForces tyreForces annotation (extent=[-20, 12; 46, 64]);
    equation 
      
      tyreForces.v_x = time;
      tyreForces.v_y = 0;
      tyreForces.w = time*4 + time*time*0.1;
      tyreForces.contact = true;
      tyreForces.gamma1 = 0;
      tyreForces.Fz = 3000;
      
    end TyreForceTest;
  end MFTyre;
  
end Wheels;


package Drivers 
  extends Modelica.Icons.Library;
  
  annotation (Coordsys(
      extent=[0, 0; 442, 394], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.45, 
      y=0.01, 
      width=0.35, 
      height=0.49, 
      library=1, 
      autolayout=1));
  model LaneChange 
    parameter Real t1=0.5 "start time of sine1";
    parameter Real t2=3.5 "start time of sine2";
    parameter Real p1=2 "period of sine1";
    parameter Real p2=2 "period of sine2";
    parameter Real a1=70*3.14/180 "amplitude of sine1";
    parameter Real a2=-70*3.14/180 "amplitude of sine2";
    parameter Boolean generateEvent=false "generate event for discontinuities";
    
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.15, 
        y=0.18, 
        width=0.6, 
        height=0.6), 
      Icon(
        Rectangle(extent=[-100, 100; 100, -100]), 
        Text(extent=[-80, 10; 100, 82], string="Lane "), 
        Text(extent=[-88, -8; 90, -88], string="change")));
  public 
    Modelica.Blocks.Interfaces.OutPort outPort
      annotation (extent=[100, -10; 120, 10]);
  equation 
    outPort.signal[1] = if generateEvent then if time < t1 then 0 else if time
       < t1 + p1 then a1*sin((2*Modelica.Constants.pi/p1)*(time - t1)) else if 
      time < t2 then 0 else if time < t2 + p2 then a2*sin((2*Modelica.Constants
      .pi/p2)*(time - t2)) else 0 else noEvent(if time < t1 then 0 else if time
       < t1 + p1 then a1*sin((2*Modelica.Constants.pi/p1)*(time - t1)) else if 
      time < t2 then 0 else if time < t2 + p2 then a2*sin((2*Modelica.Constants
      .pi/p2)*(time - t2)) else 0);
    
  end LaneChange;
  
  model OpenLoopDriver 
    import SI = Modelica.SIunits;
    extends Interfaces.DriverInterface;
    
    // STEERING
    
    parameter Real kLaneChange=1 
      "|Steering|LaneChange| Gain of Lane Change output, use this to turn on/off output";
    parameter Real t1LaneChange=0.5 
      "|Steering|LaneChange| start time of sine1 ";
    parameter Real t2LaneChange=3.5 "|Steering|LaneChange|start time of sine2";
    parameter Real p1LaneChange=2 "|Steering|LaneChange|period of sine1";
    parameter Real p2LaneChange=2 "|Steering|LaneChange| period of sine2";
    parameter Real a1LaneChange=0.5 "|Steering|LaneChange| amplitude of sine1";
    parameter Real a2LaneChange=-0.5 "|Steering|LaneChange| amplitude of sine2";
    parameter Boolean generateEvent=false 
      "|Steering|LaneChange| generate event for discontinuities";
    
    parameter Real kRampSteering=0 
      "|Steering|Ramp| Gain of Ramp output, use this to turn on/off output";
    parameter SI.Position heightRampSteering=0.5 
      "|Steering|Ramp| Height of ramp";
    parameter Real durationRampSteering(min=Modelica.Constants.SMALL) = 2 
      "|Steering|Ramp| Duration of ramp";
    parameter Real offsetRampSteering=0 "|Steering|Ramp| Ramp offset";
    parameter SI.Time startTimeRampSteering=0 
      "|Steering|Ramp| Output = offset for time < startTime";
    
    parameter Real kTableSteering=0 
      "|Steering|TimeTable| Gain of Table output, use this to turn on/off output";
    parameter Real tableSteering[:, :]=[0, 0; 1, 1; 2, 4] 
      "|Steering|TimeTable| Table matrix (time = first column)";
    
    // DRIVE
    parameter Real kRampDrive=0 
      "|Drive|Ramp| Gain of Ramp output, use this to turn on/off output";
    parameter SI.Position heightRampDrive=50 "|Drive|Ramp| Height of ramp";
    parameter Real durationRampDrive(min=Modelica.Constants.SMALL) = 2 
      "|Drive|Ramp| Duration of ramp";
    parameter Real offsetRampDrive=0 "|Drive|Ramp| Ramp offset";
    parameter SI.Time startTimeRampDrive=0 
      "|Drive|Ramp| Output = offset for time < startTime";
    
    parameter Real kTableDrive=0 
      "|Drive|TimeTable| Gain of Table output, use this to turn on/off output";
    parameter Real tableDrive[:, :]=[0, 0; 1, 1; 2, 4] 
      "|Drive|TimeTable| Table matrix (time = first column)";
    
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[1, 1], 
        component=[20, 20]), 
      Icon, 
      Window(
        x=0.4, 
        y=0.4, 
        width=0.6, 
        height=0.6), 
      Diagram);
    Modelica.Mechanics.Rotational.Position steeringWheelPosition(
      f_crit=10, 
      phi(stateSelect=StateSelect.always), 
      w(stateSelect=StateSelect.always)) annotation (extent=[60, 20; 80, 40]);
    Drivers.LaneChange laneChangeSteering(
      t1=t1LaneChange, 
      t2=t2LaneChange, 
      p1=p1LaneChange, 
      p2=p2LaneChange, 
      a1=a1LaneChange, 
      a2=a2LaneChange, 
      generateEvent=generateEvent) annotation (extent=[-50, 70; -30, 90]);
    Modelica.Blocks.Sources.Ramp rampSteering(
      height={heightRampSteering}, 
      offset={offsetRampSteering}, 
      startTime={startTimeRampSteering}, 
      duration={durationRampSteering}) annotation (extent=[-50, 40; -30, 60]);
    Modelica.Blocks.Sources.TimeTable timeTableSteering(table=tableSteering)
      annotation (extent=[-50, 10; -30, 30]);
    Modelica.Blocks.Sources.TimeTable TimeTableDrive(table=tableDrive)
      annotation (extent=[-50, -90; -30, -70]);
    Modelica.Blocks.Math.Add3 addSteering(
      k1=kLaneChange, 
      k2=kRampSteering, 
      k3=kTableSteering) annotation (extent=[0, 30; 20, 70]);
    Modelica.Blocks.Sources.Ramp rampDrive(
      duration={durationRampDrive}, 
      offset={offsetRampDrive}, 
      startTime={startTimeRampDrive}, 
      height={heightRampDrive}) annotation (extent=[-50, -60; -30, -40]);
    Modelica.Blocks.Math.Add addDrive(k1=kRampDrive, k2=kTableDrive)
      annotation (extent=[0, -80; 20, -50]);
    Modelica.Mechanics.Rotational.Position acceleratorBrakePosition(f_crit=10)
      annotation (extent=[50, -50; 70, -30]);
  equation 
    
    connect(timeTableSteering.outPort, addSteering.inPort3)
      annotation (points=[-29, 20; -10, 20; -3, 32], style(color=3));
    connect(laneChangeSteering.outPort, addSteering.inPort1)
      annotation (points=[-29, 80; -10, 80; -3, 67], style(color=3));
    connect(rampSteering.outPort, addSteering.inPort2)
      annotation (points=[-29, 50; -3, 50], style(color=3));
    connect(addSteering.outPort, steeringWheelPosition.inPort)
      annotation (points=[21, 50; 40, 50; 57, 29], style(color=3));
    connect(rampDrive.outPort, addDrive.inPort1)
      annotation (points=[-29, -50; -10, -50; -4, -56], style(color=3));
    connect(TimeTableDrive.outPort, addDrive.inPort2)
      annotation (points=[-29, -80; -10, -80; -2, -75], style(color=3));
    connect(addDrive.outPort, acceleratorBrakePosition.inPort)
      annotation (points=[20, -66; 48, -40], style(color=3));
    connect(acceleratorBrakePosition.flange_b, flange_Long)
      annotation (points=[70, -39; 105, -31], style(color=0));
    connect(steeringWheelPosition.flange_b, flange_Lat)
      annotation (points=[79, 29; 106, 28], style(color=0));
  end OpenLoopDriver;
  
  package Interfaces "Driver Interfaces" 
    extends Modelica.Icons.Library;
    model DriverInterface 
      extends LongitudinalDriverInterface;
      extends LateralDriverInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon, 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Diagram, 
        Documentation(info="Basic interface for drivers. The signals should be interpreted as follows:
flange_Lat: the lateral control, normally the steering wheel axle but could also be left-right motion of a joystick.
flange_Long: the longitudinal control, normally positive if accellerator pedal is pushed and negative if brake pedal is pushed but could also be forward-backward motion of a joystick.
The advantage of using flanges for the longitudinal motion interface instead of signal is that flanges better mimic the actual driver-vehicle interface and opens up for better interaction ability. As an example it is possible to model a coming feature to increase the stiffness of the accelerator pedal when a the driver commands a maneouvre with a high power consumption.
The advantage of using flanges for the lateral motion interface instead of signals is that it is possible to give \"steering feel\" feedback to the driver.
Possible problems may occur if both driver and pedals/steering wheel/joystick apply forces without having an own inertia. This could be the case when you combine an advanced driver that commands torque and not angle and a simple pedal set/driver interpreter that only reads the pedal/steering wheel/joystick angle.
Relevant extensions to this interface is ClosedLoopDriverInterface that makes it possible to model the drivers sensing. Also relevant could be to add a clutch pedal to handle manually sear shifted cars. Though, it could be discussed werther this should be modelled as a driver task or if it could be added as an isolated gear shift strategy, depending on drivers intentions read via this interface. A situation where it would be relevant to add the clutch pedal to the driver is when the capabilities of the driver is limited (stress etc.) but in those cases, it might also be neccesary to take a more detailed driver-vehicle interface into account. This issue is quite open.
Furthermore it would be logical to let this class be inherited from two other interface classes LongitudinalDriverInterface and LateralDriverInterface.
"));
    equation 
      
    end DriverInterface;
    
    model ClosedLoopDriverInterface 
      import SI = Modelica.SIunits;
      extends DriverInterface;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon, 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Diagram, 
        Documentation(info=
              "Extension of the DriverInterface that allows seensing of the vehicle motion via frame_a."));
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-130, -16; -100, 14]);
    equation 
      
    end ClosedLoopDriverInterface;
    
    model LateralDriverInterface 
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-59, 53; -48, -13; -6, -3; 29, -35; 39, -23], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-48, -13; -7, 0; 28, -24; 39, -11], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-70, 80; -50, 53], style(color=0)), 
          Line(points=[-58, 47; -26, 26; 2, 46], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[7, 54; 0, 22; 2, 22; 9, 53; 7, 54], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[5, 39; 70, 23; 70, 21; 5, 37; 5, 39], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[70, 22; 100, 30], style(
              color=0, 
              pattern=2, 
              fillColor=1, 
              fillPattern=1))), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Diagram, 
        Documentation(info="Base interface for lateral drivers. The signals should be interpreted as follows:
flange_Lat: the lateral control, normally the steering wheel axle but could also be left-right motion of a joystick.
The advantage of using flanges for the lateral motion interface instead of signals is that it is possible to give \"steering feel\" feedback to the driver. The driver commands can then interact directly with the chassis's steering or with a steer-by wire control system.
Possible problems may occur if both driver and pedals/steering wheel/joystick apply forces without having an own inertia. This could be the case when you combine an advanced driver that commands torque and not angle and a simple pedal set/driver interpreter that only reads the pedal/steering wheel/joystick angle.
"));
    public 
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_Lat
        annotation (extent=[100, 20; 120, 40]);
    equation 
      
    end LateralDriverInterface;
    
    model LongitudinalDriverInterface 
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-59, 53; -48, -13; -6, -3; 29, -35; 39, -23], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-48, -13; -7, 0; 28, -24; 39, -11], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[-70, 80; -50, 53], style(color=0)), 
          Line(points=[-58, 47; -26, 26; 2, 46], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[40, -28; 45, -32; 47, -35; 48, -39; 50, -39; 49, -34; 
                 46, -30; 40, -26; 40, -28], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[41, -16; 46, -20; 48, -23; 49, -27; 51, -27; 50, -22; 
                 47, -18; 41, -14; 41, -16], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[42, -21; 29, -39; 31, -40; 44, -22; 42, -21], style(
              color=0, 
              fillColor=58, 
              fillPattern=1)), 
          Polygon(points=[43, -9; 30, -27; 32, -28; 45, -10; 43, -9], style(
              color=0, 
              fillColor=1, 
              fillPattern=1)), 
          Line(points=[50, -30; 100, -30], style(
              color=0, 
              pattern=2, 
              fillColor=1, 
              fillPattern=1))), 
        Window(
          x=0.4, 
          y=0.4, 
          width=0.6, 
          height=0.6), 
        Diagram, 
        Documentation(info="Base interface for longitudinal drivers without feedback. The signals should be interpreted as follows:
flange_Long: the longitudinal control, normally positive if accellerator pedal is pushed and negative if brake pedal is pushed but could also be forward-backward motion of a joystick.
The advantage of using flanges for the longitudinal motion interface instead of signal is that flanges better mimic the actual driver-vehicle interface and opens up for better interaction ability. As an example it is possible to model a coming feature to increase the stiffness of the accelerator pedal when a the driver commands a maneouvre with a high power consumption.
Possible problems may occur if both driver and pedals/steering wheel/joystick apply forces without having an own inertia. This could be the case when you combine an advanced driver that commands torque and not angle and a simple pedal set/driver interpreter that only reads the pedal/steering wheel/joystick angle.
"));
    public 
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_Long
        annotation (extent=[100, -40; 120, -20]);
    equation 
      
    end LongitudinalDriverInterface;
  end Interfaces;
end Drivers;


package Utilities 
  import Modelica.SIunits;
  extends Modelica.Icons.Library;
  
  annotation (Coordsys(
      extent=[0, 0; 616, 570], 
      grid=[1, 1], 
      component=[20, 20]), Window(
      x=0, 
      y=0.38, 
      width=0.39, 
      height=0.52, 
      library=1, 
      autolayout=1));
  
  
  model RotationalSum "Ideal planetary gear box" 
    parameter Real ratio_a1=1 "gear ratio a1 to b";
    parameter Real ratio_a2=1 "gear ratio a2 to b";
    Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a2 
      "sun flange (flange axis directed INTO cut plane)"
      annotation (extent=[-110, -50; -90, -30]);
    Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a1 
      "carrier flange (flange axis directed INTO cut plane)"
      annotation (extent=[-90, 30; -110, 50]);
    Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b 
      "ring flange (flange axis directed OUT OF cut plane)"
      annotation (extent=[110, -10; 90, 10]);
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[1, 1], 
        component=[20, 20]), 
      Window(
        x=0.25, 
        y=0.01, 
        width=0.3, 
        height=0.73), 
      Documentation(info="<HTML>
<p>
The IdealPlanetary gear box is an ideal gear without inertia,
elasticity, damping or backlash consisting
of an inner <b>sun</b> wheel, an outer <b>ring</b> wheel and a
<b>planet</b> wheel located between sun and ring wheel. The bearing
of the planet wheel shaft is fixed in the planet <b>carrier</b>.
The component can be connected to other elements at the
sun, ring and/or carrier flanges. It is not possible to connect
to the planet wheel. If inertia shall not be neglected,
the sun, ring and carrier inertias can be easily added by attaching
inertias (= model Inertia) to the corresponding connectors.
The inertias of the planet wheels are always neglected.
</p>
<p>
The icon of the planetary gear signals that the sun and carrier
flanges are on the left side and the ring flange is on the right side
of the gear box. However, this component is generic and is valid
independantly how the flanges are actually placed (e.g. sun wheel
may be placed on the right side instead on the left side in reality).
</p>
<p>
The ideal planetary gearbox is uniquely defined by the ratio
of the number of ring teeth zr with respect to the number of
sun teeth zs. For example, if there are 100 ring teeth and
50 sun teeth then ratio = zr/zs = 2. The number of planet teeth
zp has to fulfill the following relationship:
</p>
<pre>
   <b>zp := (zr - zs) / 2</b>
</pre>
<p>
Therefore, in the above example zp = 25 is required.
</p>
<p>
According to the overall convention, the positive direction of all
vectors, especially the absolute angular velocities and cut-torques
in the flanges, are along the axis vector displayed in the icon.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>July 5, 1999</i>
       by <a href=\"http://www.op.dlr.de/~otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</HTML>
"), 
      Icon(
        Rectangle(extent=[0, -30; -90, -50], style(
            color=8, 
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Rectangle(extent=[90, 10; 45, -10], style(
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Text(extent=[0, 168; 0, 108], string="%name"), 
        Text(
          extent=[0, -112; 0, -163], 
          string="ratio=%ratio_a1 , %ratio_a2", 
          style(color=0)), 
        Rectangle(extent=[0, 50; -90, 30], style(
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Ellipse(extent=[-50, 50; 50, -50], style(fillColor=7)), 
        Text(extent=[-50, 50; 50, -50], string="+")), 
      Diagram(
        Polygon(points=[58, 130; 28, 140; 28, 120; 58, 130], style(
            color=10, 
            fillColor=10, 
            fillPattern=1)), 
        Line(points=[-52, 130; 28, 130], style(
            color=0, 
            fillColor=10, 
            fillPattern=1)), 
        Line(points=[139, -1; 110, -1], style(color=0, fillColor=0)), 
        Polygon(points=[139, 2; 149, -1; 139, -4; 139, 2], style(color=0, 
              fillColor=0)), 
        Text(extent=[112, 16; 148, 6], string="ring.tau"), 
        Text(extent=[114, -4; 148, -14], string="ring.phi")));
  equation 
    
    flange_b.phi = ratio_a1*flange_a1.phi + ratio_a2*flange_a2.phi;
    
    flange_b.tau = 1/ratio_a1*flange_a1.tau;
    flange_b.tau = 1/ratio_a2*flange_a2.tau;
    
  end RotationalSum;
  
  
  model Shape "Different visual shapes (box, cylinder, sphere, ...)" 
    parameter String shapeType="box" 
      "Type of shape (box, sphere, cylinder, pipecylinder, cone, pipe, beam, gearwheel, wirebox, spring)";
    input SI.Position r0[3]={0,0,0} 
      "Position vector from object frame origin to shape origin";
    input SI.Position lengthDirection[3]={1,0,0} 
      "Vector in length direction, resolved in object frame.";
    input SI.Position widthDirection[3]={0,1,0} 
      "Vector in width direction, resolved in object frame.";
    input SI.Position length=0 "length of visual object.";
    input SI.Position width=0 "Width of visual object.";
    input SI.Position height=0 "Height of visual object.";
    input Real Material[4]={1,0,0,0.5} "Color and specular coefficient.";
    input Real Extra=0.0 "Additional parameter for cone and pipe.";
    input Real S[3, 3] "Transformation matrix to inertial frame.";
    input SI.Position r[3] 
      "Position vector from origin of world frame to origin of object frame, resolved in world frame";
  protected 
    output Real Form annotation (Hide=false);
    output Real rxvisobj[3] annotation (Hide=false);
    output Real ryvisobj[3] annotation (Hide=false);
    output Real rvisobj[3] annotation (Hide=false);
    output Real size[3] annotation (Hide=false);
    output Real material annotation (Hide=false);
    output Real extra annotation (Hide=false);
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Icon(
        Rectangle(extent=[-100, -100; 80, 60], style(color=3, fillColor=7)), 
        Polygon(points=[-100, 60; -80, 100; 100, 100; 80, 60; -100, 60], style(
              color=3, fillColor=8)), 
        Polygon(points=[100, 100; 100, -60; 80, -100; 80, 60; 100, 100], style(
              color=3, fillColor=9)), 
        Text(
          extent=[-100, -100; 80, 60], 
          style(color=0), 
          string="%shapeType"), 
        Text(extent=[-132, 160; 128, 100], string="%name")), 
      Window(
        x=0.28, 
        y=0.06, 
        width=0.6, 
        height=0.69));
  equation 
    /* Outputs to file. */
    Form = (987000 + PackShape(shapeType))*1E20;
    rxvisobj = S[:, 1];
    ryvisobj = S[:, 2];
    rvisobj = r + S*r0;
    size = {length,width,height};
    material = PackMaterial(Material[1], Material[2], Material[3], Material[4]);
    extra = Extra;
  end Shape;
  
  package Joints "Special joints used in the chassis modelling." 
    import Modelica.SIunits;
    
    model JointUPS 
      "Universal - prismatic - spherical joint aggregation (no constraints and no states)"
       
      
      import SI = Modelica.SIunits;
      parameter Real n_a[3]={0,0,1} 
        "Axis of revolute joint 1 of universal joint resolved in frame_a";
      
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -15; -90, 15]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_b
        annotation (extent=[120, -15; 90, 15]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_pa
        annotation (extent=[-75, 130; -45, 100], rotation=-90);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_pb
        annotation (extent=[45, 130; 75, 100], rotation=-90);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.35, 
          y=0.31, 
          width=0.6, 
          height=0.6), 
        Documentation(info="<html>
<p>
This component consists of a universal joint at frame_a,
a spherical joint at frame_b and a prismatic joint along the
line connecting the origin of frame_a and the origin of frame_b. 
This joint introduces neither constraints nor state variables. 
The universal joint is defined in the following way:
<p>
<ul>
<li> The rotation axis of revolute joint 1 is along parameter
     vector n_a which is fixed in frame_a<li>
<li> The rotation axis of revolute joint 2 (= unit vector e_y) is perpendicular to
     n_a and to the line connecting the universal and the spherical joint
     (= unit vector e_x).
</ul>
<p>
Note, a singularity takes place when n_a = e_x, i.e., when n_a is 
along the line connecting the universal and the spherical joint. 
Therefore, if possible n_a should be selected in such a way that it
is perpendicular to e_x in the initial configuration (i.e., the 
distance to the singularity is as large as possible).
</p>
<p>
An additional <b>frame_pa</b> is present. It is defined in the
following way:
</p>
<ul>
<li> frame_pa is fixed to the \"left\" part of the prismatic joint
     (i.e., unit vectors e_x and e_y are constant vectors in freme_pa).</li>
<li> The origin of frame_pa and of frame_a are identical.</li>
<li> At the initial time, frame_pa is parallel to frame_a.</li>
</ul>
Finally, a frame_pb is present. It is defined in the following way:
</p>
<ul>
<li> The origin of frame_pb is fixed in the origin of frame_b, i.e.,
     in the spherical joint.</li>
<li> frame_pb is always parallel to frame_pa.</li>
</ul>
</html> 
"), 
        Icon(
          Text(
            extent=[-119, 134; -80, 94], 
            style(color=10), 
            string="pa"), 
          Text(
            extent=[80, 132; 119, 92], 
            style(color=10), 
            string="pb"), 
          Ellipse(extent=[-90, -40; -9, 40], style(color=9, fillColor=8)), 
          Ellipse(extent=[-80, -30; -19, 29], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-50, 42; -9, -40], style(color=7, fillColor=7)), 
          Line(points=[-50, 40; -50, -40], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-73, -17; -24, 21], style(color=9, fillColor=8)), 
          Ellipse(extent=[-64, -12; -30, 15], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-62, -20; -79, 3; -59, 25; -35, 27; -62, -20], style(
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-50, 40; -50, -10], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-108, 28; -32, 94], style(
              color=0, 
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-39, 20; -59, -15], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[11, -40; 91, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[21, -29; 81, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[11, 40; 32, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[34, 14; 63, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[11, -40; 91, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[21, -29; 81, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[1, 42; 54, -45], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[36, 14; 65, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[-25, -14; 4, 13], style(pattern=0, fillColor=8)), 
          Rectangle(extent=[-25, 13; 4, 17], style(
              pattern=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[4, -8; 37, 6], style(pattern=0, fillColor=8)), 
          Rectangle(extent=[4, 5; 38, 9], style(
              pattern=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[4, -14; 4, 17], style(color=0)), 
          Text(
            extent=[-82, 80; -51, 59], 
            style(gradient=0, fillPattern=0), 
            string="na"), 
          Line(points=[-26, 0; -26, 90; -60, 90; -60, 100], style(color=10, 
                thickness=2)), 
          Line(points=[50, 0; 50, 90; 60, 90; 60, 100], style(color=10, 
                thickness=2)), 
          Text(extent=[-136, -56; 152, -108], string="%name")), 
        Diagram(
          Ellipse(extent=[-90, -40; -9, 40], style(color=9, fillColor=8)), 
          Ellipse(extent=[-80, -30; -19, 29], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-50, 41; -9, -41], style(color=7, fillColor=7)), 
          Line(points=[-50, 40; -50, -40], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-73, -17; -24, 21], style(color=9, fillColor=8)), 
          Ellipse(extent=[-64, -12; -30, 15], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-62, -20; -79, 3; -59, 25; -35, 27; -62, -20], style(
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-50, 40; -50, -10], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-108, 28; -32, 94], style(
              color=0, 
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-39, 20; -59, -15], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[11, -40; 91, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[21, -29; 81, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[11, 40; 32, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[34, 14; 63, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[11, -40; 91, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[21, -29; 81, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[1, 42; 52, -44], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[36, 14; 65, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[-25, -14; 4, 13], style(pattern=0, fillColor=8)), 
          Rectangle(extent=[-25, 13; 4, 17], style(
              pattern=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[4, -8; 37, 6], style(pattern=0, fillColor=8)), 
          Rectangle(extent=[4, 5; 38, 9], style(
              pattern=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[4, -14; 4, 17], style(color=0)), 
          Line(points=[-26, 0; -26, 90; -60, 90; -60, 100], style(color=10, 
                thickness=2)), 
          Line(points=[-50, -1; 5, -1], style(fillColor=0, fillPattern=1)), 
          Text(
            extent=[-18, -3; 13, -24], 
            style(gradient=0, fillPattern=0), 
            string="e_x"), 
          Polygon(points=[17, -1; 5, 2; 5, -4; 17, -1], style(fillPattern=7)), 
          Line(points=[-40, 19; -20, 57], style(fillColor=0, fillPattern=1)), 
          Text(
            extent=[-10, 77; 21, 56], 
            style(gradient=0, fillPattern=0), 
            string="e_y"), 
          Polygon(points=[-15, 66; -23, 58; -17, 55; -15, 66], style(
                fillPattern=7)), 
          Line(points=[-50, 41; -50, 65], style(fillColor=0, fillPattern=1)), 
          Polygon(points=[-50, 77; -54, 65; -46, 65; -50, 77], style(
                fillPattern=7)), 
          Text(
            extent=[-86, 80; -55, 59], 
            style(gradient=0, fillPattern=0), 
            string="n_a"), 
          Text(extent=[-30, 11; 5, -1], string="frame_c"), 
          Line(points=[50, 0; 50, 90; 60, 90; 60, 100], style(color=10, 
                thickness=2))));
      
      Real S_rel[3, 3] "Transformation matrix from frame_a to frame_b";
      SI.Position r_rela[3] 
        "Vector from origin of frame_a to origin of frame_b, resolved in frame_a";
      SI.Velocity v_rela[3] "= der(r_rela)";
      SI.Acceleration a_rela[3] "= der(v_rela)";
      SI.Power P "Total power stored in this element (should be zero)";
      
    protected 
      SI.Velocity v_aux[3] "= cross(frame_a.w, r_rela)";
      SI.Length norm_r_rela "length of vector r_rel_a";
      SI.Length norm_n_y "length of vector n_y";
      Real e_x[3] 
        "Unit vector from origin of frame_a to origin of frame_b, resolved in frame_a";
      Real e_y[3] 
        "Unit vector perpendicular to n_a and to e_x, resolved in frame_a";
      Real e_z[3] 
        "Unit vector perpendicular to e_x and to e_z, resolved in frame_a";
      Real n_y[3] "Vector perpendicular to n_a and to e_x, resolved in frame_a";
      Real S_pa_a[3, 3] "Transformation matrix from frame_a to frame_pa";
      Real S_pb_b[3, 3] "Transformation matrix from frame_b to frame_pb";
      Real e_y2[3] "= e_y/norm_r_rela";
      SI.AngularVelocity w_rel_pa[3] 
        "Angular velocity of frame_pa with respect to frame_a, resolved in frame_pa";
      SI.AngularAcceleration z_rel_pa[3] 
        "Angular acceleration of frame_pa with respect to frame_a, resolved in frame_pa";
      Real W[3, 3](each unit="rad/m") "w_rel_c = W*v_rela";
      SI.Velocity v_rela_scaled[3] "= v_rela/norm_r_rela";
      Real der_e_x[3] "= der(e_x)";
      Real der_e_y[3] "= der(e_y)";
      Real der_e_z[3] "= der(e_z)";
      Real aux1[3] "= cross(n_a, e_x)";
      Real norm_aux1 "=sqrt(aux1*aux1)";
      Real aux2;
      Real aux3[3];
      SI.Torque t_cd_a[3] "= frame_pa.t + frame_pb.t, resolved in frame_a";
      SI.Force f_bd_a[3] "= frame_b.f + frame_pb.f, resolved in frame_a";
      Real WW[3, 3];
    protected 
      parameter Real S_pa_c[3, 3](each fixed=false) 
        "Transformation matrix from frame_c to frame_pa";
    initial equation 
      S_pa_c = [e_x, e_y, e_z];
    equation 
      S_rel = transpose(frame_b.S)*frame_a.S;
      r_rela = transpose(frame_a.S)*(frame_b.r0 - frame_a.r0);
      v_aux = cross(frame_a.w, r_rela);
      v_rela = transpose(S_rel)*frame_b.v - frame_a.v - v_aux;
      a_rela = transpose(S_rel)*frame_b.a - frame_a.a - cross(frame_a.z, r_rela)
         - cross(frame_a.w, v_aux + 2*v_rela);
      
      /* Compute relative transformation matrix S_pa_a (from frame_a to frame_pa) and
     absolute transformation matrices frame_pa.S, frame_pb.S
  */
      norm_r_rela = noEvent(sqrt(r_rela*r_rela));
      e_x = r_rela/norm_r_rela;
      n_y = cross(n_a, e_x);
      norm_n_y = noEvent(sqrt(n_y*n_y));
      e_y = n_y/norm_n_y;
      e_z = cross(e_x, e_y);
      S_pa_a = S_pa_c*transpose([e_x, e_y, e_z]);
      frame_pa.S = frame_a.S*transpose(S_pa_a);
      frame_pb.S = frame_pa.S;
      
      /* Compute relative angular velocity between frame_a and frame_pa
     (the derivation is long, but the result is simple)
  */
      e_y2 = e_y/norm_r_rela;
      aux1 = cross(n_a, e_x);
      norm_aux1 = noEvent(sqrt(aux1*aux1));
      aux2 = n_a*e_x;
      W = [aux2/norm_aux1*transpose([e_y2]); -transpose([e_z/norm_r_rela]); 
        transpose([e_y2])];
      WW = S_pa_c*W;
      w_rel_pa = WW*v_rela;
      frame_pa.w = S_pa_a*frame_a.w + w_rel_pa;
      frame_pb.w = frame_pa.w;
      
      /* Compute relative angular acceleration between frame_a and frame_pa
     (the derivation is long)
  */
      v_rela_scaled = v_rela/norm_r_rela;
      der_e_x = v_rela_scaled - (e_x*v_rela_scaled)*e_x;
      aux3 = cross(n_a, der_e_x)/norm_n_y;
      der_e_y = aux3 - (e_y*aux3)*e_y;
      der_e_z = cross(der_e_x, e_y) + cross(e_x, der_e_y);
      z_rel_pa = S_pa_c*(W*(a_rela - v_rela*(e_x*v_rela_scaled)) + [((n_a*
        der_e_x)*transpose([e_y]) + aux2*transpose([der_e_y]))/norm_aux1; -
        transpose([der_e_z]); transpose([der_e_y])]*v_rela_scaled);
      frame_pa.z = S_pa_a*frame_a.z + z_rel_pa + cross(frame_pa.w, w_rel_pa);
      frame_pb.z = frame_pa.z;
      
      /* Set remaining kinematic quantities of frame_pa and frame_pb */
      frame_pa.r0 = frame_a.r0;
      frame_pa.v = S_pa_a*frame_a.v;
      frame_pa.a = S_pa_a*frame_a.a;
      
      S_pb_b = transpose(frame_pb.S)*frame_b.S;
      frame_pb.r0 = frame_b.r0;
      frame_pb.v = S_pb_b*frame_b.v;
      frame_pb.a = S_pb_b*frame_b.a;
      
      /* Force and torque balance */
      t_cd_a = transpose(S_pa_a)*(frame_pa.t + frame_pb.t);
      f_bd_a = -transpose(WW)*t_cd_a;
      frame_b.f = -transpose(S_pb_b)*frame_pb.f + S_rel*f_bd_a;
      frame_b.t = zeros(3);
      frame_a.f = -transpose(S_pa_a)*frame_pa.f - f_bd_a;
      frame_a.t = -t_cd_a - cross(r_rela, f_bd_a);
      
      /* Total power (should be zero) */
      P = frame_a.f*frame_a.v + frame_a.t*frame_a.w + frame_b.f*frame_b.v + 
        frame_b.t*frame_b.w + frame_pa.f*frame_pa.v + frame_pa.t*frame_pa.w + 
        frame_pb.f*frame_pb.v + frame_pb.t*frame_pb.w;
      
    end JointUPS;
    
    model JointRSU 
      "Revolute - spherical - universal - joint aggregation (no constraints, no states)"
       
      
      import SI = Modelica.SIunits;
      
      extends ModelicaAdditions.MultiBody.Interfaces.Interact2;
      parameter Real n_a[3]={0,0,1} 
        "Axis of rotation of revolute joint resolved in frame_a";
      parameter Real n_b[3]={0,0,1} 
        "First rotation axis of universal joint fixed and resolved in frame_b";
      parameter SI.Position r_a[3] 
        "Position vector from origin of frame_a to spherical joint at initial time, resolved in frame_a";
      final parameter SI.Position rL_init0[3](each fixed=false) 
        "Position vector from spherical to universal joint at initial time, resolved in inertial system";
      final parameter SI.Position L(fixed=false) 
        "Distance between universal and spherical joint";
      SI.Power P "Total power flowing into this element (has to be zero)";
      Real c;
      
      annotation (
        Diagram, 
        Icon(
          Ellipse(extent=[40, -25; 90, 25], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[45, -19; 85, 19], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-16, -25; 34, 25], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-11, -20; 29, 20], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[13, 34; 60, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[57, 8; 73, -8], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[1, 8; 17, -8], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[-50, 7; -15, -6], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[-90, -30; -73, 30], style(
              color=8, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[-65, -30; -50, 30], style(
              color=8, 
              gradient=2, 
              fillColor=8)), 
          Text(extent=[-139, -55; 149, -107], string="%name"), 
          Line(points=[-50, 29; -50, 70; -80, 70; -80, 100], style(color=0)), 
          Line(points=[-15, 7; -15, 70; 0, 70; 0, 100], style(color=0)), 
          Line(points=[58, 3; 58, 85; 80, 85; 80, 100], style(color=0)), 
          Text(
            extent=[100, 131; 139, 91], 
            style(color=10), 
            string="c"), 
          Text(
            extent=[-151, 138; -112, 98], 
            style(color=10), 
            string="ta"), 
          Text(
            extent=[14, 131; 53, 91], 
            style(color=10), 
            string="tb"), 
          Rectangle(extent=[-73, 10; -65, -10], style(
              color=0, 
              gradient=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-90, 30; -73, -30], style(color=0)), 
          Rectangle(extent=[-65, 30; -50, -30], style(color=0)), 
          Rectangle(extent=[15, 5; 58, -5], style(gradient=2, fillColor=8))), 
        Coordsys(grid=[1, 1], component=[20, 20]), 
        Documentation(info="<html>
<p>
This joint aggregation assumes that it is used in a kinematic loop which
becomes a tree-structure when the JointRSU element is removed. In this
case JointRSU analytically computes the unknown revolute joint angle by
analytically solving a non-linear equation. As a result the kinematic
loop is analytically solved.
</p>
<p>
At the initial time, the coordinate systems
frame_ta and frame_tb are parallel to frame_a,
frame_c is parallel to frame_b and the angle of the revolute joint
(= rev.q) is zero.
</p>
</html>"));
      
      ModelicaAdditions.MultiBody.Parts.FrameTranslation frameTranslation(r=r_a)
        annotation (extent=[-20, -10; 0, 10]);
      DependentRevolute rev(
        n=n_a, 
        L=L, 
        q_guess=0) "dependent revolute joint"
        annotation (extent=[-70, -10; -50, 10]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_c
        annotation (extent=[65, 130; 95, 100], rotation=-90);
      JointUS jointUS(
        L=L, 
        n_a=n_b, 
        kinematicConstraint=false) annotation (extent=[50, -10; 30, 10]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_ta
        annotation (extent=[-94, 130; -64, 100], rotation=-90);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_tb
        annotation (extent=[-15, 130; 15, 100], rotation=-90);
    initial equation 
      rL_init0 = frame_b.r0 - (frame_a.r0 + frame_a.S*r_a);
      L = sqrt(rL_init0*rL_init0);
    equation 
      rev.relativePosition_a.signal = vector([r_rela; v_rela; a_rela]);
      rev.relativePosition_b.signal = vector([r_a; zeros(3); zeros(3)]);
      P = frame_a.f*frame_a.v + frame_a.t*frame_a.w + frame_b.f*frame_b.v + 
        frame_b.t*frame_b.w + frame_c.f*frame_c.v + frame_c.t*frame_c.w + 
        frame_ta.f*frame_ta.v + frame_ta.t*frame_ta.w + frame_tb.f*frame_tb.v
         + frame_tb.t*frame_tb.w;
      
      /* Compute force in the rod of jointUS */
      c = (cross(n_a, r_a)*(jointUS.S_rel*jointUS.nx));
      jointUS.fRod = -n_a*(frame_ta.t + frame_tb.t + cross(r_a, frame_tb.f - 
        jointUS.fb_b1))/noEvent(if abs(c) < 1.e-10 then 1.e-10 else c);
      connect(frameTranslation.frame_b, jointUS.frame_b)
        annotation (points=[0, 0; 30, 0]);
      connect(jointUS.frame_a, frame_b) annotation (points=[51, 0; 96, 0]);
      connect(jointUS.frame_c, frame_c)
        annotation (points=[43.9, 11.5; 44, 40; 80, 40; 80, 100]);
      connect(frame_ta, frameTranslation.frame_a)
        annotation (points=[-80, 115; -80, 50; -30, 50; -30, 0; -20, 0]);
      connect(frameTranslation.frame_b, frame_tb)
        annotation (points=[1, 0; 10, 0; 10, 50; 0, 50; 0, 115]);
      connect(rev.frame_a, frame_a) annotation (points=[-71, 0; -96, 0]);
      connect(rev.frame_b, frameTranslation.frame_a)
        annotation (points=[-49, 0; -20, 0]);
    end JointRSU;
    
    model JointUS "Universal - Spherical joint aggregation (one constraint)" 
      import SI = Modelica.SIunits;
      extends ModelicaAdditions.MultiBody.Interfaces.CutJoint;
      parameter SI.Length L=1 
        "length of the rod (distance origin of frame_a to origin of frame_b)";
      parameter Real n_a[3]={0,0,1} 
        "First rotation axis of universal joint resolved in frame_a";
      parameter Boolean kinematicConstraint=true 
        "|Advanced|| = false, if used together with DependentRevolute to solve loop analytically";
      SI.Force fRod "Constraint force in direction of the rod";
      SI.Power P "Total power flowing into this element (has to be zero)";
      
    protected 
      Real Sc[3, 3];
      SI.Position r0c[3];
      Real vc[3];
      Real wc[3];
      Real ac[3];
      Real zc[3];
      Real b1[3];
      Real bd[3];
      Real bdd[3];
      Real normb;
      Real bb;
      Real nx[3];
      Real ny[3];
      Real nz[3];
      Real S_relc[3, 3];
      Real nxd[3];
      Real nyd[3];
      Real nzd[3];
      SI.AngularVelocity w_relc[3];
      Real nxdd[3];
      Real nydd[3];
      Real nzdd[3];
      Real z_relc[3];
      Real constraintResidue;
      Real constraintResidue_d;
      Real constraintResidue_dd;
      SI.Torque tc_a[3] "Torque frame_c.t resolved in frame_a";
      SI.Force fb_a[3] "Force frame_b.f resolved in frame_a";
      SI.Force fb_a1[3];
      SI.Force fb_b1[3];
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.19, 
          y=0.01, 
          width=0.81, 
          height=0.81), 
        Documentation(info="<html>
<p>
This component consists of a universal joint at frame_a and
a spherical joint at frame_b that are connected together with
a rigid rod. This joint aggregation has no mass and no inertia
and introduces the constraint that the distance
between the origin of frame_a and the origin of frame_b is constant
(= parameter L). The universal joint is defined in the following way:
<p>
<ul>
<li> The rotation axis of revolute joint 1 is along parameter
     vector n_a which is fixed in frame_a<li>
<li> The rotation axis of revolute joint 2 (= unit vector e_y) is perpendicular to
     n_a and to the line connecting the universal and the spherical joint
     (= unit vector e_x).
</ul>
<p>
Note, a singularity takes place when n_a = e_x, i.e., when n_a is 
along the line connecting the universal and the spherical joint. 
Therefore, if possible n_a should be selected in such a way that it
is perpendicular to e_x in the initial configuration (i.e., the 
distance to the singularity is as large as possible).
</p>
<p>
An additional <b>frame_c</b> is present. It is defined in the
following way:
</p>
<ul>
<li> frame_c is fixed to the rod connecting the origin of frame_a and the
     origin of frame_b (i.e., unit vectors e_x and e_y are constant 
     vectors in freme_c).</li>
<li> The origin of frame_c and of frame_a are identical.</li>
<li> At the initial time, frame_c is parallel to frame_a.</li>
</ul>
<p>
Note, this joint aggregation can be used in many cases also, if
in reality a rod with spherical joints at each end are present.
Such a system has an additional degree of freedom to rotate
the rod along its axis. In practice this rotation is usually
of no interested and is mathematically removed by replacing one
of the spherical joints by a universal joint.
</p>
</html>
"), 
        Icon(
          Text(
            extent=[-128, -57; 132, -115], 
            style(color=3), 
            string="%name"), 
          Ellipse(extent=[-90, -40; -9, 40], style(color=9, fillColor=8)), 
          Ellipse(extent=[-80, -30; -19, 29], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-49, 41; 1, -44], style(color=7, fillColor=7)), 
          Line(points=[-50, 40; -50, -40], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-73, -17; -24, 21], style(color=9, fillColor=8)), 
          Ellipse(extent=[-64, -12; -30, 15], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-62, -20; -79, 3; -59, 25; -35, 27; -62, -20], style(
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-50, 40; -50, -10], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-108, 28; -32, 94], style(
              color=0, 
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-39, 20; -59, -15], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[9, -38; 89, 42], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[21, -29; 81, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[11, 40; 32, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[34, 14; 63, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[4, -39; 84, 41], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[18, -27; 78, 32], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[0, 44; 52, -44], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[36, 14; 65, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[-26, -8; 38, 8], style(
              pattern=0, 
              gradient=2, 
              fillColor=8)), 
          Line(points=[-26, -1; -26, 82; -40, 82; -40, 102], style(color=10, 
                thickness=2)), 
          Text(
            extent=[-96, 134; -57, 94], 
            style(color=10), 
            string="c"), 
          Text(
            extent=[-10, 98; 144, 60], 
            string="L=%L", 
            style(color=0))), 
        Diagram(
          Line(points=[-50, -70; 35, -70]), 
          Polygon(points=[50, -70; 35, -64; 35, -76; 50, -70], style(
              color=3, 
              fillColor=3, 
              fillPattern=1)), 
          Text(extent=[-16, -67; 24, -102], string="L"), 
          Ellipse(extent=[-90, -42; -9, 38], style(color=9, fillColor=8)), 
          Ellipse(extent=[-80, -32; -19, 27], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-50, 39; -9, -43], style(color=7, fillColor=7)), 
          Line(points=[-50, 38; -50, -42], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-73, -19; -24, 19], style(color=9, fillColor=8)), 
          Ellipse(extent=[-64, -14; -30, 13], style(
              color=9, 
              fillColor=7, 
              fillPattern=1)), 
          Polygon(points=[-62, -22; -79, 1; -59, 23; -35, 25; -62, -22], style(
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-50, 38; -50, -12], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-108, 26; -32, 92], style(
              color=0, 
              pattern=0, 
              fillColor=7, 
              fillPattern=1)), 
          Line(points=[-39, 18; -59, -17], style(
              color=0, 
              thickness=2, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[9, -40; 89, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[21, -31; 81, 28], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[11, 38; 32, -42], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[34, 12; 63, -16], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[4, -41; 84, 39], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[18, -29; 78, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[1, 40; 52, -46], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[36, 12; 65, -16], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[-26, -10; 38, 6], style(
              pattern=0, 
              gradient=2, 
              fillColor=8)), 
          Line(points=[-26, -3; -26, 80; -40, 80; -40, 100], style(color=10, 
                thickness=2)), 
          Line(points=[-50, -3; 5, -3], style(fillColor=0, fillPattern=1)), 
          Text(
            extent=[-18, -5; 13, -26], 
            style(gradient=0, fillPattern=0), 
            string="e_x"), 
          Polygon(points=[17, -3; 5, 0; 5, -6; 17, -3], style(fillPattern=7)), 
          Line(points=[-40, 17; -20, 55], style(fillColor=0, fillPattern=1)), 
          Text(
            extent=[-10, 75; 21, 54], 
            style(gradient=0, fillPattern=0), 
            string="e_y"), 
          Polygon(points=[-15, 64; -23, 56; -17, 53; -15, 64], style(
                fillPattern=7)), 
          Line(points=[-50, 39; -50, 63], style(fillColor=0, fillPattern=1)), 
          Polygon(points=[-50, 75; -54, 63; -46, 63; -50, 75], style(
                fillPattern=7)), 
          Text(
            extent=[-88, 81; -57, 60], 
            style(gradient=0, fillPattern=0), 
            string="n_a"), 
          Line(points=[-50, -42; -50, -74]), 
          Line(points=[51, -3; 51, -73])));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_c(
        S=Sc, 
        r0=r0c, 
        v=vc, 
        w=wc, 
        a=ac, 
        z=zc) annotation (extent=[-54, 130; -24, 100], rotation=-90);
    protected 
      parameter Real S_init[3, 3](each fixed=false) 
        "Transformation matrix from [e_x, e_y, e_z] to frame_a at initializiation";
    initial equation 
      S_init = [nx, ny, nz];
      // Constraint equations on position, velocity and acceleration level
    equation 
      P = frame_a.f*frame_a.v + frame_a.t*frame_a.w + frame_b.f*frame_b.v + 
        frame_b.t*frame_b.w + frame_c.f*frame_c.v + frame_c.t*frame_c.w;
      if kinematicConstraint then
        constraintResidue = (r_rela*r_rela - L*L)/2;
        constraintResidue_d = r_rela*v_rela;
        constraintResidue_dd = r_rela*a_rela + v_rela*v_rela;
        constrain(constraintResidue, constraintResidue_d, constraintResidue_dd);
        // constraintResidue = 0;
      else
        constraintResidue = 0;
        constraintResidue_d = 0;
        constraintResidue_dd = 0;
      end if;
      
      /*Compute relative transformation matrix S_relc (cut a  cut c) and
 absolute transformation matrix Sc (cut c  inertial system)*/
      nx = r_rela/L;
      b1 = cross(n_a, nx);
      normb = sqrt(b1*b1);
      ny = b1/normb;
      nz = cross(nx, ny);
      S_relc = S_init*transpose([nx, ny, nz]);
      Sc = Sa*transpose(S_relc);
      
      /*Compute relative angular velocity w_relc and absolute angular velocity wc
 (both w_relc and wc are resolved in cut-frame C):
 w_relc = vec(S_relc*der(S_relc')), where h=vec([  0, -h3,  h2
                                               h3,   0, -h1
                                              -h2,  h1,   0 ])
 nxd = der(nx), nxdd = der(nxd)
 nyd = der(ny), nydd = der(nyd)
 nzd = der(nz), nzdd = der(nzd)
 nyd    = der( b/normb )
        = der(b)/normb + b*der(1/normb)
        = der(b)/normb - b*( ny'*der(b)/normb ) / normb
        = der(b)/normb - ( ny'*der(b)/normb )*ny
        = bd - (ny'*bd)*ny
*/
      nxd = v_rela/L;
      bd = cross(n_a, nxd)/normb;
      bb = ny*bd;
      nyd = bd - bb*ny;
      nzd = cross(nxd, ny) + cross(nx, nyd);
      w_relc = S_init*{nz*nyd,nx*nzd,ny*nxd};
      wc = S_relc*wa + w_relc;
      
      /*Compute relative angular acceleration z_relc and absolute angular
 acceleration zc, with z_relc = der(w_relc).
 der(bd)  = cross(n_a,nxdd)/normb - cross(n_a,nxd)*der(1/normb)
          = cross(n_a,nxdd)/normb + cross(n_a,nxd)*( ny'*der(b)/normb ) / normb
          = cross(n_a,nxdd)/normb + bd*(ny'*bd)
 der(nyd) = der(bd) - der(bb)*ny - bb*der(ny)
          = der(bd) - (nyd'*bd + ny'*der(bd))*ny - bb*nyd
*/
      nxdd = a_rela/L;
      bdd = cross(n_a, nxdd)/normb - bb*bd;
      nydd = bdd - (nyd*bd + ny*bdd)*ny - bb*nyd;
      
      nzdd = cross(nxdd, ny) + cross(nx, nydd) + 2*cross(nxd, nyd);
      z_relc = S_init*{nz*nydd + nzd*nyd,nx*nzdd + nxd*nzd,ny*nxdd + nyd*nxd};
      zc = S_relc*za + cross(wc, w_relc) + z_relc;
      
      /*Compute translational quantities of cut c*/
      r0c = r0a;
      vc = S_relc*va;
      ac = S_relc*aa;
      
      /*Compute cut-forces/torques at frame_a/b */
      tc_a = transpose(S_relc)*frame_c.t;
      fb_a1 = -((tc_a*n_a)/(L*normb))*ny + ((tc_a*ny)/L)*nz;
      fb_b1 = S_rel*fb_a1;
      fb_a = fb_a1 - fRod*nx;
      frame_b.f = S_rel*fb_a;
      frame_a.f = -fb_a - transpose(S_relc)*frame_c.f;
      frame_b.t = zeros(3);
      frame_a.t = -tc_a - cross(r_rela, fb_a);
    end JointUS;
    
    model DependentRevolute 
      "Revolute joint where the rotation angle is computed from a length constraint"
       
      
      import Modelica.Math.*;
      extends ModelicaAdditions.MultiBody.Interfaces.TreeJoint;
      parameter Real n[3]={0,0,1} 
        "Axis of rotation resolved in frame_a (= same as in frame_b)";
      parameter Real q0=0 "Rotation angle offset (see info) [deg]";
      parameter SI.Position L=1 "length of connecting rod";
      parameter Real q_guess=0 "Guess for q at initial time in [deg]";
      
      final parameter Boolean positiveBranch(fixed=false) 
        "Based on q_guess, selection of one of the two solutions";
      SI.Position r1_a[3] 
        "Vector from frame_a to ConnectingRod frame_a, resolved in frame_a";
      SI.Position r1_b[3];
      SI.Position r2_b[3] 
        "Vector from frame_b to ConnectingRod frame_b, resolved in frame_b";
      SI.Position nr1;
      SI.Position nr2;
      
      // Constraint equation: A*cos(phi) + B*sin(phi) + C = 0
      SI.Length A;
      SI.Length B;
      SI.Length C;
      Real k1;
      Real k2;
      
      SI.Angle q(stateSelect=StateSelect.never);
      SI.AngularVelocity qd(stateSelect=StateSelect.never);
      SI.AngularAcceleration qdd;
      Real q_deg;
      Real nn[3];
      Real sinq;
      Real cosq;
      
      //Real LL;
      //Real delta;
      // Real qd2;
      Real v1_a[3];
      Real v1_b[3];
      Real v2_b[3];
      SI.Position r_diff_b[3];
      SI.Velocity v_diff_b[3];
      Real a1_a[3];
      Real a2_b[3];
      Real der_r1_b[3];
      Real der_r_diff_b[3];
      Real der_v_diff_b[3];
      Real k3[3];
      Real k4;
      Real k4a;
      Real qd2;
      Real qdd2;
      
      Real constraintResidue;
      Real constraintResidue_d;
      Real constraintResidue_dd;
      
      Modelica.Mechanics.Rotational.Interfaces.Flange_a axis
        annotation (extent=[10, 80; -10, 60]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing
        annotation (extent=[-70, 80; -50, 60]);
      Modelica.Blocks.Interfaces.InPort relativePosition_a(n=9)
        annotation (extent=[-120, -74; -90, -45], layer="icon");
      Modelica.Blocks.Interfaces.InPort relativePosition_b(n=9)
        annotation (extent=[120, -45; 90, -74], layer="icon");
      function selectBranch 
        "determine branch which is closest to initial angle q_guess" 
        
        import Modelica.Math.*;
        input SI.Position L "length of connetion rod";
        input Real q_guess "guess of initial angle in [deg]";
        input Real n[3] 
          "unit vector along axis of rotation, resolved in frame_a/frame_b";
        input SI.Position r1[3] 
          "position vector from frame_a to connectingRod, resolved in frame_a";
        input SI.Position r2[3] 
          "position vector from frame_b to connectingRod, resolved in frame_b";
        output Boolean positiveBranch "branch of the initial solution";
      protected 
        Real nr1;
        Real nr2;
        Real A;
        Real B;
        Real C;
        Real k1;
        Real k2;
        Real cosq1;
        Real sinq1;
        Real cosq2;
        Real sinq2;
        Real q1;
        Real q2;
      algorithm 
        nr1 := n*r1;
        nr2 := n*r2;
        A := -2*(r2*r1 - nr1*nr2);
        B := 2*r2*cross(n, r1);
        C := r1*r1 + r2*r2 - L*L - 2*nr1*nr2;
        
        k1 := A*A + B*B;
        k2 := sqrt(k1 - C*C);
        
        cosq1 := (-A*C + B*k2)/k1;
        sinq1 := (-B*C - A*k2)/k1;
        q1 := atan2(sinq1, cosq1);
        
        cosq2 := (-A*C - B*k2)/k1;
        sinq2 := (-B*C + A*k2)/k1;
        q2 := atan2(sinq2, cosq2);
        
        if abs(q1 - q_guess) <= abs(q2 - q_guess) then
          positiveBranch := true;
        else
          positiveBranch := false;
        end if;
      end selectBranch;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.09, 
          y=0, 
          width=0.83, 
          height=0.96), 
        Documentation(info="<HTML>
<p>
Joint where frame_b rotates around axis n which is fixed in frame_a.
The joint axis has an additional flange where it can be
driven with elements of the Modelica.Mechanics.Rotational library.
The relative angle q [rad] and the relative angular velocity
qd [rad/s] are used as state variables.
</p>
<p>
The following parameters are used to define the joint:
</p>
<pre>
  n : Axis of rotation resolved in frame_a (= same as in frame_b).
      n  must not necessarily be a unit vector. E.g.,
         n = {0, 0, 1} or n = {1, 0, 1}
  q0: Rotation angle offset in [deg].
      If q=q0, frame_a and frame_b are identical.
  startValueFixed: true, if start values of q, qd are fixed.
</pre>
</HTML>
"), 
        Icon(
          Rectangle(extent=[-89, -60; -29, 60], style(
              color=8, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[30, -60; 90, 60], style(
              color=8, 
              gradient=2, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-29, 10; 11, -10], style(
              color=0, 
              gradient=0, 
              fillColor=8, 
              fillPattern=1)), 
          Polygon(points=[10, 30; 30, 50; 30, -50; 10, -30; 10, 30], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Text(extent=[-100, -138; 100, -79], string="%name=%n"), 
          Line(points=[-20, 70; -50, 70], style(color=0)), 
          Line(points=[-20, 80; -20, 60], style(color=0)), 
          Line(points=[20, 80; 20, 60], style(color=0)), 
          Line(points=[20, 70; 41, 70], style(color=0)), 
          Rectangle(extent=[-10, 60; 10, 50], style(
              color=8, 
              gradient=1, 
              fillColor=8)), 
          Polygon(points=[-10, 30; 10, 30; 30, 50; -30, 50; -10, 30], style(
              color=0, 
              fillColor=8, 
              fillPattern=1))), 
        Diagram(
          Rectangle(extent=[-90, -60; -30, 60], style(
              color=8, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[30, -60; 90, 60], style(
              color=8, 
              gradient=2, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-30, 10; 10, -10], style(
              color=0, 
              gradient=0, 
              fillColor=8, 
              fillPattern=1)), 
          Polygon(points=[10, 30; 30, 50; 30, -50; 10, -30; 10, 30], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Line(points=[-20, 70; -50, 70], style(color=0)), 
          Line(points=[-20, 80; -20, 60], style(color=0)), 
          Line(points=[20, 80; 20, 60], style(color=0)), 
          Line(points=[20, 70; 41, 70], style(color=0)), 
          Polygon(points=[-10, 30; 10, 30; 30, 50; -30, 50; -10, 30], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-10, 60; 10, 50], style(
              color=8, 
              gradient=1, 
              fillColor=8))));
      
    initial equation 
      positiveBranch = selectBranch(L, q_guess, nn, r1_a, r2_b);
    equation 
      axis.phi = q;
      bearing.phi = 0;
      
      /*rotation matrix*/
      nn = n/sqrt(n*n);
      S_rel = [nn]*transpose([nn]) + (identity(3) - [nn]*transpose([nn]))*cosq
         - skew(nn)*sinq;
      
      /*other kinematic quantities*/
      r_rela = zeros(3);
      v_rela = zeros(3);
      a_rela = zeros(3);
      w_rela = nn*qd2;
      z_rela = nn*qdd2;
      
      /* Transform the kinematic quantities from frame_a to frame_b and the
     force and torque acting at frame_b to frame_a
     (= general equations of a "TreeJoint" specialized to this class).
  */
      Sb = Sa*transpose(S_rel);
      r0b = r0a;
      
      vb = S_rel*va;
      wb = S_rel*(wa + w_rela);
      
      ab = S_rel*aa;
      zb = S_rel*(za + z_rela + cross(wa, w_rela));
      
      fa = transpose(S_rel)*fb;
      ta = transpose(S_rel)*tb;
      
      // d'Alemberts principle
      // axis.tau = nn*tb;
      
      // Loop close conditions
      r1_a = relativePosition_a.signal[1:3];
      r2_b = relativePosition_b.signal[1:3];
      
      // L = sqrt((S_rel*r1_a - r2_b)*(S_rel*r1_a - r2_b))
      nr1 = nn*r1_a;
      nr2 = nn*r2_b;
      A = -2*(r2_b*r1_a - nr1*nr2);
      B = 2*r2_b*cross(nn, r1_a);
      C = r1_a*r1_a + r2_b*r2_b - L*L - 2*nr1*nr2;
      
      k1 = A*A + B*B;
      k2 = noEvent(sqrt(k1 - C*C));
      
      cosq = (-A*C + (if positiveBranch then B else -B)*k2)/k1;
      sinq = (-B*C + (if positiveBranch then -A else A)*k2)/k1;
      
      //delta = A*cosq + B*sinq + C;
      
      //q = atan2(sinqq, cosqq);
      //q = atan(sinq/cosq);
      
      v1_a = relativePosition_a.signal[4:6];
      v2_b = relativePosition_b.signal[4:6];
      r1_b = S_rel*r1_a;
      v1_b = S_rel*v1_a;
      r_diff_b = r1_b - r2_b;
      v_diff_b = v1_b - v2_b;
      k4a = r_diff_b*cross(nn, r1_b);
      k4 = noEvent(if abs(k4a) > 1.e-10 then k4a else 1.e-10);
      qd2 = r_diff_b*v_diff_b/k4;
      
      a1_a = relativePosition_a.signal[7:9];
      a2_b = relativePosition_b.signal[7:9];
      k3 = qd*cross(nn, r1_b);
      
      der_r1_b = v1_b - k3;
      der_r_diff_b = der_r1_b - v2_b;
      der_v_diff_b = S_rel*a1_a - qd*cross(nn, v1_b) - a2_b;
      
      qdd2 = (der_r_diff_b*(v_diff_b - qd*cross(nn, r1_b)) + r_diff_b*(
        der_v_diff_b - qd*cross(nn, der_r1_b)))/k4;
      
      qd = der(q);
      qdd = der(qd);
      q_deg = q*180/Modelica.Constants.pi;
      constraintResidue = q - atan2(sinq, cosq);
      constraintResidue_d = qd - qd2;
      constraintResidue_dd = qdd - qdd2;
      constrain(constraintResidue, constraintResidue_d, constraintResidue_dd);
    end DependentRevolute;
    
    model RelDistance2 
      "Distance between flange_a and flange_b is converted to translational cuts."
       
      
      import SI = Modelica.SIunits;
      parameter SI.Position s0=0 
        "Relative distance offset of translational joint (distance frame_a -> frame_b = s0 + axis.s)";
      extends ModelicaAdditions.MultiBody.Interfaces.LineForce;
      
      SI.Position res1;
      SI.Position res2;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.35, 
          y=0.31, 
          width=0.6, 
          height=0.6), 
        Documentation(info=" 
"), 
        Icon(
          Text(extent=[-100, 80; 98, 138], string="%name"), 
          Ellipse(extent=[-90, -40; -10, 40], style(
              color=8, 
              pattern=0, 
              gradient=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-80, -30; -20, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[10, -40; 90, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[20, -29; 80, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[10, 40; 31, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-64, 15; -35, -13], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[33, 14; 62, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-50, 3; -50, -90], style(color=0, thickness=4)), 
          Ellipse(extent=[-80, -30; -20, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-39, 40; 1, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[10, -40; 90, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[20, -29; 80, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-1, 40; 31, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-64, 15; -35, -13], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[33, 14; 62, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[50, 4; 50, -89], style(color=0, thickness=4)), 
          Rectangle(extent=[-31, 3; -21, -5], style(gradient=2, fillColor=8)), 
          Line(points=[-50, 3; -50, -90], style(color=0, thickness=4)), 
          Rectangle(extent=[-14, 3; -4, -5], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[3, 3; 13, -5], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[19, 3; 29, -5], style(gradient=2, fillColor=8))), 
        Diagram);
      
      Modelica.Mechanics.Translational.Interfaces.Flange_b bearing
        annotation (extent=[-40, -90; -60, -110], layer="icon");
      Modelica.Mechanics.Translational.Interfaces.Flange_a axis
        annotation (extent=[39, -90; 60, -111], layer="icon");
    equation 
      res1 = axis.s + s0 - s;
      res2 = der(axis.s) - sd;
      constrain(res1, res2);
      bearing.s = 0;
      axis.f = -f;
    end RelDistance2;
    
    model FreeMotion 
      "Free motion joint (6 degrees-of-freedom, used in spanning tree)" 
      
      parameter Boolean enforceStates=true 
        "|Advanced||if true, then all StateSelect.always is set for all 6 DOF";
      
      import SI = Modelica.SIunits;
      extends ModelicaAdditions.MultiBody.Interfaces.TreeJoint(r_rela(
            stateSelect=if enforceStates then StateSelect.always else 
              StateSelect.prefer), v_rela(stateSelect=if enforceStates then 
              StateSelect.always else StateSelect.prefer));
      SI.Angle phi[3](stateSelect=if enforceStates then StateSelect.always
             else StateSelect.prefer) 
        "Cardan angles from a frame fixed in frame_a to frame_b";
      SI.AngularVelocity phi_d[3](stateSelect=if enforceStates then StateSelect
            .always else StateSelect.prefer);
      SI.Velocity vaux[3];
      Real s1;
      Real s2;
      Real s3;
      Real c1;
      Real c2;
      Real c3;
      SI.AngularAcceleration phi_dd[3];
      SI.AngularVelocity phi_d12;
      SI.AngularVelocity phi_d13;
      SI.AngularVelocity phi_d23;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.16, 
          y=0, 
          width=0.65, 
          height=0.79), 
        Documentation(info="<HTML> 
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is just used to define the desired states to be used.
The joint is realized in 
such a way, that a singularity cannot occur. This is achieved because
the Cardan angles are defined between a frame_fix fixed in frame_a and
frame_b. Whenever the Cardan angles are near a singularity, the
integration is stopped and frame_fix is changed, such that the Cardan
angles are far away from the singularity. The following state
variables are used:
</p>
<pre>
  r_rela[3]: Distance vector from the origin of frame_a to the origin
             of frame_b, resolved in frame_a in [m].
  phi[3]   : Cardan angles, also called Tait-Bryan angles, i.e.,
             rotate around 1-, 2-, 3-axis in [rad] from intermediate
             frame_fix, which is fixed in frame_a, to frame_b. Initially, frame_fix
             is identical to frame_a. If phi[2] is near its singularity (= pi/2 or -pi/2),
             the frame_fix and phi are changed, such that phi[2] is far away from 
             its singularity.
  v_rela[3]: = der(r_rela); relative velocity of frame_b with respect to frame_a
             resolved in frame_a in [m/s].
  w_rela[3]: Relative angular velocity of frame_b with respect to frame_a
             resolved in frame_a in [rad/s].
</pre>
</HTML>
"), 
        Icon(
          Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 
                48], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0, 
                fillColor=8)), 
          Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0, 
                fillColor=8)), 
          Text(
            extent=[100, -40; -100, -100], 
            string="%name", 
            style(color=3)), 
          Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8))), 
        Diagram(
          Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 
                48], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0, 
                fillColor=8)), 
          Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0, 
                fillColor=8)), 
          Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8))));
    equation 
      
      /* Determine sines and cosines of the Cardan angles */
      s1 = Modelica.Math.sin(phi[1]);
      s2 = Modelica.Math.sin(phi[2]);
      s3 = Modelica.Math.sin(phi[3]);
      c1 = Modelica.Math.cos(phi[1]);
      c2 = Modelica.Math.cos(phi[2]);
      c3 = Modelica.Math.cos(phi[3]);
      
      /* Relative transformation matrix
       S_phi = [ c3, s3, 0; 
                -s3, c3, 0; 
                  0, 0, 1]*[c2, 0, -s2; 
                             0, 1, 0; 
                            s2, 0, c2]*[1, 0, 0; 
                                        0, c1, s1; 
                                        0, -s1, c1];
  */
      S_rel = [c2*c3, c1*s3 + s1*s2*c3, s1*s3 - c1*s2*c3; -c2*s3, c1*c3 - s1*s2
        *s3, s1*c3 + c1*s2*s3; s2, -s1*c2, c1*c2];
      
      // Kinematic differential equations for translational motion
      der(r_rela) = v_rela;
      der(v_rela) = a_rela;
      
      // Kinematic differential equations for rotational motion      
      der(phi) = phi_d;
      der(phi_d) = phi_dd;
      
      /*
  phi_d = {w_rela[1] + (s1*w_rela[2] - c1*w_rela[3])*s2/c2,c1*w_rela[2] + s1*
    w_rela[3],(-s1*w_rela[2] + c1*w_rela[3])/c2};
*/
      w_rela = {phi_d[1] + s2*phi_d[3],c1*phi_d[2] - s1*c2*phi_d[3],s1*phi_d[2]
         + c1*c2*phi_d[3]};
      
      phi_d12 = phi_d[1]*phi_d[2];
      phi_d13 = phi_d[1]*phi_d[3];
      phi_d23 = phi_d[2]*phi_d[3];
      z_rela = {phi_dd[1] + s2*phi_dd[3] + c2*phi_d23,c1*phi_dd[2] - s1*c2*
        phi_dd[3] - s1*phi_d12 - c1*c2*phi_d13 + s1*s2*phi_d23,s1*phi_dd[2] + 
        c1*c2*phi_dd[3] + c1*phi_d12 - s1*c2*phi_d13 - c1*s2*phi_d23};
      
      // Kinematic relationships
      frame_b.S = frame_a.S*transpose(S_rel);
      frame_b.r0 = frame_a.r0 + frame_a.S*r_rela;
      
      vaux = cross(frame_a.w, r_rela);
      frame_b.v = S_rel*(frame_a.v + v_rela + vaux);
      frame_b.w = S_rel*(frame_a.w + w_rela);
      
      frame_b.a = S_rel*(frame_a.a + cross(frame_a.z, r_rela) + cross(frame_a.w, 
         vaux + 2*v_rela) + a_rela);
      frame_b.z = S_rel*(frame_a.z + cross(frame_a.w, w_rela) + z_rela);
      
      // cut-forces and cut-torques are zero
      frame_a.f = zeros(3);
      frame_a.t = zeros(3);
      frame_b.f = zeros(3);
      frame_b.t = zeros(3);
    end FreeMotion;
    
    model FreeMotion4 
      "Free motion joint (6 degrees-of-freedom, used in spanning tree)" 
      
      import SI = Modelica.SIunits;
      SI.Position r_rela[3](each stateSelect=StateSelect.always) = {t_x.q,t_y.q,
        t_z.q} "Position vector from frame_a to frame_b, resolved in frame_a";
      SI.Velocity v_rela[3](each stateSelect=StateSelect.always) = {t_x.qd,t_y.
        qd,t_z.qd} "= der(r_rela)";
      SI.Angle phi[3](each stateSelect=StateSelect.always) = {r_x.q,r_y.q,r_z.q}
         "Cardan angles from frame_a to frame_b";
      SI.AngularVelocity w_rela[3](each stateSelect=StateSelect.always) = {r_x.
        qd,r_y.qd,r_z.qd} "= der(phi)";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.16, 
          y=0, 
          width=0.65, 
          height=0.79), 
        Documentation(info="<HTML> 
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is just used to define the desired states to be used.
The joint is realized in 
such a way, that a singularity cannot occur. This is achieved because
the Cardan angles are defined between a frame_fix fixed in frame_a and
frame_b. Whenever the Cardan angles are near a singularity, the
integration is stopped and frame_fix is changed, such that the Cardan
angles are far away from the singularity. The following state
variables are used:
</p>
<pre>
  r_rela[3]: Distance vector from the origin of frame_a to the origin
             of frame_b, resolved in frame_a in [m].
  phi[3]   : Cardan angles, also called Tait-Bryan angles, i.e.,
             rotate around 1-, 2-, 3-axis in [rad] from intermediate
             frame_fix, which is fixed in frame_a, to frame_b. Initially, frame_fix
             is identical to frame_a. If phi[2] is near its singularity (= pi/2 or -pi/2),
             the frame_fix and phi are changed, such that phi[2] is far away from 
             its singularity.
  v_rela[3]: = der(r_rela); relative velocity of frame_b with respect to frame_a
             resolved in frame_a in [m/s].
  w_rela[3]: Relative angular velocity of frame_b with respect to frame_a
             resolved in frame_a in [rad/s].
</pre>
</HTML>
"), 
        Icon(
          Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 
                48], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0, 
                fillColor=8)), 
          Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0, 
                fillColor=8)), 
          Text(
            extent=[100, -40; -100, -100], 
            string="%name", 
            style(color=3)), 
          Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8))), 
        Diagram);
      
      ModelicaAdditions.MultiBody.Joints.Revolute r_y(n={0,1,0})
        annotation (extent=[30, 10; 50, 30], rotation=90);
      ModelicaAdditions.MultiBody.Joints.Revolute r_z(n={0,0,1})
        annotation (extent=[50, 40; 70, 60], rotation=0);
      ModelicaAdditions.MultiBody.Joints.Revolute r_x(n={1,0,0})
        annotation (extent=[3, -10; 23, 10], rotation=0);
      ModelicaAdditions.MultiBody.Joints.Prismatic t_y(n={0,1,0})
        annotation (extent=[-50, 10; -30, 30], rotation=90);
      ModelicaAdditions.MultiBody.Joints.Prismatic t_x
        annotation (extent=[-80, -10; -60, 10]);
      ModelicaAdditions.MultiBody.Joints.Prismatic t_z(n={0,0,1})
        annotation (extent=[-30, 40; -10, 60]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -15; -90, 15]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_b
        annotation (extent=[90, -15; 120, 15]);
    equation 
      
      connect(t_x.frame_a, frame_a) annotation (points=[-80, 0; -95, 0]);
      connect(t_x.frame_b, t_y.frame_a)
        annotation (points=[-59, 0; -40, 0; -40, 9]);
      connect(t_y.frame_b, t_z.frame_a)
        annotation (points=[-40, 31; -40, 50; -31, 50]);
      connect(t_z.frame_b, r_x.frame_a)
        annotation (points=[-9, 50; -4, 50; -4, 0; 2.5, 0]);
      connect(r_x.frame_b, r_y.frame_a)
        annotation (points=[23.5, 0; 40, 0; 40, 10]);
      connect(r_y.frame_b, r_z.frame_a)
        annotation (points=[40, 29.5; 40, 50; 49, 50]);
      connect(r_z.frame_b, frame_b)
        annotation (points=[70, 50; 80, 50; 80, 0; 93, 0]);
    end FreeMotion4;
    
    model FreeMotion3 
      "Free motion joint (6 degrees-of-freedom, used in spanning tree)" 
      
      
        // S_rel needs a correct start value, because pre(S_rel) is referenced below
      extends ModelicaAdditions.MultiBody.Interfaces.TreeJoint(S_rel(start=
              identity(3)));
      SI.Angle phi[3] "Cardan angles from a frame fixed in frame_a to frame_b";
    protected 
      constant SI.Angle phi2_critical_deg=80 
        "angle in [deg] too close to singularity. Redefine S_fix and phi";
      constant SI.Angle phi2_critical=phi2_critical_deg*Modelica.Constants.pi/
          180.0;
      constant Real c2_small=1.e-5 
        "if cos(phi[2]) < c2_small, c2_small is used as guard against zero division";
      SI.Velocity vaux[3];
      Real s1;
      Real s2;
      Real s3;
      Real c1;
      Real c2;
      Real c2a;
      Real c3;
      Boolean switch_state;
      Real S_phi[3, 3] "S_rel = S_phi(phi)*S_fix";
      Real S_fix[3, 3](start=identity(3)) "S_rel = S_phi(phi)*S_fix";
      SI.AngularVelocity w_fix[3] 
        "Relative angular velocity resolved in intermediate frame S_fix";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.16, 
          y=0, 
          width=0.65, 
          height=0.79), 
        Documentation(info="<HTML> 
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is just used to define the desired states to be used.
The joint is realized in 
such a way, that a singularity cannot occur. This is achieved because
the Cardan angles are defined between a frame_fix fixed in frame_a and
frame_b. Whenever the Cardan angles are near a singularity, the
integration is stopped and frame_fix is changed, such that the Cardan
angles are far away from the singularity. The following state
variables are used:
</p>
<pre>
  r_rela[3]: Distance vector from the origin of frame_a to the origin
             of frame_b, resolved in frame_a in [m].
  phi[3]   : Cardan angles, also called Tait-Bryan angles, i.e.,
             rotate around 1-, 2-, 3-axis in [rad] from intermediate
             frame_fix, which is fixed in frame_a, to frame_b. Initially, frame_fix
             is identical to frame_a. If phi[2] is near its singularity (= pi/2 or -pi/2),
             the frame_fix and phi are changed, such that phi[2] is far away from 
             its singularity.
  v_rela[3]: = der(r_rela); relative velocity of frame_b with respect to frame_a
             resolved in frame_a in [m/s].
  w_rela[3]: Relative angular velocity of frame_b with respect to frame_a
             resolved in frame_a in [rad/s].
</pre>
</HTML>
"), 
        Icon(
          Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 
                48], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0, 
                fillColor=8)), 
          Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0, 
                fillColor=8)), 
          Text(
            extent=[100, -40; -100, -100], 
            string="%name", 
            style(color=3)), 
          Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8)), 
          Text(extent=[-76, 26; 53, -26], string="a,z=0")), 
        Diagram(
          Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 
                48], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0, 
                fillColor=8)), 
          Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0, 
                fillColor=8)), 
          Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8))));
    equation 
      
      /* Determine sines and cosines of the Cardan angles */
      s1 = Modelica.Math.sin(phi[1]);
      s2 = Modelica.Math.sin(phi[2]);
      s3 = Modelica.Math.sin(phi[3]);
      c1 = Modelica.Math.cos(phi[1]);
      c2a = Modelica.Math.cos(phi[2]);
      c3 = Modelica.Math.cos(phi[3]);
      
      /* Below, some expressions are divided by c2. By construction, it is not possible
     that c2=0, during continuous simulation. However, at initial time and when
     large numerical errors occur, c2=0 is possible, which would result in a division
     by zero. The following statement is a guard against this unlikely situation.
  */
      c2 = if noEvent(c2a > c2_small or c2a < -c2_small) then c2a else if 
        noEvent(c2a >= 0) then c2_small else -c2_small;
      
      /* Relative transformation matrix
       S_phi = [ c3, s3, 0; 
                -s3, c3, 0; 
                  0, 0, 1]*[c2, 0, -s2; 
                             0, 1, 0; 
                            s2, 0, c2]*[1, 0, 0; 
                                        0, c1, s1; 
                                        0, -s1, c1];
  */
      switch_state = phi[2] >= phi2_critical or phi[2] <= -phi2_critical;
      S_fix = identity(3);
      /*when switch_state then
    S_fix = pre(S_rel);
    reinit(phi, zeros(3));
  end when;
  */
      S_phi = [c2*c3, c1*s3 + s1*s2*c3, s1*s3 - c1*s2*c3; -c2*s3, c1*c3 - s1*s2
        *s3, s1*c3 + c1*s2*s3; s2, -s1*c2, c1*c2];
      S_rel = S_phi*S_fix;
      
      // Kinematic differential equations for translational motion
      der(r_rela) = v_rela;
      //der(v_rela) = a_rela;
      a_rela = zeros(3);
      // Kinematic differential equations for rotational motion    
      w_fix = S_fix*w_rela;
      der(phi) = {w_fix[1] + (s1*w_fix[2] - c1*w_fix[3])*s2/c2,c1*w_fix[2] + s1
        *w_fix[3],(-s1*w_fix[2] + c1*w_fix[3])/c2};
      //der(w_rela) = z_rela;
      z_rela = zeros(3);
      
      // Kinematic relationships
      frame_b.S = frame_a.S*transpose(S_rel);
      frame_b.r0 = frame_a.r0 + frame_a.S*r_rela;
      
      vaux = cross(frame_a.w, r_rela);
      frame_b.v = S_rel*(frame_a.v + v_rela + vaux);
      frame_b.w = S_rel*(frame_a.w + w_rela);
      
      frame_b.a = S_rel*(frame_a.a + cross(frame_a.z, r_rela) + cross(frame_a.w, 
         vaux + 2*v_rela) + a_rela);
      frame_b.z = S_rel*(frame_a.z + cross(frame_a.w, w_rela) + z_rela);
      
      // cut-forces and cut-torques are zero
      frame_a.f = zeros(3);
      frame_a.t = zeros(3);
      frame_b.f = zeros(3);
      frame_b.t = zeros(3);
    end FreeMotion3;
    
    model Spherical 
      "Spherical joint described by three Cardan angles (3 degrees-of-freedom, used in spanning tree)"
       
      
      import Modelica.Math.*;
      
      
        // S_rel needs a correct start value, because pre(S_rel) is referenced below
      extends ModelicaAdditions.MultiBody.Interfaces.TreeJoint;
      //(S_rel(start=identity(3)));
      SI.Angle phi[3](stateSelect=StateSelect.never) 
        "Cardan angles from a frame fixed in frame_a to frame_b";
      
    protected 
      constant SI.Angle phi2_critical_deg=80 
        "angle in [deg] too close to singularity. Redefine S_fix and phi";
      constant SI.Angle phi2_critical=phi2_critical_deg*Modelica.Constants.pi/
          180.0;
      constant Real c2_small=1.e-5 
        "if cos(phi[2]) < c2_small, c2_small is used as guard against zero division";
      Real s1;
      Real s2;
      Real s3;
      Real c1;
      Real c2;
      Real c2a;
      Real c3;
      //Boolean switch_state;
      Real S_phi[3, 3] "S_rel = S_phi(phi)*S_fix";
      Real S_fix[3, 3];
      //(start=identity(3)) "S_rel = S_phi(phi)*S_fix";
      SI.AngularVelocity w_fix[3] 
        "Relative angular velocity resolved in intermediate frame S_fix";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.02, 
          y=0.16, 
          width=0.65, 
          height=0.79), 
        Documentation(info="<HTML>
<p>
Joint where the origins of frame_a and frame_b always coincide, and
the frames are rotating against each other. The joint is realized in
such a way, that a singularity cannot occur. This is achieved because
the Cardan angles are defined between a frame_fix fixed in frame_a and
frame_b. Whenever the Cardan angles are near a singularity, the
integration is stopped and frame_fix is changed, such that the Cardan
angles are far away from the singularity. The following state
variables are used:
</p>
<pre>
  phi[3]   : Cardan angles, also called Tait-Bryan angles, i.e.,
             rotate around 1-, 2-, 3-axis in [rad] from intermediate
             frame_fix, which is fixed in frame_a, to frame_b. Initially, frame_fix
             is identical to frame_a. If phi[2] is near its singularity (= pi/2 or -pi/2),
             the frame_fix and phi are changed, such that phi[2] is far away from
             its singularity.
  w_rela[3]: Relative angular velocity of frame_b with respect to frame_a
             resolved in frame_a in [rad/s].
</pre>
</HTML>
"), 
        Icon(
          Ellipse(extent=[-60, -60; 60, 60], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-43, -47; 47, 43], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-25, 25; 25, -25], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Text(extent=[-100, -140; 100, -80], string="%name"), 
          Rectangle(extent=[25, 60; 60, -60], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[22, 10; 90, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-90, 10; -60, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8, 
              fillPattern=1))), 
        Diagram(
          Ellipse(extent=[-60, -60; 60, 60], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-43, -47; 47, 43], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-25, 25; 25, -25], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Rectangle(extent=[25, 60; 60, -60], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[22, 10; 90, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8, 
              fillPattern=1)), 
          Rectangle(extent=[-90, 10; -60, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8, 
              fillPattern=1))));
    equation 
      
      /* Determine sines and cosines of the Cardan angles */
      s1 = sin(phi[1]);
      s2 = sin(phi[2]);
      s3 = sin(phi[3]);
      c1 = cos(phi[1]);
      c2a = cos(phi[2]);
      c3 = cos(phi[3]);
      
      /* Below, some expressions are divided by c2. By construction, it is not possible
     that c2=0, during continuous simulation. However, at initial time and when
     large numerical errors occur, c2=0 is possible, which would result in a division
     by zero. The following statement is a guard against this unlikely situation.
  */
      c2 = c2a;
      
      //if noEvent(c2a > c2_small or c2a < -c2_small) then c2a else if noEvent(
      //    c2a >= 0) then c2_small else -c2_small;
      
      /* Relative transformation matrix
       S_phi = [ c3, s3, 0;
                -s3, c3, 0;
                  0, 0, 1]*[c2, 0, -s2;
                             0, 1, 0;
                            s2, 0, c2]*[1, 0, 0;
                                        0, c1, s1;
                                        0, -s1, c1];
  */
      //switch_state = phi[2] >= phi2_critical or phi[2] <= -phi2_critical;
      /*when switch_state then
    S_fix = pre(S_rel);
    reinit(phi, zeros(3));
  end when;*/
      S_fix = identity(3);
      
      S_phi = [c2*c3, c1*s3 + s1*s2*c3, s1*s3 - c1*s2*c3; -c2*s3, c1*c3 - s1*s2
        *s3, s1*c3 + c1*s2*s3; s2, -s1*c2, c1*c2];
      S_rel = S_phi*S_fix;
      
      // No translational movement
      r_rela = zeros(3);
      v_rela = zeros(3);
      a_rela = zeros(3);
      
      // Kinematic differential equations for rotational motion
      w_fix = S_fix*w_rela;
      der(phi) = {w_fix[1] + (s1*w_fix[2] - c1*w_fix[3])*s2/c2,c1*w_fix[2] + s1
        *w_fix[3],(-s1*w_fix[2] + c1*w_fix[3])/c2};
      der(w_rela) = z_rela;
      
      // Kinematic relationships
      frame_b.S = frame_a.S*transpose(S_rel);
      frame_b.r0 = frame_a.r0;
      
      frame_b.v = S_rel*frame_a.v;
      frame_b.w = S_rel*(frame_a.w + w_rela);
      
      frame_b.a = S_rel*frame_a.a;
      frame_b.z = S_rel*(frame_a.z + cross(frame_a.w, w_rela) + z_rela);
      
      // cut-torques are zero
      frame_a.f = -transpose(S_rel)*frame_b.f;
      frame_a.t = zeros(3);
      frame_b.t = zeros(3);
    end Spherical;
    
    model FreeMotion2 
      "Free motion joint (6 degrees-of-freedom, used in spanning tree)" 
      
      
        // S_rel needs a correct start value, because pre(S_rel) is referenced below
      extends ModelicaAdditions.MultiBody.Interfaces.TreeJoint(S_rel(start=
              identity(3)));
      SI.Angle phi[3] "Cardan angles from a frame fixed in frame_a to frame_b";
    protected 
      constant SI.Angle phi2_critical_deg=80 
        "angle in [deg] too close to singularity. Redefine S_fix and phi";
      constant SI.Angle phi2_critical=phi2_critical_deg*Modelica.Constants.pi/
          180.0;
      constant Real c2_small=1.e-5 
        "if cos(phi[2]) < c2_small, c2_small is used as guard against zero division";
      SI.Velocity vaux[3];
      Real s1;
      Real s2;
      Real s3;
      Real c1;
      Real c2;
      Real c2a;
      Real c3;
      //Boolean switch_state;
      Real S_phi[3, 3] "S_rel = S_phi(phi)*S_fix";
      Real S_fix[3, 3](start=identity(3)) "S_rel = S_phi(phi)*S_fix";
      SI.AngularVelocity w_fix[3] 
        "Relative angular velocity resolved in intermediate frame S_fix";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.16, 
          y=0, 
          width=0.65, 
          height=0.79), 
        Documentation(info="<HTML> 
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is just used to define the desired states to be used.
The joint is realized in 
such a way, that a singularity cannot occur. This is achieved because
the Cardan angles are defined between a frame_fix fixed in frame_a and
frame_b. Whenever the Cardan angles are near a singularity, the
integration is stopped and frame_fix is changed, such that the Cardan
angles are far away from the singularity. The following state
variables are used:
</p>
<pre>
  r_rela[3]: Distance vector from the origin of frame_a to the origin
             of frame_b, resolved in frame_a in [m].
  phi[3]   : Cardan angles, also called Tait-Bryan angles, i.e.,
             rotate around 1-, 2-, 3-axis in [rad] from intermediate
             frame_fix, which is fixed in frame_a, to frame_b. Initially, frame_fix
             is identical to frame_a. If phi[2] is near its singularity (= pi/2 or -pi/2),
             the frame_fix and phi are changed, such that phi[2] is far away from 
             its singularity.
  v_rela[3]: = der(r_rela); relative velocity of frame_b with respect to frame_a
             resolved in frame_a in [m/s].
  w_rela[3]: Relative angular velocity of frame_b with respect to frame_a
             resolved in frame_a in [rad/s].
</pre>
</HTML>
"), 
        Icon(
          Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 
                48], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0, 
                fillColor=8)), 
          Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0, 
                fillColor=8)), 
          Text(
            extent=[100, -40; -100, -100], 
            string="%name", 
            style(color=3)), 
          Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8))), 
        Diagram(
          Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 
                48], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0, 
                fillColor=8)), 
          Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0, 
                fillColor=8)), 
          Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8)), 
          Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8))));
    equation 
      
      /* Determine sines and cosines of the Cardan angles */
      s1 = Modelica.Math.sin(phi[1]);
      s2 = Modelica.Math.sin(phi[2]);
      s3 = Modelica.Math.sin(phi[3]);
      c1 = Modelica.Math.cos(phi[1]);
      c2a = Modelica.Math.cos(phi[2]);
      c3 = Modelica.Math.cos(phi[3]);
      
      /* Below, some expressions are divided by c2. By construction, it is not possible
     that c2=0, during continuous simulation. However, at initial time and when
     large numerical errors occur, c2=0 is possible, which would result in a division
     by zero. The following statement is a guard against this unlikely situation.
  */
      
      
        //c2 = if noEvent(c2a > c2_small or c2a < -c2_small) then c2a else if noEvent(
      //  c2a >= 0) then c2_small else -c2_small;
      
      c2 = c2a;
      
      /* Relative transformation matrix
       S_phi = [ c3, s3, 0; 
                -s3, c3, 0; 
                  0, 0, 1]*[c2, 0, -s2; 
                             0, 1, 0; 
                            s2, 0, c2]*[1, 0, 0; 
                                        0, c1, s1; 
                                        0, -s1, c1];
  */
      //switch_state = phi[2] >= phi2_critical or phi[2] <= -phi2_critical;
      S_fix = identity(3);
      /*when switch_state then
    S_fix = pre(S_rel);
    reinit(phi, zeros(3));
  end when;
  */
      S_phi = [c2*c3, c1*s3 + s1*s2*c3, s1*s3 - c1*s2*c3; -c2*s3, c1*c3 - s1*s2
        *s3, s1*c3 + c1*s2*s3; s2, -s1*c2, c1*c2];
      S_rel = S_phi*S_fix;
      
      // Kinematic differential equations for translational motion
      der(r_rela) = v_rela;
      der(v_rela) = a_rela;
      
      // Kinematic differential equations for rotational motion    
      w_fix = S_fix*w_rela;
      der(phi) = {w_fix[1] + (s1*w_fix[2] - c1*w_fix[3])*s2/c2,c1*w_fix[2] + s1
        *w_fix[3],(-s1*w_fix[2] + c1*w_fix[3])/c2};
      der(w_rela) = z_rela;
      
      // Kinematic relationships
      frame_b.S = frame_a.S*transpose(S_rel);
      frame_b.r0 = frame_a.r0 + frame_a.S*r_rela;
      
      vaux = cross(frame_a.w, r_rela);
      frame_b.v = S_rel*(frame_a.v + v_rela + vaux);
      frame_b.w = S_rel*(frame_a.w + w_rela);
      
      frame_b.a = S_rel*(frame_a.a + cross(frame_a.z, r_rela) + cross(frame_a.w, 
         vaux + 2*v_rela) + a_rela);
      frame_b.z = S_rel*(frame_a.z + cross(frame_a.w, w_rela) + z_rela);
      
      // cut-forces and cut-torques are zero
      frame_a.f = zeros(3);
      frame_a.t = zeros(3);
      frame_b.f = zeros(3);
      frame_b.t = zeros(3);
    end FreeMotion2;
    
    model RelDistance 
      "Distance between flange_a and flange_b is converted to translational cuts."
       
      
      extends ModelicaAdditions.MultiBody.Interfaces.LineForce;
      Real aux[3];
      SI.Position res1;
      SI.Position res2;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.35, 
          y=0.31, 
          width=0.6, 
          height=0.6), 
        Documentation(info=" 
"), 
        Icon(
          Text(extent=[-100, 80; 98, 138], string="%name"), 
          Ellipse(extent=[-90, -40; -10, 40], style(
              color=8, 
              pattern=0, 
              gradient=0, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[-80, -30; -20, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[10, -40; 90, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[20, -29; 80, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[10, 40; 31, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-64, 15; -35, -13], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[33, 14; 62, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[-50, 3; -50, -90], style(color=0, thickness=4)), 
          Ellipse(extent=[-80, -30; -20, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-39, 40; 1, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[10, -40; 90, 40], style(
              color=8, 
              fillColor=8, 
              fillPattern=1)), 
          Ellipse(extent=[20, -29; 80, 30], style(
              color=8, 
              fillColor=7, 
              fillPattern=1)), 
          Rectangle(extent=[-1, 40; 31, -40], style(
              color=7, 
              fillColor=7, 
              fillPattern=1)), 
          Ellipse(extent=[-64, 15; -35, -13], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Ellipse(extent=[33, 14; 62, -14], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Line(points=[50, 4; 50, -89], style(color=0, thickness=4)), 
          Rectangle(extent=[-31, 3; -21, -5], style(gradient=2, fillColor=8)), 
          Line(points=[-50, 3; -50, -90], style(color=0, thickness=4)), 
          Rectangle(extent=[-14, 3; -4, -5], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[3, 3; 13, -5], style(gradient=2, fillColor=8)), 
          Rectangle(extent=[19, 3; 29, -5], style(gradient=2, fillColor=8))), 
        Diagram);
      
      Modelica.Mechanics.Translational.Interfaces.Flange_b bearing
        annotation (extent=[-40, -90; -60, -110], layer="icon");
      Modelica.Mechanics.Translational.Interfaces.Flange_a axis
        annotation (extent=[39, -90; 60, -111], layer="icon");
    equation 
      
      res1 = axis.s + s;
      res2 = der(axis.s) + sd;
      constrain(res1, res2);
      
      bearing.s = 0;
      
      axis.f = f;
      
      aux = na*s;
    end RelDistance;
    
  end Joints;
  
  package Forces 
    "Modelica library to model forces and torques between interacting bodies, such as e.g. springs and dampers"
     
    
    extends Modelica.Icons.Library;
    annotation (
      Coordsys(
        extent=[0, 0; 442, 386], 
        grid=[1, 1], 
        component=[20, 20]), 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.28, 
        height=0.42, 
        library=1, 
        autolayout=1), 
      Documentation(info="<HTML> 
<p>
This library provides force generators such as non-linear spring-dampers of various kinds that are not included in the standard libraries for Mechanics.
</p>
<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.fkt.kth.se/FDN/homepage_johan.html\">Johan Andreasson</a><br>
    <a href=\"http://www.fkt.kth.se/FDN/\">Division of Vehicle Dynamics (FDN)</a><br>
    Royal Institute of Technology (KTH)<br>
    Teknikringen 8c<br>
    100 44 Stockholm<br>
    Sweden<br>
    email: <A HREF=\"mailto:Johan@fkt.kth.se\">Johan@fkt.kth.se</A><br>
</dl>
</HTML> 
"));
    model SpringDamperTable1D 
      extends Modelica.Mechanics.Translational.Interfaces.Compliant;
      
      parameter SI.Position s_rel0=0 "Unstretched spring length";
      
      parameter Real[:, 2] fc=data.fc "Table to be interpolated";
      parameter Real[:, 2] fd=data.fd "Table to be interpolated";
      
      parameter String tableName_C=data.tableName_C 
        "table name on file or in function usertab(optional)";
      parameter String fileName_C=data.fileName_C 
        "file where matrix is stored (optional)";
      parameter Real icol_C[:]=data.icol_C 
        "columns of table to be interpolated";
      parameter String tableName_D=data.tableName_D 
        "table name on file or in function usertab(optional)";
      parameter String fileName_D=data.fileName_D 
        "file where matrix is stored (optional)";
      parameter Real icol_D[:]=data.icol_D 
        "columns of table to be interpolated";
      SI.Velocity v_rel "relative velocity between flange L and R";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Documentation(info="<HTML>
<p>
This class provides a spring damper where the forces generated by the spring and the damper is read from a table and interpolated. The table can be given as parameters or with a record.
</p>
<pre>
  s_rel0:       Unstretched spring length
  tableName_C:  table name on file or in function usertab(optional) (spring force)
  fileName_C:   NoName\" \"file where matrix is stored (optional) (spring force)
  icol_C:       columns of table to be interpolated (spring force)
  tableName_D:  table name on file or in function usertab(optional) (damper force) 
  fileName_D:   NoName\" \"file where matrix is stored (optional) (damper force)
  icol_D:       columns of table to be interpolated (damper force)
  forceTable: To set the force generation via a record, add a component named [name] of class VehicleDynamics.Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"), 
        Icon(
          Line(points=[-80, 69; -80, -70], style(color=0)), 
          Line(points=[80, 70; 80, -70], style(color=0)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[80, 0; 90, 0], style(color=0)), 
          Line(points=[-60, 40; -60, -40; 60, -40; 60, 40; 30, 40; 30, -40; -30, 
                 -40; -30, 40; -60, 40; -60, 20; 60, 20; 60, 0; -60, 0; -60, -20; 
                 60, -20; 60, -40; -60, -40; -60, 40; 60, 40; 60, -40], style(
                color=8)), 
          Line(points=[0, 40; 0, -40], style(color=8)), 
          Rectangle(extent=[-60, 40; -30, 20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 20; -30, 0], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 0; -30, -20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, -20; -30, -40], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Line(points=[-80, -70; -60, -70; -45, -100; -15, -40; 15, -100; 45, -40; 
                 60, -70; 80, -70], style(color=0)), 
          Line(points=[-80, 70; -52, 70], style(color=0)), 
          Rectangle(extent=[-52, 91; 38, 49], style(color=0, fillColor=8)), 
          Line(points=[-52, 91; 68, 91], style(color=0)), 
          Line(points=[-51, 49; 69, 49], style(color=0)), 
          Line(points=[38, 70; 80, 70], style(color=0)), 
          Line(points=[-57, 0; 23, 0], style(color=0, fillColor=10)), 
          Polygon(points=[53, 0; 23, 10; 23, -10; 53, 0], style(color=0, 
                fillColor=10))), 
        Diagram(
          Polygon(points=[-120, 3; -110, 0; -120, -3; -120, 3], style(color=0, 
                fillColor=0)), 
          Line(points=[-120, 0; -149, 0], style(color=0, fillColor=0)), 
          Line(points=[139, -1; 110, -1], style(color=0, fillColor=0)), 
          Polygon(points=[139, 2; 149, -1; 139, -4; 139, 2], style(color=0, 
                fillColor=0)), 
          Text(extent=[-149, 19; -118, 3], string="flange axis"), 
          Text(extent=[119, 18; 150, 2], string="flange axis"), 
          Line(points=[-90, 0; -80, 0], style(color=0))), 
        Window(
          x=0.05, 
          y=0.07, 
          width=0.89, 
          height=0.81));
      
    public 
      ModelicaAdditions.Tables.CombiTable1D Dt(
        tableName=tableName_D, 
        fileName=fileName_D, 
        icol=icol_D, 
        table=fd) annotation (extent=[-20, 40; 20, 80]);
      ModelicaAdditions.Tables.CombiTable1D Ct(
        tableName=tableName_C, 
        fileName=fileName_C, 
        icol=icol_C, 
        table=fc) annotation (extent=[-20, -20; 20, 20]);
      parameter Utilities.ForceTable1D data
        annotation (extent=[-20, -80; 20, -40]);
    equation 
      v_rel = der(s_rel);
      
      Ct.inPort.signal[1] = s_rel - s_rel0;
      Dt.inPort.signal[1] = v_rel;
      
      f = Ct.outPort.signal[1] + Dt.outPort.signal[1];
    end SpringDamperTable1D;
    
    package Utilities 
      extends Modelica.Icons.Library;
      annotation (Coordsys(
          extent=[0, 0; 632, 426], 
          grid=[1, 1], 
          component=[20, 20]), Window(
          x=0, 
          y=0.6, 
          width=0.4, 
          height=0.4, 
          library=1, 
          autolayout=1));
      block RotationVector 
        input Real[3, 3] S;
        output Real[3] THETA;
        parameter Boolean linear=false 
          "turn on of linearisation of cosine and sine";
        
        parameter Real eps=0.0000001 "Guard against division by zero";
        
      protected 
        parameter Integer n3=if linear then 0 else 3;
        parameter Integer n1=if linear then 0 else 1;
        
        Real k[n3] "Direction of THETA";
        Real sinTHETA[n1] "sine of abs(THETA)";
        Real cosTHETA[n1] "cosine of abs(THETA)";
        Real aux[n3] "auxillary vector";
        
        annotation (
          Coordsys(
            extent=[-200, -200; 200, 200], 
            grid=[2, 2], 
            component=[20, 20]), 
          Icon(
            Rectangle(extent=[-180, 180; 180, -180]), 
            Ellipse(extent=[76, 38; 136, -60]), 
            Line(points=[72, -20; 84, -12; 128, -12; 140, -4]), 
            Line(points=[-54, 36; -56, 48; -58, 52; -66, 60; -78, 66; -92, 68; 
                  -108, 66; -122, 60; -128, 52; -132, 38; -132, 30; -128, 22; -122, 
                   16; -112, 12; -100, 8; -92, 6]), 
            Line(points=[-92, 6; -84, 4; -76, 2; -64, -4; -56, -12; -52, -26; -54, 
                   -36; -58, -46; -66, -52; -72, -56; -84, -60; -102, -60; -114, 
                   -58; -124, -54; -132, -46; -138, -38; -140, -30; -140, -24]), 
               
            Line(points=[84, 50; 130, 50]), 
            Line(points=[-20, 0; 40, 0]), 
            Line(points=[30, 8; 40, 0]), 
            Line(points=[30, -8; 40, 0]), 
            Text(extent=[76, 168; 172, 110], string="3D")), 
          Window(
            x=0.11, 
            y=0.23, 
            width=0.35, 
            height=0.49), 
          Documentation(info="This block calculates the rotation vector THETA 
from a transformation matrix S according to:
Assume  S=
    [nx, sx, ax;
     ny, sy, ay;
     nz, sz, az]=
    Rot(k,theta)=
    [kx*kx*vT+cT,    ky*kx*vT-kz*sT, kz*kx*vT+ky*sT;
     kx*ky*vT+kz*sT, ky*ky*vT+cT     kz*ky*vT-kx*sT;
     kx*kz*vT-ky*sT, ky*kz*vT+kx*sT, kz*kz*vT+cT];
where   vT=1-cT, cT=cos(theta) and sT=sin(theta).
Then    nx+sy+az=(kx*kx+ky*ky+kz*kz)*vT+3*cT 
    ->  
    cT=1/2*(nx+sy+az-1);
Additionally,
    sz-ay=(ky*kz*vT+kx*sT)-(kz*ky*vT-kx*sT)=2*kx*sT;
    ax-nz=(kz*kx*vT+ky*sT)-(kx*kz*vT-ky*sT)=2*ky*sT;
    ny-sx=(kx*ky*vT+kz*sT)-(ky*kx*vT-kz*sT)=2*kz*sT;
    ->
    sT*sT=1/4*((sz-ay)(sz-ay)+(ax-nz)(ax-nz)+(ny-sx)(ny-sx));
    and
    k=1/(2*sT)*{sz-ay,ax-nz,ny-sx};
Since   THETA=k*theta=(-k)*(-theta) any from the roots coult be coosen,
    choose positive root:
    ->
    sT=1/2*sqrt((sz-ay)(sz-ay)+(ax-nz)(ax-nz)+(ny-sx)(ny-sx));
Finnally,
    theta=atan2(sT,cT)
    ->
    THETA=k*theta.
    
    
    
"));
      equation 
        
        if (linear) then
          //THETA = {S[2, 3],-S[1, 3],S[1, 2]};
          THETA = -{S[3, 2],S[1, 3],S[2, 1]};
          
        else
          /*Calculate cos(THETA)*/
          cosTHETA[1] = 0.5*(S[1, 1] + S[2, 2] + S[3, 3] - 1);
          
          /*Calculate sin(THETA)*/
          aux[1] = (S[3, 2] - S[2, 3]);
          aux[2] = (S[1, 3] - S[3, 1]);
          aux[3] = (S[2, 1] - S[1, 2]);
          
          sinTHETA[1] = 0.5*sqrt(aux*aux);
          
          /*Calculate k, prevent zero division*/
          if noEvent(abs(sinTHETA[1]) < eps) then
            k[1] = 1;
            k[2] = 0;
            k[3] = 0;
          else
            k[1] = 0.5/sinTHETA[1]*aux[1];
            k[2] = 0.5/sinTHETA[1]*aux[2];
            k[3] = 0.5/sinTHETA[1]*aux[3];
          end if;
          /*Calculate THETA*/
          identity(n3)*THETA = Modelica.Math.atan2(sinTHETA[1], cosTHETA[1])*k;
        end if;
        
      end RotationVector;
      
      block Rotation 
        parameter Boolean linear=false;
        
        input Real[2, 2] S;
        output Real THETA;
        annotation (
          Coordsys(
            extent=[-200, -200; 200, 200], 
            grid=[2, 2], 
            component=[20, 20]), 
          Window(
            x=0.4, 
            y=0.4, 
            width=0.6, 
            height=0.6), 
          Icon(
            Rectangle(extent=[-180, 180; 180, -180]), 
            Ellipse(extent=[76, 38; 136, -60]), 
            Line(points=[72, -20; 84, -12; 128, -12; 140, -4]), 
            Line(points=[-54, 36; -56, 48; -58, 52; -66, 60; -78, 66; -92, 68; 
                  -108, 66; -122, 60; -128, 52; -132, 38; -132, 30; -128, 22; -122, 
                   16; -112, 12; -100, 8; -92, 6]), 
            Line(points=[-92, 6; -84, 4; -76, 2; -64, -4; -56, -12; -52, -26; -54, 
                   -36; -58, -46; -66, -52; -72, -56; -84, -60; -102, -60; -114, 
                   -58; -124, -54; -132, -46; -138, -38; -140, -30; -140, -24]), 
               
            Line(points=[-20, 0; 40, 0]), 
            Line(points=[30, 8; 40, 0]), 
            Line(points=[30, -8; 40, 0]), 
            Text(extent=[76, 168; 172, 110], string="2D")));
      equation 
        
        THETA = if linear then S[1, 2] else Modelica.Math.atan2(S[1, 2], S[1, 1]);
      end Rotation;
      
      model MatrixFromVector 
        input Real[3] x;
        //Temporarilly, this should be Real[:]
        output Real[size(x, 1), size(x, 1)] Y;
        
        parameter Real[1, :] grid=[-1, 0, 1];
        parameter Real[:, 9] values=[10000, 0, 0, 0, 10000, 0, 0, 0, 10000; 
            5000, 0, 0, 0, 5000, 0, 0, 0, 5000; 10000, 0, 0, 0, 10000, 0, 0, 0, 
             10000];
        
        //paramter handling must be improved
        
        
          //here should be added parameters so that tables from files could be read, see CombiTable1D 
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), 
          Diagram, 
          Icon(
            Text(
              extent=[-92, 32; 88, -36], 
              string="A=f(x)", 
              style(color=0)), 
            Text(
              extent=[20, 46; 60, 10], 
              string="_", 
              style(color=0)), 
            Rectangle(extent=[-90, 34; 92, -90]), 
            Text(extent=[-100, 100; 100, 34], string="%name")), 
          Window(
            x=0.02, 
            y=0.08, 
            width=0.6, 
            height=0.6));
        ModelicaAdditions.Tables.CombiTable1D x1(table=[transpose(grid), values[
              :, 1], values[:, 4], values[:, 7]], icol={2,3,4})
          annotation (extent=[-20, 40; 20, 80]);
        ModelicaAdditions.Tables.CombiTable1D x2(table=[transpose(grid), values[
              :, 2], values[:, 5], values[:, 8]], icol={2,3,4})
          annotation (extent=[-20, -20; 20, 20]);
        ModelicaAdditions.Tables.CombiTable1D x3(table=[transpose(grid), values[
              :, 3], values[:, 6], values[:, 9]], icol={2,3,4})
          annotation (extent=[-20, -80; 20, -40]);
      equation 
        x1.inPort.signal = {x[1],x[1],x[1]};
        x2.inPort.signal = {x[2],x[2],x[2]};
        x3.inPort.signal = {x[3],x[3],x[3]};
        
        Y = [x1.outPort.signal[1], x2.outPort.signal[1], x3.outPort.signal[1]; 
          x1.outPort.signal[2], x2.outPort.signal[2], x3.outPort.signal[2]; x1.
          outPort.signal[3], x2.outPort.signal[3], x3.outPort.signal[3]];
        
      end MatrixFromVector;
      
      record MatrixFromVectorData 
        //this record should provide the data neccessary for MatrixFromVector
        annotation (Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]));
      end MatrixFromVectorData;
      
      record ForceTable1D 
        parameter Real[:, 2] fc=[-1, -10000; -0.5, -3300; 0, 0; 0.5, 3300; 1, 
            10000];
        parameter Real[:, 2] fd=[-1, -1000; -0.5, -330; 0, 0; 0.5, 330; 1, 1000];
        
        parameter String tableName_C="NoName" 
          "table name on file or in function usertab(optional)";
        parameter String fileName_C="NoName" 
          "file where matrix is stored (optional)";
        parameter Real icol_C[:]={2} "columns of table to be interpolated";
        parameter String tableName_D="NoName" 
          "table name on file or in function usertab(optional)";
        parameter String fileName_D="NoName" 
          "file where matrix is stored (optional)";
        parameter Real icol_D[:]={2} "columns of table to be interpolated";
        
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), 
          Icon(
            Rectangle(extent=[-102, 104; 100, -100], style(color=0, fillColor=
                    51)), 
            Text(extent=[-102, -14; 100, -100], string="%name"), 
            Text(extent=[-100, 100; 100, 18], string="Data")), 
          Window(
            x=0.03, 
            y=0.02, 
            width=0.92, 
            height=0.95));
        
      end ForceTable1D;
      
      record Coefficients3DLinear 
        parameter Real[3, 3] Ct=[10000, 0, 0; 0, 10000, 0; 0, 0, 10000];
        parameter Real[3, 3] Dt=0.0001*[10000, 0, 0; 0, 10000, 0; 0, 0, 10000];
        parameter Real[3, 3] Cr=[10000, 0, 0; 0, 10000, 0; 0, 0, 10000];
        parameter Real[3, 3] Dr=[10000, 0, 0; 0, 10000, 0; 0, 0, 10000];
        parameter Real[6, 6] C=10000*[10000, 0, 0, 0, 0, 0; 0, 10000, 0, 0, 0, 
            0; 0, 0, 10000, 0, 0, 0; 0, 0, 0, 10000, 0, 0; 0, 0, 0, 0, 10000, 0; 
             0, 0, 0, 0, 0, 10000];
        parameter Real[6, 6] D=[10000, 0, 0, 0, 0, 0; 0, 10000, 0, 0, 0, 0; 0, 
            0, 10000, 0, 0, 0; 0, 0, 0, 10000, 0, 0; 0, 0, 0, 0, 10000, 0; 0, 0, 
             0, 0, 0, 10000];
        annotation (Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), Icon(
            Text(extent=[-84, -4; 76, -44], string="c=f(s)"), 
            Text(extent=[-84, -44; 76, -84], string="d=f(der(s))"), 
            Text(extent=[-100, 100; 100, 18], string="%name"), 
            Rectangle(extent=[-102, 104; 100, -100], style(color=0, fillColor=
                    51)), 
            Text(extent=[-100, 100; 100, 18], string="%name"), 
            Text(extent=[-90, -6; 86, -88], string="C[6,6], D[6,6]")));
        
      end Coefficients3DLinear;
      
      record ForceTable3D 
        parameter Real[:, 2] r1=[-0.1, 10000000; -0.08, 1000000; -0.06, 100000; 
             -0.04, 100000; 0, 0; 0.04, -100000; 0.06, -100000; 0.08, -1000000; 
             0.1, -10000000];
        parameter Real[:, 2] r2=[-0.1, 10000000; -0.08, 1000000; -0.06, 100000; 
             -0.04, 100000; 0, 0; 0.04, -100000; 0.06, -100000; 0.08, -1000000; 
             0.1, -10000000];
        parameter Real[:, 2] r3=[-0.1, 10000000; -0.08, 1000000; -0.06, 100000; 
             -0.04, 100000; 0, 0; 0.04, -100000; 0.06, -100000; 0.08, -1000000; 
             0.1, -10000000];
        
        parameter Real v1[:, 2]=[-0.1, 200000; -0.08, 20000; -0.06, 2000; -0.04, 
             2000; 0, 0; 0.04, -2000; 0.06, -2000; 0.08, -20000; 0.1, -200000];
        parameter Real v2[:, 2]=[-0.1, 200000; -0.08, 20000; -0.06, 2000; -0.04, 
             2000; 0, 0; 0.04, -2000; 0.06, -2000; 0.08, -20000; 0.1, -200000];
        parameter Real v3[:, 2]=[-0.1, 200000; -0.08, 20000; -0.06, 2000; -0.04, 
             2000; 0, 0; 0.04, -2000; 0.06, -2000; 0.08, -20000; 0.1, -200000];
        
        parameter SI.Angle phi1[:, 2]=[-0.1, 10000000; -0.08, 1000000; -0.06, 
            100000; -0.04, 100000; 0, 0; 0.04, -100000; 0.06, -100000; 0.08, -1000000; 
             0.1, -10000000];
        parameter SI.Angle phi2[:, 2]=[-0.1, 10000000; -0.08, 1000000; -0.06, 
            100000; -0.04, 100000; 0, 0; 0.04, -100000; 0.06, -100000; 0.08, -1000000; 
             0.1, -10000000];
        parameter SI.Angle phi3[:, 2]=[-0.1, 10000000; -0.08, 1000000; -0.06, 
            100000; -0.04, 100000; 0, 0; 0.04, -100000; 0.06, -100000; 0.08, -1000000; 
             0.1, -10000000];
        
        parameter Real w1[:, 2]=[-0.1, 200000; -0.08, 20000; -0.06, 2000; -0.04, 
             2000; 0, 0; 0.04, -2000; 0.06, -2000; 0.08, -20000; 0.1, -200000];
        parameter Real w2[:, 2]=[-0.1, 200000; -0.08, 20000; -0.06, 2000; -0.04, 
             2000; 0, 0; 0.04, -2000; 0.06, -2000; 0.08, -20000; 0.1, -200000];
        parameter Real w3[:, 2]=[-0.1, 200000; -0.08, 20000; -0.06, 2000; -0.04, 
             2000; 0, 0; 0.04, -2000; 0.06, -2000; 0.08, -20000; 0.1, -200000];
        
        annotation (Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]));
      end ForceTable3D;
      
      block RotationVector2 
        input Real[3, 3] S;
        output Real[3] THETA;
        
        parameter Real eps=0.0000001 "Guard against division by zero";
      protected 
        Real k[3] "Direction of THETA";
        Real sinTHETA "sine of abs(THETA)";
        Real cosTHETA "cosine of abs(THETA)";
        
        Real aux[3] "auxillary vector";
        annotation (
          Coordsys(
            extent=[-200, -200; 200, 200], 
            grid=[2, 2], 
            component=[20, 20]), 
          Icon(
            Rectangle(extent=[-180, 180; 180, -180]), 
            Ellipse(extent=[76, 38; 136, -60]), 
            Line(points=[72, -20; 84, -12; 128, -12; 140, -4]), 
            Line(points=[-54, 36; -56, 48; -58, 52; -66, 60; -78, 66; -92, 68; 
                  -108, 66; -122, 60; -128, 52; -132, 38; -132, 30; -128, 22; -122, 
                   16; -112, 12; -100, 8; -92, 6]), 
            Line(points=[-92, 6; -84, 4; -76, 2; -64, -4; -56, -12; -52, -26; -54, 
                   -36; -58, -46; -66, -52; -72, -56; -84, -60; -102, -60; -114, 
                   -58; -124, -54; -132, -46; -138, -38; -140, -30; -140, -24]), 
               
            Line(points=[84, 50; 130, 50]), 
            Line(points=[-20, 0; 40, 0]), 
            Line(points=[30, 8; 40, 0]), 
            Line(points=[30, -8; 40, 0]), 
            Text(extent=[76, 168; 172, 110], string="3D")), 
          Window(
            x=0.11, 
            y=0.23, 
            width=0.35, 
            height=0.49), 
          Documentation(info="This block calculates the rotation vector THETA 
from a transformation matrix S according to:
Assume  S=
    [nx, sx, ax;
     ny, sy, ay;
     nz, sz, az]=
    Rot(k,theta)=
    [kx*kx*vT+cT,    ky*kx*vT-kz*sT, kz*kx*vT+ky*sT;
     kx*ky*vT+kz*sT, ky*ky*vT+cT     kz*ky*vT-kx*sT;
     kx*kz*vT-ky*sT, ky*kz*vT+kx*sT, kz*kz*vT+cT];
where   vT=1-cT, cT=cos(theta) and sT=sin(theta).
Then    nx+sy+az=(kx*kx+ky*ky+kz*kz)*vT+3*cT 
    ->  
    cT=1/2*(nx+sy+az-1);
Additionally,
    sz-ay=(ky*kz*vT+kx*sT)-(kz*ky*vT-kx*sT)=2*kx*sT;
    ax-nz=(kz*kx*vT+ky*sT)-(kx*kz*vT-ky*sT)=2*ky*sT;
    ny-sx=(kx*ky*vT+kz*sT)-(ky*kx*vT-kz*sT)=2*kz*sT;
    ->
    sT*sT=1/4*((sz-ay)(sz-ay)+(ax-nz)(ax-nz)+(ny-sx)(ny-sx));
    and
    k=1/(2*sT)*{sz-ay,ax-nz,ny-sx};
Since   THETA=k*theta=(-k)*(-theta) any from the roots coult be coosen,
    choose positive root:
    ->
    sT=1/2*sqrt((sz-ay)(sz-ay)+(ax-nz)(ax-nz)+(ny-sx)(ny-sx));
Finnally,
    theta=atan2(sT,cT)
    ->
    THETA=k*theta.
    
    
    
"));
      equation 
        
        /*Calculate cos(THETA)*/
        cosTHETA = 0.5*(S[1, 1] + S[2, 2] + S[3, 3] - 1);
        
        /*Calculate sin(THETA)*/
        aux = {(S[3, 2] - S[2, 3]),(S[1, 3] - S[3, 1]),(S[2, 1] - S[1, 2])};
        
        sinTHETA = 0.5*sqrt(aux*aux);
        
        /*Calculate k, prevent zero division*/
        k = noEvent(if (abs(sinTHETA) < eps) then {1,0,0} else 0.5/sinTHETA*aux);
        
        /*Calculate THETA*/
        THETA = Modelica.Math.atan2(sinTHETA, cosTHETA)*k;
        
      end RotationVector2;
      
      block RotationVectorInterface 
        input Real[3, 3] S;
        output Real[3] THETA;
        parameter Boolean linear=false 
          "turn on of linearisation of cosine and sine";
        
        parameter Real eps=0.0000001 "Guard against division by zero";
        
      protected 
        parameter Integer n3=if linear then 0 else 3;
        parameter Integer n1=if linear then 0 else 1;
        
        Real k[n3] "Direction of THETA";
        Real sinTHETA[n1] "sine of abs(THETA)";
        Real cosTHETA[n1] "cosine of abs(THETA)";
        Real aux[n3] "auxillary vector";
        
        annotation (
          Coordsys(
            extent=[-200, -200; 200, 200], 
            grid=[2, 2], 
            component=[20, 20]), 
          Icon(
            Rectangle(extent=[-180, 180; 180, -180]), 
            Ellipse(extent=[76, 38; 136, -60]), 
            Line(points=[72, -20; 84, -12; 128, -12; 140, -4]), 
            Line(points=[-54, 36; -56, 48; -58, 52; -66, 60; -78, 66; -92, 68; 
                  -108, 66; -122, 60; -128, 52; -132, 38; -132, 30; -128, 22; -122, 
                   16; -112, 12; -100, 8; -92, 6]), 
            Line(points=[-92, 6; -84, 4; -76, 2; -64, -4; -56, -12; -52, -26; -54, 
                   -36; -58, -46; -66, -52; -72, -56; -84, -60; -102, -60; -114, 
                   -58; -124, -54; -132, -46; -138, -38; -140, -30; -140, -24]), 
               
            Line(points=[84, 50; 130, 50]), 
            Line(points=[-20, 0; 40, 0]), 
            Line(points=[30, 8; 40, 0]), 
            Line(points=[30, -8; 40, 0]), 
            Text(extent=[76, 168; 172, 110], string="3D")), 
          Window(
            x=0.11, 
            y=0.23, 
            width=0.35, 
            height=0.49), 
          Documentation(info="This block calculates the rotation vector THETA 
from a transformation matrix S according to:
Assume  S=
    [nx, sx, ax;
     ny, sy, ay;
     nz, sz, az]=
    Rot(k,theta)=
    [kx*kx*vT+cT,    ky*kx*vT-kz*sT, kz*kx*vT+ky*sT;
     kx*ky*vT+kz*sT, ky*ky*vT+cT     kz*ky*vT-kx*sT;
     kx*kz*vT-ky*sT, ky*kz*vT+kx*sT, kz*kz*vT+cT];
where   vT=1-cT, cT=cos(theta) and sT=sin(theta).
Then    nx+sy+az=(kx*kx+ky*ky+kz*kz)*vT+3*cT 
    ->  
    cT=1/2*(nx+sy+az-1);
Additionally,
    sz-ay=(ky*kz*vT+kx*sT)-(kz*ky*vT-kx*sT)=2*kx*sT;
    ax-nz=(kz*kx*vT+ky*sT)-(kx*kz*vT-ky*sT)=2*ky*sT;
    ny-sx=(kx*ky*vT+kz*sT)-(ky*kx*vT-kz*sT)=2*kz*sT;
    ->
    sT*sT=1/4*((sz-ay)(sz-ay)+(ax-nz)(ax-nz)+(ny-sx)(ny-sx));
    and
    k=1/(2*sT)*{sz-ay,ax-nz,ny-sx};
Since   THETA=k*theta=(-k)*(-theta) any from the roots coult be coosen,
    choose positive root:
    ->
    sT=1/2*sqrt((sz-ay)(sz-ay)+(ax-nz)(ax-nz)+(ny-sx)(ny-sx));
Finnally,
    theta=atan2(sT,cT)
    ->
    THETA=k*theta.
    
    
    
"));
      equation 
        
        if (linear) then
          //THETA = {S[2, 3],-S[1, 3],S[1, 2]};
          THETA = -{S[3, 2],S[1, 3],S[2, 1]};
          
        else
          /*Calculate cos(THETA)*/
          cosTHETA[1] = 0.5*(S[1, 1] + S[2, 2] + S[3, 3] - 1);
          
          /*Calculate sin(THETA)*/
          aux[1] = (S[3, 2] - S[2, 3]);
          aux[2] = (S[1, 3] - S[3, 1]);
          aux[3] = (S[2, 1] - S[1, 2]);
          
          sinTHETA[1] = 0.5*sqrt(aux*aux);
          
          /*Calculate k, prevent zero division*/
          if noEvent(abs(sinTHETA[1]) < eps) then
            k[1] = 1;
            k[2] = 0;
            k[3] = 0;
          else
            k[1] = 0.5/sinTHETA[1]*aux[1];
            k[2] = 0.5/sinTHETA[1]*aux[2];
            k[3] = 0.5/sinTHETA[1]*aux[3];
          end if;
          /*Calculate THETA*/
          identity(n3)*THETA = Modelica.Math.atan2(sinTHETA[1], cosTHETA[1])*k;
        end if;
        
      end RotationVectorInterface;
      
      model Shape "Different visual shapes (box, cylinder, sphere, ...)" 
        parameter String shapeType="box" 
          "Type of shape (box, sphere, cylinder, pipecylinder, cone, pipe, beam, gearwheel, wirebox, spring)";
        input SI.Position r0[3]={0,0,0} 
          "Position vector from object frame origin to shape origin";
        input SI.Position lengthDirection[3]={1,0,0} 
          "Vector in length direction, resolved in object frame.";
        input SI.Position widthDirection[3]={0,1,0} 
          "Vector in width direction, resolved in object frame.";
        input SI.Position length=0 "length of visual object.";
        input SI.Position width=0 "Width of visual object.";
        input SI.Position height=0 "Height of visual object.";
        input Real Material[4]={1,0,0,0.5} "Color and specular coefficient.";
        input Real Extra=0.0 "Additional parameter for cone and pipe.";
        input Real S[3, 3] "Transformation matrix to inertial frame.";
        input SI.Position r[3] 
          "Position vector from origin of world frame to origin of object frame, resolved in world frame";
      protected 
        output Real Form annotation (Hide=false);
        output Real rxvisobj[3] annotation (Hide=false);
        output Real ryvisobj[3] annotation (Hide=false);
        output Real rvisobj[3] annotation (Hide=false);
        output Real size[3] annotation (Hide=false);
        output Real material annotation (Hide=false);
        output Real extra annotation (Hide=false);
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[2, 2], 
            component=[20, 20]), 
          Icon(
            Rectangle(extent=[-100, -100; 80, 60], style(color=3, fillColor=7)), 
               
            Polygon(points=[-100, 60; -80, 100; 100, 100; 80, 60; -100, 60], 
                style(color=3, fillColor=8)), 
            Polygon(points=[100, 100; 100, -60; 80, -100; 80, 60; 100, 100], 
                style(color=3, fillColor=9)), 
            Text(
              extent=[-100, -100; 80, 60], 
              style(color=0), 
              string="%shapeType"), 
            Text(extent=[-132, 160; 128, 100], string="%name")), 
          Window(
            x=0.28, 
            y=0.06, 
            width=0.6, 
            height=0.69));
      equation 
        
        /* Outputs to file. */
        Form = (987000 + PackShape(shapeType))*1E20;
        rxvisobj = S[:, 1];
        ryvisobj = S[:, 2];
        rvisobj = r + S*r0;
        size = {length,width,height};
        material = PackMaterial(Material[1], Material[2], Material[3], Material[
          4]);
        extra = Extra;
      end Shape;
      
      model BushingAnimation 
        "SpringDamperTableLine3D - nonlinear translational spring-damper model to use with ModelicaAdditions.MultiBody"
         
        
        parameter Boolean animation=true "True, if animation shall be enabled";
        parameter SI.Position width=0.1 
          "|Animation|if animation = true| Width bushing";
        parameter SI.Position length=0.1 
          "|Animation|if animation = true| Width bushing";
        parameter SI.Position height=0.1 
          "|Animation|if animation = true| Width bushing";
        parameter Real material[4]={0.2,0.2,0.2,0.2} 
          "|Animation|if animation = true| Color and specular coefficient of spring";
        
      protected 
        Real material2[4];
        
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100], 
            grid=[1, 1], 
            component=[20, 20]), 
          Diagram, 
          Icon, 
          Window(
            x=0.13, 
            y=0.07, 
            width=0.55, 
            height=0.48), 
          Documentation(info="<HTML>
<p>
This class provides a translational spring damper where the forces generated by the spring and the damper is read from a table and interpolated. The table can be given as parameters or with a record.
</p>
<pre>
  s_rel0:       Unstretched spring length
  tableName_C:  table name on file or in function usertab(optional) (spring force)
  fileName_C:   NoName\" \"file where matrix is stored (optional) (spring force)
  icol_C:       columns of table to be interpolated (spring force)
  tableName_D:  table name on file or in function usertab(optional) (damper force) 
  fileName_D:   NoName\" \"file where matrix is stored (optional) (damper force)
  icol_D:       columns of table to be interpolated (damper force)
  forceTable: To set the force generation via a record, add a component named [name] of class VehicleDynamics.Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
        
        SI.Position r0a[3];
        SI.Position r0b[3];
        Real[3, 3] Sa;
        Real[3, 3] Sb;
        
      protected 
        parameter Integer ndim=if animation then 1 else 0;
      protected 
        Utilities.Shape shapeA[ndim](
          each shapeType="box", 
          each Material=material, 
          each length=length/2, 
          each width=width, 
          each height=width, 
          each lengthDirection={1,0,0}, 
          each widthDirection={0,1,0}, 
          each r=r0a, 
          each S=Sa) annotation (extent=[-80, -80; -60, -60]);
        Utilities.Shape shapeB[ndim](
          each shapeType="box", 
          each Material=material, 
          each length=length/2, 
          each width=width, 
          each height=width, 
          each lengthDirection={1,0,0}, 
          each widthDirection={0,1,0}, 
          each r=r0b, 
          each S=Sb) annotation (extent=[-80, -80; -60, -60]);
      equation 
        
      end BushingAnimation;
    end Utilities;
    
    model SpringDamperTable3D 
      "SpringDamperTableLine3D - nonlinear translational and rotational spring-damper model to use with ModelicaAdditions.MultiBody"
       
      
      extends ModelicaAdditions.MultiBody.Interfaces.Interact2;
      parameter SI.Position[3] r_rela0={0,0,0} 
        "Unstretched translational spring vector";
      parameter SI.Angle[3] phi_rela0={0,0,0} 
        "Unstretched rotational spring vector";
      
    protected 
      SI.Angle[3] phi_rela;
    public 
      parameter Boolean animation=true "True, if animation shall be enabled";
      parameter SI.Position width=0.1 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position length=0.1 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position height=0.1 
        "|Animation|if animation = true| Width bushing";
      parameter Real material[4]={0.2,0.2,0.2,0.2} 
        "|Animation|if animation = true| Color and specular coefficient of spring";
      
    protected 
      parameter Integer ndim=if animation then 1 else 0;
    protected 
      Utilities.Shape shapeA[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=-length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=r0a, 
        each S=Sa) annotation (extent=[-80, -80; -60, -60]);
      Utilities.Shape shapeB[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=r0b, 
        each S=Sb) annotation (extent=[-80, -80; -60, -60]);
      
    protected 
      VehicleDynamics.Utilities.Forces.Utilities.RotationVector rotationVector
        annotation (extent=[-20, -22; 20, 20]);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-80, 69; -80, -70], style(color=0)), 
          Line(points=[80, 70; 80, -70], style(color=0)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[80, 0; 90, 0], style(color=0)), 
          Line(points=[-60, 40; -60, -40; 60, -40; 60, 40; 30, 40; 30, -40; -30, 
                 -40; -30, 40; -60, 40; -60, 20; 60, 20; 60, 0; -60, 0; -60, -20; 
                 60, -20; 60, -40; -60, -40; -60, 40; 60, 40; 60, -40], style(
                color=8)), 
          Line(points=[0, 40; 0, -40], style(color=8)), 
          Rectangle(extent=[-60, 40; -30, 20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 20; -30, 0], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 0; -30, -20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, -20; -30, -40], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Line(points=[-80, -70; -60, -70; -45, -100; -15, -40; 15, -100; 45, -40; 
                 60, -70; 80, -70], style(color=0)), 
          Polygon(points=[53, 0; 23, 10; 23, -10; 53, 0], style(color=0, 
                fillColor=10)), 
          Line(points=[-80, 70; -52, 70], style(color=0)), 
          Rectangle(extent=[-52, 91; 38, 49], style(color=0, fillColor=8)), 
          Line(points=[-52, 91; 68, 91], style(color=0)), 
          Line(points=[-51, 49; 69, 49], style(color=0)), 
          Line(points=[38, 70; 80, 70], style(color=0)), 
          Line(points=[-57, 0; 23, 0], style(color=0, fillColor=10)), 
          Line(points=[-10, -30; 1, -22; 9, -9; 9, 7; 2, 20; -9, 27], style(
                color=10)), 
          Polygon(points=[-10, 27; -2, 14; 5, 25; -9, 27; -10, 27], style(color
                =0, fillColor=10))), 
        Window(
          x=0.05, 
          y=0.19, 
          width=0.51, 
          height=0.7), 
        Documentation(info="<HTML>
<p>
This class provides a translational and rotational spring damper where the forces generated by the spring and the damper is read from a table and interpolated. The table can be given as parameters or with a record.
In this version, no cross-coupling effects are considered, i.e. forces and torques are only generated in the direction of deflection.
</p>
<pre>
This must be fixed together with the parameterisation.
  s_rel0a:      Unstretched spring length
  tableName_C:  table name on file or in function usertab(optional) (spring force)
  fileName_C:   NoName\" \"file where matrix is stored (optional) (spring force)
  icol_C:       columns of table to be interpolated (spring force)
  tableName_D:  table name on file or in function usertab(optional) (damper force) 
  fileName_D:   NoName\" \"file where matrix is stored (optional) (damper force)
  icol_D:       columns of table to be interpolated (damper force)
  forceTable: To set the force generation via a record, add a component named [name] of classVehicleDynamics.Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      ModelicaAdditions.Tables.CombiTable1D r1(table=data.r1)
        annotation (extent=[-60, 56; -40, 76]);
      ModelicaAdditions.Tables.CombiTable1D phi1(table=data.phi1)
        annotation (extent=[24, 57; 44, 77]);
      ModelicaAdditions.Tables.CombiTable1D v1(table=data.v1)
        annotation (extent=[-60, -34; -40, -14]);
      ModelicaAdditions.Tables.CombiTable1D w1(table=data.w1)
        annotation (extent=[44, -34; 64, -14]);
      Utilities.ForceTable3D data annotation (extent=[-28, -72; -8, -52]);
      ModelicaAdditions.Tables.CombiTable1D r2(table=data.r2)
        annotation (extent=[-60, 34; -40, 54]);
      ModelicaAdditions.Tables.CombiTable1D r3(table=data.r3)
        annotation (extent=[-60, 3; -40, 23]);
      ModelicaAdditions.Tables.CombiTable1D v2(table=data.v2)
        annotation (extent=[-69, -58; -49, -38]);
      ModelicaAdditions.Tables.CombiTable1D v3(table=data.v3)
        annotation (extent=[-64, -84; -44, -64]);
      ModelicaAdditions.Tables.CombiTable1D phi2(table=data.phi2)
        annotation (extent=[39, 21; 59, 41]);
      ModelicaAdditions.Tables.CombiTable1D phi3(table=data.phi3)
        annotation (extent=[78, 34; 98, 54]);
      ModelicaAdditions.Tables.CombiTable1D w2(table=data.w2)
        annotation (extent=[42, -58; 62, -38]);
      ModelicaAdditions.Tables.CombiTable1D w3(table=data.w3)
        annotation (extent=[60, -81; 80, -61]);
    equation 
      
      fb = S_rel*fa;
      tb = S_rel*ta - S_rel*cross(r_rela, fa);
      
      {r1.inPort.signal[1],r2.inPort.signal[1],r3.inPort.signal[1]} = r_rela - 
        r_rela0;
      {v1.inPort.signal[1],v2.inPort.signal[1],v3.inPort.signal[1]} = v_rela;
      
      {phi1.inPort.signal[1],phi2.inPort.signal[1],phi3.inPort.signal[1]} = 
        phi_rela - phi_rela0;
      {w1.inPort.signal[1],w2.inPort.signal[1],w3.inPort.signal[1]} = w_rela;
      
      rotationVector.S = S_rel;
      rotationVector.THETA = phi_rela;
      
      fa = {r1.outPort.signal[1] + v1.outPort.signal[1],r2.outPort.signal[1] + 
        v2.outPort.signal[1],r3.outPort.signal[1] + v3.outPort.signal[1]};
      ta = {phi1.outPort.signal[1] + w1.outPort.signal[1],phi2.outPort.signal[1]
         + w2.outPort.signal[1],phi3.outPort.signal[1] + w3.outPort.signal[1]};
    end SpringDamperTable3D;
    
    model SpringDamperTableLine3D 
      "SpringDamperTableLine3D - nonlinear translational spring-damper model to use with ModelicaAdditions.MultiBody"
       
      
      parameter Boolean animation=true "True, if animation shall be enabled";
      parameter SI.Position width=0.09 
        "|Animation|if animation = true| Width of spring";
      parameter Real coilWidth=width/10 
        "|Animation|if animation = true| Width of spring coil";
      parameter Integer numberOfWindings=5 
        "|Animation|if animation = true| Number of spring windings";
      parameter Real material[4]={0.5,0.5,0.5,0.8} 
        "|Animation|if animation = true| Color and specular coefficient of spring";
      
      parameter SI.Position s_rel0=0 "Unstretched spring length";
      
      parameter Real[:, 2] fc=data.fc "Table to be interpolated";
      parameter Real[:, 2] fd=data.fd "Table to be interpolated";
      
      parameter String tableName_C=data.tableName_C 
        "table name on file or in function usertab(optional)";
      parameter String fileName_C=data.fileName_C 
        "file where matrix is stored (optional)";
      parameter Real icol_C[:]=data.icol_C 
        "columns of table to be interpolated";
      parameter String tableName_D=data.tableName_D 
        "table name on file or in function usertab(optional)";
      parameter String fileName_D=data.fileName_D 
        "file where matrix is stored (optional)";
      parameter Real icol_D[:]=data.icol_D 
        "columns of table to be interpolated";
      
      parameter Real n1[3]={0,0,1} "orthogonal to y-axis of shapes";
      
    protected 
      Real material2[4];
      extends ModelicaAdditions.MultiBody.Interfaces.LineForce;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Diagram, 
        Icon(
          Line(points=[-80, 69; -80, -70], style(color=0)), 
          Line(points=[80, 70; 80, -70], style(color=0)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[80, 0; 90, 0], style(color=0)), 
          Line(points=[-60, 40; -60, -40; 60, -40; 60, 40; 30, 40; 30, -40; -30, 
                 -40; -30, 40; -60, 40; -60, 20; 60, 20; 60, 0; -60, 0; -60, -20; 
                 60, -20; 60, -40; -60, -40; -60, 40; 60, 40; 60, -40], style(
                color=8)), 
          Line(points=[0, 40; 0, -40], style(color=8)), 
          Rectangle(extent=[-60, 40; -30, 20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 20; -30, 0], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 0; -30, -20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, -20; -30, -40], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Line(points=[-80, -70; -60, -70; -45, -100; -15, -40; 15, -100; 45, -40; 
                 60, -70; 80, -70], style(color=0)), 
          Polygon(points=[53, 0; 23, 10; 23, -10; 53, 0], style(color=0, 
                fillColor=10)), 
          Line(points=[-80, 70; -52, 70], style(color=0)), 
          Rectangle(extent=[-52, 91; 38, 49], style(color=0, fillColor=8)), 
          Line(points=[-52, 91; 68, 91], style(color=0)), 
          Line(points=[-51, 49; 69, 49], style(color=0)), 
          Line(points=[38, 70; 80, 70], style(color=0)), 
          Line(points=[-57, 0; 23, 0], style(color=0, fillColor=10))), 
        Window(
          x=0.13, 
          y=0.07, 
          width=0.55, 
          height=0.48), 
        Documentation(info="<HTML>
<p>
This class provides a translational spring damper where the forces generated by the spring and the damper is read from a table and interpolated. The table can be given as parameters or with a record.
</p>
<pre>
  s_rel0:       Unstretched spring length
  tableName_C:  table name on file or in function usertab(optional) (spring force)
  fileName_C:   NoName\" \"file where matrix is stored (optional) (spring force)
  icol_C:       columns of table to be interpolated (spring force)
  tableName_D:  table name on file or in function usertab(optional) (damper force) 
  fileName_D:   NoName\" \"file where matrix is stored (optional) (damper force)
  icol_D:       columns of table to be interpolated (damper force)
  forceTable: To set the force generation via a record, add a component named [name] of classVehicleDynamics.Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
    public 
      Utilities.ForceTable1D data annotation (extent=[-20, -80; 20, -40]);
      ModelicaAdditions.Tables.CombiTable1D Dt(
        tableName=tableName_D, 
        fileName=fileName_D, 
        icol=icol_D, 
        table=fd) annotation (extent=[-20, 40; 20, 80]);
      ModelicaAdditions.Tables.CombiTable1D Ct(
        tableName=tableName_C, 
        fileName=fileName_C, 
        icol=icol_C, 
        table=fc) annotation (extent=[-20, -20; 20, 20]);
      
    protected 
      Real nx[3];
      Real[3] b1;
      Real normb;
      Real[3] ny;
      Real[3] nz;
      Real[3, 3] S_relc;
      Real[3, 3] Sc;
      
    protected 
      parameter Integer ndim=if animation then 1 else 0;
    protected 
      Utilities.Shape shape[ndim](
        each shapeType="spring", 
        each Material=material, 
        each length=s, 
        each width=width, 
        each height=coilWidth*2, 
        each lengthDirection=na, 
        each widthDirection={1,1,-2}, 
        each Extra=numberOfWindings, 
        each r=frame_a.r0, 
        each S=Sc) annotation (extent=[-80, -80; -60, -60]);
      Utilities.Shape damperShapeA[ndim](
        each length=s, 
        each lengthDirection=na, 
        each widthDirection={1,1,-2}, 
        each r=frame_a.r0, 
        each S=Sc, 
        each shapeType="cylinder", 
        each width=width/2, 
        each height=width/2, 
        each Material=material2) annotation (extent=[30, -80; 50, -60]);
    public 
      VisualVector force(
        category="force", 
        r=r0a, 
        Size=fa, 
        S=Sa) annotation (extent=[-80, 80; -60, 60]);
    protected 
      Utilities.Shape damperShapeB[ndim](
        each Material=material, 
        each lengthDirection=na, 
        each widthDirection={1,1,-2}, 
        each S=Sc, 
        each shapeType="cylinder", 
        each length=-s_rel0/2, 
        each r=frame_b.r0, 
        each width=width*0.75, 
        each height=width*0.75) annotation (extent=[60, -80; 80, -60]);
    equation 
      
      Ct.inPort.signal[1] = s - s_rel0;
      Dt.inPort.signal[1] = sd;
      
      nx = na;
      b1 = cross(n1, nx);
      normb = sqrt(b1*b1);
      ny = b1/normb;
      nz = cross(nx, ny);
      S_relc = transpose([nx, ny, nz]);
      Sc = Sa*transpose(S_relc);
      
      material2[1] = if (s < -s_rel0/2) then 0 else if (s < 1.5*s_rel0) then -0.5
         + s/s_rel0 else 1;
      material2[2] = 0.5;
      material2[3] = 1 - material2[1];
      material2[4] = 0.9;
      
      f = Ct.outPort.signal[1] + Dt.outPort.signal[1];
    end SpringDamperTableLine3D;
    
    model SpringDamperTableRot1D 
      "SpringDamperTable1D - nonlinear spring-damper model to use with Modelica.Mechanics.Rotational"
       
      
      extends Modelica.Mechanics.Rotational.Interfaces.Compliant;
      
      parameter SI.Angle phi_rel0=0 "Unstretched spring length";
      
      parameter Real[:, 2] fc=data.fc "Table to be interpolated";
      parameter Real[:, 2] fd=data.fd "Table to be interpolated";
      
      parameter String tableName_C=data.tableName_C 
        "table name on file or in function usertab(optional)";
      parameter String fileName_C=data.fileName_C 
        "file where matrix is stored (optional)";
      parameter Real icol_C[:]=data.icol_C 
        "columns of table to be interpolated";
      parameter String tableName_D=data.tableName_D 
        "table name on file or in function usertab(optional)";
      parameter String fileName_D=data.fileName_D 
        "file where matrix is stored (optional)";
      parameter Real icol_D[:]=data.icol_D 
        "columns of table to be interpolated";
      
      SI.Velocity w_rel "relative velocity between flange L and R";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Documentation(info="<HTML>
<p>
This class provides a rotational spring damper where the forces generated by the spring and the damper is read from a table and interpolated. The table can be given as parameters or with a record.
</p>
<pre>
  phi_rel0:     Unstretched spring length
  tableName_C:  table name on file or in function usertab(optional) (spring force)
  fileName_C:   NoName\" \"file where matrix is stored (optional) (spring force)
  icol_C:       columns of table to be interpolated (spring force)
  tableName_D:  table name on file or in function usertab(optional) (damper force) 
  fileName_D:   NoName\" \"file where matrix is stored (optional) (damper force)
  icol_D:       columns of table to be interpolated (damper force)
  forceTable: To set the force generation via a record, add a component named [name] of classVehicleDynamics.Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"), 
        Icon(
          Line(points=[-80, 69; -80, -70], style(color=0)), 
          Line(points=[80, 70; 80, -70], style(color=0)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[80, 0; 90, 0], style(color=0)), 
          Line(points=[-60, 40; -60, -40; 60, -40; 60, 40; 30, 40; 30, -40; -30, 
                 -40; -30, 40; -60, 40; -60, 20; 60, 20; 60, 0; -60, 0; -60, -20; 
                 60, -20; 60, -40; -60, -40; -60, 40; 60, 40; 60, -40], style(
                color=8)), 
          Line(points=[0, 40; 0, -40], style(color=8)), 
          Rectangle(extent=[-60, 40; -30, 20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 20; -30, 0], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, 0; -30, -20], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Rectangle(extent=[-60, -20; -30, -40], style(
              color=8, 
              fillColor=52, 
              fillPattern=1)), 
          Line(points=[-80, -70; -60, -70; -45, -100; -15, -40; 15, -100; 45, -40; 
                 60, -70; 80, -70], style(color=0)), 
          Line(points=[-80, 70; -52, 70], style(color=0)), 
          Rectangle(extent=[-52, 91; 38, 49], style(color=0, fillColor=8)), 
          Line(points=[-52, 91; 68, 91], style(color=0)), 
          Line(points=[-51, 49; 69, 49], style(color=0)), 
          Line(points=[38, 70; 80, 70], style(color=0)), 
          Line(points=[-10, -30; 1, -22; 9, -9; 9, 7; 2, 20; -9, 27], style(
                color=10)), 
          Polygon(points=[-10, 27; -2, 14; 5, 25; -9, 27; -10, 27], style(color
                =0, fillColor=10))), 
        Diagram(
          Polygon(points=[-120, 3; -110, 0; -120, -3; -120, 3], style(color=0, 
                fillColor=0)), 
          Line(points=[-120, 0; -149, 0], style(color=0, fillColor=0)), 
          Line(points=[139, -1; 110, -1], style(color=0, fillColor=0)), 
          Polygon(points=[139, 2; 149, -1; 139, -4; 139, 2], style(color=0, 
                fillColor=0)), 
          Line(points=[-90, 0; -80, 0], style(color=0))), 
        Window(
          x=0.1, 
          y=0.17, 
          width=0.89, 
          height=0.63));
      
    public 
      ModelicaAdditions.Tables.CombiTable1D Dr(
        tableName=tableName_D, 
        fileName=fileName_D, 
        icol=icol_D, 
        table=fd) annotation (extent=[-20, 40; 20, 80]);
      ModelicaAdditions.Tables.CombiTable1D Cr(
        tableName=tableName_C, 
        fileName=fileName_C, 
        icol=icol_C, 
        table=fc) annotation (extent=[-20, -20; 20, 20]);
      parameter Utilities.ForceTable1D data
        annotation (extent=[-20, -80; 20, -40]);
    equation 
      w_rel = der(phi_rel);
      
      Cr.inPort.signal[1] = phi_rel - phi_rel0;
      Dr.inPort.signal[1] = w_rel;
      
      tau = Cr.outPort.signal[1] + Dr.outPort.signal[1];
    end SpringDamperTableRot1D;
    
    model SpringDamperLinear3D 
      extends ModelicaAdditions.MultiBody.Interfaces.Interact2;
      parameter SI.Position[3] r_rela0={0,0,0} 
        "Unstretched translational spring vector";
      parameter SI.Angle[3] phi_rela0={0,0,0} 
        "Unstretched rotational spring vector";
      
    protected 
      SI.Angle[3] phi_rela;
    public 
      VehicleDynamics.Utilities.Forces.Utilities.RotationVector rotationVector
        annotation (extent=[-20, -22; 20, 20]);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-80, 69; -80, -70], style(color=0)), 
          Line(points=[80, 70; 80, -70], style(color=0)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[80, 0; 90, 0], style(color=0)), 
          Line(points=[-80, -70; -60, -70; -45, -100; -15, -40; 15, -100; 45, -40; 
                 60, -70; 80, -70], style(color=0)), 
          Line(points=[-80, 70; -52, 70], style(color=0)), 
          Rectangle(extent=[-52, 91; 38, 49], style(color=0, fillColor=8)), 
          Line(points=[-52, 91; 68, 91], style(color=0)), 
          Line(points=[-51, 49; 69, 49], style(color=0)), 
          Line(points=[38, 70; 80, 70], style(color=0)), 
          Line(points=[-45, 37; -45, -33]), 
          Line(points=[-45, -33; 33, -33]), 
          Polygon(points=[33, -33; 25, -29; 25, -37; 33, -33], style(fillColor=
                  73)), 
          Polygon(points=[-45, 37; -48, 28; -42, 28; -45, 36; -45, 37], style(
                fillColor=73)), 
          Line(points=[-45, -33; 24, 22], style(color=0, thickness=2)), 
          Polygon(points=[53, 0; 23, 10; 23, -10; 53, 0], style(color=0, 
                fillColor=10)), 
          Line(points=[-57, 0; 23, 0], style(color=0, fillColor=10)), 
          Line(points=[-9, -32; 2, -24; 10, -11; 10, 5; 3, 18; -8, 25], style(
                color=10)), 
          Polygon(points=[-11, 26; -3, 13; 4, 24; -10, 26; -11, 26], style(
                color=0, fillColor=10))), 
        Window(
          x=0.03, 
          y=0.19, 
          width=0.28, 
          height=0.42), 
        Documentation(info="<HTML>
<p>
This class provides a translational and rotational linear spring damper. No correlation effects between rotation and translation is considered here!
</p>
<pre>
  s_rel0:       Unstretched spring length
  tableName_C:  table name on file or in function usertab(optional) (spring force)
  fileName_C:   NoName\" \"file where matrix is stored (optional) (spring force)
  icol_C:       columns of table to be interpolated (spring force)
  tableName_D:  table name on file or in function usertab(optional) (damper force) 
  fileName_D:   NoName\" \"file where matrix is stored (optional) (damper force)
  icol_D:       columns of table to be interpolated (damper force)
  forceTable: To set the force generation via a record, add a component named [name] of classVehicleDynamics.Utilities.Forces.Utilities.ForceTable1D to the model and write forceTable=[name] in the modifiers row
  Mass and inertia properties not yet solved!  
</pre>
</HTML>
"));
      
    public 
      parameter Boolean animation=true "True, if animation shall be enabled";
      parameter SI.Position width=0.1 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position length=0.1 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position height=0.1 
        "|Animation|if animation = true| Width bushing";
      parameter Real material[4]={0.2,0.2,0.2,0.2} 
        "|Animation|if animation = true| Color and specular coefficient of spring";
      
    protected 
      parameter Integer ndim=if animation then 1 else 0;
    protected 
      Utilities.Shape shapeA[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=-length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=r0a, 
        each S=Sa) annotation (extent=[-80, -80; -60, -60]);
      Utilities.Shape shapeB[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=r0b, 
        each S=Sb) annotation (extent=[-80, -80; -60, -60]);
      
    protected 
      Utilities.Coefficients3DLinear data annotation (extent=[61, 61; 101, 101]);
    equation 
      
      [fa; ta] = -data.C*[phi_rela - phi_rela0; r_rela - r_rela0] - data.D*[
        v_rela; w_rela];
      //fa = -data.Ct*(r_rela - r_rela0) - data.Dt*v_rela;
      //ta = -data.Cr*(phi_rela - phi_rela0) - data.Dr*w_rela;
      
      fb = S_rel*fa;
      tb = S_rel*ta - S_rel*cross(r_rela, fa);
      
      rotationVector.S = S_rel;
      rotationVector.THETA = phi_rela;
    end SpringDamperLinear3D;
    
    model ElastoGap "1D translational spring damper combination with gap" 
      
      extends Modelica.Mechanics.Translational.Interfaces.Compliant;
      parameter SI.Position s_rel0=0 "unstretched spring length";
      parameter Real c(
        final unit="N/m", 
        final min=0) = 1 "spring constant";
      parameter Real d(
        final unit="N/ (m/s)", 
        final min=0) = 1 "damping constant";
      SI.Velocity v_rel "relative velocity between flange L and R";
      Boolean Contact "false, if s_rel > l ";
      Real s1;
      Real v1;
      Real aux;
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.23, 
          y=0.04, 
          width=0.62, 
          height=0.69), 
        Documentation(info="<HTML>
<p>
A linear translational spring damper combination that can lose contact with flange_a, thus, the spring-damper can only push.
Unlike the Modelica.Mechanics.Translational.ElastoGap this component takes the dynamics of the damper into account. This removes the problem with the ElastoGap generating unphysical force spikes in the pullinig direction. The difference is illustrated i the figures below.</p>
<p>
<IMG SRC=\"../Images/ElastoStandard.png\" ALT=\"Standard ElastoGap\"> </p><p><b>Figure
1</b> The standard ElastoGap.</p>
<p>
<IMG SRC=\"../Images/ElastoNew.png\" ALT=\"New ElastoGap\"> </p><p><b>Figure
2</b> The new ElastoGap takes the dynamics of the damper into account which gives a different motion.</p>
<p>
A suggestion is instead to make two verion, push and pull, wher push would be a proper ElastoGap and pull would be typically a \"rubber spring.\"
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>-date--</i>
       by <a href=\"http://www.fkt.kht.se/FDN/\">Johan Andreasson</a>:<br>
       Realised.</li>
</ul>
</HTML>
"), 
        Diagram(
          Line(points=[-100, 0; -50, 0], style(color=58)), 
          Line(points=[-48, 34; -48, -46], style(color=0, thickness=4)), 
          Line(points=[8, 40; 8, 2], style(color=0)), 
          Line(points=[-2, 0; 38, 0; 38, 44; -2, 44], style(color=0)), 
          Line(points=[38, 22; 72, 22], style(color=0)), 
          Line(points=[-12, -38; -12, 20], style(color=0, thickness=4)), 
          Line(points=[-12, 22; 8, 22], style(color=0)), 
          Line(points=[-12, -38; -2, -38], style(color=0)), 
          Line(points=[72, 0; 90, 0], style(color=58)), 
          Line(points=[72, 22; 72, -42], style(color=0)), 
          Line(points=[-2, -38; 10, -28; 22, -48; 38, -28; 50, -48; 64, -28; 72, 
                 -40], style(color=0)), 
          Rectangle(extent=[8, 44; 38, 0], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Text(
            extent=[-28, -80; 12, -55], 
            string="s_rel", 
            style(color=3)), 
          Line(points=[-100, -29; -100, -61], style(color=0)), 
          Line(points=[100, -61; 100, -28], style(color=0)), 
          Line(points=[-98, -60; 98, -60], style(color=0)), 
          Polygon(points=[-101, -60; -96, -59; -96, -61; -101, -60], style(
              color=0, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[100, -60; 95, -61; 95, -59; 100, -60], style(
              color=0, 
              fillColor=0, 
              fillPattern=1))), 
        Icon(
          Line(points=[-100, 0; -50, 0], style(color=58)), 
          Line(points=[-48, 34; -48, -46], style(color=0, thickness=4)), 
          Line(points=[8, 40; 8, 2], style(color=0)), 
          Line(points=[-2, 0; 38, 0; 38, 44; -2, 44], style(color=0)), 
          Line(points=[38, 22; 72, 22], style(color=0)), 
          Line(points=[-12, -38; -12, 20], style(color=0, thickness=4)), 
          Line(points=[-12, 22; 8, 22], style(color=0)), 
          Line(points=[-12, -38; -2, -38], style(color=0)), 
          Line(points=[74, 0; 100, 0], style(color=58)), 
          Line(points=[72, 22; 72, -42], style(color=0)), 
          Line(points=[-2, -38; 10, -28; 22, -48; 38, -28; 50, -48; 64, -28; 72, 
                 -40], style(color=0)), 
          Rectangle(extent=[8, 44; 38, 0], style(
              color=0, 
              fillColor=8, 
              fillPattern=1)), 
          Line(points=[-60, -90; 20, -90], style(color=0, fillColor=10)), 
          Polygon(points=[50, -90; 20, -80; 20, -100; 50, -90], style(color=10, 
                 fillColor=10)), 
          Text(extent=[0, 120; 0, 60], string="%name")));
    equation 
      
      v_rel = der(s_rel);
      //  Contact = s_rel < s_rel0;
      //  f = if Contact then (c*(s_rel - s_rel0) + d*v_rel) else 0;
      
      v1 = der(s1);
      aux = -s1*c/d;
      v1 = if (v_rel < -s1*c/d and Contact) then v_rel else -s1*c/d;
      
      Contact = s1 >= s_rel;
      
      f = if Contact then (c*(s_rel - s_rel0) + d*v_rel) else 0;
      
    end ElastoGap;
    
    package Examples 
      extends Modelica.Icons.Library;
      model ElastoTest 
        parameter SI.Mass m=25;
        parameter Real g=9.81;
        parameter Real t=0.1;
        parameter Real a=0.2;
        parameter Real c=200000;
        parameter Real d=10000;
        parameter SI.Mass m1=400;
        parameter Real c1=20000;
        parameter Real d1=2000;
        
        VehicleDynamics.Utilities.Forces.ElastoGap ElastoGap1(c=c, d=d)
          annotation (extent=[-102, -12; -82, 8], rotation=90);
        Modelica.Mechanics.Translational.SlidingMass SlidingMass1(m=m)
          annotation (extent=[-102, 28; -82, 48], rotation=90);
        annotation (Diagram);
        Modelica.Mechanics.Translational.Force Force1
          annotation (extent=[-62, 18; -42, 38], rotation=180);
        Modelica.Blocks.Sources.Constant Constant1(k={-m*9.82})
          annotation (extent=[-22, 18; -2, 38], rotation=180);
        Modelica.Mechanics.Translational.Position Position1(f_crit=200)
          annotation (extent=[-102, -52; -82, -32], rotation=90);
        Modelica.Blocks.Sources.SawTooth SawTooth1(amplitude={a}, period={t})
          annotation (extent=[-122, -72; -102, -52]);
        Modelica.Mechanics.Translational.SlidingMass SlidingMass2(m=m1)
          annotation (extent=[-102, 68; -82, 88], rotation=90);
        Modelica.Mechanics.Translational.Force Force2
          annotation (extent=[-62, 58; -42, 78], rotation=180);
        Modelica.Blocks.Sources.Constant Constant2(k={-m1*9.82})
          annotation (extent=[-22, 58; -2, 78], rotation=180);
        Modelica.Mechanics.Translational.SpringDamper SpringDamper1(c=c1, d=d1)
          annotation (extent=[-124, 48; -104, 68], rotation=90);
        Modelica.Mechanics.Translational.SlidingMass SlidingMass3(m=m)
          annotation (extent=[30, 20; 50, 40], rotation=90);
        Modelica.Mechanics.Translational.Force Force3
          annotation (extent=[70, 10; 90, 30], rotation=180);
        Modelica.Blocks.Sources.Constant Constant3(k={-m*9.82})
          annotation (extent=[110, 10; 130, 30], rotation=180);
        Modelica.Mechanics.Translational.Position Position2(f_crit=200)
          annotation (extent=[30, -60; 50, -40], rotation=90);
        Modelica.Blocks.Sources.SawTooth SawTooth2(amplitude={a}, period={t})
          annotation (extent=[10, -80; 30, -60]);
        Modelica.Mechanics.Translational.SlidingMass SlidingMass4(m=m1)
          annotation (extent=[30, 60; 50, 80], rotation=90);
        Modelica.Mechanics.Translational.Force Force4
          annotation (extent=[70, 50; 90, 70], rotation=180);
        Modelica.Blocks.Sources.Constant Constant4(k={-m1*9.82})
          annotation (extent=[110, 50; 130, 70], rotation=180);
        Modelica.Mechanics.Translational.SpringDamper SpringDamper2(c=c1, d=d1)
          annotation (extent=[8, 40; 28, 60], rotation=90);
        Modelica.Mechanics.Translational.ElastoGap ElastoGap2(c=c, d=d)
          annotation (extent=[30, -20; 50, 0], rotation=90);
      equation 
        connect(SlidingMass1.flange_a, ElastoGap1.flange_b)
          annotation (points=[-92, 28; -92, 8], style(color=58));
        connect(Force1.flange_b, SlidingMass1.flange_a)
          annotation (points=[-62, 28; -92, 28], style(color=58));
        connect(Position1.flange_b, ElastoGap1.flange_a)
          annotation (points=[-92, -32; -92, -12], style(color=58));
        connect(SawTooth1.outPort, Position1.inPort)
          annotation (points=[-100, -62; -92, -54], style(color=3));
        connect(Constant1.outPort, Force1.inPort)
          annotation (points=[-24, 28; -40, 28], style(color=3));
        connect(Force2.flange_b, SlidingMass2.flange_a)
          annotation (points=[-62, 68; -92, 68], style(color=58));
        connect(Constant2.outPort, Force2.inPort)
          annotation (points=[-24, 68; -40, 68], style(color=3));
        connect(SpringDamper1.flange_a, SlidingMass1.flange_b)
          annotation (points=[-114, 48; -92, 48], style(color=58));
        connect(SpringDamper1.flange_b, SlidingMass2.flange_a)
          annotation (points=[-114, 68; -92, 68], style(color=58));
        connect(Force3.flange_b, SlidingMass3.flange_a)
          annotation (points=[70, 20; 40, 20], style(color=58));
        connect(SawTooth2.outPort, Position2.inPort)
          annotation (points=[32, -70; 40, -62], style(color=3));
        connect(Constant3.outPort, Force3.inPort)
          annotation (points=[108, 20; 92, 20], style(color=3));
        connect(Force4.flange_b, SlidingMass4.flange_a)
          annotation (points=[70, 60; 40, 60], style(color=58));
        connect(Constant4.outPort, Force4.inPort)
          annotation (points=[108, 60; 92, 60], style(color=3));
        connect(SpringDamper2.flange_a, SlidingMass3.flange_b)
          annotation (points=[18, 40; 40, 40], style(color=58));
        connect(SpringDamper2.flange_b, SlidingMass4.flange_a)
          annotation (points=[18, 60; 40, 60], style(color=58));
        connect(ElastoGap2.flange_a, Position2.flange_b)
          annotation (points=[40, -20; 40, -40], style(color=58));
        connect(ElastoGap2.flange_b, SlidingMass3.flange_a)
          annotation (points=[40, 0; 40, 20], style(color=58));
      end ElastoTest;
      
      model VerticalTyre 
        Modelica.Mechanics.Translational.ElastoGap ElastoGap1(c=200000, d=200)
          annotation (extent=[-40, -20; -20, 0], rotation=90);
        Modelica.Mechanics.Translational.SlidingMass SlidingMass1(m=25, L=0.6)
          annotation (extent=[-40, 10; -20, 30], rotation=90);
        Modelica.Mechanics.Translational.SpringDamper SpringDamper1(c=20000, d=
              2000) annotation (extent=[-40, 40; -20, 60], rotation=90);
        Modelica.Mechanics.Translational.SlidingMass SlidingMass2(m=400, L=0.3)
          annotation (extent=[-40, 70; -20, 90], rotation=90);
        annotation (Diagram);
        Modelica.Mechanics.Translational.Position Position1
          annotation (extent=[-40, -50; -20, -30], rotation=90);
        Modelica.Blocks.Sources.SawTooth SawTooth1(amplitude={0.6})
          annotation (extent=[-80, -80; -60, -60]);
        Modelica.Mechanics.Translational.Force Force1
          annotation (extent=[0, 0; 20, 20], rotation=180);
        Modelica.Mechanics.Translational.Force Force2
          annotation (extent=[0, 60; 20, 80], rotation=180);
        Modelica.Blocks.Sources.Constant Constant1(k={-25*9.82})
          annotation (extent=[40, 0; 60, 20], rotation=180);
        Modelica.Blocks.Sources.Constant Constant2(k={-400*9.82})
          annotation (extent=[40, 60; 60, 80], rotation=180);
      equation 
        connect(SlidingMass2.flange_a, SpringDamper1.flange_b)
          annotation (points=[-30, 70; -30, 60], style(color=58));
        connect(SpringDamper1.flange_a, SlidingMass1.flange_b)
          annotation (points=[-30, 40; -30, 30], style(color=58));
        connect(SlidingMass1.flange_a, ElastoGap1.flange_b)
          annotation (points=[-30, 10; -30, 0], style(color=58));
        connect(Position1.flange_b, ElastoGap1.flange_a)
          annotation (points=[-30, -30; -30, -20], style(color=58));
        connect(SawTooth1.outPort, Position1.inPort)
          annotation (points=[-58, -70; -30, -70; -30, -54], style(color=3));
        connect(Force2.flange_b, SlidingMass2.flange_a)
          annotation (points=[0, 70; -30, 70], style(color=58));
        connect(Force1.flange_b, SlidingMass1.flange_a)
          annotation (points=[0, 10; -30, 10], style(color=58));
        connect(Constant1.outPort, Force1.inPort)
          annotation (points=[38, 10; 24, 10], style(color=3));
        connect(Constant2.outPort, Force2.inPort)
          annotation (points=[40, 70; 24, 70], style(color=3));
      end VerticalTyre;
      
    end Examples;
    
    model SpringDamperLinearBump3D_freeMotion 
      "Spring-damper in parallel with bumps at ends." 
      import SI = Modelica.SIunits;
      parameter SI.Position[3] r_rela0={0,0,0} 
        "Unstretched translational spring vector";
      parameter SI.Position[3] r_min={-0.05,-0.03,-0.06} 
        "vector to bump stop from reference point in negative direction";
      parameter SI.Position[3] r_max={0.05,0.03,0.06} 
        "vector to bump stop from reference point in positive direction";
      parameter Real bumpFactor=1000 
        "scaleFactor of spring force at after bump";
      parameter SI.Angle[3] phi_rela0={0,0,0} 
        "Unstretched rotational spring vector";
      
      parameter Real[6, 6] C=[10000, 0, 0, 0, 0, 0; 0, 10000, 0, 0, 0, 0; 0, 0, 
           10000, 0, 0, 0; 0, 0, 0, 10000, 0, 0; 0, 0, 0, 0, 10000, 0; 0, 0, 0, 
           0, 0, 10000] "|Forces|Linear|Stiffness matrix";
      parameter Real[6, 6] D=[10000, 0, 0, 0, 0, 0; 0, 10000, 0, 0, 0, 0; 0, 0, 
           10000, 0, 0, 0; 0, 0, 0, 10000, 0, 0; 0, 0, 0, 0, 10000, 0; 0, 0, 0, 
           0, 0, 10000] "|Forces|Linear|Damping matrix";
      
      parameter Boolean linear=false 
        "turn on/off linearisation of cosine and sine in rotation vector equation";
      
      parameter Real eps=0.0000001 "Guard against division by zero";
      
      parameter Boolean animation=true "True, if animation shall be enabled";
      parameter SI.Position width=r_max[2] - r_min[2] 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position length=r_max[1] - r_min[1] 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position height=r_max[3] - r_min[3] 
        "|Animation|if animation = true| Width bushing";
      parameter Real material[4]={0.2,0.2,0.2,0.2} 
        "|Animation|if animation = true| Color and specular coefficient of spring";
      Real S_rel[3, 3];
      SI.Position r_rela[3](start=zeros(3));
      SI.Velocity v_rela[3];
      SI.AngularVelocity w_rela[3];
    protected 
      parameter Integer ndim=if animation then 1 else 0;
    protected 
      Utilities.Shape shapeA[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=-length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=frame_a.r0, 
        each S=frame_a.S) annotation (extent=[-80, -80; -60, -60]);
      Utilities.Shape shapeB[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=frame_b.r0, 
        each S=frame_b.S) annotation (extent=[-80, -80; -60, -60]);
      
    protected 
      SI.Angle[3] phi_rela;
      Real[3] f_bump;
      SI.Angle phi[3];
      Real s1;
      Real s2;
      Real s3;
      Real c1;
      Real c2;
      Real c3;
      
    public 
      Utilities.RotationVector rotationVector(linear=linear, eps=eps)
        annotation (extent=[-20, -21; 20, 21]);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-80, 69; -80, -70], style(color=0, thickness=2)), 
          Line(points=[80, 70; 80, -70], style(color=0, thickness=2)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[80, 0; 90, 0], style(color=0)), 
          Line(points=[-80, -70; -60, -70; -45, -100; -15, -40; 15, -100; 45, -40; 
                 60, -70; 80, -70], style(color=0)), 
          Line(points=[-80, 70; -52, 70], style(color=0)), 
          Rectangle(extent=[-52, 91; 38, 49], style(color=0, fillColor=8)), 
          Line(points=[-52, 91; 68, 91], style(color=0)), 
          Line(points=[-51, 49; 69, 49], style(color=0)), 
          Line(points=[38, 70; 80, 70], style(color=0)), 
          Line(points=[-45, 37; -45, -33]), 
          Line(points=[-45, -33; 33, -33]), 
          Polygon(points=[33, -33; 25, -29; 25, -37; 33, -33], style(fillColor=
                  73)), 
          Polygon(points=[-45, 37; -48, 28; -42, 28; -45, 36; -45, 37], style(
                fillColor=73)), 
          Line(points=[-45, -33; 0, -13], style(color=0, thickness=2)), 
          Polygon(points=[53, 0; 23, 10; 23, -10; 53, 0], style(color=0, 
                fillColor=10)), 
          Line(points=[-57, 0; 23, 0], style(color=0, fillColor=10)), 
          Line(points=[-9, -32; 2, -24; 10, -11; 10, 5; 3, 18; -8, 25], style(
                color=10)), 
          Polygon(points=[-11, 26; -3, 13; 4, 24; -10, 26; -11, 26], style(
                color=0, fillColor=10)), 
          Line(points=[0, -13; 10, 45], style(color=0, thickness=2)), 
          Text(extent=[-114, -110; 121, -157], string="%name"), 
          Polygon(points=[80, 104; 60, 86; 88, 74; 80, 104], style(color=0, 
                fillColor=8)), 
          Line(points=[-75, 77; -63, 107; -38, 129; -6, 138; 30, 134; 51, 115; 
                70, 94], style(
              color=9, 
              thickness=2, 
              fillColor=8)), 
          Text(
            extent=[-129, -23; -90, -63], 
            string="a", 
            style(color=10)), 
          Text(
            extent=[90, -24; 129, -64], 
            string="b", 
            style(color=10))), 
        Window(
          x=0.02, 
          y=0.01, 
          width=0.92, 
          height=0.94), 
        Documentation(info="<HTML>
<p>
This class provides a translational and rotational linear spring damper with bumpstops. No correlation effects between rotation and translation is considered here. 
The bumps are modelled according to the following. If the relative distance r_rela exceeds the intervall set up by the parameters r_min and r_max, the spring-damper
generated force is amplified with a factor bumpFactor. The reason that not only the spring force is amplified is because it tends to generate high velocities.
</p>
</HTML>
"));
      
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -15; -90, 15]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_b
        annotation (extent=[90, -16; 120, 14]);
    equation 
      /* Determine sines and cosines of the Cardan angles */
      s1 = Modelica.Math.sin(phi[1]);
      s2 = Modelica.Math.sin(phi[2]);
      s3 = Modelica.Math.sin(phi[3]);
      c1 = Modelica.Math.cos(phi[1]);
      c2 = Modelica.Math.cos(phi[2]);
      c3 = Modelica.Math.cos(phi[3]);
      
      /* Relative transformation matrix
       S_rel = [ c3, s3, 0; 
                -s3, c3, 0; 
                  0, 0, 1]*[c2, 0, -s2; 
                             0, 1, 0; 
                            s2, 0, c2]*[1, 0, 0; 
                                        0, c1, s1; 
                                        0, -s1, c1];
  */
      S_rel = [c2*c3, c1*s3 + s1*s2*c3, s1*s3 - c1*s2*c3; -c2*s3, c1*c3 - s1*s2
        *s3, s1*c3 + c1*s2*s3; s2, -s1*c2, c1*c2];
      
      // Kinematic relationships
      frame_b.S = frame_a.S*transpose(S_rel);
      frame_b.r0 = frame_a.r0 + frame_a.S*r_rela;
      frame_b.a = zeros(3);
      frame_b.z = zeros(3);
      
      // Kinematic differential equations for translational motion
      der(r_rela) = v_rela;
      
      // Kinematic differential equations for rotational motion
      /*
  if initial() then
    der(phi) = w_rela;
    frame_b.v = zeros(3);
    frame_b.w = zeros(3);
    f_bump = zeros(3);
    [frame_a.f; frame_a.t] = -C*[r_rela - r_rela0; phi_rela - phi_rela0];
  else
*/
      /*
  der(phi) = {w_rela[1] + (s1*w_rela[2] - c1*w_rela[3])*s2/c2,c1*w_rela[2] + s1
    *w_rela[3],(-s1*w_rela[2] + c1*w_rela[3])/c2};
 */
      w_rela = {der(phi[1]) + s2*der(phi[3]),c1*der(phi[2]) - s1*c2*der(phi[3]),
        s1*der(phi[2]) + c1*c2*der(phi[3])};
      
      frame_b.v = S_rel*(frame_a.v + v_rela + cross(frame_a.w, r_rela));
      frame_b.w = S_rel*(frame_a.w + w_rela);
      
      f_bump[1] = if (r_rela[1] - r_rela0[1] > r_max[1]) then C[1, 1]*
        bumpFactor*(r_rela[1] - r_rela0[1] - r_max[1]) + D[1, 1]*bumpFactor*
        v_rela[1] else if (r_rela[1] - r_rela0[1] > r_min[1]) then 0 else C[1, 
        1]*bumpFactor*(r_rela[1] - r_min[1]) + D[1, 1]*bumpFactor*v_rela[1];
      f_bump[2] = if (r_rela[2] - r_rela0[2] > r_max[2]) then C[2, 2]*
        bumpFactor*(r_rela[2] - r_rela0[2] - r_max[2]) + D[2, 2]*bumpFactor*
        v_rela[2] else if (r_rela[2] - r_rela0[2] > r_min[2]) then 0 else C[2, 
        2]*bumpFactor*(r_rela[2] - r_min[2]) + D[2, 2]*bumpFactor*v_rela[2];
      f_bump[3] = if (r_rela[3] - r_rela0[3] > r_max[3]) then C[3, 3]*
        bumpFactor*(r_rela[3] - r_rela0[3] - r_max[3]) + D[3, 3]*bumpFactor*
        v_rela[3] else if (r_rela[3] - r_rela0[3] > r_min[3]) then 0 else C[3, 
        3]*bumpFactor*(r_rela[3] - r_min[3]) + D[3, 3]*bumpFactor*v_rela[3];
      [frame_a.f + f_bump; frame_a.t] = -C*[r_rela - r_rela0; phi_rela - 
        phi_rela0] - D*[v_rela; {der(phi[1]) + s2*der(phi[3]),c1*der(phi[2]) - 
        s1*c2*der(phi[3]),s1*der(phi[2]) + c1*c2*der(phi[3])}];
      /*
  end if;
*/
      
      -frame_b.f = S_rel*frame_a.f;
      -frame_b.t = S_rel*(frame_a.t - cross(r_rela, frame_a.f));
      
      rotationVector.S = S_rel;
      rotationVector.THETA = phi_rela;
      /*
initial equation 
  der(r_rela) = zeros(3);
  der(phi) = zeros(3);
*/
    end SpringDamperLinearBump3D_freeMotion;
    
    model SpringDamperLinearBump3D 
      "Spring-damper in parallel with bumps at ends." 
      import SI = Modelica.SIunits;
      extends ModelicaAdditions.MultiBody.Interfaces.TwoNoTreeFrames;
      parameter SI.Position[3] r_rela0={0,0,0} 
        "Unstretched translational spring vector";
      parameter SI.Position[3] r_min={-0.05,-0.03,-0.06} 
        "vector to bump stop from reference point in negative direction";
      parameter SI.Position[3] r_max={0.05,0.03,0.06} 
        "vector to bump stop from reference point in positive direction";
      parameter Real bumpFactor=1000 
        "scaleFactor of spring force at after bump";
      parameter SI.Angle[3] phi_rela0={0,0,0} 
        "Unstretched rotational spring vector";
      
      parameter Real[6, 6] C=[10000, 0, 0, 0, 0, 0; 0, 10000, 0, 0, 0, 0; 0, 0, 
           10000, 0, 0, 0; 0, 0, 0, 10000, 0, 0; 0, 0, 0, 0, 10000, 0; 0, 0, 0, 
           0, 0, 10000] "|Forces|Linear|Stiffness matrix";
      parameter Real[6, 6] D=[10000, 0, 0, 0, 0, 0; 0, 10000, 0, 0, 0, 0; 0, 0, 
           10000, 0, 0, 0; 0, 0, 0, 10000, 0, 0; 0, 0, 0, 0, 10000, 0; 0, 0, 0, 
           0, 0, 10000] "|Forces|Linear|Damping matrix";
      
      parameter Boolean linear=false 
        "turn on/off linearisation of cosine and sine in rotation vector equation";
      
      parameter Real eps=0.0000001 "Guard against division by zero";
      
      parameter Boolean animation=true "True, if animation shall be enabled";
      parameter SI.Position width=r_max[2] - r_min[2] 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position length=r_max[1] - r_min[1] 
        "|Animation|if animation = true| Width bushing";
      parameter SI.Position height=r_max[3] - r_min[3] 
        "|Animation|if animation = true| Width bushing";
      parameter Real material[4]={0.2,0.2,0.2,0.2} 
        "|Animation|if animation = true| Color and specular coefficient of spring";
      
      Real S_rel[3, 3];
      SI.Position r_rela[3];
      SI.Velocity v_rela[3];
      SI.AngularVelocity w_rela[3];
      SI.Velocity vaux[3];
      
    protected 
      parameter Integer ndim=if animation then 1 else 0;
    public 
      Utilities.Shape shapeA[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=-length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=r0a, 
        each S=Sa) annotation (extent=[-80, -80; -60, -60]);
      Utilities.Shape shapeB[ndim](
        each shapeType="box", 
        each Material=material, 
        each length=length/2, 
        each width=width, 
        each height=width, 
        each lengthDirection={1,0,0}, 
        each widthDirection={0,1,0}, 
        each r=r0b, 
        each S=Sb) annotation (extent=[-80, -80; -60, -60]);
      
    protected 
      SI.Angle[3] phi_rela;
      Real[3] f_bump;
    public 
      Utilities.RotationVector rotationVector(linear=linear, eps=eps)
        annotation (extent=[-20, -21; 20, 21]);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-80, 69; -80, -70], style(color=0, thickness=2)), 
          Line(points=[80, 70; 80, -70], style(color=0, thickness=2)), 
          Line(points=[-90, 0; -80, 0], style(color=0)), 
          Line(points=[80, 0; 90, 0], style(color=0)), 
          Line(points=[-80, -70; -60, -70; -45, -100; -15, -40; 15, -100; 45, -40; 
                 60, -70; 80, -70], style(color=0)), 
          Line(points=[-80, 70; -52, 70], style(color=0)), 
          Rectangle(extent=[-52, 91; 38, 49], style(color=0, fillColor=8)), 
          Line(points=[-52, 91; 68, 91], style(color=0)), 
          Line(points=[-51, 49; 69, 49], style(color=0)), 
          Line(points=[38, 70; 80, 70], style(color=0)), 
          Line(points=[-45, 37; -45, -33]), 
          Line(points=[-45, -33; 33, -33]), 
          Polygon(points=[33, -33; 25, -29; 25, -37; 33, -33], style(fillColor=
                  73)), 
          Polygon(points=[-45, 37; -48, 28; -42, 28; -45, 36; -45, 37], style(
                fillColor=73)), 
          Line(points=[-45, -33; 0, -13], style(color=0, thickness=2)), 
          Polygon(points=[53, 0; 23, 10; 23, -10; 53, 0], style(color=0, 
                fillColor=10)), 
          Line(points=[-57, 0; 23, 0], style(color=0, fillColor=10)), 
          Line(points=[-9, -32; 2, -24; 10, -11; 10, 5; 3, 18; -8, 25], style(
                color=10)), 
          Polygon(points=[-11, 26; -3, 13; 4, 24; -10, 26; -11, 26], style(
                color=0, fillColor=10)), 
          Line(points=[0, -13; 10, 45], style(color=0, thickness=2))), 
        Window(
          x=0.02, 
          y=0.01, 
          width=0.92, 
          height=0.94), 
        Documentation(info="<HTML>
<p>
This class provides a translational and rotational linear spring damper with bumpstops. No correlation effects between rotation and translation is considered here. 
The bumps are modelled according to the following. If the relative distance r_rela exceeds the intervall set up by the parameters r_min and r_max, the spring-damper
generated force is amplified with a factor bumpFactor. The reason that not only the spring force is amplified is because it tends to generate high velocities.
</p>
</HTML>
"));
    equation 
      
      S_rel = transpose(Sb)*Sa;
      r_rela = transpose(Sa)*(r0b - r0a);
      
      /*  
  if initial() then
    vaux = zeros(3);
    v_rela = zeros(3);
    w_rela = zeros(3);
    f_bump = zeros(3);
    [fa; ta] = -C*[r_rela - r_rela0; phi_rela - phi_rela0];
  else
*/
      vaux = cross(wa, r_rela);
      v_rela = transpose(S_rel)*vb - va - vaux;
      w_rela = transpose(S_rel)*wb - wa;
      f_bump[1] = if (r_rela[1] - r_rela0[1] > r_max[1]) then C[1, 1]*
        bumpFactor*(r_rela[1] - r_rela0[1] - r_max[1]) + D[1, 1]*bumpFactor*
        v_rela[1] else if (r_rela[1] - r_rela0[1] > r_min[1]) then 0 else C[1, 
        1]*bumpFactor*(r_rela[1] - r_min[1]) + D[1, 1]*bumpFactor*v_rela[1];
      f_bump[2] = if (r_rela[2] - r_rela0[2] > r_max[2]) then C[2, 2]*
        bumpFactor*(r_rela[2] - r_rela0[2] - r_max[2]) + D[2, 2]*bumpFactor*
        v_rela[2] else if (r_rela[2] - r_rela0[2] > r_min[2]) then 0 else C[2, 
        2]*bumpFactor*(r_rela[2] - r_min[2]) + D[2, 2]*bumpFactor*v_rela[2];
      f_bump[3] = if (r_rela[3] - r_rela0[3] > r_max[3]) then C[3, 3]*
        bumpFactor*(r_rela[3] - r_rela0[3] - r_max[3]) + D[3, 3]*bumpFactor*
        v_rela[3] else if (r_rela[3] - r_rela0[3] > r_min[3]) then 0 else C[3, 
        3]*bumpFactor*(r_rela[3] - r_min[3]) + D[3, 3]*bumpFactor*v_rela[3];
      
      [fa + f_bump; ta] = -C*[r_rela - r_rela0; phi_rela - phi_rela0] - D*[
        v_rela; w_rela];
      /*
  end if;
*/
      
      fb = S_rel*fa;
      tb = S_rel*ta - S_rel*cross(r_rela, fa);
      
      rotationVector.S = S_rel;
      rotationVector.THETA = phi_rela;
      
    end SpringDamperLinearBump3D;
  end Forces;
  
  package Functions 
    function EWM 
      input Real a[:];
      input Real b[size(a, 1)];
      output Real c[size(a, 1)];
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Documentation(info="Element wise multiplication of vectors.
"), 
        Window(
          x=0.07, 
          y=0.1, 
          width=0.36, 
          height=0.6), 
        Icon(
          Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=70)), 
             
          Text(
            extent=[-100, 100; 100, 20], 
            string="EWM(a,b)", 
            style(color=7)), 
          Text(
            extent=[-100, 40; 100, -40], 
            string="=", 
            style(color=7)), 
          Text(
            extent=[-100, -20; 100, -100], 
            string="a.*b", 
            style(color=7))));
    algorithm 
      
      for i in 1:size(a, 1) loop
        c[i] := a[i]*b[i];
      end for;
    end EWM;
    
    function from_nxy "Return orientation object from n_x and n_y vectors" 
      extends Modelica.Icons.Function;
      input Real n_x[3] 
        "Vector in direction of x-axis of frame 2, resolved in frame 1";
      input Real n_y[3] 
        "Vector in direction of y-axis of frame 2, resolved in frame 1";
      output Real S[3, 3] "Orientation matrix to rotate frame 1 into frame 2";
    protected 
      Real abs_n_x=sqrt(n_x*n_x);
      Real e_x[3]=if abs_n_x < 1.e-10 then {1,0,0} else n_x/abs_n_x;
      Real n_z_aux[3]=cross(e_x, n_y);
      Real n_y_aux[3]=if n_z_aux*n_z_aux > 1.0e-6 then n_y else (if abs(e_x[1])
           > 1.0e-6 then {0,1,0} else {1,0,0});
      Real e_z_aux[3]=cross(e_x, n_y_aux);
      Real e_z[3]=e_z_aux/sqrt(e_z_aux*e_z_aux);
      Real e_y[3]=cross(e_z, e_x);
    algorithm 
      S := {e_x[:],e_y[:],cross(e_x[:], e_y[:])};
      annotation (Documentation(info="<html>
<p>
It is assumed that the two input vectors n_x and n_y are 
resolved in frame 1 and are directed along the x and y axis
of frame 2 (i.e., n_x and n_y are orthogonal to each other)
The function returns the orientation object R to rotate from 
frame 1 to frame 2. 
</p>
<p>
The function is robust in the sense that it returns always
an orientation object R, even if n_y is not orthogonal to n_x.
This is performed in the following way:
</p>
<p>
If n_x and n_y are not orthogonal to each other, first a unit
vector e_y is determined that is orthogonal to n_x and is lying
in the plane spanned by n_x and n_y. If n_x and n_y are parallel 
or nearly parallel to each other, a vector e_y is selected 
arbitrarily such that e_x and e_y are orthogonal to each.
</p>
</html>"));
    end from_nxy;
    
    function length "Return length of a vector" 
      extends Modelica.Icons.Function;
      input Real r[:] "Vector";
      output Real r_length "length of vector r";
    algorithm 
      r_length := sqrt(r*r);
    end length;
    
    function normalize "Return normalized vector such that length = 1" 
      extends Modelica.Icons.Function;
      input Real r[:] "Vector";
      output Real r_unitlength[size(r, 1)] 
        "Input vector r normalized to length=1";
    algorithm 
      r_unitlength := if length(r) > 1.e-10 then r/length(r) else r;
    end normalize;
  end Functions;
  
  package Parts 
    model FrameAxesNoAcceleration 
      "Identical to FrameAxes but acceleration is set to zero" 
      import SI = Modelica.SIunits;
      
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -15; -90, 15]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_b frame_b
        annotation (extent=[120, 15; 90, -15], rotation=180);
      parameter SI.Position r[3]={0,0,0} 
        "Vector from frame_a to frame_b resolved in frame_a";
      parameter Real nx[3]={1,0,0} "x-axis of frame_b resolved in frame_a";
      parameter Real ny[3]={0,1,0} "y-axis of frame_b resolved in frame_a";
      parameter String label1="x" "Label of horizontal axis in icon";
      parameter String label2="y" "Label of vertical axis in icon";
      final parameter Real nz[3]=cross(nx, ny);
      final parameter Real nnz[3]=nz/sqrt(nz*nz);
      final parameter Real nnx[3]=nx/sqrt(nx*nx);
      final parameter Real nny[3]=cross(nnz, nnx);
      final parameter Real S_rel[3, 3]=transpose([nnx, nny, nnz]) 
        "relative transformation matrix from frame_a to frame_b";
      annotation (
        Icon(
          Rectangle(extent=[-90, 5; 90, -5], style(color=0, fillColor=0)), 
          Line(points=[80, 20; 129, 50], style(color=0)), 
          Line(points=[80, 20; 57, 59], style(color=0)), 
          Polygon(points=[158, 69; 121, 62; 136, 40; 158, 69], style(color=0, 
                fillColor=0)), 
          Polygon(points=[34, 96; 46, 50; 68, 65; 34, 96], style(color=0, 
                fillColor=0)), 
          Text(
            extent=[3, 97; 31, 75], 
            string="%label2", 
            style(color=0)), 
          Text(
            extent=[129, 102; 157, 80], 
            string="%label1", 
            style(color=0)), 
          Text(extent=[-137, -91; 136, -35], string="%name=%r"), 
          Text(
            extent=[-83, 41; 57, 13], 
            string="a=0", 
            style(color=0))), 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.24, 
          y=0.37, 
          width=0.6, 
          height=0.6), 
        Diagram);
    equation 
      frame_b.S = frame_a.S*transpose(S_rel);
      frame_b.r0 = frame_a.r0 + frame_a.S*r;
      frame_b.v = S_rel*(frame_a.v + cross(frame_a.w, r));
      frame_b.w = S_rel*frame_a.w;
      frame_b.a = zeros(3);
      frame_b.z = zeros(3);
      
      /* Transform the force and torque acting at frame_b to frame_a */
      frame_a.f = transpose(S_rel)*(-frame_b.f);
      frame_a.t = transpose(S_rel)*(-frame_b.t) + cross(r, frame_a.f);
    end FrameAxesNoAcceleration;
    
    model Body "Rigid body with visual shape (also used for animation)" 
      extends ModelicaAdditions.MultiBody.Interfaces.TwoTreeFrames;
      parameter Boolean mass=true 
        "|Mass and Inertia| |True if mass should be considered";
      parameter Boolean animation=true 
        "|Animation||True if part should be animated";
      
      parameter SI.Position r[3]={0.2,0,0} 
        "|Geometry||Vector from frame_a to frame_b, resolved in frame_a";
      parameter Real rho(min=0) = 3.2 
        "|Mass and Inertia|Default box parameters|Density of material [g/cm^3]";
      
      parameter String shape="box" "|Animation||shape";
      parameter SI.Length height=0.02 "|Animation||height of shape";
      parameter SI.Length width=0.02 "|Animation||width of shape";
      parameter SI.Position r0[3]={0,0,0} 
        "|Animation||Vector from frame_a to shape frame, resolved in frame_a";
      parameter SI.Length length=Utilities.Functions.length(r - r0) 
        "|Animation||length of shape";
      parameter SI.Position lengthDirection[3]=r - r0 
        "|Animation||Vector in direction of cylinder axis, resolved in frame_a";
      parameter SI.Position widthDirection[3]={0,0,1} 
        "|Animation||Vector in direction perpendiculare to cylinder axis, resolved in frame_a";
      parameter Real material[4]={0.5,0.5,0.5,0.5} 
        "|Animation||Color and specular coefficient";
      parameter Real extra=1.0 "|Animation||Additional parameter";
      
      parameter SI.Position rCM[3]=r0 + length/2*Utilities.Functions.normalize(
          lengthDirection) 
        "|Mass and Inertia|General parameters|Vector from frame_a to center of mass, resolved in frame_a";
      parameter SI.Mass m=1000*rho*length*width*height 
        "|Mass and Inertia|General parameters|Mass of body [kg]";
      parameter SI.Inertia I11=I[1, 1] 
        "|Mass and Inertia|General parameters|(1,1) element of inertia tensor";
      parameter SI.Inertia I22=I[2, 2] 
        "|Mass and Inertia|General parameters|(2,2) element of inertia tensor";
      parameter SI.Inertia I33=I[3, 3] 
        "|Mass and Inertia|General parameters|(3,3) element of inertia tensor";
      parameter SI.Inertia I21=I[2, 1] 
        "|Mass and Inertia|General parameters|(2,1) element of inertia tensor";
      parameter SI.Inertia I31=I[3, 1] 
        "|Mass and Inertia|General parameters|(3,1) element of inertia tensor";
      parameter SI.Inertia I32=I[3, 2] 
        "|Mass and Inertia|General parameters|(3,2) element of inertia tensor";
      
    protected 
      parameter Real S[3, 3]=Utilities.Functions.from_nxy(lengthDirection, 
          widthDirection);
      parameter SI.Inertia I[3, 3]=S*diagonal({m*(width*width + height*height),
          m*(length*length + height*height),m*(length*length + width*width)}/12)
          *transpose(S);
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.29, 
          y=0.12, 
          width=0.61, 
          height=0.68), 
        Documentation(info="
Rigid body with two cuts and an additional shape used in animation.
All the mass and inertia properties must be supplied with respect
to the center of mass. All parameter vectors are resolved in frame_a
in [m].
Parameters:
  r[3]         : Position vector from the origin of frame_a to the origin of
                 frame_b.
  rCM[3]       : Position vector from the origin of frame_a to the center of
                 mass.
  m            : Mass of body in [kg].
  Ixy          : Elements (x,y) of the inertia tensor of the body with respect to
                 the center of mass, resolved in the inertial frame in the home
                 position in [kgm^2]. The following elements can be defined:
                 I11, I22, I33, I21, I31, I32.
                 (Move the inertial from into the center of mass by translation
                 and resolve the inertia tensor in this frame).
  Shape        : Type of visual shape. Predefined shapes are
                   Shape = \"box\"
                           \"sphere\"
                           \"cylinder\"
                           \"cone\"
                           \"pipe\"
                           \"beam\"
                           \"gearwheel\"
                           \"wirebox\"
                 External shapes are specified as DXF-files (only 3DFace is supported).
                 External shapes must be named \"1\", \"2\"  etc. The corresponding definitions
                 should be in files \"1.dxf\", \"2.dxf\" etc.Since the DXF-files contain color
                 and dimensions for the individual faces, the corresponding information
                 in the model is currently ignored, but the specular coefficient is
                 utilized. The DXF-files must be found in the current directory.
  r0[3]        : Position vector from the origin of frame_a to the origin
                 of the visual object.
  LengthDirection[3]: Unit vector in direction of length (will be normalized).
  WidthDirection[3] : Unit vector in direction of width (will be normalized).
  Length       : Length of visual shape in [m].
  Width        : Width of visual shape in [m].
  Height       : Height of visual shape in direction of a vector which is
                 orthogonal to  LengthDirection  and  WidthDirection.
  Material[4]  : = {r, g, b, specular} specifies the color of the shape.
                 [r,g,b] affects the color of diffuse and ambient reflected
                 light. Specular is a coefficient defining white specular
                 reflection. What color that is reflected also depends on the
                 color of the light sources. Note, r g, b and specular are
                 given in the range 0 . 1. Specular=1 gives a metallic appearance.
  Extra        : Additional parameter for cone and pipe.
                 \"pipe\"     : A hollow cylinder. Extra = InnerRadius/Radius of the cylinder.
                 \"gearwheel\": A ring with gear wheel teeth. Extra = number of teeth.
"), 
        Icon(
          Rectangle(extent=[-90, 4; -49, -5], style(
              color=0, 
              gradient=2, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[-50, 34; -50, -60; 0, -40; 40, -60; 40, 36; 0, 12; -50, 
                 34], style(
              color=7, 
              fillColor=46, 
              fillPattern=1)), 
          Polygon(points=[-50, 34; -30, 46; 10, 30; 60, 62; 40, 36; 0, 12; -50, 
                 34], style(
              color=7, 
              fillColor=75, 
              fillPattern=1)), 
          Polygon(points=[59, 61; 40, 36; 40, -60; 58, -36; 59, 61], style(
              color=7, 
              fillColor=75, 
              fillPattern=1)), 
          Text(extent=[-100, 120; 100, 61], string="%name%"), 
          Rectangle(extent=[49, 4; 90, -5], style(
              color=0, 
              gradient=2, 
              fillColor=0, 
              fillPattern=1))));
      
    public 
      ModelicaAdditions.MultiBody.Parts.FrameTranslation translation(r=r)
        annotation (extent=[-40, -40; 40, 40]);
      ModelicaAdditions.MultiBody.Parts.Shape vis[ndimVis](
        each Shape=shape, 
        each r0=r0, 
        each LengthDirection=lengthDirection, 
        each WidthDirection=widthDirection, 
        each Length=length, 
        each Width=width, 
        each Height=height, 
        each Material=material, 
        each Extra=extra) annotation (extent=[-20, -90; 20, -50]);
    protected 
      final parameter Integer ndimMass=if mass then 1 else 0;
      final parameter Integer ndimVis=if animation then 1 else 0;
    public 
      ModelicaAdditions.MultiBody.Parts.Body body[ndimMass](
        each rCM=rCM, 
        each m=m, 
        each I11=I11, 
        each I22=I22, 
        each I33=I33, 
        each I21=I21, 
        each I31=I31, 
        each I32=I32) annotation (extent=[-19, 50; 21, 90]);
    equation 
      
      for i in 1:ndimMass loop
        connect(body[i].frame_a, frame_a);
      end for;
      for i in 1:ndimVis loop
        connect(vis[i].frame_a, frame_a);
      end for;
      connect(frame_a, translation.frame_a)
        annotation (points=[-105, 0; -42, 0]);
      connect(frame_b, translation.frame_b) annotation (points=[105, 0; 42, 0]);
    end Body;
    
    model Shape "Rigid body with visual shape (also used for animation)" 
      extends ModelicaAdditions.MultiBody.Interfaces.OneFrame_a;
      parameter Boolean animation=true 
        "|Animation||True if part should be animated";
      parameter Real Material[4]={1,0,0,0.5} 
        "|Animation||Color and specular coefficient";
      parameter String shape="box" "|Animation||shape";
      parameter SI.Length height=0.03 "|Animation||height of shape";
      parameter SI.Length width=0.03 "|Animation||width of shape";
      parameter SI.Length length=0.03 "|Animation||length of shape";
      parameter SI.Position r0[3]={0,0,0} 
        "|Animation||Vector from frame_a to shape frame, resolved in frame_a";
      parameter SI.Position lengthDirection[3]={1,0,0} 
        "|Animation||Vector in direction of cylinder axis, resolved in frame_a";
      parameter SI.Position widthDirection[3]={0,0,1} 
        "|Animation||Vector in direction perpendiculare to cylinder axis, resolved in frame_a";
      parameter Real material[4]={0.5,0.5,0.5,0.5} 
        "|Animation||Color and specular coefficient";
      parameter Real extra=1.0 "|Animation||Additional parameter";
      
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.29, 
          y=0.12, 
          width=0.61, 
          height=0.68), 
        Documentation(info="
Rigid body with two cuts and an additional shape used in animation.
All the mass and inertia properties must be supplied with respect
to the center of mass. All parameter vectors are resolved in frame_a
in [m].
Parameters:
  Shape        : Type of visual shape. Predefined shapes are
                   Shape = \"box\"
                           \"sphere\"
                           \"cylinder\"
                           \"cone\"
                           \"pipe\"
                           \"beam\"
                           \"gearwheel\"
                           \"wirebox\"
                 External shapes are specified as DXF-files (only 3DFace is supported).
                 External shapes must be named \"1\", \"2\"  etc. The corresponding definitions
                 should be in files \"1.dxf\", \"2.dxf\" etc.Since the DXF-files contain color
                 and dimensions for the individual faces, the corresponding information
                 in the model is currently ignored, but the specular coefficient is
                 utilized. The DXF-files must be found in the current directory.
  r0[3]        : Position vector from the origin of frame_a to the origin
                 of the visual object.
  LengthDirection[3]: Unit vector in direction of length (will be normalized).
  WidthDirection[3] : Unit vector in direction of width (will be normalized).
  Length       : Length of visual shape in [m].
  Width        : Width of visual shape in [m].
  Height       : Height of visual shape in direction of a vector which is
                 orthogonal to  LengthDirection  and  WidthDirection.
  Material[4]  : = {r, g, b, specular} specifies the color of the shape.
                 [r,g,b] affects the color of diffuse and ambient reflected
                 light. Specular is a coefficient defining white specular
                 reflection. What color that is reflected also depends on the
                 color of the light sources. Note, r g, b and specular are
                 given in the range 0 . 1. Specular=1 gives a metallic appearance.
  Extra        : Additional parameter for cone and pipe.
                 \"pipe\"     : A hollow cylinder. Extra = InnerRadius/Radius of the cylinder.
                 \"gearwheel\": A ring with gear wheel teeth. Extra = number of teeth.
"), 
        Icon(
          Rectangle(extent=[-90, 4; -49, -5], style(
              color=0, 
              gradient=2, 
              fillColor=0, 
              fillPattern=1)), 
          Polygon(points=[-50, 34; -50, -60; 0, -40; 40, -60; 40, 36; 0, 12; -50, 
                 34], style(
              color=7, 
              fillColor=46, 
              fillPattern=1)), 
          Polygon(points=[-50, 34; -30, 46; 10, 30; 60, 62; 40, 36; 0, 12; -50, 
                 34], style(
              color=7, 
              fillColor=75, 
              fillPattern=1)), 
          Polygon(points=[59, 61; 40, 36; 40, -60; 58, -36; 59, 61], style(
              color=7, 
              fillColor=75, 
              fillPattern=1)), 
          Text(extent=[-100, 120; 100, 61], string="%name%")), 
        Diagram);
      
      ModelicaAdditions.MultiBody.Parts.Shape vis[ndimVis](
        each Shape=shape, 
        each r0=r0, 
        each LengthDirection=lengthDirection, 
        each WidthDirection=widthDirection, 
        each Length=length, 
        each Width=width, 
        each Height=height, 
        each Material=material, 
        each Extra=extra) annotation (extent=[-20, -90; 20, -50]);
    protected 
      final parameter Integer ndimVis=if animation then 1 else 0;
    equation 
      
      for i in 1:ndimVis loop
        connect(vis[i].frame_a, frame_a);
      end for;
    end Shape;
    
    model Inertia3D 
      "1D-rotational component with inertia taking into account 3D-effects" 
      
      import SI = Modelica.SIunits;
      extends Modelica.Mechanics.Rotational.Interfaces.Rigid(phi(fixed=
              startValueFixed));
      parameter SI.Inertia J=1 "Moment of inertia";
      parameter Real n[3]={1,0,0} "Axle of rotation resolved in bearing_frame";
      parameter Boolean exact=true 
        "true, if exact calculations; false if influence of bearing acceleration is neglected";
      parameter Boolean startValueFixed=false;
      
      SI.AngularVelocity w 
        "Angular velocity of component with respect to bearing_frame";
      SI.AngularAcceleration a 
        "Angular acceleration of component with respect to bearing_frame";
      
    protected 
      SI.Inertia nJ[3]=J*n/sqrt(n*n);
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-80, -25; -60, -25], style(color=0)), 
          Line(points=[60, -25; 80, -25], style(color=0)), 
          Line(points=[-70, -25; -70, -70], style(color=0)), 
          Line(points=[70, -25; 70, -70], style(color=0)), 
          Line(points=[-80, 25; -60, 25], style(color=0)), 
          Line(points=[60, 25; 80, 25], style(color=0)), 
          Line(points=[-70, 45; -70, 25], style(color=0)), 
          Line(points=[70, 45; 70, 25], style(color=0)), 
          Line(points=[-70, -70; 70, -70], style(color=0)), 
          Rectangle(extent=[-50, 50; 50, -50], style(
              color=0, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[-90, 10; -50, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[50, 10; 90, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8)), 
          Text(extent=[0, 117; 0, 57], string="%name"), 
          Text(
            extent=[-1, -115; -1, -165], 
            string="J=%J", 
            style(color=0))), 
        Diagram(
          Line(points=[-80, -25; -60, -25], style(color=0)), 
          Line(points=[60, -25; 80, -25], style(color=0)), 
          Line(points=[-70, -25; -70, -70], style(color=0)), 
          Line(points=[70, -25; 70, -70], style(color=0)), 
          Line(points=[-80, 25; -60, 25], style(color=0)), 
          Line(points=[60, 25; 80, 25], style(color=0)), 
          Line(points=[-70, 45; -70, 25], style(color=0)), 
          Line(points=[70, 45; 70, 25], style(color=0)), 
          Line(points=[-70, -70; 70, -70], style(color=0)), 
          Rectangle(extent=[-50, 50; 50, -50], style(
              color=0, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[-90, 10; -50, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8)), 
          Rectangle(extent=[50, 10; 90, -10], style(
              color=0, 
              gradient=2, 
              fillColor=8)), 
          Polygon(points=[116, -81; 96, -76; 96, -86; 116, -81], style(color=10, 
                 fillColor=10)), 
          Line(points=[26, -81; 97, -81], style(color=10, fillColor=10)), 
          Text(
            extent=[30, -87; 98, -100], 
            string="rotation axis", 
            style(color=10)), 
          Polygon(points=[9, 73; 19, 70; 9, 67; 9, 73], style(color=0, 
                fillColor=0)), 
          Line(points=[9, 70; -21, 70], style(color=0, fillColor=0)), 
          Text(extent=[25, 77; 77, 65], string="w = der(phi) ")), 
        Window(
          x=0.09, 
          y=0.11, 
          width=0.65, 
          height=0.74));
    public 
      ModelicaAdditions.MultiBody.Interfaces.Frame_a bearing_frame annotation (
        extent=[-16, -70; 14, -100], 
        rotation=-90, 
        layer="icon");
    equation 
      
      w = der(phi);
      a = der(w);
      
      if exact then
        J*a = flange_a.tau + flange_b.tau - nJ*bearing_frame.z;
      else
        J*a = flange_a.tau + flange_b.tau;
      end if;
      
      // reaction torque in bearing
      bearing_frame.t = nJ*a + cross(bearing_frame.w, nJ)*w;
      bearing_frame.f = zeros(3);
    end Inertia3D;
  end Parts;
  
  block Filter_CritDamping 
    "Output the input signal filtered with an n-th order filter with critical damping"
     
    
    extends Modelica.Blocks.Interfaces.SISO;
    
    parameter Integer n=2 "Order of filter";
    parameter SI.Frequency f=1 "Cut-off frequency";
    final parameter SI.AngularVelocity w=2*f*Modelica.Constants.pi 
      "Angular frequency";
    output Real x[n] "Filter states";
    annotation (
      Icon(
        Line(points=[-80.6897, 77.6256; -80.6897, -90.3744], style(color=8)), 
        Polygon(points=[-79.7044, 90.6305; -87.7044, 68.6305; -71.7044, 68.6305; 
               -79.7044, 88.6305; -79.7044, 90.6305], style(
            color=8, 
            fillColor=8, 
            fillPattern=1)), 
        Line(points=[-90, -80; 82, -80], style(color=8)), 
        Polygon(points=[90, -80; 68, -72; 68, -88; 90, -80], style(
            color=8, 
            fillColor=8, 
            fillPattern=1)), 
        Text(
          extent=[0, 0; 60, -60], 
          string="PTn", 
          style(color=8)), 
        Line(points=[-80.7599, -80.5082; -70.7599, -74.5082; -56, -60; -48, -42; 
               -42, -18; -36, 4; -26, 20; -10.7599, 34.9018; -0.759907, 38.8218; 
               9.24009, 41.6818; 19.2401, 43.7818; 29.2401, 45.3118; 39.2401, 
              46.4318; 49.2401, 47.2518; 59.2401, 47.8518; 69.2401, 48.2918; 
              79.2401, 48.6118]), 
        Text(
          extent=[-70, 94; 26, 48], 
          string="%n", 
          style(color=8)), 
        Text(
          extent=[8, -106; 8, -146], 
          string="f=%f", 
          style(color=0))), 
      Diagram(
        Line(points=[40, 0; -40, 0], style(color=0)), 
        Text(
          extent=[-55, 55; 55, 5], 
          string="1", 
          style(color=0)), 
        Rectangle(extent=[-60, 60; 60, -60]), 
        Line(points=[-100, 0; -60, 0]), 
        Line(points=[60, 0; 100, 0]), 
        Text(
          extent=[-54, -6; 44, -56], 
          string="(T s + 1)", 
          style(color=0)), 
        Text(
          extent=[38, -10; 58, -30], 
          string="n", 
          style(color=0))), 
      Documentation(info="<html>
<p>This block defines the transfer function between the
input u=inPort.signal[1] and the output y=outPort.signal[1]
as an n-th order filter with <i>critical damping</i>
characteristics and cut-off frequency f=1/T. It is
implemented as a series of first order filters.</p>
<p>If transients at the simulation start shall be avoided
the states x need to be initialized with the start value of
the input.</p>
<pre>
               k
     y = ------------- * u
         (T * s + 1)^n
</pre>
<p><b>Release Notes:</b></p>
<ul>
<li><i>October 17, 2002</i>
       by <a href=\"http://www.robotic.dlr.de/Christian.Schweiger/\">Christian Schweiger</a>:<br>
       Changed to block. Therefore changed variable <b>x</b> to output variable.</li>
<li><i>June 28, 2001</i>
       by Michael Th&uuml;mmel:<br>
       Realized.</li>
</ul>
</HTML>
"));
  equation 
    
    for i in 1:n loop
      der(x[i]) = ((if i == 1 then u else x[i - 1]) - x[i])*w;
    end for;
    
    y = x[n];
    
  initial equation 
    y = u;
    for i in 1:n - 1 loop
      der(x[i]) = 0;
    end for;
  end Filter_CritDamping;
  
  model FilteredPosition 
    "Forced movement of a flange according to a reference angle signal" 
    
    import SI = Modelica.SIunits;
    parameter SI.Frequency f_crit=50 
      "Critical (cut-off) frequency of filter to filter input signal";
    parameter Integer order(min=2) = 2 "Order of filter";
  public 
    Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b
      annotation (extent=[90, -10; 110, 10]);
    Modelica.Blocks.Interfaces.InPort angle(final n=1, redeclare type 
        SignalType = SI.Angle) "Flange angle as input signal"
      annotation (extent=[-140, -20; -100, 20]);
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Window(
        x=0.33, 
        y=0.01, 
        width=0.57, 
        height=0.72), 
      Documentation(info="<HTML>
<p>
The input signal angle.signal[1] defines the <b>reference
angle</b> <b>phi_ref(t)</b> in [rad]. Flange <b>flange_b</b> is <b>forced</b>
to move according to this reference motion. The reference angle is
 <b>filtered</b> and the second derivative
    of the filtered curve is used to compute the reference acceleration
    of the flange. This second derivative is <b>not</b> computed by
    numerical differentiation but by an appropriate realization of the
    filter. For filtering, an n-th order filter with critical
    damping characteristic is used.
    The critical frequency (also called cut-off frequency) of the
    filter is defined via parameter <b>f_crit</b> in [Hz]. This value
    should be selected in such a way that it is higher as the essential
    low frequencies in the signal. The filter is initialized
    in steady state.
</p>
</HTML>
"), 
      Icon(
        Text(
          extent=[-80, -40; -80, -100], 
          string="phi", 
          style(color=0)), 
        Text(extent=[-100, 120; 98, 60], string="%name"), 
        Rectangle(extent=[-100, 20; 90, -20], style(
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Line(points=[-30, -32; 30, -32], style(color=0)), 
        Line(points=[0, 52; 0, 32], style(color=0)), 
        Line(points=[-29, 32; 30, 32], style(color=0)), 
        Line(points=[0, -32; 0, -60], style(color=0))), 
      Diagram);
    
    Modelica.Mechanics.Rotational.Position position(exact=true)
      annotation (extent=[40, -10; 60, 10]);
    Filter_CritDamping filter(n=order, f=f_crit)
      annotation (extent=[-40, -10; -20, 10]);
  equation 
    connect(position.flange_b, flange_b) annotation (points=[60, 0; 98, 0], 
        style(
        color=0, 
        fillColor=7, 
        fillPattern=1));
    connect(filter.inPort, angle)
      annotation (points=[-42, 0; -104, 0], style(color=3));
    connect(filter.outPort, position.inPort)
      annotation (points=[-20, 0; 36, 0], style(color=3));
  end FilteredPosition;
  
  encapsulated package Polynomial "Components to work with polynomials" 
    
    import Modelica.Blocks;
    import Modelica.Icons;
    import Thermal.MoveToModelica;
    
    extends Icons.Library;
    
    /*
   The following function gives currently an error during translation since
   c=c seem to not work for arrays with unknown dimensions
           function getPolynomial
  input Real u_min;
  input Real u_max;
  input Real c[:];
  output ScaledPolynomial poly(
    u_min=u_min,
    u_max=u_max,
    c=c);
      annotation (Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]));
           end getPolynomial;
*/
    
    record ScaledPolynomial 
      "Polynomial y_scaled=f( (u-u_min)/(u_max - u_min) )" 
      
      replaceable type uType = Real "Type of abscissa u";
      replaceable type yType = Real "Type of ordinate y";
      parameter uType u_min=0 
        "Minimum value of u for which polynomial is valid";
      parameter uType u_max=1 
        "Maximum value of u for which polynomial is valid";
      parameter yType y_min=0 
        "Minimum value of y for which polynomial is valid";
      parameter yType y_max=1 
        "Maximum value of y for which polynomial is valid";
      parameter Real c[:]={1} 
        "Coefficients of polynomial scaled with u_min, u_max, y_min, y_max";
      parameter yType y_umin "Value of y at u_min";
      parameter Real der_y_umin "First derivative of y at u_min";
      parameter yType y_umax "Value of y at u_max";
      parameter Real der_y_umax "First derivative of y at u_max";
      parameter Real original[:, 2] 
        "Original data from which polynomial has been constructed [u(i), y(i)] pairs (data is not used)";
      annotation (
        Icon(
          Rectangle(extent=[-100, 50; 100, -100], style(color=0, fillColor=7)), 
             
          Text(extent=[-127, 119; 127, 59], string="%name"), 
          Line(points=[-100, -50; 100, -50], style(color=0)), 
          Line(points=[-100, 0; 100, 0], style(color=0)), 
          Line(points=[0, 50; 0, -100], style(color=0)), 
          Text(
            extent=[-94, 45; -6, 6], 
            string="u_min", 
            style(color=10)), 
          Text(
            extent=[-94, -5; -6, -44], 
            string="u_max", 
            style(color=10)), 
          Text(
            extent=[-96, -57; -8, -96], 
            string="c", 
            style(color=10))), 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.21, 
          y=0.19, 
          width=0.6, 
          height=0.6), 
        Documentation(info="The polynomial y = f(u) is defined as
  du := (u - u_min) / (u_max - u_min);
  dy := c[1]*du^n + c[2]*du^(n-1) + ... + c[n]*du + c[n+1];
  y  := y_min + (y_max - y_min)*dy;
where u is the abszissa, y is the ordinate and c is the vector
of coefficients of the scaled polynomial.
As a result, the abszissa du and the ordinate dy of the
scaled polynomial p are in the range 0.1.
Outside of this range, the polynomial is replaced by the tangents
at the u_min and the u_max points of the polynomial.
"));
      
    end ScaledPolynomial;
    
    block Evaluation "Evaluate scaled polynomial with input signal" 
      
      extends Blocks.Interfaces.SISO;
      parameter ScaledPolynomial poly "Scaled polynomial"
        annotation (extent=[-80, 60; -40, 100]);
    protected 
      Real du=(u - poly.u_min)/(poly.u_max - poly.u_min);
      Real dy;
      Real w[size(poly.c, 1) - 1] "Auxiliary vector";
      annotation (
        Icon(
          Polygon(points=[0, 90; -8, 68; 8, 68; 0, 90], style(color=8, 
                fillColor=8)), 
          Text(
            extent=[-86, 80; -14, 32], 
            string="poly", 
            style(color=8)), 
          Line(points=[0, -80; 0, 68], style(color=8)), 
          Line(points=[-90, 0; 68, 0], style(color=8)), 
          Line(points=[-80, -80; -76, -65.4; -71.2, -51.4; -65.5, -38.8; -59.1, 
                 -28.1; -51.1, -18.7; -41.4, -11.4; -27.7, -5.5; -4.42, -0.653; 
                 24.5, 4.57; 39, 10.1; 49.4, 17.2; 57.5, 25.9; 63.9, 35.8; 69.5, 
                 47.4; 74.4, 60.4; 78.4, 73.8; 80, 80], style(color=0)), 
          Polygon(points=[90, 0; 68, 8; 68, -8; 90, 0], style(color=8, 
                fillColor=8))), 
        Diagram(
          Line(points=[0, -90; 0, 84], style(color=8)), 
          Text(
            extent=[9, 98; 34, 78], 
            string="outPort", 
            style(color=9)), 
          Polygon(points=[0, 100; -6, 84; 6, 84; 0, 100], style(color=8, 
                fillColor=8)), 
          Line(points=[-100, 0; 84, 0], style(color=8)), 
          Polygon(points=[100, 0; 84, 6; 84, -6; 100, 0], style(color=8, 
                fillColor=8)), 
          Line(points=[-80, -80; -76, -65.4; -71.2, -51.4; -65.5, -38.8; -59.1, 
                 -28.1; -51.1, -18.7; -41.4, -11.4; -27.7, -5.5; -4.42, -0.653; 
                 24.5, 4.57; 39, 10.1; 49.4, 17.2; 57.5, 25.9; 63.9, 35.8; 69.5, 
                 47.4; 74.4, 60.4; 78.4, 73.8; 80, 80], style(color=0)), 
          Text(
            extent=[72, -6; 96, -26], 
            string="inPort", 
            style(color=9))), 
        Documentation(info="<HTML>
<p>
This blocks computes the output as <b>polynomial</b> value of the input:
</p>
<pre>
   du = (u - u_min)/(u_max - u_min);
   dy = c[1]*du^n + c[2]*du^(n-1) + ... + c[n]*du + c[n+1];
   y  = y_min + (y_max - y_min)*dy;
   where
      <b>u   : input</b> signal (= inPort.signal[1])
      <b>y   : output</b> signal (= outPort.signal[1])
      <b>c[:]: coefficients</b> of scaled polynomial as vector with n+1 elements
</pre>
<p>
If u is smaller as u_min or larger as u_max, the output is the tangent
on the polynominal at the corresponding point:
</p>
<pre>
   if u < u_min then 
      y = y_umin + der_y_umin*(u - u_min);
   elseif u > u_max then
      y = y_umax + der_y_umax*(u - u_max);
   end if;
</pre>
<p>
Example:
</p>
<pre>
   c := {2,3,1};
   u := 2
   results in
     y := 2*2^2 + 3*2 + 1
       := 15
</pre>
</HTML>
"), 
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Window(
          x=0.04, 
          y=0.01, 
          width=0.72, 
          height=0.72));
    equation 
      
      // Evaluate polynomial with Horners scheme
      [w; dy] = [poly.c] + du*[0; w];
      y = if noEvent(u < poly.u_min) then poly.y_umin + poly.der_y_umin*(u - 
        poly.u_min) else if noEvent(u > poly.u_max) then poly.y_umax + poly.
        der_y_umax*(u - poly.u_max) else poly.y_min + (poly.y_max - poly.y_min)
        *dy;
    end Evaluation;
    annotation (Coordsys(
        extent=[0, 0; 491, 168], 
        grid=[1, 1], 
        component=[20, 20]), Window(
        x=0.12, 
        y=0.13, 
        width=0.49, 
        height=0.28, 
        library=1, 
        autolayout=1));
    function value "Evaluate polynomial with Horner's scheme" 
      extends Icons.Function;
      input ScaledPolynomial poly;
      input Real u "Value at which polynomial poly shall be evaluated";
      output Real y "Polynomial value at u";
    protected 
      Real du=(u - poly.u_min)/(poly.u_max - poly.u_min);
    algorithm 
      if u < poly.u_min then
        y := poly.y_umin + poly.der_y_umin*(u - poly.u_min);
      elseif u > poly.u_max then
        y := poly.y_umax + poly.der_y_umax + (u - poly.u_max);
      else
        y := poly.c[1];
        for j in 2:size(u, 1) loop
          y := poly.c[j] + du*y;
        end for;
        y := poly.y_min + (poly.y_max - poly.y_min)*y;
      end if;
    end value;
    
    function newScaledPolynomial "Generate a new ScaledPolynomial record" 
      extends Icons.Function;
      input Real u_min=0 "Minimum value of u for which polynomial is valid";
      input Real u_max=1 "Maximum value of u for which polynomial is valid";
      input Real y_min=0 "Minimum value of y for which polynomial is valid";
      input Real y_max=1 "Maximum value of y for which polynomial is valid";
      
      input Real c[:]={1} 
        "Coefficients of polynomial scaled with u_min, u_max, y_min, y_max";
      input Real y_umin "Value of y at u_min";
      input Real der_y_umin "First derivative of y at u_min";
      input Real y_umax "Value of y at u_max";
      input Real der_y_umax "First derivative of y at u_max";
      input Real original[:, 2] 
        "Original data from which polynomial has been constructed [u(i), y(i)] pairs (data is not used)";
      output ScaledPolynomial result(
        u_min=u_min, 
        u_max=u_max, 
        y_min=y_min, 
        y_max=y_max, 
        c=c, 
        y_umin=y_umin, 
        der_y_umin=der_y_umin, 
        y_umax=y_umax, 
        der_y_umax=der_y_umax, 
        original=original);
    end newScaledPolynomial;
  end Polynomial;
  
  package Sensors 
    model CamberAngle 
      extends ModelicaAdditions.MultiBody.Interfaces.Interact2;
      
      Real camberAngle;
      Real casterAngle;
      Real toeInAngle;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Icon(
          Line(points=[-60, 20; -76, 68], style(thickness=2)), 
          Polygon(points=[-76, 68; -74, 54; -72, 62; -76, 68], style(fillColor=
                  73)), 
          Line(points=[-60, 20; -72, -32], style(thickness=2)), 
          Polygon(points=[-72, -20; -72, -32; -68, -26; -72, -20], style(
                fillColor=73)), 
          Line(points=[-60, 20; 30, -26], style(color=0, pattern=3)), 
          Line(points=[30, -26; 6, -62], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Line(points=[30, -26; 14, 16], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Polygon(points=[14, 16; 14, 4; 20, 10; 14, 16], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Polygon(points=[6, -62; 10, -50; 14, -56; 6, -62], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Polygon(points=[70, -50; 72, -44; 80, -52; 70, -50], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Line(points=[30, -26; 80, -52], style(
              color=41, 
              thickness=2, 
              fillColor=41)), 
          Line(points=[-10, -6; -4, -2; 2, 4; 6, 10; 8, 16; 8, 20], style(color
                =0, thickness=2)), 
          Line(points=[-60, 20; 32, 20], style(color=0, pattern=3)), 
          Polygon(points=[-6, 20; -16, 24; -16, 16; -6, 20], style(fillColor=73)), 
             
          Line(points=[-60, 20; -6, 20], style(thickness=2)), 
          Polygon(points=[-10, -6; -6, 0; 0, -2; -10, -6], style(
              color=0, 
              thickness=2, 
              fillColor=0)), 
          Rectangle(extent=[-90, 90; 90, -90])), 
        Window(
          x=0.25, 
          y=0.23, 
          width=0.6, 
          height=0.6), 
        Documentation(info="This component measues the angles between two frames. If used between the hub and the ground, camber, caster and toe-in can be mesaured-
"));
    equation 
      
      camberAngle = Modelica.Math.acos(Sa[3, :]*Sb[3, :]);
      casterAngle = Modelica.Math.acos(Sa[2, :]*Sb[2, :]);
      toeInAngle = Modelica.Math.acos(Sa[1, :]*Sb[1, :]);
      ta = zeros(3);
      tb = zeros(3);
      fa = zeros(3);
      fb = zeros(3);
    end CamberAngle;
    
    model RelPositionSensor "Relative position sensor" 
      SI.Position r_rela[3];
      SI.Velocity v_rela[3];
      Real a_rela[3];
      Real vaux[3];
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), 
        Window(
          x=0.04, 
          y=0, 
          width=0.86, 
          height=0.78), 
        Icon(
          Text(extent=[-162, 31; 163, 91], string="%name"), 
          Rectangle(extent=[-80, -10; -60, -90], style(color=6, fillColor=6)), 
          Rectangle(extent=[-90, 10; 90, -10], style(color=3, fillColor=6)), 
          Line(points=[-70, -11; -70, -91], style(color=3)), 
          Text(
            extent=[-60, -82; 90, -111], 
            string="r_rela", 
            style(color=0))), 
        Diagram);
      
      Modelica.Blocks.Interfaces.OutPort relativePosition(final n=9) 
        annotation (
        extent=[-80, -112; -60, -92], 
        rotation=-90, 
        layer="icon");
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_a
        annotation (extent=[-120, -14; -90, 15], layer="icon");
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_b annotation (
        extent=[90, 15; 120, -15], 
        rotation=-180, 
        layer="icon");
    equation 
      /*relative position vectors*/
      r_rela = transpose(frame_a.S)*(frame_b.r0 - frame_a.r0);
      vaux = cross(frame_a.w, r_rela);
      v_rela = transpose(frame_a.S)*(frame_b.S*frame_b.v) - frame_a.v - vaux;
      a_rela = transpose(frame_a.S)*(frame_b.S*frame_b.a) - frame_a.a - cross(
        frame_a.z, r_rela) - cross(frame_a.w, vaux + 2*v_rela);
      
      [relativePosition.signal] = [r_rela; v_rela; a_rela];
      
      frame_a.f = zeros(3);
      frame_b.f = zeros(3);
      frame_a.t = zeros(3);
      frame_b.t = zeros(3);
    end RelPositionSensor;
  end Sensors;
end Utilities;


package Environments 
  extends Modelica.Icons.Library;
  
  annotation (Coordsys(
      extent=[0, 0; 442, 386], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.08, 
      y=0.13, 
      width=0.44, 
      height=0.65, 
      library=1, 
      autolayout=1));
  package Interfaces 
    annotation (Coordsys(
        extent=[0, 0; 440, 450], 
        grid=[2, 2], 
        component=[20, 20]), Window(
        x=0.45, 
        y=0.01, 
        width=0.28, 
        height=0.42, 
        library=1, 
        autolayout=1));
    partial block TyreRoadInterface 
      input Real x "x position";
      input Real y "y position";
      //input Real v "velocity";
      output Real z "Road altitude";
      output Real[3] n "Road normal";
      output Real mue "Road adhesion";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), 
        Icon(
          Rectangle(extent=[-100, 40; 100, -40.2]), 
          Polygon(points=[-25, 36.2; -99.2, -20; -99.2, -38.8; 99, -38.8; 99, -20; 
                 20, 36.2; -25, 36.2], style(color=10, fillColor=10)), 
          Polygon(points=[-7.2, -34; 7.8, -34; 4.4, 0; -4.2, 0; -7.2, -34], 
              style(color=8, fillColor=8)), 
          Polygon(points=[-3.2, 16; 3.2, 16; 2, 30; -2, 30; -3.2, 16], style(
                color=8, fillColor=8)), 
          Text(extent=[-100, 100; 100, 40], string="%name")), 
        Window(
          x=0.45, 
          y=0.01, 
          width=0.35, 
          height=0.49));
    end TyreRoadInterface;
    
    partial block WeatherInterface 
      input SI.Position[3] r "position";
      
      output Real[3] v_wind "Wind speed in global system";
      output Real rho "Air density";
      
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[2, 2], 
          component=[20, 20]), Icon(
          Rectangle(extent=[-100, 40; 100, -40.2], style(fillColor=75)), 
          Text(extent=[-100, 100; 100, 40], string="%name"), 
          Ellipse(extent=[-64, -2; -30, -26], style(pattern=0, fillColor=7)), 
          Ellipse(extent=[-56, 16; 36, -26], style(pattern=0, fillColor=7)), 
          Ellipse(extent=[-48, 26; -14, 2], style(pattern=0, fillColor=7)), 
          Ellipse(extent=[-40, -12; -6, -36], style(pattern=0, fillColor=7)), 
          Ellipse(extent=[-16, -4; 28, -28], style(pattern=0, fillColor=7)), 
          Ellipse(extent=[16, 6; 50, -18], style(pattern=0, fillColor=7)), 
          Ellipse(extent=[-22, 22; 32, -2], style(pattern=0, fillColor=7))));
      
    end WeatherInterface;
  end Interfaces;
  
  block SplitMueRoad 
    extends Interfaces.TyreRoadInterface;
    parameter Real mue1=0.7;
    parameter Real mue2=0.1;
    parameter Real mue3=0.7;
    parameter SI.Position x0=-200;
    parameter SI.Position x1=50;
    parameter SI.Position x2=100;
    parameter SI.Position x3=200;
    parameter SI.Position y0=14;
    
  protected 
    parameter Real eps=1;
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Icon, 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.35, 
        height=0.49), 
      Diagram(
        Rectangle(extent=[-88, 94; 96, -12], style(color=0, fillColor=58)), 
        Rectangle(extent=[-88, 60; -34, 20], style(
            color=0, 
            fillColor=0, 
            fillPattern=1)), 
        Rectangle(extent=[-34, 60; 22, 20], style(
            color=0, 
            fillColor=8, 
            fillPattern=1)), 
        Rectangle(extent=[22, 60; 96, 20], style(
            color=0, 
            fillColor=0, 
            fillPattern=1)), 
        Text(
          extent=[-94, -22; -72, -42], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="x"), 
        Text(
          extent=[-86, -28; -64, -48], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="0"), 
        Text(
          extent=[-44, -22; -22, -42], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="x"), 
        Text(
          extent=[-36, -28; -14, -48], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="1"), 
        Text(
          extent=[10, -22; 32, -42], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="x"), 
        Text(
          extent=[18, -28; 40, -48], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="2"), 
        Text(
          extent=[84, -22; 106, -42], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="x"), 
        Text(
          extent=[92, -28; 114, -48], 
          style(
            color=0, 
            fillColor=8, 
            fillPattern=1), 
          string="3")));
    parameter Real a=-0.05;
  protected 
    VisualShape section1(
      Height=0.1, 
      r0={x0,0,0.02}, 
      Length=x1 - x0, 
      Width=y0, 
      Material={1/mue1,1/mue1,1/mue1,1/mue1}*0.1)
      annotation (extent=[-72, 30; -52, 50]);
    VisualShape section2(
      Height=0.1, 
      r0={x1,0,0.02}, 
      Length=x2 - x1, 
      Width=y0, 
      Material={1/mue2,1/mue2,1/mue2,1/mue2}*0.1)
      annotation (extent=[-14, 32; 6, 52]);
    VisualShape section3(
      Height=0.1, 
      r0={x2,0,0.02}, 
      Length=x3 - x2, 
      Width=y0, 
      Material={1/mue3,1/mue3,1/mue3,1/mue3}*0.1)
      annotation (extent=[48, 30; 68, 50]);
    VisualShape suroundings(
      Height=0.1, 
      Material={0.4,0.6,0.1,0.3}, 
      r0={x0,0,0.02}, 
      Length=x3 - x0, 
      Width=y0*4) annotation (extent=[-76, 72; -56, 92]);
  equation 
    mue = noEvent(if (x < x1) then mue1 else (if (x < x2) then mue2 else mue3));
    n = {0,0,1};
    z = 0;
    
    section1.S = identity(3);
    section1.r = {0,0,a};
    section2.S = identity(3);
    section2.r = {0,0,a};
    section3.S = identity(3);
    section3.r = {0,0,a};
    suroundings.S = identity(3);
    suroundings.r = {0,0,a - 0.1};
    
  end SplitMueRoad;
  
  block NoGraphicsRoad 
    extends Interfaces.TyreRoadInterface;
    
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), 
      Icon, 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.35, 
        height=0.49), 
      Diagram);
  equation 
    
    mue = 0.7;
    n = {0,0,1};
    z = 0;
    
  end NoGraphicsRoad;
  
  block ConstantWind 
    extends Interfaces.WeatherInterface;
    
    parameter SI.Velocity[3] v_Wind={0,0,0} "Wind velocity";
    parameter SI.Density rho_air=1 "Air density";
    annotation (Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[2, 2], 
        component=[20, 20]), Window(
        x=0.45, 
        y=0.01, 
        width=0.35, 
        height=0.49));
  equation 
    v_wind = v_Wind;
    rho = rho_air;
  end ConstantWind;
end Environments;


package PowerTrain 
  extends Modelica.Icons.Library;
  package Interfaces "Interface base classes for powertrains" 
    extends Modelica.Icons.Library;
    model PowerTrain2WInterface 
      
      parameter Real k=1 
        "Gain factor from accelerator angle to drive troque apply [Nm/rad]";
      
      annotation (Diagram, Icon);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_2
        annotation (extent=[50, 50; 70, 70]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_1
        annotation (extent=[50, -70; 70, -50]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_Long
        annotation (extent=[-110, -40; -90, -20]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-110, 20; -90, 40]);
    end PowerTrain2WInterface;
    
    model PowerTrain4WInterface 
      
      parameter Real k=1 
        "Gain factor from accelerator angle to drive troque apply [Nm/rad]";
      
      annotation (Diagram, Icon);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_2
        annotation (extent=[-70, 50; -50, 70]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_1
        annotation (extent=[-70, -70; -50, -50]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_Long
        annotation (extent=[-110, -40; -90, -20]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_3
        annotation (extent=[50, 50; 70, 70]);
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_4
        annotation (extent=[50, -70; 70, -50]);
      ModelicaAdditions.MultiBody.Interfaces.Frame_a frame_C
        annotation (extent=[-110, 20; -90, 40]);
    end PowerTrain4WInterface;
  end Interfaces;
  
  model SimplePowerTrain 
    extends Interfaces.PowerTrain2WInterface;
    parameter Real k=1 
      "Gain factor from accelerator angle to drive troque apply [Nm/rad]";
    
    Differential differential annotation (extent=[80, -20; 40, 20], rotation=90);
    Modelica.Mechanics.Rotational.Torque torque
      annotation (extent=[-8, -20; 32, 20]);
    annotation (Diagram, Icon(
        Rectangle(extent=[54, -10; 66, -50], style(
            color=0, 
            gradient=1, 
            fillColor=8, 
            fillPattern=1)), 
        Rectangle(extent=[54, 50; 66, 10], style(
            color=0, 
            gradient=1, 
            fillColor=8, 
            fillPattern=1)), 
        Polygon(points=[40, 6; 40, 6; 40, -6; 54, -18; 66, -18; 66, 20; 54, 20; 
               40, 6], style(color=0, fillColor=8)), 
        Rectangle(extent=[-26, 6; 40, -6], style(
            color=0, 
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Line(points=[-80, 0; -66, 12; -52, 14; -42, 12; -28, 0; -28, 0], style(
            color=0, 
            fillColor=8, 
            fillPattern=1)), 
        Polygon(points=[-36, 2; -26, -2; -30, 8; -36, 2], style(
            color=0, 
            fillColor=0, 
            fillPattern=1)), 
        Text(extent=[40, -18; 80, -46], string="L")));
    Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor
      annotation (extent=[-78, -10; -58, 10]);
    Modelica.Blocks.Math.Gain gain(k={k})
      annotation (extent=[-50, -10; -30, 10]);
  equation 
    connect(torque.flange_b, differential.flange_engine)
      annotation (points=[32, 0; 40, -1.22461e-015], style(color=0));
    connect(gain.inPort, angleSensor.outPort)
      annotation (points=[-52, 0; -57, 0], style(color=3));
    connect(gain.outPort, torque.inPort)
      annotation (points=[-29, 0; -12, 0], style(color=3));
    connect(angleSensor.flange_a, flange_Long)
      annotation (points=[-78, 0; -100, -30], style(color=0));
    connect(differential.flange_left, flange_1)
      annotation (points=[60, -20; 60, -60], style(color=0));
    connect(differential.flange_right, flange_2)
      annotation (points=[60, 20; 60, 60], style(color=0));
  end SimplePowerTrain;
  
  model Differential "Differential gear box" 
    parameter Real ratio=1 "gear ratio";
    Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_engine 
      "Flange of engine" annotation (extent=[-10, -110; 10, -90]);
    Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_left 
      "Left (wheel) flange" annotation (extent=[-110, -10; -90, 10]);
    Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_right 
      "right (wheel) flange" annotation (extent=[90, -10; 110, 10]);
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[1, 1], 
        component=[20, 20]), 
      Window(
        x=0.06, 
        y=0.03, 
        width=0.73, 
        height=0.69), 
      Icon(
        Text(extent=[-100, 76; 100, 26], string="%name=%ratio"), 
        Rectangle(extent=[-90, 10; -50, -10], style(
            color=0, 
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Polygon(points=[-50, 20; -50, -20; -20, -50; 20, -50; 50, -20; 50, 20; 
              -50, 20], style(
            color=9, 
            fillColor=9, 
            fillPattern=1)), 
        Rectangle(extent=[90, 10; 50, -10], style(
            color=0, 
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Rectangle(extent=[-10, -50; 10, -90], style(
            color=0, 
            gradient=1, 
            fillColor=8, 
            fillPattern=1))), 
      Diagram(
        Rectangle(extent=[-10, -50; 10, -90], style(
            color=0, 
            gradient=1, 
            fillColor=8, 
            fillPattern=1)), 
        Rectangle(extent=[90, 10; 50, -10], style(
            color=0, 
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Rectangle(extent=[-90, 10; -50, -10], style(
            color=0, 
            gradient=2, 
            fillColor=8, 
            fillPattern=1)), 
        Polygon(points=[-50, 20; -50, -20; -20, -50; 20, -50; 50, -20; 50, 20; 
              -50, 20], style(
            color=9, 
            fillColor=9, 
            fillPattern=1)), 
        Polygon(points=[126, 3; 136, 0; 126, -3; 126, 3; 126, 3], style(color=0, 
               fillColor=0)), 
        Line(points=[126, 0; 110, 0], style(color=0, fillColor=0)), 
        Text(extent=[110, 18; 140, 1], string="flange axis"), 
        Line(points=[-110, 0; -126, 0], style(color=0, fillColor=0)), 
        Polygon(points=[-126, 3; -136, 0; -126, -3; -126, 3; -126, 3], style(
              color=0, fillColor=0)), 
        Text(extent=[-140, 20; -110, 3], string="flange axis"), 
        Text(extent=[5, -120; 35, -137], string="flange axis"), 
        Line(points=[0, -140; 0, -120], style(color=0, fillColor=0)), 
        Polygon(points=[-3, -120; 0, -110; 3, -120; -3, -120], style(
            color=0, 
            fillColor=0, 
            fillPattern=1))), 
      Documentation(info="
<HTML>
<p>
The differential gear box is an ideal gear without inertia, splitting
the driving torque of an engine into equal parts for two driven
output flanges which usually drive the wheels of a vehicle. An optional
gear ratio is supported (ratio > 1 means that the output flanges rotate
slower than the engine flange). If inertia has to be considered,
the gear inertias have to be added to the wheel shafts.
If an efficiency factor should be considered, attach model
Modelica.Mechanics.Rotational.GearEfficiency at flange_engine.
</p>
<p><b>Release Notes:</b></p>
<ul>
<li><i>June 30, 1999</i>
       by <a href=\"http://www.op.dlr.de/~otter/\">Martin Otter</a>:<br>
       Realized.
</li>
</ul>
</HTML>
"));
  equation 
    flange_engine.phi = (flange_left.phi + flange_right.phi)*ratio/2;
    flange_left.tau = flange_right.tau;
    -ratio*flange_engine.tau = flange_left.tau + flange_right.tau;
  end Differential;
end PowerTrain;


package Aerodynamics 
  extends Modelica.Icons.Library;
  annotation (Coordsys(
      extent=[0, 0; 504, 364], 
      grid=[1, 1], 
      component=[20, 20]), Window(
      x=0, 
      y=0.6, 
      width=0.4, 
      height=0.4, 
      library=1, 
      autolayout=1));
  model BasicAirResistance 
    extends Interfaces.AerodynamicsBase;
    
    parameter Real c_w_x=0.32;
    parameter Real a_x=1.2;
    parameter Real c_w_y=0.52;
    parameter Real a_y=2.2;
    
    SI.Velocity v_tot[3];
    SI.Velocity v_x;
    SI.Velocity v_y;
    SI.Velocity v_wind[3];
    SI.Density rho;
    outer block Weather = Environments.Interfaces.WeatherInterface;
    Weather weather;
    
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100], 
        grid=[1, 1], 
        component=[20, 20]), 
      Icon, 
      Window(
        x=0.45, 
        y=0.01, 
        width=0.35, 
        height=0.49));
  equation 
    weather.v_wind = v_wind;
    weather.rho = rho;
    weather.r = r0a;
    
    v_tot = va - transpose(Sa)*v_wind;
    
    v_x = v_tot[1];
    v_y = v_tot[2];
    fa = {c_w_x*rho*a_x*v_x*v_x,0,0};
    ta = zeros(3);
    //to be extended later
  end BasicAirResistance;
  
  package Interfaces 
    partial model AerodynamicsBase 
      extends ModelicaAdditions.MultiBody.Interfaces.OneFrame_a;
      annotation (Coordsys(
          extent=[-100, -100; 100, 100], 
          grid=[1, 1], 
          component=[20, 20]), Icon(
          Polygon(points=[-55, 81; -69, 57; -53, 36; -53, 51; -40, 56; -25, 51; 
                 -15, 48; 1, 48; 12, 53; 21, 58; 32, 58; 41, 54; 49, 50; 55, 49; 
                 62, 53; 69, 57; 69, 69; 57, 62; 50, 63; 39, 70; 21, 70; 9, 65; 
                 4, 62; -5, 59; -10, 59; -20, 62; -25, 62; -34, 65; -38, 67; -44, 
                 67; -46, 65; -53, 63; -55, 81], style(fillColor=73)), 
          Polygon(points=[-54, -40; -68, -64; -52, -85; -52, -70; -39, -65; -24, 
                 -70; -14, -73; 2, -73; 13, -68; 22, -63; 33, -63; 42, -67; 50, 
                 -71; 56, -72; 63, -68; 70, -64; 70, -52; 58, -59; 51, -58; 40, 
                 -51; 22, -51; 10, -56; 5, -59; -4, -62; -9, -62; -19, -59; -24, 
                 -59; -33, -56; -37, -54; -43, -54; -45, -56; -52, -58; -54, -40], 
               style(fillColor=73)), 
          Text(extent=[-99, 81; 99, 137], string="%name")));
    end AerodynamicsBase;
  end Interfaces;
end Aerodynamics;
end VehicleDynamics;
